/* cnic_drv.h: Broadcom CNIC core network driver.
 *
 * Copyright (c) 2008 Broadcom Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation.
 *
 */


#ifndef CNIC_DRV_H
#define CNIC_DRV_H

#if !defined(__iomem)
#define __iomem
#endif

struct kwqe;
struct kcqe;
struct kwqe_16;

struct cnic_ops {
	struct module	*cnic_owner;
	void		(*cnic_stop)(void *);
	void		(*cnic_start)(void *);
	int		(*cnic_handler)(void *, void *);
	unsigned long	reserved[2];
};

#define MAX_CNIC_VEC	8

struct cnic_irq {
	unsigned int	vector;
	void		*status_blk;
	u32		status_blk_num;
	u32		irq_flags;
#define CNIC_IRQ_FL_MSIX		0x00000001
};

struct cnic_eth_dev {
	u32		drv_flags;
#define CNIC_DRV_FL_USING_MSIX		0x00000001
	u32		chip_id;
	u32		max_kwqe_pending;
	struct pci_dev	*pdev;
	void __iomem	*io_base;

	u32		ctx_tbl_offset;
	u32		ctx_tbl_len;
	int		ctx_blk_size;
	u32		starting_cid;

	int		num_irq;
	struct cnic_irq	irq_arr[MAX_CNIC_VEC];
	int		(*drv_submit_kwqes_32)(struct net_device *,
					       struct kwqe *[], u32);
	int		(*drv_submit_kwqes_16)(struct net_device *,
					       struct kwqe_16 *[], u32);
	void		(*drv_reg_wr)(struct net_device *, u32, u32);
	u32		(*drv_reg_rd)(struct net_device *, u32);
	int		(*drv_ctx_wr)(struct net_device *, u32, u32, u32);
	int		(*drv_ctx_tbl_wr)(struct net_device *, u32, dma_addr_t);
	unsigned long	reserved[2];
};

#ifdef __VMKLNX__
extern struct cnic_eth_dev *bnx2_register_cnic(struct net_device *,
	struct cnic_ops *, void *);
extern int bnx2_unregister_cnic(struct net_device *);
#else
extern struct cnic_eth_dev *bnx2_register_cnic(struct net_device *,
	struct cnic_ops *, void *) __attribute__((weak));
extern int bnx2_unregister_cnic(struct net_device *) __attribute__((weak));

extern struct cnic_eth_dev *bnx2x_register_cnic(struct net_device *,
	struct cnic_ops *, void *) __attribute__((weak));
extern int bnx2x_unregister_cnic(struct net_device *) __attribute__((weak));
#endif

#endif
