/*
 * This file is part of the Chelsio T3 Ethernet driver.
 *
 * Copyright (C) 2003-2007 Chelsio Communications.  All rights reserved.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the LICENSE file included in this
 * release for licensing terms and conditions.
 */
#ifndef __CXGB3_COMPAT_H
#define __CXGB3_COMPAT_H

#include <linux/version.h>
#include "common.h"
#include <linux/pci.h>

#include <linux/debugfs.h>

static inline int t3_os_pci_save_state(adapter_t *adapter)
{
	return pci_save_state(adapter->pdev);
}

static inline int t3_os_pci_restore_state(adapter_t *adapter)
{
	return pci_restore_state(adapter->pdev);
}


#if !defined(NETEVENT)
struct notifier_block;

static inline void register_netevent_notifier(struct notifier_block *nb)
{}

static inline void unregister_netevent_notifier(struct notifier_block *nb)
{}

#if defined(CONFIG_TCP_OFFLOAD_MODULE) && !defined(CONFIG_IA64)
#define OFLD_USE_KPROBES
#endif

#else
extern int netdev_nit;
#endif



typedef irqreturn_t (*intr_handler_t)(int, void *, struct pt_regs *);
#define DECLARE_INTR_HANDLER(handler, irq, cookie, regs) \
	static irqreturn_t handler(int irq, void *cookie, struct pt_regs *regs)

intr_handler_t t3_intr_handler(struct adapter *adap, int polling);
static inline void t3_poll_handler(struct adapter *adapter,
				   struct sge_qset *qs)
{
	t3_intr_handler(adapter, qs->rspq.polling) (0,
		(adapter->flags & USING_MSIX) ? (void *)qs : (void *)adapter,
		NULL);
}

#define CHECKSUM_PARTIAL CHECKSUM_HW
#define CHECKSUM_COMPLETE CHECKSUM_HW

#ifndef I_PRIVATE
#define i_private u.generic_ip
#endif


#define DECLARE_TASK_FUNC(task, task_param) \
	static void task(void *task_param)

#define WORK2ADAP(task_param, task) task_param
#define DELWORK2ADAP(task_param, task) task_param
#define WORK2T3CDATA(task_param, task) task_param

#define delayed_work work_struct

#define T3_INIT_WORK INIT_WORK
#define T3_INIT_DELAYED_WORK INIT_WORK


#if defined(CONFIG_FW_LOADER) || defined(CONFIG_FW_LOADER_MODULE)
#include <linux/firmware.h>
#else
struct firmware {
	size_t size;
	u8 *data;
};

struct device;

static inline int request_firmware(const struct firmware **firmware_p,
				   char *name,
				   struct device *device)
{
	printk(KERN_WARNING
	       "FW_LOADER not set in this kernel. FW upgrade aborted.\n");
	return -1;
}

static inline void release_firmware(const struct firmware *fw)
{}
#endif /* FW_LOADER */

#ifndef NIPQUAD_FMT
#define NIPQUAD_FMT "%u.%u.%u.%u"
#endif

/* sysfs compatibility */

#define to_net_dev(class) container_of(class, struct net_device, class_dev)

#define cxgb3_compat_device class_device

#define CXGB3_SHOW_FUNC(func, d, attr, buf)			\
	static ssize_t func(struct cxgb3_compat_device *d,	\
			    char *buf)				\

#define CXGB3_STORE_FUNC(func, d, attr, buf, len)		\
	static ssize_t func(struct cxgb3_compat_device *d,	\
			    const char *buf,			\
			    size_t len)

#define CXGB3_DEVICE_ATTR(_name,_mode,_show,_store)		\
struct class_device_attribute dev_attr_##_name = 		\
	__ATTR(_name,_mode,_show,_store)

static inline struct kobject *net2kobj(struct net_device *dev)
{
	return &dev->class_dev.kobj;
}


#if !defined(VLANGRP)
#include <linux/if_vlan.h>
static inline struct net_device *vlan_group_get_device(struct vlan_group *vg,
						       int vlan_id)
{
	return vg->vlan_devices[vlan_id];
}
#endif /* VLANGRP */

#if !defined(for_each_netdev)
#define for_each_netdev(d) \
	for (d = dev_base; d; d = d->next)
#endif

#include <linux/ip.h>

static inline void cxgb3_set_skb_header(struct sk_buff *skb,
					struct iphdr *ip_hdr,
					int offset)
{
	skb_set_network_header(skb, offset);
}

#if defined(ETHTOOL_GPERMADDR)
#define CXGB3_ETHTOOL_GPERMADDR ETHTOOL_GPERMADDR
#endif

#if !defined(TRANSPORT_HEADER)
#define transport_header h.raw
#endif
#endif
