/*
 * Brocade Fibre Channel HBA Driver
 * Copyright (c) 2006-2007 Brocade Communications Systems, Inc.
 * All rights reserved.
 */
#include "aen/bfa_aen.h"
#include "cs/bfa_debug.h"
#include "bfad_aen_cb.h"
#include "bfa_os_inc.h"

#define	BFA_TRC_AEN_AEN	1

BFA_TRC_FILE(AEN, AEN);

void            bfa_os_gettimeofday(bfa_timeval_t *tv);
uint32_t        bfa_os_get_instance_id(bfad_t *bfad);

/**
 * Assumes bfad has already allocated memory for this aen instance
 */
int32_t
bfa_aen_init(bfa_aen_t *aen, bfad_t *bfad)
{
	aen->bfad = bfad;
	aen->write_index = 0;
	aen->read_index = 0;
	aen->max_entry = bfa_aen_get_max_cfg_entry();

	if (aen->max_entry < 2) {
		/*
		 * Log this error
		 */
		bfa_trc(&aen->bfad->bfa_hal, aen->max_entry);
		bfa_assert(0);
		return -1;
	}

	return 0;
}

/**
 * HAL/FCS/OS driver API for posting an Async Event
 */
int32_t
bfa_aen_post(bfa_aen_t *aen, bfa_aen_category_t aen_category,
	     bfa_aen_event_t aen_type, bfa_aen_data_t *aen_data)
{
	int32_t         rc = 0;
	int32_t         write_index = aen->write_index;

	bfa_os_gettimeofday(&aen->list[write_index].aen_tv);
	aen->list[write_index].bfad_num = bfa_os_get_instance_id(aen->bfad);
	aen->list[write_index].aen_category = aen_category;
	aen->list[write_index].aen_type = aen_type;
	if (aen_data)
		/* Legally, it can be NULL!! */
		memcpy(&aen->list[write_index].aen_data, aen_data,
		       sizeof(bfa_aen_data_t));

	switch (aen_category) {
	case BFA_AEN_CAT_ADAPTER:
	case BFA_AEN_CAT_PORT:
	case BFA_AEN_CAT_LPORT:
	case BFA_AEN_CAT_RPORT:
	case BFA_AEN_CAT_ITNIM:
	case BFA_AEN_CAT_ITNTM:
	case BFA_AEN_CAT_IPFC:
	case BFA_AEN_CAT_AUDIT:
	case BFA_AEN_CAT_IOC:
		break;
	default:
		/*
		 * Log this error & continue?
		 */
		bfa_trc(&aen->bfad->bfa_hal, aen_category);
		rc = -1;
		break;
	}

	/* Now put in the list */
	aen->write_index = (write_index + 1) % aen->max_entry;

	if (aen->write_index == aen->read_index)
		aen->read_index = (aen->read_index + 1) % aen->max_entry;

	/* And let OS driver(the consumer) know */
	bfad_aen_cb_notify(aen->bfad);

	bfa_trc(&aen->bfad->bfa_hal, aen_category);
	bfa_trc(&aen->bfad->bfa_hal, bfa_aen_fetch_count(aen));

	return rc;
}

/**
 * OS driver API for fetching an Event from the list
 */
int32_t
bfa_aen_fetch(bfa_aen_t *aen, bfa_aen_entry_t *aen_entry,
	      int32_t entry_list_size)
{
	int32_t         fetch_count = 0;

	bfa_assert(aen_entry);

	while ((entry_list_size - fetch_count) > 0) {
		if (aen->write_index == aen->read_index)
			return fetch_count;
		else {
			memcpy(&aen_entry[fetch_count],
			       &aen->list[aen->read_index],
			       sizeof(bfa_aen_entry_t));
			aen->read_index = (aen->read_index + 1) %
					 aen->max_entry;
			fetch_count++;
		}
	}

	return fetch_count;
}

int32_t 
bfa_aen_get_inst(bfa_aen_t *aen)
{
	return bfa_os_get_instance_id(aen->bfad);
}

static inline uint8_t 
n2b(uint8_t n)
{
	if (n <= 0x09) return (n + '0');
	else return (n - 0x0a + 'a');
}

char *
wwn2str(char *buf, int bufsize, uint64_t wwn)
{
	char *ptr;
	union {
	wwn_t           wwn;
	uint8_t         byte[sizeof(wwn_t)];
	} w;
	int i = sizeof(wwn_t);

	ptr = buf + bufsize;
	*--ptr = '\0';
	bufsize--;

	w.wwn = wwn;
	while (i > 0 ) {
		i--;
		*--ptr = n2b(w.byte[i] & 0x0f);
		*--ptr = n2b((w.byte[i] >> 4) & 0x0f);
		if (i > 0)
			*--ptr = ':';
	}	

	return ptr;
}
