/* 
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 * 
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfa_hcb_ioim_macros.h BFA HAL IOIM driver interface macros.
 */

#ifndef __BFA_HCB_IOIM_MACROS_H__
#define __BFA_HCB_IOIM_MACROS_H__

#include <bfa_os.h>
/*
 * #include <linux/dma-mapping.h>
 *
 * #include <scsi/scsi.h> #include <scsi/scsi_cmnd.h> #include
 * <scsi/scsi_device.h> #include <scsi/scsi_host.h>
 */

/*
 * task attribute values in FCP-2 FCP_CMND IU
 */
#define SIMPLE_Q    0
#define HEAD_OF_Q   1
#define ORDERED_Q   2
#define ACA_Q       4
#define UNTAGGED    5

static inline lun_t
bfad_int_to_lun(uint32_t luno)
{
	union {
		uint16_t        scsi_lun[4];
		lun_t           bfa_lun;
	} lun;

	lun.bfa_lun     = 0;
	lun.scsi_lun[0] = bfa_os_htons(luno);

	return (lun.bfa_lun);
}

/**
 * Get LUN for the I/O request
 */
#define bfa_hcb_ioim_get_lun(__dio)	\
	bfad_int_to_lun(((struct scsi_cmnd *)__dio)->device->lun)

/**
 * Get CDB for the I/O request
 */
static inline uint8_t *
bfa_hcb_ioim_get_cdb(bfad_ioim_t *dio)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;

	return ((uint8_t *) cmnd->cmnd);
}

/**
 * Get I/O direction (read/write) for the I/O request
 */
static inline fcp_iodir_t
bfa_hcb_ioim_get_iodir(bfad_ioim_t *dio)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;
	enum dma_data_direction dmadir;

	dmadir = cmnd->sc_data_direction;
	if (dmadir == DMA_TO_DEVICE)
		return FCP_IODIR_WRITE;
	else if (dmadir == DMA_FROM_DEVICE)
		return FCP_IODIR_READ;
	else
		return FCP_IODIR_NONE;
}

/**
 * Get IO size in bytes for the I/O request
 */
static inline uint32_t
bfa_hcb_ioim_get_size(bfad_ioim_t *dio)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;

	return (cmnd->request_bufflen);
}

/**
 * Get timeout for the I/O request
 */
static inline uint8_t
bfa_hcb_ioim_get_timeout(bfad_ioim_t *dio)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;
	/*
	 * TBD: need a timeout for scsi passthru
	 */
	if (cmnd->device->host == NULL)
		return 4;

	return 0;
}

/**
 * Get SG element for the I/O request given the SG element index
 */
static inline bfa_addr_t
bfa_hcb_ioim_get_sgaddr(bfad_ioim_t *dio, int sgeid)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;
	struct scatterlist *sge;
	uint64_t        addr;

	if (cmnd->use_sg) {
		sge = (struct scatterlist *)cmnd->request_buffer + sgeid;
		addr = (uint64_t) sg_dma_address(sge);
	} else {
#ifdef __VMKERNEL_MODULE__
		addr = (uint64_t) cmnd->request_bufferMA;
#else
		addr = (uint64_t) cmnd->SCp.dma_handle;
#endif
	}

	return (*(bfa_addr_t *) &addr);
}

static inline uint32_t
bfa_hcb_ioim_get_sglen(bfad_ioim_t *dio, int sgeid)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;
	struct scatterlist *sge;
	uint32_t        len;

	if (cmnd->use_sg) {
		sge = (struct scatterlist *)cmnd->request_buffer + sgeid;
		len = sg_dma_len(sge);
	} else {
		len = cmnd->request_bufflen;
	}

	return len;
}

/**
 * Get Command Reference Number for the I/O request. 0 if none.
 */
static inline uint8_t
bfa_hcb_ioim_get_crn(bfad_ioim_t *dio)
{
	return 0;
}

/**
 * Get SAM-3 priority for the I/O request. 0 is default.
 */
static inline uint8_t
bfa_hcb_ioim_get_priority(bfad_ioim_t *dio)
{
	return 0;
}

/**
 * Get task attributes for the I/O request. Default is FCP_TASK_ATTR_SIMPLE(0).
 */
static inline uint8_t
bfa_hcb_ioim_get_taskattr(bfad_ioim_t *dio)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;
	uint8_t         task_attr = UNTAGGED;

	if (cmnd->device->tagged_supported) {
		switch (cmnd->tag) {
		case HEAD_OF_QUEUE_TAG:
			task_attr = HEAD_OF_Q;
			break;
		case ORDERED_QUEUE_TAG:
			task_attr = ORDERED_Q;
			break;
		default:
			task_attr = SIMPLE_Q;
			break;
		}
	}

	return task_attr;
}

/**
 * Get CDB length in bytes for the I/O request. Default is FCP_CMND_CDB_LEN(16).
 */
static inline uint8_t
bfa_hcb_ioim_get_cdblen(bfad_ioim_t *dio)
{
	struct scsi_cmnd *cmnd = (struct scsi_cmnd *)dio;

	return (cmnd->cmd_len);
}

/**
 * @}
 */

#endif /* __BFA_HCB_IOIM_MACROS_H__ */
