/* 
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 * 
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfad_2.6.c Linux driver PCI interface module.
 */
#define __BFAD_2_6_C__

#include "bfad_drv.h"
#include "bfad_im.h"
#include "bfad_tm.h"
#include "bfad_ipfc.h"
#include "bfad_trcmod.h"
#include "bfad_sysfs.h"


module_param(os_name, charp, S_IRUGO | S_IWUSR);
module_param(os_patch, charp, S_IRUGO | S_IWUSR);
module_param(host_name, charp, S_IRUGO | S_IWUSR);
module_param(num_rports, int, S_IRUGO | S_IWUSR);
module_param(num_ios, int, S_IRUGO | S_IWUSR);
module_param(num_tms, int, S_IRUGO | S_IWUSR);
module_param(num_fcxps, int, S_IRUGO | S_IWUSR);
module_param(num_ufbufs, int, S_IRUGO | S_IWUSR);
module_param(reqq_size, int, S_IRUGO | S_IWUSR);
module_param(rspq_size, int, S_IRUGO | S_IWUSR);
module_param(num_sgpgs, int, S_IRUGO | S_IWUSR);
module_param(rport_del_timeout, int, S_IRUGO | S_IWUSR);
module_param(msix_disable, int, S_IRUGO | S_IWUSR);
module_param(bfa_ioc_queue_depth, int, S_IRUGO | S_IWUSR);
module_param(bfa_lun_queue_depth, int, S_IRUGO | S_IWUSR);
module_param(bfa_io_max_sge, int, S_IRUGO | S_IWUSR);
module_param(log_level, int, S_IRUGO | S_IWUSR);

#ifdef CONFIG_PM
int             bfad_os_pci_save_state(struct pci_dev *pdev);
int             bfad_os_pci_restore_state(struct pci_dev *pdev);
#endif
int             bfad_pci_probe(struct pci_dev *pdev,
			       const struct pci_device_id *pid);
void            bfad_pci_remove(struct pci_dev *pdev);

bfad_t         *
bfad_os_find_bfad_by_path(char *path)
{
	bfad_t         *bfad = NULL, *rc = NULL;
	char           *ioc_path;

	down(&bfad_sem);
	list_for_each_entry(bfad, &bfad_list, list_entry) {
		ioc_path = bfad_os_sysfs_get_ioc_path(bfad);
		if (strstr(path, ioc_path)) {
			rc = bfad;
			kfree(ioc_path);
			break;
		}
		kfree(ioc_path);
	}
	up(&bfad_sem);
	return rc;
}

void
bfad_os_pci_unregister_driver(struct pci_driver *drv)
{
	pci_unregister_driver(drv);
}

/*
 * Not necessary for PCIE
 */
void
bfad_os_pci_set_mwi(struct pci_dev *pdev)
{
	return;
}

#ifdef CONFIG_PM
/**
 * @brief
 * PCI suspend entry.
 */
static int
bfad_os_suspend(struct pci_dev *pdev, pm_message_t state)
{
	bfad_t         *bfad = pci_get_drvdata(pdev);
	pci_power_t     device_state;

	/*
	 * Find the appropriate device state from pm_message_t state. This is
	 * Liunx version dependent. Pay attention to the difference before and
	 * after kernel version 2.6.12. Refer to LSI's mpt_suspend() function.
	 * Typicall we shoud get D3Host for PM suspend operation.
	 */
	device_state = pci_choose_state(pdev, state);

	/*
	 * save the PCI configuration space content. Note: pci_save_state() is
	 * linux version sensitive, we need to look at the difference between
	 * kernel version before and after 2.6.9.
	 * TBD: need to investigate if saving 256 bytes is enough for PCIE
	 * in the case of kernel version 2.6.9 or earlier
	 */
	bfad_os_pci_save_state(pdev);

	init_completion(&bfad->suspend);

	/*
	 * call HAL function to bring the BFAD into suspend state. This should
	 * disable and clear pending interrupt. Writing to the configuration
	 * space can be handled by direct linux pci function call
	 * pci_set_power_state().
	 */
	bfa_hal_pm_ctl(&bfad->bfa_hal, (bfa_hal_pm_cmd_t) device_state);

	/*
	 * wait to be waked up
	 */
	wait_for_completion(&bfad->suspend);

	pci_disable_device(pdev);

	/*
	 * This will write the BFAD's PCI configuration space, and bring the
	 * device to the desired power state, typcially D3hot
	 */
	pci_set_power_state(pdev, device_state);

	return 0;
}

/**
 * @brief
 * PCI resume entry.
 */
static int
bfad_os_resume(struct pci_dev *pdev)
{
	bfad_t         *bfad = pci_get_drvdata(pdev);

	/*
	 * Resume the device power state to D0
	 */
	pci_set_power_state(pdev, 0);

	/*
	 * Restore all device PCI configuation space to its original state,
	 * Note that simiar to pci_save_state(), this function is Linux version
	 * sensitive, pay attention to the difference between before 2.6.9 and
	 * after.
	 */
	bfad_os_pci_restore_state(pdev);

	if (pci_enable_device(pdev))
		goto out;

	/*
	 * Bring back the BFAD back into operational state. Enalbe interrupts,
	 * and do we just need a bfa_hal_start()?
	 */
	bfa_hal_pm_ctl(&bfad->bfa_hal, 0);

	return 0;

out:
	return 1;

}
#endif

#ifdef SUPPORT_PCI_AER
/*
 * PCI error recovery support, this fearure is only availbe for kernel
 * .6.16 or later.
 */
/**
 * @brief
 * PCI Error Recovery entry, error detected.
 */
static          pci_ers_result_t
bfad_pci_error_detected(struct pci_dev *pdev, pci_channel_state_t state)
{
	bfad_t         *bfad = pci_get_drvdata(pdev);

	switch (state) {
	case pci_channel_io_perm_failure:
		/*
		 * PCI bus has permanently failed. This is
		 * unrecoveralbe, Do we need to do the cleanup like PCI
		 * remove? or kernel will automatically do it?
		 */
		return PCI_ERS_RESULT_DISCONNECT;

	case pci_channel_io_frozen:

	default:

		init_completion(&bfad->comp);
		bfa_fcs_exit(&bfad->bfa_fcs);
		bfa_hal_stop(&bfad->bfa_hal);
		/*
		 * TBD: Wait for BFAD to be stopped, How long should
		 * wait?, or wait forever?
		 */
		wait_for_completion_timeout(&bfad->comp, BFAD_STOP_TIMEOUT);

		pci_disable_device(pdev);

		break;
	}

	/*
	 * Request a slot reset
	 */
	return PCI_ERS_RESULT_NEED_RESET;
}

/**
 * @brief
 * PCI Error Recovery entry, re-initialize the chip.
 */
static          pci_ers_result_t
bfad_pci_slot_reset(struct pci_dev *pdev)
{
	bfad_t         *bfad = pci_get_drvdata(pdev);
	int             retval;

	if (pci_enable_device(pdev))
		goto out;

	pci_set_master(pdev);

	retval = pci_set_mwi(pdev);

	if (retval)
		dev_printk(KERN_WARNING, &pdev->dev,
			   "Warning: pci_set_mwi returned %d\n", retval);

	if (pci_set_dma_mask(bfad->pcidev, DMA_64BIT_MASK) != 0)
		if (pci_set_dma_mask(bfad->pcidev, DMA_32BIT_MASK) != 0)
			goto out_disable_device;

	BFAD_ENABLE_PCIE_AER(pdev);

	/*
	 * HAL bfad should still keep all the BFAD and port config info
	 */
	bfa_hal_start(&bfad->bfa_hal);

	/*
	 * TBD: may need to wait until bfad_start completes
	 */

	return PCI_ERS_RESULT_RECOVERED;

out_disable_device:
	pci_disable_device(pdev);
out:
	return PCI_ERS_RESULT_DISCONNECT;

}

/*
 * PCI error recovery support, this fearure is only available for kernel
 * .6.16 or later. At this point, we only define error_detected and
 * slot_reset.
 */
static struct pci_error_handlers bfad_err_handler = {
	.error_detected = bfad_pci_error_detected,
	.slot_reset = bfad_pci_slot_reset,
};
#endif

static int      __devinit
bfad_os_pci_probe(struct pci_dev *pdev, const struct pci_device_id *pid)
{
	return (bfad_pci_probe(pdev, pid));
}

static void
bfad_os_pci_remove(struct pci_dev *pdev)
{
	bfad_pci_remove(pdev);
}

static struct pci_driver bfad_pci_driver = {
	.name = BFAD_DRIVER_NAME,
	.id_table = bfad_id_table,
	.probe = bfad_os_pci_probe,
	.remove = __devexit_p(bfad_os_pci_remove),
#ifdef CONFIG_PM
	.suspend = bfad_os_suspend,
	.resume = bfad_os_resume,
#endif

#ifdef SUPPORT_PCI_AER
	/*
	 * PCI error recovery support, this fearure is only available for
	 * kernel 2.6.16 or later.
	 */
	.err_handler = &bfad_err_handler,
#endif
};

struct pci_driver *bfad_pci_driver_p = &bfad_pci_driver;

/*
 * @}
 */
