/* 
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 * 
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfa_attr.c Linux driver configuration interface module.
 */

#include "bfa_os.h"
#include "bfad_drv.h"
#include "bfad_im.h"
#include "bfad_trcmod.h"
#include <scsi/scsi_transport.h>
#include "bfad_attr.h"

#if (LINUX_VERSION_CODE != KERNEL_VERSION(2, 6, 5))
/**
 * @dg FC_transport_template FC transport template
 * @{
 */

/**
 * @brief
 * FC transport template entry, get SCSI target port ID.
 */
void
bfad_im_get_starget_port_id(struct scsi_target *starget)
{
	struct Scsi_Host *shost;
	bfad_im_port_t *im_port;
	bfad_t         *bfad;
	bfad_itnim_t   *itnim = NULL;
	uint32_t        fc_id = -1;
	unsigned long   flags;

	shost = bfad_os_starget_to_shost(starget);
	im_port = (bfad_im_port_t *) shost->hostdata[0];
	bfad = im_port->bfad;
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	/*
	 * Search the mapped list for this target ID
	 */
	list_for_each_entry(itnim, &im_port->itnim_mapped_list, list_entry) {
		if (starget->id == itnim->scsi_tgt_id) {
			fc_id = bfa_fcs_itnim_get_fcid(&itnim->fcs_itnim);
			break;
		}
	}
	fc_starget_port_id(starget) = fc_id;
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
}

/**
 * @brief
 * FC transport template entry, get SCSI target nwwn.
 */
void
bfad_im_get_starget_node_name(struct scsi_target *starget)
{
	struct Scsi_Host *shost;
	bfad_im_port_t *im_port;
	bfad_t         *bfad;
	bfad_itnim_t   *itnim = NULL;
	u64             node_name = 0;
	unsigned long   flags;

	shost = bfad_os_starget_to_shost(starget);
	im_port = (bfad_im_port_t *) shost->hostdata[0];
	bfad = im_port->bfad;
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	/*
	 * Search the mapped list for this target ID
	 */
	list_for_each_entry(itnim, &im_port->itnim_mapped_list, list_entry) {
		if (starget->id == itnim->scsi_tgt_id) {
			node_name = bfa_fcs_itnim_get_nwwn(&itnim->fcs_itnim);
			break;
		}
	}
	fc_starget_node_name(starget) = node_name;
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
}

/**
 * @brief
 * FC transport template entry, get SCSI target pwwn.
 */
void
bfad_im_get_starget_port_name(struct scsi_target *starget)
{
	struct Scsi_Host *shost;
	bfad_im_port_t *im_port;
	bfad_t         *bfad;
	bfad_itnim_t   *itnim = NULL;
	u64             port_name = 0;
	unsigned long   flags;

	shost = bfad_os_starget_to_shost(starget);
	im_port = (bfad_im_port_t *) shost->hostdata[0];
	bfad = im_port->bfad;
	spin_lock_irqsave(&bfad->bfad_lock, flags);
	/*
	 * Search the mapped list for this target ID
	 */
	list_for_each_entry(itnim, &im_port->itnim_mapped_list, list_entry) {
		if (starget->id == itnim->scsi_tgt_id) {
			port_name = bfa_fcs_itnim_get_pwwn(&itnim->fcs_itnim);
			break;
		}
	}
	fc_starget_port_name(starget) = port_name;
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
}

/**
 * @brief
 * FC transport template entry, get SCSI host port ID.
 */
void
bfad_im_get_host_port_id(struct Scsi_Host *shost)
{
	bfad_im_port_t *im_port = (bfad_im_port_t *) shost->hostdata[0];
	bfad_port_t    *port = im_port->port;

	fc_host_port_id(shost) =
			bfa_os_hton3b(bfa_fcs_port_get_fcid(port->fcs_port));
}

/**
 * @brief
 * FC transport template entry, issue a LIP.
 */
int
bfad_im_issue_fc_host_lip(struct Scsi_Host *shost)
{
	/*
	 * TBD: need to know how to issue a LIP, through mailbox interface?
	 */

	return 0;
}

/**
 * @}
 */
#endif
