/* 
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 * 
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFAD_IM_H__
#define __BFAD_IM_H__

#include "hal/bfa_hal_ioim.h"
#include "fcs/bfa_fcs_fcpim.h"

#define FCPI_NAME " fcpim"

typedef struct bfad_im_s bfad_im_t;
typedef struct bfad_im_port_s bfad_im_port_t;

bfa_status_t    bfad_im_module_init(void);
void            bfad_im_module_exit(void);
bfa_status_t    bfad_im_probe(bfad_t *bfad);
void            bfad_im_probe_undo(bfad_t *bfad);
void            bfad_im_probe_post(bfad_im_t *im);
bfa_status_t    bfad_im_port_new(bfad_t *bfad, bfad_port_t *port);
void            bfad_im_port_delete(bfad_t *bfad, bfad_port_t *port);
void            bfad_im_port_online(bfad_t *bfad, bfad_port_t *port);
void            bfad_im_port_offline(bfad_t *bfad, bfad_port_t *port);
void            bfad_im_port_clean(bfad_im_port_t *im_port);
int             bfad_im_scsi_host_alloc(bfad_t *bfad,
					bfad_im_port_t *im_port);
void            bfad_im_scsi_host_free(bfad_t *bfad, bfad_im_port_t *im_port);

#define MAX_FCP_TARGET 256
#define BFAD_TARGET_RESET_TMO 60
#define BFAD_LUN_RESET_TMO 60
#define ScsiResult(host_code, scsi_code) (((host_code) << 16) | scsi_code)

/*
 * itnim flags
 */
#define ITNIM_MAPPED		0x00000001

#define SCSI_TASK_MGMT		0x00000001
#define IO_DONE_BIT			0

typedef struct {
	struct bfad_itnim_s *itnim;
} bfad_itnim_data_t;

struct bfad_im_port_s {
	bfad_t         *bfad;
	bfad_port_t    *port;
	struct work_struct port_delete_work;
	int             idr_id;
	uint16_t        cur_scsi_id;
	struct list_head binding_list;
	struct Scsi_Host *shost;
	struct list_head itnim_mapped_list;
};

typedef enum {
	ITNIM_STATE_ONLINE,
	ITNIM_STATE_OFFLINE,
	ITNIM_STATE_FREE,
} bfad_itnim_state_t;

/*
 * Per itnim data structure
 */
struct bfad_itnim_s {
	struct list_head list_entry;
	bfa_fcs_itnim_t fcs_itnim;
	struct work_struct itnim_work;
	uint32_t        flags;
	bfad_itnim_state_t state;
	struct bfad_im_s *im;
	struct bfad_im_port_s *im_port;
	struct bfad_rport_s *drv_rport;
	struct fc_rport *fc_rport;
	bfa_hal_itnim_t *hal_itnim;
	uint16_t        scsi_tgt_id;
};

typedef enum {
	FCP_PWWN_BINDING = 0x1,
	FCP_NWWN_BINDING = 0x2,
	FCP_FCID_BINDING = 0x3,
} bfad_binding_type_t;

typedef struct bfad_fcp_binding {
	struct list_head list_entry;
	bfad_binding_type_t binding_type;
	uint16_t        scsi_target_id;
	uint32_t        fc_id;
	wwn_t           nwwn;
	wwn_t           pwwn;
} bfad_fcp_binding_t;

struct bfad_im_s {
	bfad_t         *bfad;
	struct workqueue_struct *drv_workq;
	char            drv_workq_name[KOBJ_NAME_LEN];
};

void            bfad_os_spin_os_lock_irq(struct Scsi_Host *);
void            bfad_os_spin_os_unlock_irq(struct Scsi_Host *);
void            bfad_os_fc_release_transp(struct Scsi_Host *shost);
struct Scsi_Host *bfad_os_scsi_host_alloc(struct scsi_host_template *,
					  bfad_im_port_t *im_port, bfad_t *);
int             bfad_os_scsi_register(struct Scsi_Host *shost,
				      bfad_im_port_t *im_port, void *key);
int             bfad_os_fc_attach(struct Scsi_Host *shost,
				  struct fc_function_template *fct);
int             bfad_os_reset_tgt(struct scsi_cmnd *cmnd);
void            bfad_os_scsi_set_pci_device(struct Scsi_Host *shost,
					    struct pci_dev *pcidev);
void            bfad_os_select_queue_depths(bfad_im_port_t *im_port);
bfa_status_t    bfad_os_thread_workq(bfad_t *bfad);
void            bfad_os_destroy_workq(bfad_im_t *im);
void            bfad_os_queue_work(void *workq, void *work);
void            bfad_os_itnim_alloc(bfad_itnim_t *itnim_drv);
void            bfad_os_itnim_process(bfad_itnim_t *itnim_drv);
void            bfad_os_scsi_unregister(struct Scsi_Host *shost);
void            bfad_os_scsi_remove_host(struct Scsi_Host *shost);
void            bfad_os_scsi_put_host(struct Scsi_Host *shost);
int             bfad_os_scsi_add_host(struct Scsi_Host *shost,
				      struct device *dev);
bfad_itnim_data_t *bfad_os_itnim_data(bfad_im_port_t *im_port,
				      struct scsi_cmnd *cmnd);
void            bfad_os_fc_host_init(bfad_im_port_t *im_port);
void            bfad_os_fc_remove_host(struct Scsi_Host *shost);
void            bfad_os_init_work(bfad_im_port_t *im_port);
int             bfad_os_dev_init(struct Scsi_Host *shost);
void            bfad_os_transp_templ(struct Scsi_Host *sh,
				     struct scsi_transport_template *stt);
dma_addr_t      bfad_os_dma_map_single(struct device *dev, void *cpu_addr,
				       size_t size,
				       enum dma_data_direction direction);
void            bfad_os_dma_unmap_single(struct device *dev,
					 dma_addr_t dma_addr, size_t size,
					 enum dma_data_direction direction);
int             bfad_os_dma_map_sg(struct device *dev, struct scatterlist *sg,
				   int nents,
				   enum dma_data_direction direction);
void            bfad_os_dma_unmap_sg(struct device *dev, struct scatterlist *sg,
				     int nhwentries,
				     enum dma_data_direction direction);
int             bfad_os_im_dma_map(bfad_t *bfad, struct scsi_cmnd *cmnd);
void            bfad_os_im_dma_unmap(bfad_t *bfad, struct scsi_cmnd *cmnd);
int             bfad_os_idr_get_new(struct idr *idp, int *id);
void            bfad_os_scsi_state_changed(struct Scsi_Host *shost);
void		bfad_os_scsi_scan(bfad_im_port_t *im_port);
void 		bfad_os_scsi_host_free(bfad_t *bfad, bfad_im_port_t *im_port);
void		bfad_wwn_to_str(wwn_t wwn, char *str);
void		bfad_fcid_to_str(uint32_t fcid, char *str);

/*
 * scsi_host_template entries
 */
int             bfad_im_queuecommand(struct scsi_cmnd *cmnd,
				     void (*done) (struct scsi_cmnd *));
const char     *bfad_im_info(struct Scsi_Host *host);
int             bfad_im_slave_alloc(struct scsi_device *sdev);
void            bfad_im_slave_destroy(struct scsi_device *sdev);
int             bfad_im_abort_handler(struct scsi_cmnd *cmnd);
int             bfad_im_reset_lun_handler(struct scsi_cmnd *cmnd);
int             bfad_im_reset_bus_handler(struct scsi_cmnd *cmnd);

extern struct scsi_host_template bfad_im_scsi_host_template;
extern struct fc_function_template bfad_im_fc_function_template;
extern struct scsi_transport_template *bfad_im_scsi_transport_template;

#endif
