/* 
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 * 
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfad_im_2.6.10.c Linux driver IM module.
 * code for Linux >= 2.6.10
 */

#include "bfad_drv.h"
#include "bfad_im.h"
#include "bfad_trcmod.h"

BFA_TRC_FILE(LDRV, IM_2_6_10);

BFAD_WORK_HANDLER(bfad_im_port_delete_handler);

void
bfad_os_init_work(bfad_im_port_t *im_port)
{
	BFAD_INIT_WORK(im_port, port_delete_work, bfad_im_port_delete_handler);

	return;
}

void
bfad_os_fc_remove_host(struct Scsi_Host *shost)
{
	fc_remove_host(shost);
}

void bfad_os_rport_online_wait(bfad_t *bfad)
{
	return;
}

void
bfad_os_scsi_scan(bfad_im_port_t *im_port)
{
	return;
}

void
bfad_os_spin_os_lock_irq(struct Scsi_Host *shost)
{
	return;
}

void
bfad_os_spin_os_unlock_irq(struct Scsi_Host *shost)
{
	return;
}

/**
 * @brief
 * Scsi_Host template entry slave_alloc
 */
int
bfad_im_slave_alloc(struct scsi_device *sdev)
{
	struct fc_rport *rport = starget_to_rport(scsi_target(sdev));

	if (!rport || fc_remote_port_chkready(rport))
		return -ENXIO;

	sdev->hostdata = rport->dd_data;

	return 0;
}

void
bfad_os_fc_host_init(bfad_im_port_t *im_port)
{
	struct Scsi_Host *host = im_port->shost;
	bfad_t         *bfad = im_port->bfad;
	bfad_port_t    *port = im_port->port;
	union attr {
		bfa_pport_attr_t pattr;
		bfa_ioc_attr_t  ioc_attr;
	} attr;

	fc_host_node_name(host) =
		bfa_os_htonll((bfa_fcs_port_get_nwwn(port->fcs_port)));
	fc_host_port_name(host) =
		bfa_os_htonll((bfa_fcs_port_get_pwwn(port->fcs_port)));
	/*
	 * Do we support class2
	 */
	fc_host_supported_classes(host) = FC_COS_CLASS3;

	memset(fc_host_supported_fc4s(host), 0,
	       sizeof(fc_host_supported_fc4s(host)));
	if (supported_fc4s & (BFA_PORT_ROLE_FCP_IM | BFA_PORT_ROLE_FCP_TM))
		/*
		 * For FCP type 0x08
		 */
		fc_host_supported_fc4s(host)[2] = 1;
	if (supported_fc4s | BFA_PORT_ROLE_FCP_IPFC)
		/*
		 * For LLC/SNAP type 0x05
		 */
		fc_host_supported_fc4s(host)[3] = 0x20;
	/*
	 * For fibre channel services type 0x20
	 */
	fc_host_supported_fc4s(host)[7] = 1;

	memset(&attr.ioc_attr, 0, sizeof(attr.ioc_attr));
	bfa_hal_ioc_get_attr(&bfad->bfa_hal, &attr.ioc_attr);
	sprintf(fc_host_symbolic_name(host), "Brocade %s FV%s DV%s",
		attr.ioc_attr.adapter_attr.model,
		attr.ioc_attr.adapter_attr.fw_ver, BFAD_DRIVER_VERSION);

	fc_host_supported_speeds(host) = 0;
	fc_host_supported_speeds(host) |=
		FC_PORTSPEED_8GBIT | FC_PORTSPEED_4GBIT | FC_PORTSPEED_2GBIT |
		FC_PORTSPEED_1GBIT;

	memset(&attr.pattr, 0, sizeof(attr.pattr));
	bfa_hal_pport_get_attr(&bfad->bfa_hal, &attr.pattr);
	fc_host_maxframe_size(host) = attr.pattr.pport_cfg.maxfrsize;
}

void
bfad_im_fc_rport_add(bfad_im_port_t *im_port, bfad_itnim_t *itnim)
{
	struct fc_rport_identifiers rport_ids;
	struct fc_rport *fc_rport;
	bfad_itnim_data_t *itnim_data;

	rport_ids.node_name = 
		bfa_os_htonll(bfa_fcs_itnim_get_nwwn(&itnim->fcs_itnim));
	rport_ids.port_name = 
		bfa_os_htonll(bfa_fcs_itnim_get_pwwn(&itnim->fcs_itnim));
	rport_ids.port_id = 
		bfa_os_hton3b(bfa_fcs_itnim_get_fcid(&itnim->fcs_itnim));
	rport_ids.roles = FC_RPORT_ROLE_UNKNOWN;

	itnim->fc_rport = fc_rport =
		fc_remote_port_add(im_port->shost, 0, &rport_ids);

	if (!fc_rport)
		return;

	fc_rport->maxframe_size =
		bfa_fcs_itnim_get_maxfrsize(&itnim->fcs_itnim);
	fc_rport->supported_classes = bfa_fcs_itnim_get_cos(&itnim->fcs_itnim);

	/*
	 * The size of the dd_data is specified by dd_fcport_size in
	 * fc_function_tempalate. It is set to sizeof(bfad_itnim_data_t *).
	 * The memory is allocated when fc_remote_port_add is called.
	 */
	itnim_data = fc_rport->dd_data;
	itnim_data->itnim = itnim;

	rport_ids.roles |= FC_RPORT_ROLE_FCP_TARGET;

	if (rport_ids.roles != FC_RPORT_ROLE_UNKNOWN)
		fc_remote_port_rolechg(fc_rport, rport_ids.roles);

	if ((fc_rport->scsi_target_id != -1)
	    && (fc_rport->scsi_target_id < MAX_FCP_TARGET))
		itnim->scsi_tgt_id = fc_rport->scsi_target_id;

	return;
}

/**
 * @brief
 * Work queue handler using FC transport service
* Context: kernel
 */
BFAD_WORK_HANDLER(bfad_im_itnim_work_handler)
{	/* work struct is the function arg */
	bfad_itnim_t   *itnim = container_of(work, bfad_itnim_t, itnim_work);
	bfad_im_t      *im = itnim->im;
	bfad_t         *bfad = im->bfad;
	bfad_im_port_t *im_port;
	unsigned long   flags;
	struct fc_rport *fc_rport;
	wwn_t wwpn;
	uint32_t fcid;
	char wwpn_str[32], fcid_str[16];

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	im_port = itnim->im_port;
	bfa_trc(bfad, itnim->state);
	switch (itnim->state) {
	case ITNIM_STATE_ONLINE:
		if (!itnim->fc_rport) {
			spin_unlock_irqrestore(&bfad->bfad_lock, flags);
			bfad_im_fc_rport_add(im_port, itnim);
			spin_lock_irqsave(&bfad->bfad_lock, flags);
			wwpn = bfa_fcs_itnim_get_pwwn(&itnim->fcs_itnim);
			fcid = bfa_fcs_itnim_get_fcid(&itnim->fcs_itnim);
			bfad_wwn_to_str(wwpn, wwpn_str);
			bfad_fcid_to_str(fcid, fcid_str);
			list_add_tail(&itnim->list_entry,
				&im_port->itnim_mapped_list);
			bfa_log(bfad->logmod, BFA_LOG_LINUX_ITNIM_ONLINE,
				im_port->shost->host_no,
				itnim->scsi_tgt_id,
				fcid_str, wwpn_str);
		} else {
			printk(KERN_WARNING
				"%s: itnim %llx is already in online state\n",
				__FUNCTION__,
				bfa_fcs_itnim_get_pwwn(&itnim->fcs_itnim));
                }

		break;
	case ITNIM_STATE_OFFLINE:
		if (itnim->fc_rport) {
			fc_rport = itnim->fc_rport;
			((bfad_itnim_data_t *) fc_rport->dd_data)->itnim = NULL;
			itnim->fc_rport = NULL;
			spin_unlock_irqrestore(&bfad->bfad_lock, flags);
			fc_remote_port_delete(fc_rport);
			spin_lock_irqsave(&bfad->bfad_lock, flags);
			wwpn = bfa_fcs_itnim_get_pwwn(&itnim->fcs_itnim);
			fcid = bfa_fcs_itnim_get_fcid(&itnim->fcs_itnim);
			bfad_wwn_to_str(wwpn, wwpn_str);
			bfad_fcid_to_str(fcid, fcid_str);
			list_del(&itnim->list_entry);
			bfa_log(bfad->logmod, BFA_LOG_LINUX_ITNIM_OFFLINE,
				im_port->shost->host_no,
				itnim->scsi_tgt_id,
				fcid_str, wwpn_str);
		}
		break;
	case ITNIM_STATE_FREE:
		if (itnim->fc_rport) {
			fc_rport = itnim->fc_rport;
			((bfad_itnim_data_t *) fc_rport->dd_data)->itnim = NULL;
			itnim->fc_rport = NULL;
			spin_unlock_irqrestore(&bfad->bfad_lock, flags);
			fc_remote_port_delete(fc_rport);
			spin_lock_irqsave(&bfad->bfad_lock, flags);
			list_del(&itnim->list_entry);
		}

		kfree(itnim);
	default:
		break;
	}

	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
}

/**
 * @brief
 * Scsi_Host template entry, queue a SCSI command to the BFAD.
 */
int
bfad_im_queuecommand(struct scsi_cmnd *cmnd, void (*done) (struct scsi_cmnd *))
{
	bfad_im_port_t *im_port = (bfad_im_port_t *) cmnd->device->host->hostdata[0];
	bfad_t         *bfad = im_port->bfad;
	bfad_itnim_data_t *itnim_data = cmnd->device->hostdata;
	bfad_itnim_t   *itnim;
	bfa_hal_ioim_t *hal_io;
	unsigned long   flags;
	int             rc;
	uint16_t        sg_cnt = 0;
	struct fc_rport *rport = starget_to_rport(scsi_target(cmnd->device));

	rc = fc_remote_port_chkready(rport);
	if (rc) {
		cmnd->result = rc; 
		done(cmnd);
                return 0;
        }

	/* Map DMA */
	rc = bfad_os_im_dma_map(bfad, cmnd);
	if (rc)
		return SCSI_MLQUEUE_HOST_BUSY;

	cmnd->scsi_done = done;
	if (cmnd->use_sg)
		sg_cnt = cmnd->use_sg;
	else if (cmnd->request_bufflen)
		sg_cnt = 1;

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	if (!(bfad->bfad_flags & BFAD_HAL_START_DONE)) {
		printk(KERN_WARNING
			"bfad%d, queuecommand %p %x failed, HAL stopped\n",
		       bfad->inst_no, cmnd, cmnd->cmnd[0]);
		cmnd->result = ScsiResult(DID_NO_CONNECT, 0);
		goto out_fail_cmd;
	}


	itnim = itnim_data->itnim;
	if (!itnim) {
		cmnd->result = ScsiResult(DID_IMM_RETRY, 0);
		goto out_fail_cmd;
	}

	hal_io = bfa_hal_ioim_alloc(&bfad->bfa_hal, (bfad_ioim_t *) cmnd,
				    itnim->hal_itnim, sg_cnt);
	if (!hal_io) {
		printk(KERN_WARNING "hal_io failure\n");
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		bfad_os_im_dma_unmap(bfad, cmnd);
		return SCSI_MLQUEUE_HOST_BUSY;
	}

	cmnd->host_scribble = (char *)hal_io;
	bfa_trc_fp(bfad, hal_io->iotag);
	bfa_hal_ioim_start(hal_io);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	return 0;

out_fail_cmd:
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);
	bfad_os_im_dma_unmap(bfad, cmnd);
	if (done)
		done(cmnd);

	return 0;
}
