/* 
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 * 
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfad_im_2.6.c Linux driver IM module.
 * common code for Linux ver 2.6.*
 */

#include "bfad_drv.h"
#include "bfad_im.h"
#include "bfad_trcmod.h"

BFA_TRC_FILE(LDRV, IM_2_6);

extern struct class_device_attribute *bfad_im_host_attrs[];

BFAD_WORK_HANDLER(bfad_im_itnim_work_handler);

/*
 * vmware & 2.4 {
 */
void
bfad_os_fc_release_transp(struct Scsi_Host *shost)
{
	return;
}

int
bfad_os_scsi_register(struct Scsi_Host *shost, bfad_im_port_t *im_port,
		      void *key)
{
	return 0;
}

void
bfad_os_scsi_unregister(struct Scsi_Host *shost)
{
	return;
}

int
bfad_os_fc_attach(struct Scsi_Host *shost, struct fc_function_template *fct)
{
	return 0;
}

int
bfad_os_reset_tgt(struct scsi_cmnd *cmnd)
{
	return 0;
}

void
bfad_os_select_queue_depths(bfad_im_port_t *im_port)
{
	return;
}

void
bfad_os_scsi_set_pci_device(struct Scsi_Host *shost, struct pci_dev *pcidev)
{
	return;
}

void
bfad_os_idr_init(struct idr *idr)
{
	return;
}

void
bfad_os_idr_destroy(struct idr *idr)
{
	return;
}

/*
 * } vmware & 2.4
 */

struct Scsi_Host *
bfad_os_scsi_host_alloc(struct scsi_host_template *sht,
			bfad_im_port_t *im_port, bfad_t *bfad)
{
	sht->sg_tablesize = bfad->cfg_data.io_max_sge; 
	return scsi_host_alloc(sht, sizeof(unsigned long));
}

void
bfad_os_scsi_put_host(struct Scsi_Host *shost)
{
	scsi_host_put(shost);
}

void
bfad_os_scsi_remove_host(struct Scsi_Host *shost)
{
	scsi_remove_host(shost);
}

void
bfad_os_queue_work(void *workq, void *work)
{
	queue_work((struct workqueue_struct *)workq,
		   (struct work_struct *)work);
}

void
bfad_os_scsi_host_free(bfad_t *bfad, bfad_im_port_t *im_port)
{
        queue_work(bfad->im->drv_workq,
                                   &im_port->port_delete_work);
}

void
bfad_os_destroy_workq(bfad_im_t *im)
{
	if (im && im->drv_workq) {
		destroy_workqueue(im->drv_workq);
		im->drv_workq = NULL;
	}
}

bfa_status_t
bfad_os_thread_workq(bfad_t *bfad)
{
	bfad_im_t      *im = bfad->im;

	bfa_trc(bfad, 0);
	snprintf(im->drv_workq_name, KOBJ_NAME_LEN, "bfad_wq_%d",
		 bfad->inst_no);
	im->drv_workq = create_singlethread_workqueue(im->drv_workq_name);
	if (!im->drv_workq)
		return BFA_STATUS_FAILED;

	return BFA_STATUS_OK;
}

int
bfad_os_scsi_add_host(struct Scsi_Host *shost, struct device *dev)
{
	return scsi_add_host(shost, dev);
}

bfad_itnim_data_t *
bfad_os_itnim_data(bfad_im_port_t *im_port, struct scsi_cmnd *cmnd)
{
	if (!cmnd || !cmnd->device)
		return 0;
	return cmnd->device->hostdata;
}

int
bfad_os_dev_init(struct Scsi_Host *shost)
{
	return 0;
}

dma_addr_t
bfad_os_dma_map_single(struct device *dev, void *cpu_addr, size_t size,
		       enum dma_data_direction direction)
{
	return dma_map_single(dev, cpu_addr, size, direction);
}

void
bfad_os_dma_unmap_single(struct device *dev, dma_addr_t dma_addr, size_t size,
			 enum dma_data_direction direction)
{
	dma_unmap_single(dev, dma_addr, size, direction);
}

int
bfad_os_dma_map_sg(struct device *dev, struct scatterlist *sg, int nents,
		   enum dma_data_direction direction)
{
	return dma_map_sg(dev, sg, nents, direction);
}

void
bfad_os_dma_unmap_sg(struct device *dev, struct scatterlist *sg, int nhwentries,
		     enum dma_data_direction direction)
{
	dma_unmap_sg(dev, sg, nhwentries, direction);
}

/**
 * @brief
 * Scsi_Host template entry.
 *
 * Description:
 * OS entry point to adjust the queue_depths on a per-device basis.
 * Called once per device during the bus scan.
 * Return non-zero if fails.
 */
static int
bfad_im_slave_configure(struct scsi_device *sdev)
{
	bfad_im_port_t *im_port = (bfad_im_port_t *) sdev->host->hostdata[0];
	bfad_t         *bfad = im_port->bfad;

	if (sdev->tagged_supported)
		scsi_activate_tcq(sdev, bfad->cfg_data.lun_queue_depth);
	else
		/*
		 * TBD: should we set queue depth to 1 or the same value as
		 * we set in scsi_host_template (cmd_per_lun/3)?
		 */
		scsi_deactivate_tcq(sdev, bfad->cfg_data.lun_queue_depth);

	/*
	 * TBD: do we need to set up dev_loss_tmo attribute here?
	 */

	return 0;
}

struct scsi_host_template bfad_im_scsi_host_template = {
	.module = THIS_MODULE,
	.name = BFAD_DRIVER_NAME,
	.info = bfad_im_info,
	.queuecommand = bfad_im_queuecommand,
	.eh_abort_handler = bfad_im_abort_handler,
	.eh_device_reset_handler = bfad_im_reset_lun_handler,
	.eh_bus_reset_handler = bfad_im_reset_bus_handler,

	.slave_alloc = bfad_im_slave_alloc,
	.slave_configure = bfad_im_slave_configure,
	.slave_destroy = bfad_im_slave_destroy,

	.this_id = -1,
	.sg_tablesize = BFAD_IO_MAX_SGE,
	.cmd_per_lun = 3,
	.use_clustering = ENABLE_CLUSTERING,
	.shost_attrs = bfad_im_host_attrs,
	.max_sectors = 0xFFFF,
};

void
bfad_im_probe_post(bfad_im_t *im)
{
	flush_workqueue(im->drv_workq);
}

bfa_status_t
bfad_im_module_init(void)
{
	bfad_im_scsi_transport_template =
		fc_attach_transport(&bfad_im_fc_function_template);
	if (!bfad_im_scsi_transport_template)
		return BFA_STATUS_ENOMEM;

	return BFA_STATUS_OK;
}

void
bfad_im_module_exit(void)
{
	if (bfad_im_scsi_transport_template)
		fc_release_transport(bfad_im_scsi_transport_template);
}

void
bfad_os_transp_templ(struct Scsi_Host *sh, struct scsi_transport_template *stt)
{
	sh->transportt = stt;
}

int
bfad_os_im_dma_map(bfad_t *bfad, struct scsi_cmnd *cmnd)
{
	int             sg_cnt, rc = 0;

	if (cmnd->use_sg) {
		sg_cnt = bfad_os_dma_map_sg((struct device *)&bfad->pcidev->dev,
					    (struct scatterlist *)cmnd->
					    request_buffer, cmnd->use_sg,
					    cmnd->sc_data_direction);
		if (sg_cnt == 0 || sg_cnt > bfad->cfg_data.io_max_sge) {
			/*
			 * log info and return
			 */
			printk(KERN_WARNING "dma_map_sg failure\n");
			rc = 1;
		}

	} else if (cmnd->request_bufflen) {
		cmnd->SCp.dma_handle =
			bfad_os_dma_map_single((struct device *)&bfad->pcidev->
					       dev, cmnd->request_buffer,
					       cmnd->request_bufflen,
					       cmnd->sc_data_direction);
	}

	return rc;
}

void
bfad_os_im_dma_unmap(bfad_t *bfad, struct scsi_cmnd *cmnd)
{
	if (cmnd->use_sg)
		bfad_os_dma_unmap_sg((struct device *)&bfad->pcidev->dev,
				     cmnd->request_buffer, cmnd->use_sg,
				     cmnd->sc_data_direction);
	else if (cmnd->request_bufflen)
		bfad_os_dma_unmap_single((struct device *)&bfad->pcidev->dev,
					 cmnd->SCp.dma_handle,
					 cmnd->request_bufflen,
					 cmnd->sc_data_direction);
}

int
bfad_os_idr_get_new(struct idr *idp, int *id)
{
	return idr_get_new(idp, NULL, id);
}

void
bfad_os_itnim_alloc(bfad_itnim_t *itnim_drv)
{
	BFAD_INIT_WORK(itnim_drv, itnim_work, bfad_im_itnim_work_handler);
}

void
bfad_os_itnim_process(bfad_itnim_t *itnim_drv)
{
	bfad_im_t      *im = itnim_drv->im;

	bfad_os_queue_work(im->drv_workq, &itnim_drv->itnim_work);
}
