/* 
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 * 
 * See LICENSE.bfa for copyright and licensing details.
 */

#include "bfad_drv.h"
#include "bfad_trcmod.h"

BFA_TRC_FILE(LDRV, INTR);

int             msix_disable = 1;

/**
 * @dg bfa_isr BFA driver interrupt functions
 * @{
 */

/**
 * @brief
 * Line based interrupt handler.
 */
irqreturn_t
bfad_intx(int irq, void *dev_id)
{
	bfad_t         *bfad = dev_id;
	bfa_q_t         doneq;
	unsigned long   flags;
	bfa_hal_intr_status_t rc;

	spin_lock_irqsave(&bfad->bfad_lock, flags);
	rc = bfa_hal_intx(&bfad->bfa_hal);
	if (rc == BFA_HAL_INTR_NOTCLAIMED) {
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		return IRQ_NONE;
	}

	bfa_hal_comp_deq(&bfad->bfa_hal, &doneq);
	spin_unlock_irqrestore(&bfad->bfad_lock, flags);

	if (!bfa_q_is_empty(&doneq)) {
		bfa_hal_comp_process(&bfad->bfa_hal, &doneq);

		spin_lock_irqsave(&bfad->bfad_lock, flags);
		bfa_hal_comp_free(&bfad->bfa_hal, &doneq);
		spin_unlock_irqrestore(&bfad->bfad_lock, flags);
		bfa_trc_fp(bfad, irq);
	}

	return IRQ_HANDLED;

}

#ifdef CONFIG_PCI_MSI
#define BFAD_MSIX_REQQ(id)						\
irqreturn_t bfad_msix_reqq##id(int irq, void *dev_id)			\
{									\
	bfad_t *bfad = dev_id;						\
									\
	bfa_trc(bfad, irq);						\
	spin_lock(&bfad->bfad_lock);					\
	bfa_hal_msix_reqq(&bfad->bfa_hal, id);				\
	spin_unlock(&bfad->bfad_lock);					\
									\
	return IRQ_HANDLED;						\
}

BFAD_MSIX_REQQ(0)
BFAD_MSIX_REQQ(1)
BFAD_MSIX_REQQ(2)
BFAD_MSIX_REQQ(3)
BFAD_MSIX_REQQ(4)
BFAD_MSIX_REQQ(5)
BFAD_MSIX_REQQ(6)
BFAD_MSIX_REQQ(7)

#define BFAD_MSIX_RSPQ(id)						\
irqreturn_t bfad_msix_rspq##id(int irq, void *dev_id)			\
{									\
	bfad_t *bfad = dev_id;						\
	bfa_q_t	doneq;							\
	bfa_hal_intr_status_t rc;					\
									\
	bfa_trc(bfad, irq);						\
	spin_lock(&bfad->bfad_lock);					\
	rc = bfa_hal_msix_rspq(&bfad->bfa_hal, id);			\
	bfa_hal_comp_deq(&bfad->bfa_hal, &doneq);			\
									\
	if (!bfa_q_is_empty(&doneq)) {					\
		spin_unlock(&bfad->bfad_lock);				\
		bfa_hal_comp_process(&bfad->bfa_hal, &doneq);		\
		spin_lock(&bfad->bfad_lock);				\
		bfa_hal_comp_free(&bfad->bfa_hal, &doneq);		\
	}								\
	spin_unlock(&bfad->bfad_lock);					\
									\
	return IRQ_HANDLED;						\
}

BFAD_MSIX_RSPQ(0)
BFAD_MSIX_RSPQ(1)
BFAD_MSIX_RSPQ(2)
BFAD_MSIX_RSPQ(3)
BFAD_MSIX_RSPQ(4)
BFAD_MSIX_RSPQ(5)
BFAD_MSIX_RSPQ(6)
BFAD_MSIX_RSPQ(7)

irqreturn_t
bfad_msix_lpu(int irq, void *dev_id)
{
	bfad_t         *bfad = dev_id;

	bfa_trc(bfad, bfad->inst_no);
	bfa_trc(bfad, irq);
	spin_lock(&bfad->bfad_lock);
	bfa_hal_msix_lpu(&bfad->bfa_hal);
	spin_unlock(&bfad->bfad_lock);

	return IRQ_HANDLED;
}

irqreturn_t
bfad_msix_errint(int irq, void *dev_id)
{
	bfad_t         *bfad = dev_id;

	bfa_trc(bfad, irq);
	spin_lock(&bfad->bfad_lock);
	bfa_hal_msix_errint(&bfad->bfa_hal);
	spin_unlock(&bfad->bfad_lock);

	return IRQ_HANDLED;
}

/*
 * Global interrupt handler table, one to one map to each bit of the chip
 * interrupt register.
 */
bfad_isr_t      bfad_msix_isr[] = {
	(bfad_isr_t) bfad_msix_reqq0,
	(bfad_isr_t) bfad_msix_reqq1,
	(bfad_isr_t) bfad_msix_reqq2,
	(bfad_isr_t) bfad_msix_reqq3,
	(bfad_isr_t) bfad_msix_reqq4,
	(bfad_isr_t) bfad_msix_reqq5,
	(bfad_isr_t) bfad_msix_reqq6,
	(bfad_isr_t) bfad_msix_reqq7,
	(bfad_isr_t) bfad_msix_rspq0,
	(bfad_isr_t) bfad_msix_rspq1,
	(bfad_isr_t) bfad_msix_rspq2,
	(bfad_isr_t) bfad_msix_rspq3,
	(bfad_isr_t) bfad_msix_rspq4,
	(bfad_isr_t) bfad_msix_rspq5,
	(bfad_isr_t) bfad_msix_rspq6,
	(bfad_isr_t) bfad_msix_rspq7,
	(bfad_isr_t) bfad_msix_errint,
	(bfad_isr_t) bfad_msix_errint,
	(bfad_isr_t) bfad_msix_errint,
	(bfad_isr_t) bfad_msix_errint,
	(bfad_isr_t) bfad_msix_lpu,
	(bfad_isr_t) bfad_msix_lpu,
	(bfad_isr_t) bfad_msix_lpu,
	(bfad_isr_t) bfad_msix_lpu,
};

/**
 * @brief
 * Initialize the MSIX entry table.
 */
void
bfad_init_msix_entry(bfad_t *bfad, struct msix_entry *msix_entries, int mask,
		     int max_bit)
{
	int             i;
	int             match = 0x00000001;

	for (i = 0, bfad->nvec = 0; i < MAX_MSIX_ENTRY; i++) {
		if (mask & match) {
			bfad->msix_tab[bfad->nvec].msix.entry = i;
			msix_entries[bfad->nvec].entry = i;
			bfad->msix_tab[bfad->nvec++].handler = bfad_msix_isr[i];
		}

		match <<= 1;
	}

}

int
bfad_install_msix_handler(bfad_t *bfad)
{
	int             i, error = 0;

	for (i = 0; i < bfad->nvec; i++) {
		error = request_irq(bfad->msix_tab[i].msix.vector,
				    bfad->msix_tab[i].handler, 0,
				    BFAD_DRIVER_NAME, bfad);
		printk(KERN_WARNING "%s: bfad%d irq %d\n", __FUNCTION__,
			 bfad->inst_no, bfad->msix_tab[i].msix.vector);

		if (error) {
			int             j;

			for (j = 0; j < i; j++)
				free_irq(bfad->msix_tab[j].msix.vector, bfad);

			return 1;
		}
	}

	return 0;
}

/**
 * @brief
 * Setup MSIX based interrupt.
 */
int
bfad_setup_intr(bfad_t *bfad)
{
	int             error = 0;

	if (!msix_disable) {
		int             mask = 0, i, num_bit = 0, max_bit = 0;
		struct msix_entry msix_entries[MAX_MSIX_ENTRY];

		/*
		 * Call HAL API to get the msix map for this PCI function.
		 */
		bfa_hal_msix_getvecs(&bfad->bfa_hal, &mask, &num_bit, &max_bit);

		/*
		 * Set up the msix entry table
		 */
		bfad_init_msix_entry(bfad, msix_entries, mask, max_bit);

		error = pci_enable_msix(bfad->pcidev, msix_entries, bfad->nvec);
		if (error) {
			/*
			 * Only error number of vector is available.
			 * We don't have a mechanism to map multiple
			 * interrupts into one vector, so even if we
			 * can try to request less vectors, we don't
			 * know how to associate interrupt events to
			 *  vectors. Linux doesn't dupicate vectors
			 * in the MSIX table for this case.
			 */

			/*
			 * Enable MSIX failed
			 */
			printk(KERN_WARNING
				"%s: enable_msix failed, %d bfad%d\n",
			       __FUNCTION__, error, bfad->inst_no);

			goto line_based;
		}

		/*
		 * Save the vectors
		 */
		for (i = 0; i < bfad->nvec; i++)
			bfad->msix_tab[i].msix.vector = msix_entries[i].vector;

		/*
		 * Set up interrupt handler for each vectors
		 */
		if (bfad_install_msix_handler(bfad)) {
			printk(KERN_WARNING "%s: install_msix failed, bfad%d\n",
			       __FUNCTION__, bfad->inst_no);
			goto line_based;
		}

		bfad->bfad_flags |= BFAD_MSIX_ON;

		goto success;
	}

line_based:
	error = 0;
	if (request_irq
	    (bfad->pcidev->irq, (bfad_isr_t) bfad_intx, BFAD_IRQ_FLAGS,
	     BFAD_DRIVER_NAME, bfad) != 0) {
		/*
		 * Enable interrupt handler failed
		 */
		return 1;
	}
success:
	return error;
}

void
bfad_remove_intr(bfad_t *bfad)
{
	int             i;

	if (bfad->bfad_flags & BFAD_MSIX_ON) {
		for (i = 0; i < bfad->nvec; i++)
			free_irq(bfad->msix_tab[i].msix.vector, bfad);

		pci_disable_msix(bfad->pcidev);
		bfad->bfad_flags &= ~BFAD_MSIX_ON;
	} else {
		free_irq(bfad->pcidev->irq, bfad);
	}
}

#else /* CONFIG_PCI_MSI */
/**
 * @brief
 * Setup line-based interrupt.
 */
int
bfad_setup_intr(bfad_t *bfad)
{
	if (request_irq
	    (bfad->pcidev->irq, (bfad_isr_t) bfad_intx, SA_SHIRQ,
	     BFAD_DRIVER_NAME, bfad) != 0) {
		/*
		 * Enable interrupt handler failed
		 */
		return 1;
	}

	return 0;
}

void
bfad_remove_intr(bfad_t *bfad)
{
	bfa_trc(bfad, bfad->pcidev->irq);
	free_irq(bfad->pcidev->irq, bfad);
}

#endif /* CONFIG_PCI_MSI */

/**
 * @}
 */
