/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfa_fcs_uf.c BFA FCS UF ( Unsolicited Frames )
 */

#include <fcs/bfa_fcs.h>
#include <hal/bfa_hal_uf.h>
#include <fcs/bfa_fcs_fabric.h>
#include <cs/bfa_stats.h>
#include "fcs.h"
#include "fcs_trcmod.h"
#include "fcs_fabric.h"

BFA_TRC_FILE(FCS, UF);

/**
 * @brief
 * 		HAL callback for unsolicited frame receive handler.
 *
 * @param[in]		cbarg		callback arg for receive handler
 * @param[in]		uf		unsolicited frame descriptor
 *
 * @return None
 */
static void
bfa_fcs_uf_recv(void *cbarg, bfa_hal_uf_t *uf)
{
	bfa_fcs_t      *fcs = (bfa_fcs_t *) cbarg;
	fchs_t         *fchs = bfa_hal_uf_get_frmbuf(uf);
	uint16_t        len = bfa_hal_uf_get_frmlen(uf);
	fc_vft_t       *vft;
	bfa_fcs_fabric_t *fabric;

	/**
	 * check for VFT header
	 */
	if (fchs->routing == FC_RTG_EXT_HDR &&
		fchs->cat_info == FC_CAT_VFT_HDR) {
		bfa_stats(fcs, uf.tagged);
		vft = bfa_hal_uf_get_frmbuf(uf);
		if (fcs->port_vfid == vft->vf_id)
			fabric = &fcs->fabric;
		else
			fabric = bfa_fcs_vf_lookup(fcs, (uint16_t) vft->vf_id);

		/**
		 * drop frame if vfid is unknown
		 */
		if (!fabric) {
			bfa_assert(0);
			bfa_stats(fcs, uf.vfid_unknown);
			bfa_hal_uf_free(uf);
			return;
		}

		/**
		 * skip vft header
		 */
		fchs = (fchs_t *) (vft + 1);
		len -= sizeof(fc_vft_t);

		bfa_trc(fcs, vft->vf_id);
	} else {
		bfa_stats(fcs, uf.untagged);
		fabric = &fcs->fabric;
	}

	bfa_trc(fcs, ((uint32_t *) fchs)[0]);
	bfa_trc(fcs, ((uint32_t *) fchs)[1]);
	bfa_trc(fcs, ((uint32_t *) fchs)[2]);
	bfa_trc(fcs, ((uint32_t *) fchs)[3]);
	bfa_trc(fcs, ((uint32_t *) fchs)[4]);
	bfa_trc(fcs, ((uint32_t *) fchs)[5]);
	bfa_trc(fcs, len);

	bfa_fcs_fabric_uf_recv(fabric, fchs, len);
	bfa_hal_uf_free(uf);
}

void
bfa_fcs_uf_modinit(bfa_fcs_t *fcs)
{
	bfa_hal_uf_recv_register(fcs->hal, bfa_fcs_uf_recv, fcs);
}

void
bfa_fcs_uf_modexit(bfa_fcs_t *fcs)
{
	bfa_fcs_modexit_comp(fcs);
}
