/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
/*
 * fcbuild.c - FC link service frame building and parsing routines
 */

#include <bfa_os_inc.h>
#include <bfa_os.h>
#include "fcbuild.h"

/*
 * static build functions
 */
static void     fc_els_rsp_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id,
				 uint16_t ox_id);
static void     fc_bls_rsp_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id,
				 uint16_t ox_id);
static fchs_t   fc_els_req_tmpl;
static fchs_t   fc_els_rsp_tmpl;
static fchs_t   fc_bls_req_tmpl;
static fchs_t   fc_bls_rsp_tmpl;
static fc_ba_acc_t ba_acc_tmpl;
static fc_plogi_t plogi_tmpl;
static fc_prli_t prli_tmpl;
static fc_rrq_t rrq_tmpl;
static fchs_t   fcp_fchs_tmpl;

void
fcbuild_init(void)
{
	/*
	 * fc_els_req_tmpl
	 */
	fc_els_req_tmpl.routing = FC_RTG_EXT_LINK;
	fc_els_req_tmpl.cat_info = FC_CAT_LD_REQUEST;
	fc_els_req_tmpl.type = FC_TYPE_ELS;
	fc_els_req_tmpl.f_ctl =
		bfa_os_hton3b(FCTL_SEQ_INI | FCTL_FS_EXCH | FCTL_END_SEQ |
			      FCTL_SI_XFER);
	fc_els_req_tmpl.rx_id = FC_RXID_ANY;

	/*
	 * fc_els_rsp_tmpl
	 */
	fc_els_rsp_tmpl.routing = FC_RTG_EXT_LINK;
	fc_els_rsp_tmpl.cat_info = FC_CAT_LD_REPLY;
	fc_els_rsp_tmpl.type = FC_TYPE_ELS;
	fc_els_rsp_tmpl.f_ctl =
		bfa_os_hton3b(FCTL_EC_RESP | FCTL_SEQ_INI | FCTL_LS_EXCH |
			      FCTL_END_SEQ | FCTL_SI_XFER);
	fc_els_rsp_tmpl.rx_id = FC_RXID_ANY;

	/*
	 * fc_bls_req_tmpl
	 */
	fc_bls_req_tmpl.routing = FC_RTG_BASIC_LINK;
	fc_bls_req_tmpl.type = FC_TYPE_BLS;
	fc_bls_req_tmpl.f_ctl = bfa_os_hton3b(FCTL_END_SEQ | FCTL_SI_XFER);
	fc_bls_req_tmpl.rx_id = FC_RXID_ANY;

	/*
	 * fc_bls_rsp_tmpl
	 */
	fc_bls_rsp_tmpl.routing = FC_RTG_BASIC_LINK;
	fc_bls_rsp_tmpl.cat_info = FC_CAT_BA_ACC;
	fc_bls_rsp_tmpl.type = FC_TYPE_BLS;
	fc_bls_rsp_tmpl.f_ctl =
		bfa_os_hton3b(FCTL_EC_RESP | FCTL_SEQ_INI | FCTL_LS_EXCH |
			      FCTL_END_SEQ | FCTL_SI_XFER);
	fc_bls_rsp_tmpl.rx_id = FC_RXID_ANY;

	/*
	 * ba_acc_tmpl
	 */
	ba_acc_tmpl.seq_id_valid = 0;
	ba_acc_tmpl.low_seq_cnt = 0;
	ba_acc_tmpl.high_seq_cnt = 0xFFFF;

	/*
	 * plogi_tmpl
	 */
	plogi_tmpl.csp.verhi = FC_PH_VER_PH_3;
	plogi_tmpl.csp.verlo = FC_PH_VER_4_3;
	plogi_tmpl.csp.bbcred = bfa_os_htons(0x0004);
	plogi_tmpl.csp.ciro = 0x1;
	plogi_tmpl.csp.cisc = 0x0;
	plogi_tmpl.csp.altbbcred = 0x0;
	plogi_tmpl.csp.conseq = bfa_os_htons(0x00FF);
	plogi_tmpl.csp.ro_bitmap = bfa_os_htons(0x0002);
	plogi_tmpl.csp.e_d_tov = bfa_os_htonl(2000);

	plogi_tmpl.class3.class_valid = 1;
	plogi_tmpl.class3.sequential = 1;
	plogi_tmpl.class3.conseq = 0xFF;
	plogi_tmpl.class3.ospx = 1;

	/*
	 * prli_tmpl
	 */
	prli_tmpl.command = FC_ELS_PRLI;
	prli_tmpl.pglen = 0x10;
	prli_tmpl.pagebytes = bfa_os_htons(0x0014);
	prli_tmpl.parampage.type = FC_TYPE_FCP;
	prli_tmpl.parampage.imagepair = 1;
	prli_tmpl.parampage.servparams.rxrdisab = 1;

	/*
	 * rrq_tmpl
	 */
	rrq_tmpl.els_cmd.els_code = FC_ELS_RRQ;

	/*
	 * fcp_fchs_tmpl
	 */
	fcp_fchs_tmpl.routing = FC_RTG_FC4_DEV_DATA;
	fcp_fchs_tmpl.cat_info = FC_CAT_UNSOLICIT_CMD;
	fcp_fchs_tmpl.type = FC_TYPE_FCP;
	fcp_fchs_tmpl.f_ctl =
		bfa_os_hton3b(FCTL_FS_EXCH | FCTL_END_SEQ | FCTL_SI_XFER);
	fcp_fchs_tmpl.seq_id = 1;
	fcp_fchs_tmpl.rx_id = FC_RXID_ANY;
}

static void
fc_gs_fchdr_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id, uint32_t ox_id)
{
	fcif_t          fcif_z0 = { 0 };
	fcif_t         *ifhdr = &fchs->fcif;

	*ifhdr = fcif_z0;
	fcif_sof_set(ifhdr, SOFi3);
	fcif_eof_set(ifhdr, EOFt);

	bfa_os_memset(fchs, 0, sizeof(fchs_t));

	fchs->routing = FC_RTG_FC4_DEV_DATA;
	fchs->cat_info = FC_CAT_UNSOLICIT_CTRL;
	fchs->type = FC_TYPE_SERVICES;
	fchs->f_ctl =
		bfa_os_hton3b(FCTL_SEQ_INI | FCTL_FS_EXCH | FCTL_END_SEQ |
			      FCTL_SI_XFER);
	fchs->rx_id = FC_RXID_ANY;
	fchs->d_id = (d_id);
	fchs->s_id = (s_id);
	fchs->ox_id = bfa_os_htons(ox_id);

	/**
	 * @todo no need to set ox_id for request
	 *       no need to set rx_id for response
	 */
}

extern void
fc_els_req_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id, uint16_t ox_id)
{
	fcif_t         *ifhdr = &fchs->fcif;

	memset(ifhdr, '\0', sizeof(fcif_t));
	fcif_sof_set(ifhdr, SOFi3);
	fcif_eof_set(ifhdr, EOFt);

	memcpy(fchs, &fc_els_req_tmpl, sizeof(fchs_t));
	fchs->d_id = (d_id);
	fchs->s_id = (s_id);
	fchs->ox_id = bfa_os_htons(ox_id);
}

static void
fc_els_rsp_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id, uint16_t ox_id)
{
	fcif_t         *ifhdr = &fchs->fcif;

	memset(ifhdr, '\0', sizeof(fcif_t));
	fcif_sof_set(ifhdr, SOFi3);
	fcif_eof_set(ifhdr, EOFt);

	memcpy(fchs, &fc_els_rsp_tmpl, sizeof(fchs_t));
	fchs->d_id = d_id;
	fchs->s_id = s_id;
	fchs->ox_id = ox_id;
}

fc_parse_status_t
fc_els_rsp_parse(fchs_t *fchs, int len)
{
	fc_els_cmd_t   *els_cmd = (fc_els_cmd_t *) (fchs + 1);
	fc_ls_rjt_t    *ls_rjt = (fc_ls_rjt_t *) els_cmd;

	len = len;

	switch (els_cmd->els_code) {
	case FC_ELS_LS_RJT:
		if (ls_rjt->reason_code == FC_LS_RJT_RSN_LOGICAL_BUSY)
			return (FC_PARSE_BUSY);
		else
			return (FC_PARSE_FAILURE);

	case FC_ELS_ACC:
		return (FC_PARSE_OK);
	}
	return (FC_PARSE_OK);
}

static void
fc_bls_rsp_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id, uint16_t ox_id)
{
	fcif_t         *ifhdr = &fchs->fcif;

	memset(ifhdr, '\0', sizeof(fcif_t));
	fcif_sof_set(ifhdr, SOFi3);
	fcif_eof_set(ifhdr, EOFt);

	memcpy(fchs, &fc_bls_rsp_tmpl, sizeof(fchs_t));
	fchs->d_id = d_id;
	fchs->s_id = s_id;
	fchs->ox_id = ox_id;
}

static          uint16_t
fc_plogi_x_build(fchs_t *fchs, void *pld, uint32_t d_id, uint32_t s_id,
		 uint16_t ox_id, wwn_t port_name, wwn_t node_name,
		 uint16_t pdu_size, uint8_t els_code)
{
	fc_plogi_t     *plogi = (fc_plogi_t *) (pld);

	memcpy(plogi, &plogi_tmpl, sizeof(fc_plogi_t));

	plogi->els_cmd.els_code = els_code;
	if (els_code == FC_ELS_PLOGI)
		fc_els_req_build(fchs, d_id, s_id, ox_id);
	else
		fc_els_rsp_build(fchs, d_id, s_id, ox_id);

	plogi->csp.rxsz = plogi->class3.rxsz = bfa_os_htons(pdu_size);
	plogi->port_name = port_name;
	plogi->node_name = node_name;

	return (sizeof(fc_plogi_t));
}

uint16_t
fc_flogi_build(fchs_t *fchs, fc_flogi_t *flogi, uint32_t s_id, uint16_t ox_id,
	       wwn_t port_name, wwn_t node_name, uint16_t pdu_size,
	       uint8_t set_npiv, uint8_t set_auth, uint16_t local_bb_credits)
{
	uint32_t        d_id = bfa_os_hton3b(FC_FABRIC_PORT);

	memcpy(flogi, &plogi_tmpl, sizeof(fc_flogi_t));

	flogi->els_cmd.els_code = FC_ELS_FLOGI;
	fc_els_req_build(fchs, d_id, s_id, ox_id);

	flogi->csp.rxsz = flogi->class3.rxsz = bfa_os_htons(pdu_size);
	flogi->port_name = port_name;
	flogi->node_name = node_name;

	/*
	 * Set the NPIV Capability Bit ( word 1, bit 31) of Common
	 * Service Parameters.
	 */
	flogi->csp.ciro = set_npiv;

	/* set AUTH capability */
	flogi->csp.security = set_auth;

	flogi->csp.bbcred = bfa_os_htons(local_bb_credits);

	return (sizeof(fc_flogi_t));
}

uint16_t
fc_flogi_acc_build(fchs_t *fchs, fc_flogi_t *flogi, uint32_t s_id,
		   uint16_t ox_id, wwn_t port_name, wwn_t node_name,
		   uint16_t pdu_size, uint16_t local_bb_credits)
{
	uint32_t        d_id = 0;

	memcpy(flogi, &plogi_tmpl, sizeof(fc_flogi_t));
	fc_els_rsp_build(fchs, d_id, s_id, ox_id);

	flogi->els_cmd.els_code = FC_ELS_ACC;
	flogi->csp.rxsz = flogi->class3.rxsz = bfa_os_htons(pdu_size);
	flogi->port_name = port_name;
	flogi->node_name = node_name;

	flogi->csp.bbcred = bfa_os_htons(local_bb_credits);

	return (sizeof(fc_flogi_t));
}

uint16_t
fc_fdisc_build(fchs_t *fchs, fc_flogi_t *flogi, uint32_t s_id, uint16_t ox_id,
	       wwn_t port_name, wwn_t node_name, uint16_t pdu_size)
{
	uint32_t        d_id = bfa_os_hton3b(FC_FABRIC_PORT);

	memcpy(flogi, &plogi_tmpl, sizeof(fc_flogi_t));

	flogi->els_cmd.els_code = FC_ELS_FDISC;
	fc_els_req_build(fchs, d_id, s_id, ox_id);

	flogi->csp.rxsz = flogi->class3.rxsz = bfa_os_htons(pdu_size);
	flogi->port_name = port_name;
	flogi->node_name = node_name;

	return (sizeof(fc_flogi_t));
}

uint16_t
fc_plogi_build(fchs_t *fchs, void *pld, uint32_t d_id, uint32_t s_id,
	       uint16_t ox_id, wwn_t port_name, wwn_t node_name,
	       uint16_t pdu_size)
{
	return fc_plogi_x_build(fchs, pld, d_id, s_id, ox_id, port_name,
				node_name, pdu_size, FC_ELS_PLOGI);
}

uint16_t
fc_plogi_acc_build(fchs_t *fchs, void *pld, uint32_t d_id, uint32_t s_id,
		   uint16_t ox_id, wwn_t port_name, wwn_t node_name,
		   uint16_t pdu_size)
{
	return fc_plogi_x_build(fchs, pld, d_id, s_id, ox_id, port_name,
				node_name, pdu_size, FC_ELS_ACC);
}

fc_parse_status_t
fc_plogi_rsp_parse(fchs_t *fchs, int len, wwn_t port_name)
{
	fc_els_cmd_t   *els_cmd = (fc_els_cmd_t *) (fchs + 1);
	fc_plogi_t     *plogi;
	fc_ls_rjt_t    *ls_rjt;

	switch (els_cmd->els_code) {
	case FC_ELS_LS_RJT:
		ls_rjt = (fc_ls_rjt_t *) (fchs + 1);
		if (ls_rjt->reason_code == FC_LS_RJT_RSN_LOGICAL_BUSY)
			return (FC_PARSE_BUSY);
		else
			return (FC_PARSE_FAILURE);
	case FC_ELS_ACC:
		plogi = (fc_plogi_t *) (fchs + 1);
		if (len < sizeof(fc_plogi_t))
			return (FC_PARSE_FAILURE);

		if (!wwn_is_equal(plogi->port_name, port_name))
			return (FC_PARSE_FAILURE);

		if (!plogi->class3.class_valid)
			return (FC_PARSE_FAILURE);

		if (bfa_os_ntohs(plogi->class3.rxsz) < (FC_MIN_PDUSZ))
			return (FC_PARSE_FAILURE);

		return (FC_PARSE_OK);
	default:
		return (FC_PARSE_FAILURE);
	}
}

fc_parse_status_t
fc_plogi_parse(fchs_t *fchs)
{
	fc_plogi_t     *plogi = (fc_plogi_t *) (fchs + 1);

	if (plogi->class3.class_valid != 1)
		return FC_PARSE_FAILURE;

	if ((bfa_os_ntohs(plogi->class3.rxsz) < FC_MIN_PDUSZ)
	    || (bfa_os_ntohs(plogi->class3.rxsz) > FC_MAX_PDUSZ)
	    || (plogi->class3.rxsz == 0))
		return (FC_PARSE_FAILURE);

	return FC_PARSE_OK;
}

uint16_t
fc_prli_build(fchs_t *fchs, void *pld, uint32_t d_id, uint32_t s_id,
	      uint16_t ox_id)
{
	fc_prli_t      *prli = (fc_prli_t *) (pld);

	fc_els_req_build(fchs, d_id, s_id, ox_id);
	memcpy(prli, &prli_tmpl, sizeof(fc_prli_t));

	prli->command = FC_ELS_PRLI;
	prli->parampage.servparams.initiator     = 1;
	prli->parampage.servparams.retry         = 1;
	prli->parampage.servparams.rec_support   = 1;
	prli->parampage.servparams.task_retry_id = 0;
	prli->parampage.servparams.confirm       = 1;

	return (sizeof(fc_prli_t));
}

uint16_t
fc_prli_acc_build(fchs_t *fchs, void *pld, uint32_t d_id, uint32_t s_id,
		  uint16_t ox_id, bfa_port_role_t role)
{
	fc_prli_t      *prli = (fc_prli_t *) (pld);

	fc_els_rsp_build(fchs, d_id, s_id, ox_id);
	memcpy(prli, &prli_tmpl, sizeof(fc_prli_t));

	prli->command = FC_ELS_ACC;

	if ((role & BFA_PORT_ROLE_FCP_TM) == BFA_PORT_ROLE_FCP_TM)
		prli->parampage.servparams.target = 1;
	else
		prli->parampage.servparams.initiator = 1;

	prli->parampage.rspcode = FC_PRLI_ACC_XQTD;

	return (sizeof(fc_prli_t));
}

fc_parse_status_t
fc_prli_rsp_parse(fc_prli_t *prli, int len)
{
	if (len < sizeof(fc_prli_t))
		return (FC_PARSE_FAILURE);

	if (prli->command != FC_ELS_ACC)
		return (FC_PARSE_FAILURE);

	if ((prli->parampage.rspcode != FC_PRLI_ACC_XQTD)
	    && (prli->parampage.rspcode != FC_PRLI_ACC_PREDEF_IMG))
		return (FC_PARSE_FAILURE);

	if (prli->parampage.servparams.target != 1)
		return (FC_PARSE_FAILURE);

	return (FC_PARSE_OK);
}

fc_parse_status_t
fc_prli_parse(fc_prli_t *prli)
{
	if (prli->parampage.type != FC_TYPE_FCP)
		return (FC_PARSE_FAILURE);

	if (!prli->parampage.imagepair)
		return (FC_PARSE_FAILURE);

	if (!prli->parampage.servparams.initiator)
		return (FC_PARSE_FAILURE);

	return (FC_PARSE_OK);
}

uint16_t
fc_logo_build(fchs_t *fchs, fc_logo_t *logo, uint32_t d_id, uint32_t s_id,
	      uint16_t ox_id, wwn_t port_name)
{
	fc_els_req_build(fchs, d_id, s_id, ox_id);

	memset(logo, '\0', sizeof(fc_logo_t));
	logo->els_cmd.els_code = FC_ELS_LOGO;
	logo->nport_id = (s_id);
	logo->orig_port_name = port_name;

	return (sizeof(fc_logo_t));
}

static          uint16_t
fc_adisc_x_build(fchs_t *fchs, fc_adisc_t *adisc, uint32_t d_id,
		 uint32_t s_id, uint16_t ox_id, wwn_t port_name,
		 wwn_t node_name, uint8_t els_code)
{
	memset(adisc, '\0', sizeof(fc_adisc_t));

	adisc->els_cmd.els_code = els_code;

	if (els_code == FC_ELS_ADISC)
		fc_els_req_build(fchs, d_id, s_id, ox_id);
	else
		fc_els_rsp_build(fchs, d_id, s_id, ox_id);

	adisc->orig_HA = 0;
	adisc->orig_port_name = port_name;
	adisc->orig_node_name = node_name;
	adisc->nport_id = (s_id);

	return (sizeof(fc_adisc_t));
}

uint16_t
fc_adisc_build(fchs_t *fchs, fc_adisc_t *adisc, uint32_t d_id, uint32_t s_id,
	       uint16_t ox_id, wwn_t port_name, wwn_t node_name)
{
	return fc_adisc_x_build(fchs, adisc, d_id, s_id, ox_id, port_name,
				node_name, FC_ELS_ADISC);
}

uint16_t
fc_adisc_acc_build(fchs_t *fchs, fc_adisc_t *adisc, uint32_t d_id,
		   uint32_t s_id, uint16_t ox_id, wwn_t port_name,
		   wwn_t node_name)
{
	return fc_adisc_x_build(fchs, adisc, d_id, s_id, ox_id, port_name,
				node_name, FC_ELS_ACC);
}

fc_parse_status_t
fc_adisc_rsp_parse(fc_adisc_t *adisc, int len, wwn_t port_name, 
				 wwn_t node_name)
{

	if (len < sizeof(fc_adisc_t))
		return (FC_PARSE_FAILURE);

	if (adisc->els_cmd.els_code != FC_ELS_ACC)
		return (FC_PARSE_FAILURE);

	if (!wwn_is_equal(adisc->orig_port_name, port_name))
		return (FC_PARSE_FAILURE);

	return (FC_PARSE_OK);
}

fc_parse_status_t
fc_adisc_parse(fchs_t *fchs, void *pld, uint32_t host_dap, wwn_t node_name,
	       wwn_t port_name)
{
	fc_adisc_t     *adisc = (fc_adisc_t *) pld;

	if (adisc->els_cmd.els_code != FC_ELS_ACC)
		return (FC_PARSE_FAILURE);

	if ((adisc->nport_id == (host_dap))
	    && wwn_is_equal(adisc->orig_port_name, port_name)
	    && wwn_is_equal(adisc->orig_node_name, node_name))
		return (FC_PARSE_OK);

	return (FC_PARSE_FAILURE);
}

fc_parse_status_t
fc_pdisc_parse(fchs_t *fchs, wwn_t node_name, wwn_t port_name)
{
	fc_plogi_t     *pdisc = (fc_plogi_t *) (fchs + 1);

	if (pdisc->class3.class_valid != 1)
		return FC_PARSE_FAILURE;

	if ((bfa_os_ntohs(pdisc->class3.rxsz) < (FC_MIN_PDUSZ - sizeof(fchs_t)))
	    || (pdisc->class3.rxsz == 0))
		return (FC_PARSE_FAILURE);

	if (!wwn_is_equal(pdisc->port_name, port_name))
		return (FC_PARSE_FAILURE);

	if (!wwn_is_equal(pdisc->node_name, node_name))
		return (FC_PARSE_FAILURE);

	return FC_PARSE_OK;
}

uint16_t
fc_abts_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id, uint16_t ox_id)
{
	fcif_t         *ifhdr = &fchs->fcif;

	memset(ifhdr, '\0', sizeof(fcif_t));
	fcif_sof_set(ifhdr, SOFi3);
	fcif_eof_set(ifhdr, EOFt);

	memcpy(fchs, &fc_bls_req_tmpl, sizeof(fchs_t));
	fchs->cat_info = FC_CAT_ABTS;
	fchs->d_id = (d_id);
	fchs->s_id = (s_id);
	fchs->ox_id = bfa_os_htons(ox_id);

	return (sizeof(fchs_t));
}

fc_parse_status_t
fc_abts_rsp_parse(fchs_t *fchs, int len)
{
	if ((fchs->cat_info == FC_CAT_BA_ACC)
	    || (fchs->cat_info == FC_CAT_BA_RJT))
		return (FC_PARSE_OK);

	return (FC_PARSE_FAILURE);
}

uint16_t
fc_rrq_build(fchs_t *fchs, fc_rrq_t *rrq, uint32_t d_id, uint32_t s_id,
	     uint16_t ox_id, uint16_t rrq_oxid)
{
	fc_els_req_build(fchs, d_id, s_id, ox_id);

	/*
	 * build rrq payload
	 */
	memcpy(rrq, &rrq_tmpl, sizeof(fc_rrq_t));
	rrq->s_id = (s_id);
	rrq->ox_id = bfa_os_htons(rrq_oxid);
	rrq->rx_id = FC_RXID_ANY;

	return (sizeof(fc_rrq_t));
}

uint16_t
fc_logo_acc_build(fchs_t *fchs, void *pld, uint32_t d_id, uint32_t s_id,
		  uint16_t ox_id)
{
	fc_els_cmd_t   *acc = pld;

	fc_els_rsp_build(fchs, d_id, s_id, ox_id);

	memset(acc, 0, sizeof(fc_els_cmd_t));
	acc->els_code = FC_ELS_ACC;

	return (sizeof(fc_els_cmd_t));
}

uint16_t
fc_ls_rjt_build(fchs_t *fchs, fc_ls_rjt_t *ls_rjt, uint32_t d_id,
		uint32_t s_id, uint16_t ox_id, uint8_t reason_code,
		uint8_t reason_code_expl)
{
	fc_els_rsp_build(fchs, d_id, s_id, ox_id);
	memset(ls_rjt, 0, sizeof(fc_ls_rjt_t));

	ls_rjt->els_cmd.els_code = FC_ELS_LS_RJT;
	ls_rjt->reason_code = reason_code;
	ls_rjt->reason_code_expl = reason_code_expl;
	ls_rjt->vendor_unique = 0x00;

	return (sizeof(fc_ls_rjt_t));
}

uint16_t
fc_ba_acc_build(fchs_t *fchs, fc_ba_acc_t *ba_acc, uint32_t d_id,
		uint32_t s_id, uint16_t ox_id, uint16_t rx_id)
{
	fc_bls_rsp_build(fchs, d_id, s_id, ox_id);

	memcpy(ba_acc, &ba_acc_tmpl, sizeof(fc_ba_acc_t));

	fchs->rx_id = rx_id;

	ba_acc->ox_id = fchs->ox_id;
	ba_acc->rx_id = fchs->rx_id;

	return (sizeof(fc_ba_acc_t));
}

uint16_t
fc_ls_acc_build(fchs_t *fchs, fc_els_cmd_t *els_cmd, uint32_t d_id,
		uint32_t s_id, uint16_t ox_id)
{
	fc_els_rsp_build(fchs, d_id, s_id, ox_id);
	memset(els_cmd, 0, sizeof(fc_els_cmd_t));
	els_cmd->els_code = FC_ELS_ACC;

	return (sizeof(fc_els_cmd_t));
}

int
fc_logout_params_pages(fchs_t *fc_frame, uint8_t els_code)
{
	int             num_pages = 0;
	fc_prlo_t      *prlo;
	fc_tprlo_t     *tprlo;

	if (els_code == FC_ELS_PRLO) {
		prlo = (fc_prlo_t *) (fc_frame + 1);
		num_pages = (bfa_os_ntohs(prlo->payload_len) - 4) / 16;
	} else {
		tprlo = (fc_tprlo_t *) (fc_frame + 1);
		num_pages = (bfa_os_ntohs(tprlo->payload_len) - 4) / 16;
	}
	return num_pages;
}

uint16_t
fc_tprlo_acc_build(fchs_t *fchs, fc_tprlo_acc_t *tprlo_acc, uint32_t d_id,
		   uint32_t s_id, uint16_t ox_id, int num_pages)
{
	int             page;

	fc_els_rsp_build(fchs, d_id, s_id, ox_id);

	memset(tprlo_acc, 0, (num_pages * 16) + 4);
	tprlo_acc->command = FC_ELS_ACC;

	tprlo_acc->page_len = 0x10;
	tprlo_acc->payload_len = bfa_os_htons((num_pages * 16) + 4);

	for (page = 0; page < num_pages; page++) {
		tprlo_acc->tprlo_acc_params[page].opa_valid = 0;
		tprlo_acc->tprlo_acc_params[page].rpa_valid = 0;
		tprlo_acc->tprlo_acc_params[page].fc4type_csp = FC_TYPE_FCP;
		tprlo_acc->tprlo_acc_params[page].orig_process_assc = 0;
		tprlo_acc->tprlo_acc_params[page].resp_process_assc = 0;
	}
	return (bfa_os_ntohs(tprlo_acc->payload_len));
}

uint16_t
fc_prlo_acc_build(fchs_t *fchs, fc_prlo_acc_t *prlo_acc, uint32_t d_id,
		  uint32_t s_id, uint16_t ox_id, int num_pages)
{
	int             page;

	fc_els_rsp_build(fchs, d_id, s_id, ox_id);

	memset(prlo_acc, 0, (num_pages * 16) + 4);
	prlo_acc->command = FC_ELS_ACC;
	prlo_acc->page_len = 0x10;
	prlo_acc->payload_len = bfa_os_htons((num_pages * 16) + 4);

	for (page = 0; page < num_pages; page++) {
		prlo_acc->prlo_acc_params[page].opa_valid = 0;
		prlo_acc->prlo_acc_params[page].rpa_valid = 0;
		prlo_acc->prlo_acc_params[page].fc4type_csp = FC_TYPE_FCP;
		prlo_acc->prlo_acc_params[page].orig_process_assc = 0;
		prlo_acc->prlo_acc_params[page].resp_process_assc = 0;
	}

	return (bfa_os_ntohs(prlo_acc->payload_len));
}

uint16_t
fc_rnid_build(fchs_t *fchs, fc_rnid_cmd_t *rnid, uint32_t d_id, uint32_t s_id,
	      uint16_t ox_id, uint32_t data_format)
{
	fc_els_req_build(fchs, d_id, s_id, ox_id);

	memset(rnid, 0, sizeof(fc_rnid_cmd_t));

	rnid->els_cmd.els_code = FC_ELS_RNID;
	rnid->node_id_data_format = data_format;

	return (sizeof(fc_rnid_cmd_t));
}

uint16_t
fc_rnid_acc_build(fchs_t *fchs, fc_rnid_acc_t *rnid_acc, uint32_t d_id,
		  uint32_t s_id, uint16_t ox_id, uint32_t data_format,
		  fc_rnid_common_id_data_t *common_id_data,
		  fc_rnid_general_topology_data_t *gen_topo_data)
{
	memset(rnid_acc, 0, sizeof(fc_rnid_acc_t));

	fc_els_rsp_build(fchs, d_id, s_id, ox_id);

	rnid_acc->els_cmd.els_code = FC_ELS_ACC;
	rnid_acc->node_id_data_format = data_format;
	rnid_acc->common_id_data_length = sizeof(fc_rnid_common_id_data_t);
	rnid_acc->common_id_data = *common_id_data;

	if (data_format == RNID_NODEID_DATA_FORMAT_DISCOVERY) {
		rnid_acc->specific_id_data_length =
			sizeof(fc_rnid_general_topology_data_t);
		rnid_acc->gen_topology_data = *gen_topo_data;
		return (sizeof(fc_rnid_acc_t));
	} else {
		return (sizeof(fc_rnid_acc_t) -
			sizeof(fc_rnid_general_topology_data_t));
	}

}

uint16_t
fc_rpsc_build(fchs_t *fchs, fc_rpsc_cmd_t *rpsc, uint32_t d_id, uint32_t s_id,
	      uint16_t ox_id)
{
	fc_els_req_build(fchs, d_id, s_id, ox_id);

	memset(rpsc, 0, sizeof(fc_rpsc_cmd_t));

	rpsc->els_cmd.els_code = FC_ELS_RPSC;
	return (sizeof(fc_rpsc_cmd_t));
}

uint16_t
fc_rpsc_acc_build(fchs_t *fchs, fc_rpsc_acc_t *rpsc_acc, uint32_t d_id,
		  uint32_t s_id, uint16_t ox_id,
		  fc_rpsc_speed_info_t *oper_speed)
{
	memset(rpsc_acc, 0, sizeof(fc_rpsc_acc_t));

	fc_els_rsp_build(fchs, d_id, s_id, ox_id);

	rpsc_acc->command = FC_ELS_ACC;
	rpsc_acc->num_entries = bfa_os_htons(1);

	rpsc_acc->speed_info[0].port_speed_cap =
		bfa_os_htons(oper_speed->port_speed_cap);

	rpsc_acc->speed_info[0].port_op_speed =
		bfa_os_htons(oper_speed->port_op_speed);

	return (sizeof(fc_rpsc_acc_t));

}

uint16_t
fcp_cmnd_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id, uint16_t ox_id,
	       fcp_iodir_t iodir, lun_t lun, uint8_t *cdb, uint32_t fcp_dl)
{
	fcif_t         *ifhdr = &fchs->fcif;
	fcp_cmnd_t     *fcp_cmnd = (fcp_cmnd_t *) (fchs + 1);

	memset(ifhdr, '\0', sizeof(fcif_t));
	fcif_sof_set(ifhdr, SOFi3);
	fcif_eof_set(ifhdr, EOFt);

	memcpy(fchs, &fcp_fchs_tmpl, sizeof(fchs_t));
	fchs->d_id = (d_id);
	fchs->s_id = (s_id);
	fchs->ox_id = bfa_os_htons(ox_id);

	memset(fcp_cmnd, '\0', sizeof(fcp_cmnd_t));
	fcp_cmnd->lun = lun;
	memcpy(&fcp_cmnd->cdb, cdb, FCP_CMND_CDB_LEN);
	fcp_cmnd->fcp_dl = bfa_os_htonl(fcp_dl);

	fcp_cmnd->iodir = iodir;

	return (sizeof(fcp_cmnd_t));
}

uint16_t
fcp_read_data_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id, uint16_t ox_id,
		    uint16_t rx_id, uint8_t seq_id, uint16_t seq_cnt,
		    uint32_t data_ro, int last_frame, uint32_t data_len)
{
	fcif_t         *ifhdr = &fchs->fcif;
	uint32_t        fill_bytes = 0;

	memset(ifhdr, '\0', sizeof(fcif_t));

	memcpy(fchs, &fcp_fchs_tmpl, sizeof(fchs_t));
	fchs->d_id = (d_id);
	fchs->s_id = (s_id);
	fchs->ox_id = bfa_os_htons(ox_id);
	fchs->rx_id = bfa_os_htons(rx_id);
	fchs->ro = bfa_os_htonl(data_ro);
	fchs->cat_info = FC_CAT_SOLICIT_DATA;

	fill_bytes = (((data_len + 3) - ((data_len + 3) % 4)) - data_len);

	if (!seq_cnt)
		fcif_sof_set(ifhdr, SOFi3);
	else
		fcif_sof_set(ifhdr, SOFn3);

	if (!last_frame) {
		fcif_eof_set(ifhdr, EOFn);

		fchs->f_ctl =
			bfa_os_hton3b(FCTL_EC_RESP | FCTL_RO_PRESENT |
				      fill_bytes);
	} else {
		fcif_eof_set(ifhdr, EOFt);

		fchs->f_ctl =
			bfa_os_hton3b(FCTL_EC_RESP | FCTL_END_SEQ |
				      FCTL_RO_PRESENT | fill_bytes);
	}

	fchs->seq_id = seq_id;
	fchs->seq_cnt = bfa_os_htons(seq_cnt);

	return (0);
}

uint16_t
fcp_write_data_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id,
		     uint16_t ox_id, uint16_t rx_id, uint8_t seq_id,
		     uint16_t seq_cnt, uint32_t data_ro, int last_frame)
{
	fcif_t         *ifhdr = &fchs->fcif;

	memset(ifhdr, '\0', sizeof(fcif_t));

	memcpy(fchs, &fcp_fchs_tmpl, sizeof(fchs_t));
	fchs->d_id = (d_id);
	fchs->s_id = (s_id);
	fchs->ox_id = bfa_os_htons(ox_id);
	fchs->rx_id = bfa_os_htons(rx_id);
	fchs->ro = bfa_os_htonl(data_ro);
	fchs->cat_info = FC_CAT_SOLICIT_DATA;

	if (!seq_cnt)
		fcif_sof_set(ifhdr, SOFi3);
	else
		fcif_sof_set(ifhdr, SOFn3);

	if (!last_frame) {
		fcif_eof_set(ifhdr, EOFn);
		fchs->f_ctl = bfa_os_hton3b(FCTL_RO_PRESENT);
	} else {
		fcif_eof_set(ifhdr, EOFt);
		fchs->f_ctl =
			bfa_os_hton3b(FCTL_END_SEQ | FCTL_SI_XFER |
				      FCTL_RO_PRESENT);
	}

	fchs->seq_id = seq_id;
	fchs->seq_cnt = bfa_os_htons(seq_cnt);

	return (0);
}

uint16_t
fcp_xfer_rdy_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id, uint8_t seq_id,
		   uint16_t seq_cnt, uint16_t ox_id, uint16_t rx_id,
		   uint32_t data_ro, uint32_t burst_len)
{
	fcif_t         *ifhdr = &fchs->fcif;
	fcp_xfer_rdy_t *xfer_rdy = (fcp_xfer_rdy_t *) (fchs + 1);

	memset(ifhdr, '\0', sizeof(fcif_t));
	fcif_sof_set(ifhdr, SOFi3);
	fcif_eof_set(ifhdr, EOFt);

	memcpy(fchs, &fcp_fchs_tmpl, sizeof(fchs_t));
	fchs->d_id = (d_id);
	fchs->s_id = (s_id);
	fchs->ox_id = bfa_os_htons(ox_id);
	fchs->cat_info = FC_CAT_DATA_DESC;
	fchs->seq_id = seq_id;
	fchs->seq_cnt = bfa_os_htons(seq_cnt);
	fchs->rx_id = bfa_os_htons(rx_id);
	fchs->f_ctl =
		bfa_os_hton3b(FCTL_EC_RESP | FCTL_SEQ_INI | FCTL_END_SEQ |
			      FCTL_SI_XFER);

	xfer_rdy->data_ro = bfa_os_htonl(data_ro);
	xfer_rdy->burst_len = bfa_os_htonl(burst_len);

	return (sizeof(fcp_xfer_rdy_t));
}

uint16_t
fcp_rsp_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id, uint16_t ox_id,
	      uint16_t rx_id, uint8_t seq_id, uint8_t scsi_status,
	      fcp_residue_t resflag, uint32_t residue, uint32_t sns_len,
	      void *sns_info, fcp_rspinfo_t *rsp_info)
{
	fcif_t         *ifhdr = &fchs->fcif;
	fcp_resp_t     *fcp_rsp = (fcp_resp_t *) (fchs + 1);
	uint32_t        rsp_len = 0;

	memset(ifhdr, '\0', sizeof(fcif_t));
	fcif_sof_set(ifhdr, SOFi3);
	fcif_eof_set(ifhdr, EOFt);

	memcpy(fchs, &fcp_fchs_tmpl, sizeof(fchs_t));
	fchs->d_id = (d_id);
	fchs->s_id = (s_id);
	fchs->ox_id = bfa_os_htons(ox_id);
	fchs->rx_id = bfa_os_htons(rx_id);
	fchs->cat_info = FC_CAT_CMD_STATUS;
	fchs->f_ctl =
		bfa_os_hton3b(FCTL_EC_RESP | FCTL_LS_EXCH | FCTL_END_SEQ |
			      FCTL_SI_XFER);

	fchs->seq_id = seq_id;

	/*
	 * FCP_RSP fields
	 */
	memset(fcp_rsp, 0, sizeof(fcp_resp_t));
	fcp_rsp->resid_flags = resflag;

	fcp_rsp->sns_len = bfa_os_htonl(sns_len);

	if (sns_len) {
		fcp_rsp->sns_len_valid = 1;
		if (rsp_info == NULL)
			memcpy((fcp_rsp + 1), sns_info, sns_len);
		else
			memcpy(((uint8_t *) (fcp_rsp + 1)) + rsp_len, sns_info,
			       sns_len);
	}

	if (rsp_info) {
		fcp_rsp->rsp_len_valid = 1;
		memcpy((fcp_rsp + 1), rsp_info, rsp_len);
		rsp_len = sizeof(fcp_rspinfo_t);
	}

	fcp_rsp->scsi_status = scsi_status;
	fcp_rsp->residue = bfa_os_htonl(residue);
	fcp_rsp->rsp_len = bfa_os_htonl(rsp_len);

	return (uint16_t) (sizeof(fcp_resp_t) - sizeof(fcp_rspinfo_t) +
			   sns_len + rsp_len);
}

uint16_t
fcp_tm_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id, uint16_t ox_id,
	     lun_t lun, uint8_t tm_flags)
{
	fcif_t         *ifhdr = &fchs->fcif;
	fcp_cmnd_t     *fcp_cmnd = (fcp_cmnd_t *) (fchs + 1);

	memset(ifhdr, '\0', sizeof(fcif_t));
	fcif_sof_set(ifhdr, SOFi3);
	fcif_eof_set(ifhdr, EOFt);

	memcpy(fchs, &fcp_fchs_tmpl, sizeof(fchs_t));
	fchs->d_id = (d_id);
	fchs->s_id = (s_id);
	fchs->ox_id = bfa_os_htons(ox_id);

	memset(fcp_cmnd, '\0', sizeof(fcp_cmnd_t));
	fcp_cmnd->lun = lun;
	fcp_cmnd->fcp_dl = 0;
	fcp_cmnd->tm_flags = tm_flags;

	return (sizeof(fcp_cmnd_t));
}

/*
 * TBD -
 * . get rid of unnecessary memsets
 */

uint16_t
fc_logo_rsp_parse(fchs_t *fchs, int len)
{
	fc_els_cmd_t   *els_cmd = (fc_els_cmd_t *) (fchs + 1);

	len = len;
	if (els_cmd->els_code != FC_ELS_ACC)
		return FC_PARSE_FAILURE;

	return FC_PARSE_OK;
}

uint16_t
fc_pdisc_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id, uint16_t ox_id,
	       wwn_t port_name, wwn_t node_name, uint16_t pdu_size)
{
	fc_plogi_t     *pdisc = (fc_plogi_t *) (fchs + 1);

	memcpy(pdisc, &plogi_tmpl, sizeof(fc_plogi_t));

	pdisc->els_cmd.els_code = FC_ELS_PDISC;
	fc_els_req_build(fchs, d_id, s_id, ox_id);

	pdisc->csp.rxsz = pdisc->class3.rxsz = bfa_os_htons(pdu_size);
	pdisc->port_name = port_name;
	pdisc->node_name = node_name;

	return (sizeof(fc_plogi_t));
}

uint16_t
fc_pdisc_rsp_parse(fchs_t *fchs, int len, wwn_t port_name)
{
	fc_plogi_t     *pdisc = (fc_plogi_t *) (fchs + 1);

	if (len < sizeof(fc_plogi_t))
		return (FC_PARSE_LEN_INVAL);

	if (pdisc->els_cmd.els_code != FC_ELS_ACC)
		return (FC_PARSE_ACC_INVAL);

	if (!wwn_is_equal(pdisc->port_name, port_name))
		return (FC_PARSE_PWWN_NOT_EQUAL);

	if (!pdisc->class3.class_valid)
		return (FC_PARSE_NWWN_NOT_EQUAL);

	if (bfa_os_ntohs(pdisc->class3.rxsz) < (FC_MIN_PDUSZ))
		return (FC_PARSE_RXSZ_INVAL);

	return (FC_PARSE_OK);
}

uint16_t
fc_prlo_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id, uint16_t ox_id,
	      int num_pages)
{
	fc_prlo_t      *prlo = (fc_prlo_t *) (fchs + 1);
	int             page;

	fc_els_req_build(fchs, d_id, s_id, ox_id);
	memset(prlo, 0, (num_pages * 16) + 4);
	prlo->command = FC_ELS_PRLO;
	prlo->page_len = 0x10;
	prlo->payload_len = bfa_os_htons((num_pages * 16) + 4);

	for (page = 0; page < num_pages; page++) {
		prlo->prlo_params[page].type = FC_TYPE_FCP;
		prlo->prlo_params[page].opa_valid = 0;
		prlo->prlo_params[page].rpa_valid = 0;
		prlo->prlo_params[page].orig_process_assc = 0;
		prlo->prlo_params[page].resp_process_assc = 0;
	}

	return (bfa_os_ntohs(prlo->payload_len));
}

uint16_t
fc_prlo_rsp_parse(fchs_t *fchs, int len)
{
	fc_prlo_acc_t  *prlo = (fc_prlo_acc_t *) (fchs + 1);
	int             num_pages = 0;
	int             page = 0;

	len = len;

	if (prlo->command != FC_ELS_ACC)
		return (FC_PARSE_FAILURE);

	num_pages = ((bfa_os_ntohs(prlo->payload_len)) - 4) / 16;

	for (page = 0; page < num_pages; page++) {
		if (prlo->prlo_acc_params[page].type != FC_TYPE_FCP)
			return FC_PARSE_FAILURE;

		if (prlo->prlo_acc_params[page].opa_valid != 0)
			return FC_PARSE_FAILURE;

		if (prlo->prlo_acc_params[page].rpa_valid != 0)
			return FC_PARSE_FAILURE;

		if (prlo->prlo_acc_params[page].orig_process_assc != 0)
			return FC_PARSE_FAILURE;

		if (prlo->prlo_acc_params[page].resp_process_assc != 0)
			return FC_PARSE_FAILURE;
	}
	return (FC_PARSE_OK);

}

uint16_t
fc_tprlo_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id, uint16_t ox_id,
	       int num_pages, fc_tprlo_type_t tprlo_type, uint32_t tpr_id)
{
	fc_tprlo_t     *tprlo = (fc_tprlo_t *) (fchs + 1);
	int             page;

	fc_els_req_build(fchs, d_id, s_id, ox_id);
	memset(tprlo, 0, (num_pages * 16) + 4);
	tprlo->command = FC_ELS_TPRLO;
	tprlo->page_len = 0x10;
	tprlo->payload_len = bfa_os_htons((num_pages * 16) + 4);

	for (page = 0; page < num_pages; page++) {
		tprlo->tprlo_params[page].type = FC_TYPE_FCP;
		tprlo->tprlo_params[page].opa_valid = 0;
		tprlo->tprlo_params[page].rpa_valid = 0;
		tprlo->tprlo_params[page].orig_process_assc = 0;
		tprlo->tprlo_params[page].resp_process_assc = 0;
		if (tprlo_type == FC_GLOBAL_LOGO) {
			tprlo->tprlo_params[page].global_process_logout = 1;
		} else if (tprlo_type == FC_TPR_LOGO) {
			tprlo->tprlo_params[page].tpo_nport_valid = 1;
			tprlo->tprlo_params[page].tpo_nport_id = (tpr_id);
		}
	}

	return (bfa_os_ntohs(tprlo->payload_len));
}

uint16_t
fc_tprlo_rsp_parse(fchs_t *fchs, int len)
{
	fc_tprlo_acc_t *tprlo = (fc_tprlo_acc_t *) (fchs + 1);
	int             num_pages = 0;
	int             page = 0;

	len = len;

	if (tprlo->command != FC_ELS_ACC)
		return (FC_PARSE_ACC_INVAL);

	num_pages = (bfa_os_ntohs(tprlo->payload_len) - 4) / 16;

	for (page = 0; page < num_pages; page++) {
		if (tprlo->tprlo_acc_params[page].type != FC_TYPE_FCP)
			return (FC_PARSE_NOT_FCP);
		if (tprlo->tprlo_acc_params[page].opa_valid != 0)
			return (FC_PARSE_OPAFLAG_INVAL);
		if (tprlo->tprlo_acc_params[page].rpa_valid != 0)
			return (FC_PARSE_RPAFLAG_INVAL);
		if (tprlo->tprlo_acc_params[page].orig_process_assc != 0)
			return (FC_PARSE_OPA_INVAL);
		if (tprlo->tprlo_acc_params[page].resp_process_assc != 0)
			return (FC_PARSE_RPA_INVAL);
	}
	return (FC_PARSE_OK);
}

fc_parse_status_t
fc_rrq_rsp_parse(fchs_t *fchs, int len)
{
	fc_els_cmd_t   *els_cmd = (fc_els_cmd_t *) (fchs + 1);

	len = len;
	if (els_cmd->els_code != FC_ELS_ACC)
		return FC_PARSE_FAILURE;

	return FC_PARSE_OK;
}

uint16_t
fc_ba_rjt_build(fchs_t *fchs, uint32_t d_id, uint32_t s_id, uint16_t ox_id,
		uint32_t reason_code, uint32_t reason_expl)
{
	fc_ba_rjt_t    *ba_rjt = (fc_ba_rjt_t *) (fchs + 1);

	fc_bls_rsp_build(fchs, d_id, s_id, ox_id);

	fchs->cat_info = FC_CAT_BA_RJT;
	ba_rjt->reason_code = reason_code;
	ba_rjt->reason_expl = reason_expl;
	return (sizeof(fc_ba_rjt_t));
}

static void
fc_gs_cthdr_build(ct_hdr_t *cthdr, uint32_t s_id, uint16_t cmd_code)
{
	bfa_os_memset(cthdr, 0, sizeof(ct_hdr_t));
	cthdr->rev_id = CT_GS3_REVISION;
	cthdr->gs_type = CT_GSTYPE_DIRSERVICE;
	cthdr->gs_sub_type = CT_GSSUBTYPE_NAMESERVER;
	cthdr->cmd_rsp_code = bfa_os_htons(cmd_code);
}

static void
fc_gs_fdmi_cthdr_build(ct_hdr_t *cthdr, uint32_t s_id, uint16_t cmd_code)
{
	bfa_os_memset(cthdr, 0, sizeof(ct_hdr_t));
	cthdr->rev_id = CT_GS3_REVISION;
	cthdr->gs_type = CT_GSTYPE_MGMTSERVICE;
	cthdr->gs_sub_type = CT_GSSUBTYPE_HBA_MGMTSERVER;
	cthdr->cmd_rsp_code = bfa_os_htons(cmd_code);
}

uint16_t
fc_gidpn_build(fchs_t *fchs, void *pyld, uint32_t s_id, uint16_t ox_id,
	       wwn_t port_name)
{

	ct_hdr_t       *cthdr = (ct_hdr_t *) pyld;
	fcgs_gidpn_req_t *gidpn = (fcgs_gidpn_req_t *) (cthdr + 1);
	uint32_t        d_id = bfa_os_hton3b(FC_NAME_SERVER);

	fc_gs_fchdr_build(fchs, d_id, s_id, ox_id);
	fc_gs_cthdr_build(cthdr, s_id, GS_GID_PN);

	bfa_os_memset(gidpn, 0, sizeof(fcgs_gidpn_req_t));
	gidpn->port_name = port_name;
	return (sizeof(fcgs_gidpn_req_t) + sizeof(ct_hdr_t));
}

uint16_t
fc_gpnid_build(fchs_t *fchs, void *pyld, uint32_t s_id, uint16_t ox_id,
	       uint32_t port_id)
{

	ct_hdr_t       *cthdr = (ct_hdr_t *) pyld;
	fcgs_gpnid_req_t *gpnid = (fcgs_gpnid_req_t *) (cthdr + 1);
	uint32_t        d_id = bfa_os_hton3b(FC_NAME_SERVER);

	fc_gs_fchdr_build(fchs, d_id, s_id, ox_id);
	fc_gs_cthdr_build(cthdr, s_id, GS_GPN_ID);

	bfa_os_memset(gpnid, 0, sizeof(fcgs_gpnid_req_t));
	gpnid->dap = port_id;
	return (sizeof(fcgs_gpnid_req_t) + sizeof(ct_hdr_t));
}

uint16_t
fc_gnnid_build(fchs_t *fchs, void *pyld, uint32_t s_id, uint16_t ox_id,
	       uint32_t port_id)
{

	ct_hdr_t       *cthdr = (ct_hdr_t *) pyld;
	fcgs_gnnid_req_t *gnnid = (fcgs_gnnid_req_t *) (cthdr + 1);
	uint32_t        d_id = bfa_os_hton3b(FC_NAME_SERVER);

	fc_gs_fchdr_build(fchs, d_id, s_id, ox_id);
	fc_gs_cthdr_build(cthdr, s_id, GS_GNN_ID);

	bfa_os_memset(gnnid, 0, sizeof(fcgs_gnnid_req_t));
	gnnid->dap = port_id;
	return (sizeof(fcgs_gnnid_req_t) + sizeof(ct_hdr_t));
}

uint16_t
fc_ct_rsp_parse(ct_hdr_t *cthdr)
{
	if (bfa_os_ntohs(cthdr->cmd_rsp_code) != CT_RSP_ACCEPT) {
		if (cthdr->reason_code == CT_RSN_LOGICAL_BUSY)
			return FC_PARSE_BUSY;
		else
			return FC_PARSE_FAILURE;
	}

	return FC_PARSE_OK;
}

uint16_t
fc_scr_build(fchs_t *fchs, fc_scr_t *scr, uint32_t s_id, uint16_t ox_id)
{
	uint32_t        d_id = bfa_os_hton3b(FC_FABRIC_CONTROLLER);

	fc_els_req_build(fchs, d_id, s_id, ox_id);

	bfa_os_memset(scr, 0, sizeof(fc_scr_t));
	scr->command = FC_ELS_SCR;
	scr->reg_func = FC_SCR_REG_FUNC_FULL;
	return (sizeof(fc_scr_t));
}

uint16_t
fc_rscn_build(fchs_t *fchs, fc_rscn_pl_t *rscn, uint32_t s_id, uint16_t ox_id)
{
	uint32_t        d_id = bfa_os_hton3b(FC_FABRIC_CONTROLLER);
	uint16_t        payldlen;

	fc_els_req_build(fchs, d_id, s_id, ox_id);
	rscn->command = FC_ELS_RSCN;
	rscn->pagelen = sizeof(rscn->event[0]);

	payldlen = sizeof(uint32_t) + rscn->pagelen;
	rscn->payldlen = bfa_os_htons(payldlen);

	rscn->event[0].format = FC_RSCN_FORMAT_PORTID;
	rscn->event[0].portid = s_id;

	return (sizeof(fc_rscn_pl_t));
}

uint16_t
fc_rftid_build(fchs_t *fchs, void *pyld, uint32_t s_id, uint16_t ox_id,
	       uint8_t fc4_type)
{
	ct_hdr_t       *cthdr = (ct_hdr_t *) pyld;
	fcgs_rftid_req_t *rftid = (fcgs_rftid_req_t *) (cthdr + 1);
	uint32_t        type_value, d_id = bfa_os_hton3b(FC_NAME_SERVER);
	uint8_t         index;

	fc_gs_fchdr_build(fchs, d_id, s_id, ox_id);
	fc_gs_cthdr_build(cthdr, s_id, GS_RFT_ID);

	bfa_os_memset(rftid, 0, sizeof(fcgs_rftid_req_t));

	rftid->dap = s_id;
	index = fc4_type >> 5;
	type_value = 1 << (fc4_type % 32);
	rftid->fc4_type[index] = bfa_os_htonl(type_value);

	return (sizeof(fcgs_rftid_req_t) + sizeof(ct_hdr_t));
}

uint16_t
fc_rftid_build_sol(fchs_t *fchs, void *pyld, uint32_t s_id, uint16_t ox_id,
		   uint8_t *fc4_bitmap, uint32_t bitmap_size)
{
	ct_hdr_t       *cthdr = (ct_hdr_t *) pyld;
	fcgs_rftid_req_t *rftid = (fcgs_rftid_req_t *) (cthdr + 1);
	uint32_t        d_id = bfa_os_hton3b(FC_NAME_SERVER);

	fc_gs_fchdr_build(fchs, d_id, s_id, ox_id);
	fc_gs_cthdr_build(cthdr, s_id, GS_RFT_ID);

	bfa_os_memset(rftid, 0, sizeof(fcgs_rftid_req_t));

	rftid->dap = s_id;
	bfa_os_memcpy((void *)rftid->fc4_type, (void *)fc4_bitmap,
(bitmap_size < 32 ? bitmap_size : 32));

	return (sizeof(fcgs_rftid_req_t) + sizeof(ct_hdr_t));
}

uint16_t
fc_rffid_build(fchs_t *fchs, void *pyld, uint32_t s_id, uint16_t ox_id,
	       uint8_t fc4_type, uint8_t fc4_ftrs)
{
	ct_hdr_t         *cthdr = (ct_hdr_t *) pyld;
	fcgs_rffid_req_t *rffid = (fcgs_rffid_req_t *) (cthdr + 1);
	uint32_t         d_id = bfa_os_hton3b(FC_NAME_SERVER);

	fc_gs_fchdr_build(fchs, d_id, s_id, ox_id);
	fc_gs_cthdr_build(cthdr, s_id, GS_RFF_ID);

	bfa_os_memset(rffid, 0, sizeof(fcgs_rffid_req_t));

	rffid->dap 		 	= s_id;
	rffid->fc4ftr_bits  = fc4_ftrs;
	rffid->fc4_type		= fc4_type;

	return (sizeof(fcgs_rffid_req_t) + sizeof(ct_hdr_t));
}

uint16_t
fc_rspnid_build(fchs_t *fchs, void *pyld, uint32_t s_id, uint16_t ox_id,
		uint8_t *name)
{

	ct_hdr_t       *cthdr = (ct_hdr_t *) pyld;
	fcgs_rspnid_req_t *rspnid = (fcgs_rspnid_req_t *) (cthdr + 1);
	uint32_t        d_id = bfa_os_hton3b(FC_NAME_SERVER);

	fc_gs_fchdr_build(fchs, d_id, s_id, ox_id);
	fc_gs_cthdr_build(cthdr, s_id, GS_RSPN_ID);

	bfa_os_memset(rspnid, 0, sizeof(fcgs_rspnid_req_t));

	rspnid->dap = s_id;
	rspnid->spn_len = (uint8_t) strlen((char *)name);
	strncpy((char *)rspnid->spn, (char *)name, rspnid->spn_len);

	return (sizeof(fcgs_rspnid_req_t) + sizeof(ct_hdr_t));
}

uint16_t
fc_gid_ft_build(fchs_t *fchs, void *pyld, uint32_t s_id, uint8_t fc4_type)
{

	ct_hdr_t       *cthdr = (ct_hdr_t *) pyld;
	fcgs_gidft_req_t *gidft = (fcgs_gidft_req_t *) (cthdr + 1);
	uint32_t        d_id = bfa_os_hton3b(FC_NAME_SERVER);

	fc_gs_fchdr_build(fchs, d_id, s_id, 0);

	fc_gs_cthdr_build(cthdr, s_id, GS_GID_FT);

	bfa_os_memset(gidft, 0, sizeof(fcgs_gidft_req_t));
	gidft->fc4_type = fc4_type;
	gidft->domain_id = 0;
	gidft->area_id = 0;

	return (sizeof(fcgs_gidft_req_t) + sizeof(ct_hdr_t));
}

uint16_t
fc_rpnid_build(fchs_t *fchs, void *pyld, uint32_t s_id, uint32_t port_id,
	       wwn_t port_name)
{
	ct_hdr_t       *cthdr = (ct_hdr_t *) pyld;
	fcgs_rpnid_req_t *rpnid = (fcgs_rpnid_req_t *) (cthdr + 1);
	uint32_t        d_id = bfa_os_hton3b(FC_NAME_SERVER);

	fc_gs_fchdr_build(fchs, d_id, s_id, 0);
	fc_gs_cthdr_build(cthdr, s_id, GS_RPN_ID);

	bfa_os_memset(rpnid, 0, sizeof(fcgs_rpnid_req_t));
	rpnid->port_id = port_id;
	rpnid->port_name = port_name;

	return (sizeof(fcgs_rpnid_req_t) + sizeof(ct_hdr_t));
}

uint16_t
fc_rnnid_build(fchs_t *fchs, void *pyld, uint32_t s_id, uint32_t port_id,
	       wwn_t node_name)
{
	ct_hdr_t       *cthdr = (ct_hdr_t *) pyld;
	fcgs_rnnid_req_t *rnnid = (fcgs_rnnid_req_t *) (cthdr + 1);
	uint32_t        d_id = bfa_os_hton3b(FC_NAME_SERVER);

	fc_gs_fchdr_build(fchs, d_id, s_id, 0);
	fc_gs_cthdr_build(cthdr, s_id, GS_RNN_ID);

	bfa_os_memset(rnnid, 0, sizeof(fcgs_rnnid_req_t));
	rnnid->port_id = port_id;
	rnnid->node_name = node_name;

	return (sizeof(fcgs_rnnid_req_t) + sizeof(ct_hdr_t));
}

uint16_t
fc_rcsid_build(fchs_t *fchs, void *pyld, uint32_t s_id, uint32_t port_id,
	       uint32_t cos)
{
	ct_hdr_t       *cthdr = (ct_hdr_t *) pyld;
	fcgs_rcsid_req_t *rcsid = (fcgs_rcsid_req_t *) (cthdr + 1);
	uint32_t        d_id = bfa_os_hton3b(FC_NAME_SERVER);

	fc_gs_fchdr_build(fchs, d_id, s_id, 0);
	fc_gs_cthdr_build(cthdr, s_id, GS_RCS_ID);

	bfa_os_memset(rcsid, 0, sizeof(fcgs_rcsid_req_t));
	rcsid->port_id = port_id;
	rcsid->cos = cos;

	return (sizeof(fcgs_rcsid_req_t) + sizeof(ct_hdr_t));
}

uint16_t
fc_rptid_build(fchs_t *fchs, void *pyld, uint32_t s_id, uint32_t port_id,
	       uint8_t port_type)
{
	ct_hdr_t       *cthdr = (ct_hdr_t *) pyld;
	fcgs_rptid_req_t *rptid = (fcgs_rptid_req_t *) (cthdr + 1);
	uint32_t        d_id = bfa_os_hton3b(FC_NAME_SERVER);

	fc_gs_fchdr_build(fchs, d_id, s_id, 0);
	fc_gs_cthdr_build(cthdr, s_id, GS_RPT_ID);

	bfa_os_memset(rptid, 0, sizeof(fcgs_rptid_req_t));
	rptid->port_id = port_id;
	rptid->port_type = port_type;

	return (sizeof(fcgs_rptid_req_t) + sizeof(ct_hdr_t));
}

uint16_t
fc_ganxt_build(fchs_t *fchs, void *pyld, uint32_t s_id, uint32_t port_id)
{
	ct_hdr_t       *cthdr = (ct_hdr_t *) pyld;
	fcgs_ganxt_req_t *ganxt = (fcgs_ganxt_req_t *) (cthdr + 1);
	uint32_t        d_id = bfa_os_hton3b(FC_NAME_SERVER);

	fc_gs_fchdr_build(fchs, d_id, s_id, 0);
	fc_gs_cthdr_build(cthdr, s_id, GS_GA_NXT);

	bfa_os_memset(ganxt, 0, sizeof(fcgs_ganxt_req_t));
	ganxt->port_id = port_id;

	return (sizeof(ct_hdr_t) + sizeof(fcgs_ganxt_req_t));
}

/*
 * Builds fc hdr and ct hdr for FDMI requests.
 */
uint16_t
fc_fdmi_reqhdr_build(fchs_t *fchs, void *pyld, uint32_t s_id,
		     uint16_t cmd_code)
{

	ct_hdr_t       *cthdr = (ct_hdr_t *) pyld;
	uint32_t        d_id = bfa_os_hton3b(FC_MGMT_SERVER);

	fc_gs_fchdr_build(fchs, d_id, s_id, 0);
	fc_gs_fdmi_cthdr_build(cthdr, s_id, cmd_code);

	return (sizeof(ct_hdr_t));
}

/*
 * Given a FC4 Type, this function returns a fc4 type bitmask
 */
void
fc_get_fc4type_bitmask(uint8_t fc4_type, uint8_t *bit_mask)
{
	uint8_t         index;
	uint32_t       *ptr = (uint32_t *) bit_mask;
	uint32_t        type_value;

	/*
* @todo : Check for bitmask size
	 */

	index = fc4_type >> 5;
	type_value = 1 << (fc4_type % 32);
	ptr[index] = bfa_os_htonl(type_value);

}
