/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
/*
 * fcbuild.h - FC link service frame building and parsing routines
 */

#ifndef __FCBUILD_H__
#define __FCBUILD_H__

#include <bfa_os_inc.h>
#include <protocol/fc.h>
#include <protocol/fcp.h>
#include <protocol/ct.h>
#include <defs/bfa_defs_port.h>
#include <defs/bfa_defs_pport.h>

/*
 * Utility Macros/functions
 */

#define fcif_sof_set(_ifhdr, _sof)	(_ifhdr)->sof = FC_ ## _sof
#define fcif_eof_set(_ifhdr, _eof)	(_ifhdr)->eof = FC_ ## _eof

#define wwn_is_equal(_wwn1, _wwn2)		\
	(memcmp(&(_wwn1), &(_wwn2), sizeof(wwn_t)) == 0)

#define fc_roundup(_l, _s) (((_l) + ((_s) - 1)) & ~((_s) - 1))

/*
 * Given the fc response length, this routine will return
 * the length of the actual payload bytes following the CT header.
 *
 * Assumes the input response length does not include the crc, eof, etc.
 */
static inline   uint32_t
fc_get_ctresp_pyld_len(uint32_t resp_len)
{
	return (resp_len - sizeof(ct_hdr_t));
}

/*
 * Convert bfa speed to rpsc speed value.
 */
static inline   fc_rpsc_op_speed_t
fc_bfa_speed_to_rpsc_operspeed(bfa_pport_speed_t op_speed)
{
	switch (op_speed) {

	case BFA_PPORT_SPEED_1GBPS:
		return RPSC_OP_SPEED_1G;

	case BFA_PPORT_SPEED_2GBPS:
		return RPSC_OP_SPEED_2G;

	case BFA_PPORT_SPEED_4GBPS:
		return RPSC_OP_SPEED_4G;

	case BFA_PPORT_SPEED_8GBPS:
		return RPSC_OP_SPEED_8G;

	default:
		return RPSC_OP_SPEED_NOT_EST;
	}
}

typedef enum {
	FC_PARSE_OK = 0,
	FC_PARSE_FAILURE = 1,
	FC_PARSE_BUSY = 2,
	FC_PARSE_LEN_INVAL,
	FC_PARSE_ACC_INVAL,
	FC_PARSE_PWWN_NOT_EQUAL,
	FC_PARSE_NWWN_NOT_EQUAL,
	FC_PARSE_RXSZ_INVAL,
	FC_PARSE_NOT_FCP,
	FC_PARSE_OPAFLAG_INVAL,
	FC_PARSE_RPAFLAG_INVAL,
	FC_PARSE_OPA_INVAL,
	FC_PARSE_RPA_INVAL,

} fc_parse_status_t;

typedef struct fc_templates {
	fchs_t          fc_els_req;
	fchs_t          fc_bls_req;
	fc_plogi_t      plogi;
	fc_rrq_t        rrq;
} fc_templates_t;

void            fcbuild_init(void);

uint16_t        fc_flogi_build(fchs_t *fchs, fc_flogi_t *flogi, uint32_t s_id,
			       uint16_t ox_id, wwn_t port_name, wwn_t node_name,
			       uint16_t pdu_size, uint8_t set_npiv, uint8_t set_auth,
			       uint16_t local_bb_credits);

uint16_t        fc_fdisc_build(fchs_t *buf, fc_flogi_t *flogi, uint32_t s_id,
			       uint16_t ox_id, wwn_t port_name, wwn_t node_name,
			       uint16_t pdu_size);

uint16_t        fc_flogi_acc_build(fchs_t *fchs, fc_flogi_t *flogi,
				   uint32_t s_id, uint16_t ox_id,
				   wwn_t port_name, wwn_t node_name,
				   uint16_t pdu_size,
				   uint16_t local_bb_credits);

uint16_t        fc_plogi_build(fchs_t *fchs, void *pld, uint32_t d_id,
			       uint32_t s_id, uint16_t ox_id, wwn_t port_name,
			       wwn_t node_name, uint16_t pdu_size);

fc_parse_status_t fc_plogi_parse(fchs_t *fchs);

uint16_t        fc_abts_build(fchs_t *buf, uint32_t d_id, uint32_t s_id,
			      uint16_t ox_id);

fc_parse_status_t fc_abts_rsp_parse(fchs_t *buf, int len);

uint16_t        fc_rrq_build(fchs_t *buf, fc_rrq_t *rrq, uint32_t d_id,
			     uint32_t s_id, uint16_t ox_id, uint16_t rrq_oxid);
fc_parse_status_t fc_rrq_rsp_parse(fchs_t *buf, int len);

uint16_t        fc_rspnid_build(fchs_t *fchs, void *pld, uint32_t s_id,
				uint16_t ox_id, uint8_t *name);

uint16_t        fc_rftid_build(fchs_t *fchs, void *pld, uint32_t s_id,
			       uint16_t ox_id, uint8_t fc4_type);

uint16_t       fc_rftid_build_sol(fchs_t *fchs, void *pyld, uint32_t s_id,
				   uint16_t ox_id, uint8_t *fc4_bitmap,
				   uint32_t bitmap_size);

uint16_t 		fc_rffid_build(fchs_t *fchs, void *pyld, uint32_t s_id, uint16_t ox_id,
	       			uint8_t fc4_type, uint8_t fc4_ftrs);

uint16_t        fc_gidpn_build(fchs_t *fchs, void *pyld, uint32_t s_id,
			       uint16_t ox_id, wwn_t port_name);

uint16_t        fc_gpnid_build(fchs_t *fchs, void *pld, uint32_t s_id,
			       uint16_t ox_id, uint32_t port_id);

uint16_t        fc_scr_build(fchs_t *fchs, fc_scr_t *scr, uint32_t s_id,
			     uint16_t ox_id);

uint16_t        fc_plogi_acc_build(fchs_t *fchs, void *pld, uint32_t d_id,
				   uint32_t s_id, uint16_t ox_id,
				   wwn_t port_name, wwn_t node_name,
				   uint16_t pdu_size);

uint16_t        fc_adisc_build(fchs_t *fchs, fc_adisc_t *adisc, uint32_t d_id,
			       uint32_t s_id, uint16_t ox_id, wwn_t port_name,
			       wwn_t node_name);

fc_parse_status_t fc_adisc_parse(fchs_t *fchs, void *pld, uint32_t host_dap,
				 wwn_t node_name, wwn_t port_name);

fc_parse_status_t fc_adisc_rsp_parse(fc_adisc_t *adisc, int len, wwn_t port_name,
									wwn_t node_name);

uint16_t        fc_adisc_acc_build(fchs_t *fchs, fc_adisc_t *adisc,
				   uint32_t d_id, uint32_t s_id, uint16_t ox_id,
				   wwn_t port_name, wwn_t node_name);
uint16_t        fc_ls_rjt_build(fchs_t *fchs, fc_ls_rjt_t *ls_rjt,
				uint32_t d_id, uint32_t s_id, uint16_t ox_id,
				uint8_t reason_code, uint8_t reason_code_expl);
uint16_t        fc_ls_acc_build(fchs_t *fchs, fc_els_cmd_t *els_cmd,
				uint32_t d_id, uint32_t s_id, uint16_t ox_id);
uint16_t        fc_prli_build(fchs_t *fchs, void *pld, uint32_t d_id,
			      uint32_t s_id, uint16_t ox_id);

fc_parse_status_t fc_prli_rsp_parse(fc_prli_t *prli, int len);

uint16_t        fc_prli_acc_build(fchs_t *fchs, void *pld, uint32_t d_id,
				  uint32_t s_id, uint16_t ox_id,
				  bfa_port_role_t role);

uint16_t        fc_rnid_build(fchs_t *fchs, fc_rnid_cmd_t *rnid,
			      uint32_t d_id, uint32_t s_id, uint16_t ox_id,
			      uint32_t data_format);

uint16_t        fc_rnid_acc_build(fchs_t *fchs, fc_rnid_acc_t *rnid_acc,
				  uint32_t d_id, uint32_t s_id, uint16_t ox_id,
				  uint32_t data_format,
				  fc_rnid_common_id_data_t *common_id_data,
				  fc_rnid_general_topology_data_t *
				  gen_topo_data);

uint16_t        fc_rpsc_build(fchs_t *fchs, fc_rpsc_cmd_t *rpsc,
			      uint32_t d_id, uint32_t s_id, uint16_t ox_id);
uint16_t        fc_rpsc_acc_build(fchs_t *fchs, fc_rpsc_acc_t *rpsc_acc,
				  uint32_t d_id, uint32_t s_id, uint16_t ox_id,
				  fc_rpsc_speed_info_t *oper_speed);
uint16_t        fc_gid_ft_build(fchs_t *fchs, void *pld, uint32_t s_id,
				uint8_t fc4_type);

uint16_t        fc_rpnid_build(fchs_t *fchs, void *pyld, uint32_t s_id,
			       uint32_t port_id, wwn_t port_name);

uint16_t        fc_rnnid_build(fchs_t *fchs, void *pyld, uint32_t s_id,
			       uint32_t port_id, wwn_t node_name);

uint16_t        fc_rcsid_build(fchs_t *fchs, void *pyld, uint32_t s_id,
			       uint32_t port_id, uint32_t cos);

uint16_t        fc_rptid_build(fchs_t *fchs, void *pyld, uint32_t s_id,
			       uint32_t port_id, uint8_t port_type);

uint16_t        fc_ganxt_build(fchs_t *fchs, void *pyld, uint32_t s_id,
			       uint32_t port_id);

uint16_t        fc_logo_build(fchs_t *fchs, fc_logo_t *logo, uint32_t d_id,
			      uint32_t s_id, uint16_t ox_id, wwn_t port_name);

uint16_t        fc_logo_acc_build(fchs_t *fchs, void *pld, uint32_t d_id,
				  uint32_t s_id, uint16_t ox_id);

uint16_t        fc_fdmi_reqhdr_build(fchs_t *fchs, void *pyld, uint32_t s_id,
				     uint16_t cmd_code);
void		fc_get_fc4type_bitmask(uint8_t fc4_type, uint8_t *bit_mask);
#endif
