/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#include <hal/bfa_hal.h>
#include <hal/bfa_hal_timer.h>
#include <hal/bfa_hal_fcxp.h>
#include <hal/bfa_hal_rport.h>
#include <hcb/bfa_hcb_fcxp.h>
#include <fcs/bfa_fcs_port.h>
#include <fcs/bfa_fcs_vport.h>
#include <fcs/bfa_fcs_rport.h>
#include <hcb/bfa_hcb_rport.h>
#include <fcb/bfa_fcb_rport.h>
#include "fcs_vport.h"
#include "fcs_port.h"
#include "fcs_rport.h"
#include "fcs_trcmod.h"

BFA_TRC_FILE(FCS, RPORT_API);

/**
 * @file rport_api.c Remote port implementation.
 */

/**
 * @dg fcs_rport_api FCS rport API.
 * @{
 */

/**
 * @brief
 * 	Direct API to add a target by port wwn. This interface is used, for
 *	example, by bios when target pwwn is known from boot lun configuration.
 */
bfa_status_t
bfa_fcs_rport_add(bfa_fcs_port_t *port, wwn_t *pwwn, bfa_fcs_rport_t *rport,
		  bfad_rport_t *rport_drv)
{
	bfa_trc(port->fcs, *pwwn);

	return BFA_STATUS_OK;
}

/**
 * @brief
 *	Direct API to remove a target and its associated resources. This
 *	interface is used, for example, by vmware driver to remove target
 *	ports from the target list for a VM.
 */
bfa_status_t
bfa_fcs_rport_remove(bfa_fcs_rport_t *rport_in)
{

	bfa_fcs_rport_t *rport;

	bfa_trc(rport_in->fcs, rport_in->pwwn);

	rport = bfa_fcs_port_get_rport_by_pwwn(rport_in->port, rport_in->pwwn);
	if (rport == NULL) {
		/*
		 * TBD Error handling
		 */
		bfa_trc(rport_in->fcs, rport_in->pid);
		return BFA_STATUS_UNKNOWN_RWWN;
	}

	/*
	 * TBD if this remote port is online, send a logo
	 */
	return BFA_STATUS_OK;

}

/**
 * @brief
 *	Remote device status for display/debug.
 */
void
bfa_fcs_rport_get_attr(bfa_fcs_rport_t *rport, bfa_rport_attr_t *rport_attr)
{
	bfa_os_memset(rport_attr, 0, sizeof(bfa_rport_attr_t));

	rport_attr->pid = rport->pid;
	rport_attr->pwwn = rport->pwwn;
	rport_attr->nwwn = rport->nwwn;
	rport_attr->cos_supported = rport->fc_cos;
	rport_attr->df_sz = rport->maxfrsize;
	rport_attr->state = bfa_fcs_rport_get_state(rport);
	rport_attr->fc_cos = rport->fc_cos;
	rport_attr->cisc = rport->cisc;
	rport_attr->scsi_function = rport->scsi_function;

	/*
	 * TODO
	 * rport->symname
	 */
}

/**
 * @brief
 * 	Per remote device statistics.
 */
void
bfa_fcs_rport_get_stats(bfa_fcs_rport_t *rport, bfa_rport_stats_t *stats)
{
	*stats = rport->stats;
}

void
bfa_fcs_rport_clear_stats(bfa_fcs_rport_t *rport)
{
	bfa_os_memset((char *)&rport->stats, 0, sizeof(bfa_rport_stats_t));
}

bfa_fcs_rport_t *
bfa_fcs_rport_lookup(bfa_fcs_port_t *port, wwn_t rpwwn)
{
	bfa_fcs_rport_t *rport;

	rport = bfa_fcs_port_get_rport_by_pwwn(port, rpwwn);
	if (rport == NULL) {
		/*
		 * TBD Error handling
		 */
	}

	return rport;
}

bfa_fcs_rport_t *
bfa_fcs_rport_lookup_by_nwwn(bfa_fcs_port_t *port, wwn_t rnwwn)
{
	bfa_fcs_rport_t *rport;

	rport = bfa_fcs_port_get_rport_by_nwwn(port, rnwwn);
	if (rport == NULL) {
		/*
		 * TBD Error handling
		 */
	}

	return rport;
}

/**
 * @}
 */
