/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#include <hal/bfa_hal.h>
#include <hal/bfa_hal_debug.h>
#include <fwi/bfa_cbreg.h>

BFA_TRC_FILE(HAL, DEBUG);

/**
 * @ig hal_debug_api
 * @{
 */

#define BFA_HAL_FWTRC_ENTS	(128)
#define BFA_HAL_FWTRC_LEN	(BFA_HAL_FWTRC_ENTS * sizeof(bfa_trc_t) + \
					(sizeof(bfa_trc_mod_t) - 	\
					BFA_TRC_MAX * sizeof(bfa_trc_t)))
#define BFA_HAL_FWTRC_OFF(_fn)	(BFA_FWI_IOC_TRC_OFF + \
					BFA_HAL_FWTRC_LEN * (_fn))

/**
 * @brief
 * 		Fetch firmware trace data.
 *
 * @param[in]		hal			HAL instance
 * @param[out]		trcdata		Firmware trace buffer
 * @param[in,out]	trclen		Firmware trace buffer len
 *
 * @retval BFA_STATUS_OK			Firmware trace is fetched.
 * @retval BFA_STATUS_INPROGRESS	Firmware trace fetch is in progress.
 */
bfa_status_t
bfa_hal_debug_fwtrc(bfa_hal_t *hal, void *trcdata, int *trclen)
{
	bfa_hal_ioc_t  *ioc = &hal->ioc;
	uint32_t        pgnum, pgoff;
	uint32_t        loff = BFA_HAL_FWTRC_OFF(hal->pcidev->pci_func);
	int             i, tlen;
	uint32_t       *tbuf = trcdata, r32;

	bfa_trc(hal, *trclen);

	pgnum = PSS_SMEM_PGNUM(loff);
	pgoff = PSS_SMEM_PGOFF(loff);
	bfa_hal_reg_write(ioc->bfa_regs.host_page_num_fn, pgnum);

	tlen = *trclen;
	if (tlen > BFA_HAL_FWTRC_LEN)
		tlen = BFA_HAL_FWTRC_LEN;
	tlen /= sizeof(uint32_t);

	bfa_trc(hal, tlen);

	for (i = 0; i < tlen; i++) {
		r32 = bfa_hal_mem_read(ioc->bfa_regs.smem_page_start, loff);
		tbuf[i] = bfa_os_ntohl(r32);
		loff += sizeof(uint32_t);

		/**
		 * handle page offset wrap around
		 */
		loff = PSS_SMEM_PGOFF(loff);
		if (loff == 0) {
			pgnum++;
			bfa_hal_reg_write(ioc->bfa_regs.host_page_num_fn,
					  pgnum);
		}
	}
	bfa_hal_reg_write(ioc->bfa_regs.host_page_num_fn, PSS_SMEM_PGNUM(0));
	bfa_trc(hal, pgnum);

	*trclen = tlen * sizeof(uint32_t);
	return BFA_STATUS_OK;
}

/**
 * @}
 */
