/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#include <hal/bfa_hal.h>
#include <fwi/bfa_cbreg.h>
#include <hal_port.h>
#include <hal_intr.h>
#include <cs/bfa_debug.h>

BFA_TRC_FILE(HAL, INTR);

extern bfa_int_func_t hal_isrs[BFA_MC_MAX];

/**
 * @ig hal_intr_api
 * @{
 */
bfa_hal_intr_status_t
bfa_hal_intx(bfa_hal_t *hal)
{
	uint32_t        intr;
	int             vec;

	intr = bfa_hal_reg_read(hal->ioc.bfa_regs.intr_status);
	if (!intr)
		return BFA_HAL_INTR_NOTCLAIMED;

	/**
	 * mailbox interrupt
	 */
	if (hal->ioc.pci_info.pci_func == 0) {
		if (intr & __HFN_INT_MBOX_LPU0)
			bfa_hal_msix_lpu(hal);
	} else {
		if (intr & __HFN_INT_MBOX_LPU1)
			bfa_hal_msix_lpu(hal);
	}

	/**
	 * RME completion queue interrupt
	 */
	for (vec = HFN_MSIX_RME_Q0; vec <= HFN_MSIX_RME_Q7; vec++)
		if (intr & (__HFN_INT_RME_Q0 << (vec - HFN_MSIX_RME_Q0)))
			bfa_hal_msix_rspq(hal, (vec - HFN_MSIX_RME_Q0));

	/**
	 * CPE completion queue interrupt
	 */
	for (vec = HFN_MSIX_CPE_Q0; vec <= HFN_MSIX_CPE_Q7; vec++)
		if (intr & (__HFN_INT_CPE_Q0 << (vec - HFN_MSIX_CPE_Q0)))
			bfa_hal_msix_reqq(hal, (vec - HFN_MSIX_CPE_Q0));

	if (intr & (__HFN_INT_ERR_EMC | __HFN_INT_ERR_LPU0 |
				__HFN_INT_ERR_LPU1 | __HFN_INT_ERR_PSS))
		bfa_hal_msix_errint(hal);

	return (BFA_HAL_INTR_CLAIMED_COMP);
}

uint32_t
sm(uint32_t *addr, uint32_t val)
{
	*addr = val;
	return *addr;
}

void
bfa_hal_intr_enable(bfa_hal_t *hal)
{
	uint32_t        intr_unmask;
	int             pci_func = hal->ioc.pci_info.pci_func;

	bfa_trc(hal, pci_func);

	intr_unmask = (__HFN_INT_ERR_EMC | __HFN_INT_ERR_LPU0 |
				__HFN_INT_ERR_LPU1 | __HFN_INT_ERR_PSS);

	if (pci_func == 0)
		intr_unmask |= (__HFN_INT_CPE_Q0 | __HFN_INT_CPE_Q1 |
				__HFN_INT_CPE_Q2 | __HFN_INT_CPE_Q3 |
				__HFN_INT_RME_Q0 | __HFN_INT_RME_Q1 |
				__HFN_INT_RME_Q2 | __HFN_INT_RME_Q3 |
				__HFN_INT_MBOX_LPU0);
	else
		intr_unmask |= (__HFN_INT_CPE_Q4 | __HFN_INT_CPE_Q5 |
				__HFN_INT_CPE_Q6 | __HFN_INT_CPE_Q7 |
				__HFN_INT_RME_Q4 | __HFN_INT_RME_Q5 |
				__HFN_INT_RME_Q6 | __HFN_INT_RME_Q7 |
				__HFN_INT_MBOX_LPU1);

	bfa_hal_reg_write(hal->ioc.bfa_regs.intr_mask, ~intr_unmask);
}

void
bfa_hal_intr_disable(bfa_hal_t *hal)
{
	bfa_hal_reg_write(hal->ioc.bfa_regs.intr_mask, -1);
}

void
bfa_hal_msix_getvecs(bfa_hal_t *hal, uint32_t *msix_vecs_bmap,
		     uint32_t *num_vecs, uint32_t *max_vec_bit)
{
#define __HFN_NUMINTS	13
	if (hal->ioc.pci_info.pci_func == 0) {
		*msix_vecs_bmap = (__HFN_INT_CPE_Q0 | __HFN_INT_CPE_Q1 |
					__HFN_INT_CPE_Q2 | __HFN_INT_CPE_Q3 |
					__HFN_INT_RME_Q0 | __HFN_INT_RME_Q1 |
					__HFN_INT_RME_Q2 | __HFN_INT_RME_Q3 |
					__HFN_INT_MBOX_LPU0);
		*max_vec_bit = __HFN_INT_MBOX_LPU0;
	} else {
		*msix_vecs_bmap = (__HFN_INT_CPE_Q4 | __HFN_INT_CPE_Q5 |
					__HFN_INT_CPE_Q6 | __HFN_INT_CPE_Q7 |
					__HFN_INT_RME_Q4 | __HFN_INT_RME_Q5 |
					__HFN_INT_RME_Q6 | __HFN_INT_RME_Q7 |
					__HFN_INT_MBOX_LPU1);
		*max_vec_bit = __HFN_INT_MBOX_LPU1;
	}

	*msix_vecs_bmap |= (__HFN_INT_ERR_EMC | __HFN_INT_ERR_LPU0 |
				__HFN_INT_ERR_LPU1 | __HFN_INT_ERR_PSS);
	*num_vecs = __HFN_NUMINTS;
}

bfa_hal_intr_status_t
bfa_hal_msix_reqq(bfa_hal_t *hal, int qid)
{
	bfa_trc(hal, qid);

	/**
	 * ack the interrupt first
	 */
	bfa_hal_reg_write(hal->ioc.bfa_regs.intr_status,
						(1 << (HFN_MSIX_CPE_Q0 + qid)));

	qid &= (BFA_IOC_MAX_CQS - 1);

	return (BFA_HAL_INTR_CLAIMED_COMP);
}

void
bfa_hal_isr_unhandled(bfa_hal_t *hal, bfa_msg_t *m)
{
	bfa_trc(hal, m->mhdr.msg_class);
	bfa_trc(hal, m->mhdr.msg_id);
	bfa_trc(hal, m->mhdr.mtag.i2htok);
	bfa_assert(0);
	bfa_trc_stop(hal->trcmod);
}

bfa_hal_intr_status_t
bfa_hal_msix_rspq(bfa_hal_t *hal, int rsp_qid)
{
	bfa_msg_t      *m;
	uint32_t        pi, ci;

	bfa_trc_fp(hal, rsp_qid);

	/**
	 * ack the interrupt first
	 */
	bfa_hal_reg_write(hal->ioc.bfa_regs.intr_status,
					  (1 << (HFN_MSIX_RME_Q0 + rsp_qid)));

	rsp_qid &= (BFA_IOC_MAX_CQS - 1);

	ci = bfa_hal_rspq_ci(hal, rsp_qid);
	pi = bfa_hal_rspq_pi(hal, rsp_qid);

	bfa_trc_fp(hal, ci);
	bfa_trc_fp(hal, pi);

	while (ci != pi) {
		m = bfa_hal_rspq_elem(hal, rsp_qid, ci);
		bfa_assert_fp(m->mhdr.msg_class < BFA_MC_MAX);

		hal_isrs[m->mhdr.msg_class] (hal, m);

		CQ_INCR(ci, hal->ioc.cfg.num_rspq_elems);
	}

	/**
	 * update CI
	 */
	bfa_hal_rspq_ci(hal, rsp_qid) = ci;
	bfa_hal_reg_write(hal->ioc.bfa_regs.rme_q_ci[rsp_qid],
						hal->ioc.rsp_cq_ci[rsp_qid]);
	bfa_os_mmiowb();

	return (BFA_HAL_INTR_CLAIMED_COMP);
}

bfa_hal_intr_status_t
bfa_hal_msix_lpu(bfa_hal_t *hal)
{
	bfa_ioc_i2h_msg_t	msg;
	uint32_t		*msgp = (uint32_t *) &msg;
	uint32_t		r32;
	int			i;
	bfa_hal_ioc_t		*ioc = &hal->ioc;

	/**
	 * read the MBOX msg
	 */
	for (i = 0; i < (sizeof(bfa_ioc_i2h_msg_t) / sizeof(uint32_t)); i++) {
		r32 = bfa_hal_reg_read(ioc->bfa_regs.lpu_mbox +
						       i * sizeof(uint32_t));
		msgp[i] = bfa_os_htonl(r32);
	}

	/**
	 * turn off mailbox interrupt by clearing mailbox status
	 */
	bfa_hal_reg_write(ioc->bfa_regs.lpu_mbox_cmd, 1);
	r32 = bfa_hal_reg_read(ioc->bfa_regs.lpu_mbox_cmd);

	/**
	 * ack the interrupt next
	 */
	if (ioc->pci_info.pci_func == 0)
		bfa_hal_reg_write(ioc->bfa_regs.intr_status, __HFN_INT_MBOX_LPU0);
	else
		bfa_hal_reg_write(ioc->bfa_regs.intr_status, __HFN_INT_MBOX_LPU1);

	bfa_trc(hal, r32);
	bfa_trc(hal, msg.mh.msg_id);

	bfa_hal_ioc_isr(ioc, &msg);

	return (BFA_HAL_INTR_CLAIMED_COMP);
}

bfa_hal_intr_status_t
bfa_hal_msix_errint(bfa_hal_t *hal)
{
	uint32_t        intr;

	intr = bfa_hal_reg_read(hal->ioc.bfa_regs.intr_status);
	bfa_trc(hal, intr);
	bfa_hal_reg_write(hal->ioc.bfa_regs.intr_status, intr);
	bfa_hal_intr_disable(hal);
	bfa_assert(0);

	return (BFA_HAL_INTR_CLAIMED_COMP);
}

/**
 * @}
 */
