/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfa_hal_ipfc.c HAL unsolicited frame receive implementation
 */

#include <hal/bfa_hal_ipfc.h>
#include <hcb/bfa_hcb_ipfc.h>

/**
 * @ig hal_ipfc_api
 * @{
 */

/**
 * @brief
 * Query memory requirement for HAL ipfc module.
 *
 * @param[in]	npkts	Max number of receive packets posted by driver.
 * @param[out]	meminfo	(kernel virtual) memory required is retuned
 * 			here. DMA memory for actual receive packets
 * 			is associated with each packet.
 *
 * @return None
 */
void
bfa_hal_ipfc_meminfo(int npkts, bfa_hal_meminfo_t *meminfo)
{
}

/**
 * @brief
 * Attach and initialize IPFC hal component.
 *
 * Once hal IPFC component is initialized, packets can be posted by driver
 * to receive inbound IPFC packets. Synchronous call.
 *
 * @param[in]	hal	HAL instance handle
 * @param[in]	dipfc	driver IPFC instance
 * @param[in]	npkts	Max number of receive packets posted by driver.
 * @param[in]	meminfo	Memory for managing IPFC packet pools.
 *
 * @return None
 */
void
bfa_hal_ipfc_attach(bfa_hal_t *hal, void *dipfc, int npkts,
						bfa_hal_meminfo_t *meminfo)
{
}

/**
 * @brief
 * Detach IPFC operations. Completion notified by bfa_hcb_ipfc_detach().
 *
 * @param[in]	hal	HAL instance handle
 *
 * @return None
 */
void
bfa_hal_ipfc_detach(bfa_hal_t *hal)
{
}

/**
 * @brief
 * Post a packet to receive IP type packets.
 *
 * Driver must post packets after attach to receive IPFC packets. Received
 * packets are notified by bfa_hcb_ipfc_pktrecv() callback. Driver should
 * replenish HAL packet pool by posting more packets as part of receive
 * processing. Driver must not post more than the total number of packets
 * specified at attach time.
 *
 * @param[in]	hal	HAL instance handle
 * @param[in]	dpkt	Driver packet handle.
 *
 * @return None
 */
void
bfa_hal_ipfc_postpkt(bfa_hal_t *hal, bfad_pkt_t *dpkt)
{
}

/**
 * @brief
 * Flush all packets posted to HAL.
 *
 * Packets posted to HAL are returned to driver using bfa_hcb_ipfc_pktrecv()
 * callback for each outstanding packet. Flushed packets are returned with
 * status BFA_HAL_IPFC_STATUS_FLUSHED.
 *
 * @param[in]	hal	HAL instance handle
 *
 * @return None
 */
void
bfa_hal_ipfc_flushpkt(bfa_hal_t *hal, bfad_pkt_t *dpkt)
{
}

/**
 * @}
 */
