/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#include <hal/bfa_hal.h>
#include <hal/bfa_hal_itnim.h>
#include <fwi/bfa_fwi_itnim.h>
#include <cs/bfa_stats.h>
#include "hal_fcpim.h"

BFA_TRC_FILE(HAL, ITNIM);

#define BFA_HAL_ITNIM_FROM_TAG(_fcpim, _tag)				\
	((_fcpim)->itnim_arr + ((_tag) & ((_fcpim)->num_itnims - 1)))

#define bfa_hal_fcpim_additn(__itnim)					\
	bfa_q_enq(&(__itnim)->fcpim->itnim_q, __itnim)
#define bfa_hal_fcpim_delitn(__itnim)	do {				\
	bfa_assert(bfa_q_is_on_q(&(__itnim)->fcpim->itnim_q, __itnim));	\
	bfa_q_qe_deq(__itnim);						\
	bfa_assert(bfa_q_is_empty(&(__itnim)->io_q));			\
	bfa_assert(bfa_q_is_empty(&(__itnim)->io_cleanup_q));		\
	bfa_assert(bfa_q_is_empty(&(__itnim)->pending_q));		\
} while (0)

#define bfa_hal_itnim_online_cb(__itnim) do {				\
	if ((__itnim)->hal->fcs)					\
		bfa_hcb_itnim_online((__itnim)->ditn);			\
	else {								\
		bfa_hcb_queue((__itnim)->hal, &(__itnim)->hcb_qe,	\
		__bfa_hcb_itnim_online, (__itnim));			\
	}								\
} while (0)

#define bfa_hal_itnim_offline_cb(__itnim) do {				\
	if ((__itnim)->hal->fcs)					\
		bfa_hcb_itnim_offline((__itnim)->ditn);			\
	else {								\
		bfa_hcb_queue((__itnim)->hal, &(__itnim)->hcb_qe,	\
		__bfa_hcb_itnim_offline, (__itnim));			\
	}								\
} while (0)

#define bfa_hal_itnim_sler_cb(__itnim) do {				\
	if ((__itnim)->hal->fcs)					\
		bfa_hcb_itnim_sler((__itnim)->ditn);			\
	else {								\
		bfa_hcb_queue((__itnim)->hal, &(__itnim)->hcb_qe,	\
		__bfa_hcb_itnim_sler, (__itnim));			\
	}								\
} while (0)

/*
 * forward declarations
 */
static void     bfa_hal_itnim_iocdisable_cleanup(bfa_hal_itnim_t *itnim);
static bfa_boolean_t bfa_hal_itnim_send_fwcreate(bfa_hal_itnim_t *itnim);
static bfa_boolean_t bfa_hal_itnim_send_fwdelete(bfa_hal_itnim_t *itnim);
static void     bfa_hal_itnim_cleanp_comp(void *itnim_cbarg);
static void     bfa_hal_itnim_cleanup(bfa_hal_itnim_t *itnim);
static void     __bfa_hcb_itnim_online(void *cbarg, bfa_boolean_t complete);
static void     __bfa_hcb_itnim_offline(void *cbarg, bfa_boolean_t complete);
static void     __bfa_hcb_itnim_sler(void *cbarg, bfa_boolean_t complete);
static void     bfa_hal_itnim_iotov_online(bfa_hal_itnim_t *itnim);
static void     bfa_hal_itnim_iotov_cleanup(bfa_hal_itnim_t *itnim);
static void     bfa_hal_itnim_iotov(void *itnim_arg);
static void     bfa_hal_itnim_iotov_start(bfa_hal_itnim_t *itnim);
static void     bfa_hal_itnim_iotov_stop(bfa_hal_itnim_t *itnim);
static void     bfa_hal_itnim_iotov_delete(bfa_hal_itnim_t *itnim);

/**
 * @dg hal_itnim_sm HAL itnim state machine
 * @img hal_itnim_sm.jpg
 * @{
 */

/**
 * @brief HAL itnim state machine events
 */
typedef enum {
	BFA_HAL_ITNIM_SM_CREATE = 1,	/* !< itnim is created */
	BFA_HAL_ITNIM_SM_ONLINE = 2,	/* !< itnim is online */
	BFA_HAL_ITNIM_SM_OFFLINE = 3,	/* !< itnim is offline */
	BFA_HAL_ITNIM_SM_FWRSP = 4,	/* !< firmware response */
	BFA_HAL_ITNIM_SM_DELETE = 5,	/* !< deleting an existing itnim */
	BFA_HAL_ITNIM_SM_CLEANUP = 6,	/* !< IO cleanup completion */
	BFA_HAL_ITNIM_SM_SLER = 7,	/* !< second level error recovery */
	BFA_HAL_ITNIM_SM_HWFAIL = 8,	/* !< IOC h/w failure event */
} bfa_hal_itnim_event_t;

static void     bfa_hal_itnim_sm_uninit(bfa_hal_itnim_t *itnim,
					bfa_hal_itnim_event_t event);
static void     bfa_hal_itnim_sm_created(bfa_hal_itnim_t *itnim,
					 bfa_hal_itnim_event_t event);
static void     bfa_hal_itnim_sm_fwcreate(bfa_hal_itnim_t *itnim,
					  bfa_hal_itnim_event_t event);
static void	bfa_hal_itnim_sm_delete_pending(bfa_hal_itnim_t *itnim,
				bfa_hal_itnim_event_t event);
static void     bfa_hal_itnim_sm_online(bfa_hal_itnim_t *itnim,
					bfa_hal_itnim_event_t event);
static void     bfa_hal_itnim_sm_sler(bfa_hal_itnim_t *itnim,
				      bfa_hal_itnim_event_t event);
static void     bfa_hal_itnim_sm_cleanup_offline(bfa_hal_itnim_t *itnim,
						 bfa_hal_itnim_event_t event);
static void     bfa_hal_itnim_sm_cleanup_delete(bfa_hal_itnim_t *itnim,
						bfa_hal_itnim_event_t event);
static void     bfa_hal_itnim_sm_fwdelete(bfa_hal_itnim_t *itnim,
					  bfa_hal_itnim_event_t event);
static void     bfa_hal_itnim_sm_offline(bfa_hal_itnim_t *itnim,
					 bfa_hal_itnim_event_t event);
static void     bfa_hal_itnim_sm_iocdisable(bfa_hal_itnim_t *itnim,
					    bfa_hal_itnim_event_t event);
static void     bfa_hal_itnim_sm_deleting(bfa_hal_itnim_t *itnim,
					  bfa_hal_itnim_event_t event);

/**
 * @brief
 * 		Beginning/unallocated state - no events expected.
 */
static void
bfa_hal_itnim_sm_uninit(bfa_hal_itnim_t *itnim, bfa_hal_itnim_event_t event)
{
	bfa_trc(itnim->hal, itnim->rport->rport_tag);
	bfa_trc(itnim->hal, event);

	switch (event) {
	case BFA_HAL_ITNIM_SM_CREATE:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_created);
		itnim->is_online = BFA_FALSE;
		bfa_hal_fcpim_additn(itnim);
		break;

	default:
		bfa_assert(0);
	}
}

/**
 * @brief
 * 		Beginning state, only online event expected.
 */
static void
bfa_hal_itnim_sm_created(bfa_hal_itnim_t *itnim, bfa_hal_itnim_event_t event)
{
	bfa_trc(itnim->hal, itnim->rport->rport_tag);
	bfa_trc(itnim->hal, event);

	switch (event) {
	case BFA_HAL_ITNIM_SM_ONLINE:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_fwcreate);
		bfa_hal_itnim_send_fwcreate(itnim);
		break;

	case BFA_HAL_ITNIM_SM_DELETE:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_uninit);
		bfa_hal_fcpim_delitn(itnim);
		break;

	case BFA_HAL_ITNIM_SM_HWFAIL:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_iocdisable);
		break;

	default:
		bfa_assert(0);
	}
}

/**
 * @brief
 * 		Waiting for itnim create response from firmware.
 */
static void
bfa_hal_itnim_sm_fwcreate(bfa_hal_itnim_t *itnim, bfa_hal_itnim_event_t event)
{
	bfa_trc(itnim->hal, itnim->rport->rport_tag);
	bfa_trc(itnim->hal, event);

	switch (event) {
	case BFA_HAL_ITNIM_SM_FWRSP:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_online);
		itnim->is_online = BFA_TRUE;
		bfa_hal_itnim_iotov_online(itnim);
		bfa_hal_itnim_online_cb(itnim);
		break;

	case BFA_HAL_ITNIM_SM_DELETE:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_delete_pending);
		break;

	case BFA_HAL_ITNIM_SM_OFFLINE:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_fwdelete);
		bfa_hal_itnim_send_fwdelete(itnim);
		break;

	case BFA_HAL_ITNIM_SM_HWFAIL:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_iocdisable);
		break;

	default:
		bfa_assert(0);
	}
}

/**
 * @brief
 * 	Waiting for itnim create response from firmware, a delete is pending.
 */
static void
bfa_hal_itnim_sm_delete_pending(bfa_hal_itnim_t *itnim,
				bfa_hal_itnim_event_t event)
{
	bfa_trc(itnim->hal, itnim->rport->rport_tag);
	bfa_trc(itnim->hal, event);

	switch (event) {
	case BFA_HAL_ITNIM_SM_FWRSP:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_deleting);
		bfa_hal_itnim_send_fwdelete(itnim);
		break;

	case BFA_HAL_ITNIM_SM_HWFAIL:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_uninit);
		bfa_hal_fcpim_delitn(itnim);
		break;

	default:
		bfa_assert(0);
	}
}

/**
 * @brief
 * 		Online state - normal parking state.
 */
static void
bfa_hal_itnim_sm_online(bfa_hal_itnim_t *itnim, bfa_hal_itnim_event_t event)
{
	bfa_trc(itnim->hal, itnim->rport->rport_tag);
	bfa_trc(itnim->hal, event);

	switch (event) {
	case BFA_HAL_ITNIM_SM_OFFLINE:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_cleanup_offline);
		itnim->is_online = BFA_FALSE;
		bfa_hal_itnim_iotov_start(itnim);
		bfa_hal_itnim_cleanup(itnim);
		break;

	case BFA_HAL_ITNIM_SM_DELETE:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_cleanup_delete);
		itnim->is_online = BFA_FALSE;
		bfa_hal_itnim_cleanup(itnim);
		break;

	case BFA_HAL_ITNIM_SM_SLER:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_sler);
		itnim->is_online = BFA_FALSE;
		bfa_hal_itnim_iotov_start(itnim);
		bfa_hal_itnim_sler_cb(itnim);
		break;

	case BFA_HAL_ITNIM_SM_HWFAIL:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_iocdisable);
		itnim->is_online = BFA_FALSE;
		bfa_hal_itnim_iotov_start(itnim);
		bfa_hal_itnim_iocdisable_cleanup(itnim);
		break;

	default:
		bfa_assert(0);
	}
}

/**
 * @brief
 * 		Second level error recovery need.
 */
static void
bfa_hal_itnim_sm_sler(bfa_hal_itnim_t *itnim, bfa_hal_itnim_event_t event)
{
	bfa_trc(itnim->hal, itnim->rport->rport_tag);
	bfa_trc(itnim->hal, event);

	switch (event) {
	case BFA_HAL_ITNIM_SM_OFFLINE:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_cleanup_offline);
		bfa_hal_itnim_cleanup(itnim);
		break;

	case BFA_HAL_ITNIM_SM_DELETE:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_cleanup_delete);
		bfa_hal_itnim_cleanup(itnim);
		bfa_hal_itnim_iotov_delete(itnim);
		break;

	case BFA_HAL_ITNIM_SM_HWFAIL:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_iocdisable);
		bfa_hal_itnim_iocdisable_cleanup(itnim);
		break;

	default:
		bfa_assert(0);
	}
}

/**
 * @brief
 * 		Going offline. Waiting for active IO cleanup.
 */
static void
bfa_hal_itnim_sm_cleanup_offline(bfa_hal_itnim_t *itnim,
				 bfa_hal_itnim_event_t event)
{
	bfa_trc(itnim->hal, itnim->rport->rport_tag);
	bfa_trc(itnim->hal, event);

	switch (event) {
	case BFA_HAL_ITNIM_SM_CLEANUP:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_fwdelete);
		bfa_hal_itnim_send_fwdelete(itnim);
		break;

	case BFA_HAL_ITNIM_SM_DELETE:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_cleanup_delete);
		bfa_hal_itnim_iotov_delete(itnim);
		break;

	case BFA_HAL_ITNIM_SM_HWFAIL:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_iocdisable);
		bfa_hal_itnim_iocdisable_cleanup(itnim);
		bfa_hal_itnim_offline_cb(itnim);
		break;

	case BFA_HAL_ITNIM_SM_SLER:
		break;

	default:
		bfa_assert(0);
	}
}

/**
 * @brief
 * 		Deleting itnim. Waiting for active IO cleanup.
 */
static void
bfa_hal_itnim_sm_cleanup_delete(bfa_hal_itnim_t *itnim,
				bfa_hal_itnim_event_t event)
{
	bfa_trc(itnim->hal, itnim->rport->rport_tag);
	bfa_trc(itnim->hal, event);

	switch (event) {
	case BFA_HAL_ITNIM_SM_CLEANUP:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_deleting);
		bfa_hal_itnim_send_fwdelete(itnim);
		break;

	case BFA_HAL_ITNIM_SM_HWFAIL:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_iocdisable);
		bfa_hal_itnim_iocdisable_cleanup(itnim);
		break;

	default:
		bfa_assert(0);
	}
}

/**
 * @brief
 * 		Rport offline. Fimrware itnim is being deleted - awaiting f/w response.
 */
static void
bfa_hal_itnim_sm_fwdelete(bfa_hal_itnim_t *itnim, bfa_hal_itnim_event_t event)
{
	bfa_trc(itnim->hal, itnim->rport->rport_tag);
	bfa_trc(itnim->hal, event);

	switch (event) {
	case BFA_HAL_ITNIM_SM_FWRSP:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_offline);
		bfa_hal_itnim_offline_cb(itnim);
		break;

	case BFA_HAL_ITNIM_SM_DELETE:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_deleting);
		break;

	case BFA_HAL_ITNIM_SM_HWFAIL:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_iocdisable);
		bfa_hal_itnim_offline_cb(itnim);
		break;

	default:
		bfa_assert(0);
	}
}

/**
 * @brief
 * 		Offline state.
 */
static void
bfa_hal_itnim_sm_offline(bfa_hal_itnim_t *itnim, bfa_hal_itnim_event_t event)
{
	bfa_trc(itnim->hal, itnim->rport->rport_tag);
	bfa_trc(itnim->hal, event);

	switch (event) {
	case BFA_HAL_ITNIM_SM_DELETE:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_uninit);
		bfa_hal_itnim_iotov_delete(itnim);
		bfa_hal_fcpim_delitn(itnim);
		break;

	case BFA_HAL_ITNIM_SM_ONLINE:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_fwcreate);
		bfa_hal_itnim_send_fwcreate(itnim);
		break;

	case BFA_HAL_ITNIM_SM_HWFAIL:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_iocdisable);
		break;

	default:
		bfa_assert(0);
	}
}

/**
 * @brief
 * 		IOC h/w failed state.
 */
static void
bfa_hal_itnim_sm_iocdisable(bfa_hal_itnim_t *itnim,
			    bfa_hal_itnim_event_t event)
{
	bfa_trc(itnim->hal, itnim->rport->rport_tag);
	bfa_trc(itnim->hal, event);

	switch (event) {
	case BFA_HAL_ITNIM_SM_DELETE:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_uninit);
		bfa_hal_itnim_iotov_delete(itnim);
		bfa_hal_fcpim_delitn(itnim);
		break;

	case BFA_HAL_ITNIM_SM_OFFLINE:
		bfa_hal_itnim_offline_cb(itnim);
		break;

	case BFA_HAL_ITNIM_SM_ONLINE:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_fwcreate);
		bfa_hal_itnim_send_fwcreate(itnim);
		break;

	default:
		bfa_assert(0);
	}
}

/**
 * @brief
 * 		Itnim is deleted, waiting for firmware response to delete.
 */
static void
bfa_hal_itnim_sm_deleting(bfa_hal_itnim_t *itnim, bfa_hal_itnim_event_t event)
{
	bfa_trc(itnim->hal, itnim->rport->rport_tag);
	bfa_trc(itnim->hal, event);

	switch (event) {
	case BFA_HAL_ITNIM_SM_FWRSP:
	case BFA_HAL_ITNIM_SM_HWFAIL:
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_uninit);
		bfa_hal_fcpim_delitn(itnim);
		break;

	default:
		bfa_assert(0);
	}
}

/**
 * @}
 */

/**
 * @dg hal_itnim_private
 * @{
 */

/**
 * @brief
 * 		Initiate cleanup of all IOs on an IOC failure.
 */
static void
bfa_hal_itnim_iocdisable_cleanup(bfa_hal_itnim_t *itnim)
{
	bfa_hal_tskim_t *tskim;
	bfa_hal_ioim_t *ioim;
	bfa_q_t        *qe, *qen;

	bfa_q_iter_safe(&itnim->tsk_q, qe, qen) {
		tskim = (bfa_hal_tskim_t *) qe;
		bfa_hal_tskim_iocdisable(tskim);
	}

	bfa_q_iter_safe(&itnim->io_q, qe, qen) {
		ioim = (bfa_hal_ioim_t *) qe;
		bfa_hal_ioim_iocdisable(ioim);
	}

	/**
	 * For IO request in pending queue, we pretend an early timeout.
	 */
	bfa_q_iter_safe(&itnim->pending_q, qe, qen) {
		ioim = (bfa_hal_ioim_t *) qe;
		bfa_hal_ioim_tov(ioim);
	}

	bfa_q_iter_safe(&itnim->io_cleanup_q, qe, qen) {
		ioim = (bfa_hal_ioim_t *) qe;
		bfa_hal_ioim_iocdisable(ioim);
	}
}

/**
 * @brief
 * 		IO cleanup completion
 */
static void
bfa_hal_itnim_cleanp_comp(void *itnim_cbarg)
{
	bfa_hal_itnim_t *itnim = itnim_cbarg;

	bfa_stats(itnim, cleanup_comps);
	bfa_sm_send_event(itnim, BFA_HAL_ITNIM_SM_CLEANUP);
}

/**
 * @brief
 * 		Initiate cleanup of all IOs.
 */
static void
bfa_hal_itnim_cleanup(bfa_hal_itnim_t *itnim)
{
	bfa_hal_ioim_t  *ioim;
	bfa_hal_tskim_t *tskim;
	bfa_q_t         *qe, *qen;

	bfa_wc_init(&itnim->wc, bfa_hal_itnim_cleanp_comp, itnim);

	bfa_q_iter_safe(&itnim->io_q, qe, qen) {
		ioim = (bfa_hal_ioim_t *) qe;

		/**
		 * Move IO to a cleanup queue from active queue so that a later
		 * TM will not pickup this IO.
		 */
		bfa_q_qe_deq(ioim);
		bfa_q_enq(&itnim->io_cleanup_q, ioim);

		bfa_wc_up(&itnim->wc);
		bfa_hal_ioim_cleanup(ioim);
	}

	bfa_q_iter_safe(&itnim->tsk_q, qe, qen) {
		tskim = (bfa_hal_tskim_t *) qe;
		bfa_wc_up(&itnim->wc);
		bfa_hal_tskim_cleanup(tskim);
	}

	bfa_wc_wait(&itnim->wc);
}

static void
__bfa_hcb_itnim_online(void *cbarg, bfa_boolean_t complete)
{
	bfa_hal_itnim_t *itnim = cbarg;

	if (complete)
		bfa_hcb_itnim_online(itnim->ditn);
}

static void
__bfa_hcb_itnim_offline(void *cbarg, bfa_boolean_t complete)
{
	bfa_hal_itnim_t *itnim = cbarg;

	if (complete)
		bfa_hcb_itnim_offline(itnim->ditn);
}

static void
__bfa_hcb_itnim_sler(void *cbarg, bfa_boolean_t complete)
{
	bfa_hal_itnim_t *itnim = cbarg;

	if (complete)
		bfa_hcb_itnim_sler(itnim->ditn);
}

/**
 * @}
 */

/**
 * @dg hal_itnim_public
 * @{
 */

void
bfa_hal_itnim_iodone(bfa_hal_itnim_t *itnim)
{
	bfa_wc_down(&itnim->wc);
}

void
bfa_hal_itnim_tskdone(bfa_hal_itnim_t *itnim)
{
	bfa_wc_down(&itnim->wc);
}

void
bfa_hal_itnim_qresume(void *cbarg)
{
	bfa_assert(0);
}

void
bfa_hal_itnim_meminfo(bfa_ioc_cfg_t *cfg, uint32_t *km_len, uint32_t *dm_len)
{
	/**
	 * ITN memory
	 */
	if (cfg->fwcfg.num_itnims < BFA_HAL_ITNIM_MIN)
		cfg->fwcfg.num_itnims = BFA_HAL_ITNIM_MIN;

	*km_len += cfg->fwcfg.num_itnims * sizeof(bfa_hal_itnim_t);
}

void
bfa_hal_itnim_attach(bfa_hal_fcpim_mod_t *fcpim, bfa_hal_meminfo_t *minfo)
{
	bfa_hal_t      *hal = fcpim->hal;
	bfa_hal_itnim_t *itnim;
	int             i;

	bfa_q_init(&fcpim->itnim_q);

	itnim = (bfa_hal_itnim_t *) bfa_hal_meminfo_kva(minfo);
	fcpim->itnim_arr = itnim;

	for (i = 0; i < fcpim->num_itnims; i++, itnim++) {
		bfa_os_memset(itnim, 0, sizeof(bfa_hal_itnim_t));
		itnim->hal = hal;
		itnim->fcpim = fcpim;
		itnim->reqq = BFA_HAL_REQQ_QOS_LO;
		itnim->rport = BFA_HAL_RPORT_FROM_TAG(hal, i);
		itnim->iotov_active = BFA_FALSE;
		bfa_q_init(&itnim->io_q);
		bfa_q_init(&itnim->io_cleanup_q);
		bfa_q_init(&itnim->pending_q);
		bfa_q_init(&itnim->tsk_q);
		bfa_sm_set_state(itnim, bfa_hal_itnim_sm_uninit);
	}

	bfa_hal_meminfo_kva(minfo) = (uint8_t *) itnim;
}

void
bfa_hal_itnim_iocdisable(bfa_hal_itnim_t *itnim)
{
	bfa_stats(itnim, ioc_disabled);
	bfa_sm_send_event(itnim, BFA_HAL_ITNIM_SM_HWFAIL);
}

static bfa_boolean_t
bfa_hal_itnim_send_fwcreate(bfa_hal_itnim_t *itnim)
{
	bfa_fwi_itnim_create_req_t *m;

	itnim->msg_no++;

	/**
	 * check for room in queue to send request now
	 */
	m = bfa_hal_reqq_next(itnim->hal, itnim->reqq);
	if (!m) {
		bfa_hal_reqq_wait(itnim->hal, itnim->reqq, &itnim->reqq_wait);
		return BFA_FALSE;
	}

	bfa_fwi_h2i_set(m->mh, BFA_MC_ITNIM, BFA_FWI_ITNIM_H2I_CREATE_REQ,
			bfa_hal_ioc_lpuid(itnim->hal));
	m->fw_handle = itnim->rport->fw_handle;
	m->class = FC_CLASS_3;
	m->seq_rec = itnim->seq_rec;
	m->msg_no = itnim->msg_no;

	/**
	 * queue I/O message to firmware
	 */
	bfa_hal_reqq_produce(itnim->hal, itnim->reqq);
	return BFA_TRUE;
}

static bfa_boolean_t
bfa_hal_itnim_send_fwdelete(bfa_hal_itnim_t *itnim)
{
	bfa_fwi_itnim_delete_req_t *m;

	/**
	 * check for room in queue to send request now
	 */
	m = bfa_hal_reqq_next(itnim->hal, itnim->reqq);
	if (!m) {
		bfa_hal_reqq_wait(itnim->hal, itnim->reqq, &itnim->reqq_wait);
		return BFA_FALSE;
	}

	bfa_fwi_h2i_set(m->mh, BFA_MC_ITNIM, BFA_FWI_ITNIM_H2I_DELETE_REQ,
			bfa_hal_ioc_lpuid(itnim->hal));
	m->fw_handle = itnim->rport->fw_handle;

	/**
	 * queue I/O message to firmware
	 */
	bfa_hal_reqq_produce(itnim->hal, itnim->reqq);
	return BFA_TRUE;
}

/**
 * Start all pending IO requests.
 */
static void
bfa_hal_itnim_iotov_online(bfa_hal_itnim_t *itnim)
{
	bfa_hal_ioim_t	*ioim;

	bfa_hal_itnim_iotov_stop(itnim);

	/**
	 * Start all pending IO requests.
	 */
	while (!bfa_q_is_empty(&itnim->pending_q)) {
		bfa_q_deq(&itnim->pending_q, &ioim);
		bfa_q_enq(&itnim->io_q, ioim);
		bfa_hal_ioim_start(ioim);
	}
}

/**
 * Fail all pending IO requests
 */
static void
bfa_hal_itnim_iotov_cleanup(bfa_hal_itnim_t *itnim)
{
	bfa_hal_ioim_t	*ioim;

	/**
	 * Fail any pending IO requests.
	 */
	while (!bfa_q_is_empty(&itnim->pending_q)) {
		bfa_q_deq(&itnim->pending_q, &ioim);
		bfa_q_enq(&ioim->fcpim->ioim_comp_q, ioim);
		bfa_hal_ioim_tov(ioim);
	}
}

/**
 * IO TOV timer callback. Fail any pending IO requests and restart timer.
 */
static void
bfa_hal_itnim_iotov(void *itnim_arg)
{
	bfa_hal_itnim_t *itnim = itnim_arg;

	itnim->iotov_active = BFA_FALSE;
	bfa_hal_itnim_iotov_cleanup(itnim);
	bfa_hal_itnim_iotov_start(itnim);
}

/**
 * Start IO TOV timer for failing back pending IO requests in offline state.
 */
static void
bfa_hal_itnim_iotov_start(bfa_hal_itnim_t *itnim)
{
	if (!itnim->fcpim->mpio_mode)  {
		bfa_assert(itnim->fcpim->path_tov > 0);
		bfa_assert(bfa_hal_itnim_is_offline(itnim));

		itnim->iotov_active = BFA_TRUE;
		bfa_hal_timer_start(itnim->hal, &itnim->timer,
			bfa_hal_itnim_iotov, itnim, itnim->fcpim->path_tov);
	}
}

/**
 * Stop IO TOV timer.
 */
static void
bfa_hal_itnim_iotov_stop(bfa_hal_itnim_t *itnim)
{
	if (itnim->iotov_active) {
		itnim->iotov_active = BFA_FALSE;
		bfa_hal_timer_stop(&itnim->timer);
	}
}

/**
 * Stop IO TOV timer.
 */
static void
bfa_hal_itnim_iotov_delete(bfa_hal_itnim_t *itnim)
{
	bfa_hal_itnim_iotov_stop(itnim);
	bfa_hal_itnim_iotov_cleanup(itnim);
}

/**
 * @}
 */

/**
 * @ig hal_itnim_public
 * @{
 */

/**
 * @brief
 * 		Itnim interrupt processing.
 */
void
bfa_hal_itnim_isr(bfa_hal_t *hal, bfa_msg_t *m)
{
	bfa_hal_fcpim_mod_t *fcpim = BFA_HAL_FCPIM_MOD(hal);
	bfa_fwi_itnim_i2h_msg_t msg;
	bfa_hal_itnim_t *itnim;

	bfa_trc(hal, m->mhdr.msg_id);

	msg.msg = m;

	switch (m->mhdr.msg_id) {
	case BFA_FWI_ITNIM_I2H_CREATE_RSP:
		itnim = BFA_HAL_ITNIM_FROM_TAG(fcpim,
					       msg.create_rsp->bfa_handle);
		bfa_assert(msg.create_rsp->status == BFA_STATUS_OK);
		bfa_stats(itnim, create_comps);
		bfa_sm_send_event(itnim, BFA_HAL_ITNIM_SM_FWRSP);
		break;

	case BFA_FWI_ITNIM_I2H_DELETE_RSP:
		itnim = BFA_HAL_ITNIM_FROM_TAG(fcpim,
					       msg.delete_rsp->bfa_handle);
		bfa_assert(msg.delete_rsp->status == BFA_STATUS_OK);
		bfa_stats(itnim, delete_comps);
		bfa_sm_send_event(itnim, BFA_HAL_ITNIM_SM_FWRSP);
		break;

	case BFA_FWI_ITNIM_I2H_SLER_EVENT:
		itnim = BFA_HAL_ITNIM_FROM_TAG(fcpim,
					       msg.sler_event->bfa_handle);
		bfa_stats(itnim, sler_events);
		bfa_sm_send_event(itnim, BFA_HAL_ITNIM_SM_SLER);
		break;

	default:
		bfa_trc(hal, m->mhdr.msg_id);
		bfa_assert(0);
	}
}

/**
 * @}
 */

/**
 * @ig hal_itnim_api
 * @{
 */

bfa_hal_itnim_t *
bfa_hal_itnim_create(bfa_hal_t *hal, bfa_hal_rport_t *rport, void *ditn)
{
	bfa_hal_fcpim_mod_t *fcpim = BFA_HAL_FCPIM_MOD(hal);
	bfa_hal_itnim_t *itnim;

	itnim = BFA_HAL_ITNIM_FROM_TAG(fcpim, rport->rport_tag);
	bfa_assert(itnim->rport == rport);

	itnim->ditn = ditn;

	bfa_stats(itnim, creates);
	bfa_sm_send_event(itnim, BFA_HAL_ITNIM_SM_CREATE);

	return (itnim);
}

void
bfa_hal_itnim_delete(bfa_hal_itnim_t *itnim)
{
	bfa_stats(itnim, deletes);
	bfa_sm_send_event(itnim, BFA_HAL_ITNIM_SM_DELETE);
}

void
bfa_hal_itnim_online(bfa_hal_itnim_t *itnim, bfa_boolean_t seq_rec)
{
	itnim->seq_rec = seq_rec;
	bfa_stats(itnim, onlines);
	bfa_sm_send_event(itnim, BFA_HAL_ITNIM_SM_ONLINE);
}

void
bfa_hal_itnim_offline(bfa_hal_itnim_t *itnim)
{
	bfa_stats(itnim, offlines);
	bfa_sm_send_event(itnim, BFA_HAL_ITNIM_SM_OFFLINE);
}

/**
 * Return true if itnim is considered offline for holding off IO request.
 * IO is not held if itnim is being deleted.
 */
bfa_boolean_t
bfa_hal_itnim_is_offline(bfa_hal_itnim_t *itnim)
{
	return (
		!itnim->fcpim->mpio_mode &&
		(bfa_sm_cmp_state(itnim, bfa_hal_itnim_sm_fwcreate) ||
		 bfa_sm_cmp_state(itnim, bfa_hal_itnim_sm_sler) ||
		 bfa_sm_cmp_state(itnim, bfa_hal_itnim_sm_cleanup_offline) ||
		 bfa_sm_cmp_state(itnim, bfa_hal_itnim_sm_fwdelete) ||
		 bfa_sm_cmp_state(itnim, bfa_hal_itnim_sm_offline) ||
		 bfa_sm_cmp_state(itnim, bfa_hal_itnim_sm_iocdisable))
	);
}

void
bfa_hal_itnim_get_stats(bfa_hal_itnim_t *itnim, bfa_itnim_hal_stats_t *stats)
{
	*stats = itnim->stats;
}

void
bfa_hal_itnim_clear_stats(bfa_hal_itnim_t *itnim)
{
	bfa_os_memset(&itnim->stats, 0, sizeof(itnim->stats));
}

/**
 * @}
 */
