/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#include "bfa_hal_ioc_priv.h"
#include <cs/bfa_debug.h>
#include <hal_priv.h>
#include <hal/bfa_hal_wwn.h>

BFA_TRC_FILE(HAL, WWN);

typedef union {
	wwn_t           wwn;
	uint8_t         byte[sizeof(wwn_t)];
} wwn_types_t;

/**
 * @ig hal_wwn_public
 * @{
 */
wwn_t
bfa_hal_wwn_get_naa1(bfa_hal_t *hal)
{
	wwn_types_t     w;

	w.wwn = hal->ioc.cfgrsp->mfg_wwn;

	if (hal->pcidev->pci_func == 1)
		w.byte[7]++;

	return w.wwn;
}

wwn_t
bfa_hal_wwn_get_naa2(bfa_hal_t *hal)
{
	wwn_types_t     w;

	w.wwn = hal->ioc.cfgrsp->mfg_wwn;

	if (hal->pcidev->pci_func == 1)
		w.byte[7]++;

	w.byte[0] = 0x20;

	return w.wwn;
}

wwn_t
bfa_hal_wwn_get_naa5(bfa_hal_t *hal, uint16_t inst)
{
	union {
		wwn_t           wwn;
		uint8_t         byte[sizeof(wwn_t)];
	} w, w5;

	bfa_trc(hal, inst);

	w.wwn = hal->ioc.cfgrsp->mfg_wwn;
	w5.byte[0] = 0x50 | w.byte[2] >> 4;
	w5.byte[1] = w.byte[2] << 4 | w.byte[3] >> 4;
	w5.byte[2] = w.byte[3] << 4 | w.byte[4] >> 4;
	w5.byte[3] = w.byte[4] << 4 | w.byte[5] >> 4;
	w5.byte[4] = w.byte[5] << 4 | w.byte[6] >> 4;
	w5.byte[5] = w.byte[6] << 4 | w.byte[7] >> 4;
	w5.byte[6] = w.byte[7] << 4 | (inst & 0x0f00) >> 8;
	w5.byte[7] = (inst & 0xff);

	return w5.wwn;
}

uint64_t
bfa_hal_get_adapter_id(bfa_hal_t *hal)
{
	return hal->ioc.cfgrsp->mfg_wwn;
}
/**
 * @}
 */
