/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#include <hal/bfa_hal.h>
#include <hal/bfa_hal_timer.h>
#include <cs/bfa_debug.h>
#include <hal/bfa_hal_debug.h>

void
bfa_hal_timer_init(bfa_hal_t *hal)
{
	bfa_q_init(&hal->timer_mod.timer_q);
}

void
bfa_hal_timer_tick(bfa_hal_t *hal)
{
	bfa_q_t        *qh = &hal->timer_mod.timer_q;
	bfa_q_t        *qe, *qe_next;
	bfa_hal_timer_t *elem;
	bfa_q_t         timedout_q;

	bfa_q_init(&timedout_q);

	bfa_hal_debug(hal);

	qe = bfa_q_next(qh);

	while (qe != qh) {
		qe_next = bfa_q_next(qe);

		elem = (bfa_hal_timer_t *) qe;
		if (elem->timeout <= BFA_HAL_TIMER_FREQ) {
			elem->timeout = 0;
			bfa_q_qe_deq(elem);
			bfa_q_enq(&timedout_q, elem);
		} else {
			elem->timeout -= BFA_HAL_TIMER_FREQ;
		}

		qe = qe_next;	/* go to next elem */
	}

	/* TODO - need unlock before calling timeout entries */

	/*
	 * Pop all the timeout entries
	 */
	while (!bfa_q_is_empty(&timedout_q)) {
		bfa_q_deq(&timedout_q, &elem);
		elem->timercb(elem->arg);
	}
}

/**
 * Should be called with lock protection
 */
void
bfa_hal_timer_start(bfa_hal_t *hal, bfa_hal_timer_t *timer,
		    void (*timercb) (void *), void *arg, unsigned int timeout)
{

	bfa_assert(timercb != 0);
	bfa_assert(!bfa_q_is_on_q(&hal->timer_mod.timer_q, timer));

	timer->timeout = timeout;
	timer->timercb = timercb;
	timer->arg = arg;

	bfa_q_enq(&hal->timer_mod.timer_q, timer);
}

/**
 * Should be called with lock protection
 */
void
bfa_hal_timer_stop(bfa_hal_timer_t *timer)
{
	bfa_assert(!bfa_q_is_empty(timer));

	bfa_q_qe_deq(timer);
}
