/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __HAL_CALLBACK_H__
#define __HAL_CALLBACK_H__

#include <cs/bfa_q.h>

typedef void    (*bfa_hcb_cbfn_t) (void *cbarg, bfa_boolean_t complete);

/**
 * Generic HAL callback element.
 */
typedef struct {
	bfa_q_t         qe;
	bfa_hcb_cbfn_t  cbfn;
	bfa_boolean_t   once;
	uint32_t		rsvd;
	void           *cbarg;
} bfa_hcb_qe_t;

#define bfa_hcb_queue(__hal, __hcb_qe, __cbfn, __cbarg) do {		\
	(__hcb_qe)->cbfn  = (__cbfn);					\
	(__hcb_qe)->cbarg = (__cbarg);					\
	bfa_q_enq(&(__hal)->comp_q, (__hcb_qe));			\
} while (0)

#define bfa_hcb_queue_once(__hal, __hcb_qe, __cbfn, __cbarg) do {	\
	(__hcb_qe)->cbfn  = (__cbfn);					\
	(__hcb_qe)->cbarg = (__cbarg);					\
	if (!(__hcb_qe)->once) {					\
		bfa_q_enq(&(__hal)->comp_q, (__hcb_qe));		\
		(__hcb_qe)->once = BFA_TRUE;				\
	}								\
} while (0)

#define bfa_hcb_queue_done(__hcb_qe) do {				\
	(__hcb_qe)->once = BFA_FALSE;					\
} while (0)

#endif /* __HAL_CALLBACK_H__ */
