/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __HAL_FCPIM_H__
#define __HAL_FCPIM_H__

#include <hal/bfa_hal_itnim.h>
#include <hal/bfa_hal_ioim.h>
#include <hal/bfa_hal_tskim.h>
#include <hcb/bfa_hcb_itnim.h>
#include <hcb/bfa_hcb_ioim.h>
#include <hcb/bfa_hcb_tskim.h>
#include <cs/bfa_wc.h>
#include "hal_sgpg.h"

#define BFA_HAL_ITNIM_MIN   64
#define BFA_HAL_ITNIM_MAX   1024

#define BFA_HAL_IOIM_MIN    512
#define BFA_HAL_IOIM_MAX    2000

#define BFA_HAL_TSKIM_MIN   64
#define BFA_HAL_TSKIM_MAX   512
#define BFA_HAL_FCPIM_PATH_TOV_DEF	(10 * 1000)	/* in millisecs */

typedef struct {
	uint32_t        total_ios;	/*!< Total IO count */
	uint32_t        qresumes;	/*!< IO waiting for CQ space */
	uint32_t        no_iotags;	/*!< NO IO contexts */
	uint32_t        io_aborts;	/*!< IO abort requests */
	uint32_t        no_tskims;	/*!< NO task management contexts */
	uint32_t        iocomp_ok;	/*!< IO completions with OK status */
	uint32_t        iocomp_underrun;	/*!< IO underrun (good) */
	uint32_t        iocomp_overrun;	/*!< IO overrun (good) */
	uint32_t        iocomp_aborted;	/*!< Aborted IO requests */
	uint32_t        iocomp_timedout;	/*!< IO timeouts */
	uint32_t        iocom_nexus_abort;	/*!< IO selection timeouts */
	uint32_t        iocom_proto_err;	/*!< IO protocol errors */
	uint32_t        iocom_dif_err;	/*!< IO SBC-3 protection errors */
	uint32_t        iocom_tm_abort;	/*!< IO aborted by TM requests */
	uint32_t        iocom_sqer_needed;	/*!< IO retry for SQ error
						 *recovery */
	uint32_t        iocom_res_free;	/*!< Delayed freeing of IO resources */
	uint32_t        iocomp_scsierr;	/*!< IO with non-good SCSI status */
	uint32_t        iocom_hostabrts;	/*!< Host IO abort requests */
	uint32_t        iocom_utags;	/*!< IO comp with unknown tags */
	uint32_t        io_cleanups;	/*!< IO implicitly aborted */
	uint32_t        io_tmaborts;	/*!< IO aborted due to TM commands */
	uint32_t        rsvd;
} bfa_fcpim_stats_t;
#define bfa_hal_fcpim_stats(__fcpim, __stats)   \
    (__fcpim)->stats.__stats ++

typedef struct bfa_hal_ioim_sp_s bfa_hal_ioim_sp_t;

typedef struct {
	bfa_hal_t		*hal;
	bfa_hal_itnim_t		*itnim_arr;
	bfa_hal_ioim_t		*ioim_arr;
	bfa_hal_ioim_sp_t	*ioim_sp_arr;
	bfa_hal_tskim_t		*tskim_arr;
	bfa_hal_dma_t		snsbase;
	int			num_itnims;
	int			num_ioim_reqs;
	int			num_tskim_reqs;
	bfa_boolean_t		mpio_mode;
	uint32_t		path_tov;
	uint16_t		q_depth;
	uint16_t		rsvd;
	bfa_q_t			itnim_q;        /*!< queue of active itnim    */
	bfa_q_t			ioim_free_q;    /*!< free IO resources        */
	bfa_q_t			ioim_resfree_q; /*!< IOs waiting for f/w      */
	bfa_q_t			ioim_comp_q;    /*!< IO global comp Q         */
	bfa_q_t			tskim_free_q;
	uint32_t		ios_active;	/*!< current active IOs	      */
	bfa_fcpim_stats_t	stats;
} bfa_hal_fcpim_mod_t;

struct bfa_hal_ioim_s;
struct bfa_hal_tskim_s;

/**
 * HAL IO (initiator mode)
 */
struct bfa_hal_ioim_s {
	bfa_q_t			qe;		/*!< queue elememt            */
	bfa_sm_t		sm;		/*!< HAL ioim state machine   */
	bfa_hal_t		*hal;		/*!< HAL module               */
	bfa_hal_fcpim_mod_t	*fcpim;		/*!< parent fcpim module      */
	bfa_hal_itnim_t		*itnim;		/*!< i-t-n nexus for this IO  */
	bfad_ioim_t		*dio;		/*!< driver IO handle         */
	uint16_t		iotag;		/*!< FWI IO tag               */
	uint16_t		abort_tag;	/*!< unqiue abort request tag */
	uint16_t		nsges;		/*!< number of SG elements    */
	uint16_t		nsgpgs;		/*!< number of SG pages       */
	bfa_hal_sgpg_t		*sgpg;		/*!< first SG page            */
	bfa_q_t			sgpg_q;		/*!< allocated SG pages       */
	bfa_hcb_qe_t		hcb_qe;		/*!< hal callback qelem       */
	bfa_hcb_cbfn_t		io_cbfn;	/*!< IO completion handler    */
	bfa_hal_ioim_sp_t	*iosp;		/*!< slow-path IO handling    */
};

struct bfa_hal_ioim_sp_s {
	bfa_msg_t		comp_rspmsg;	/*!< IO comp f/w response     */
	uint8_t			*snsinfo;	/*!< sense info for this IO   */
	bfa_hal_sgpg_wqe_t	sgpg_wqe;	/*!< waitq elem for sgpg      */
	bfa_hal_reqq_wait_t	reqq_wait;	/*!< to wait for room in reqq */
	bfa_boolean_t		abort_explicit;	/*!< aborted by OS            */
	struct bfa_hal_tskim_s	*tskim;		/*!< Relevant TM cmd          */
};

/**
 * HAL Task management command (initiator mode)
 */
struct bfa_hal_tskim_s {
	bfa_q_t              qe;
	bfa_sm_t             sm;
	bfa_hal_t            *hal;           /*!< HAL module                  */
	bfa_hal_fcpim_mod_t  *fcpim;         /*!< parent fcpim module         */
	bfa_hal_itnim_t      *itnim;         /*!< i-t-n nexus for this IO     */
	bfad_tskim_t         *dtsk;          /*!< driver task mgmt cmnd       */
	bfa_boolean_t        notify;         /*!< notify itnim on TM comp     */
	lun_t                lun;            /*!< lun if applicable           */
	fcp_tm_cmnd_t        tm_cmnd;        /*!< task management command     */
	uint16_t             tsk_tag;        /*!< FWI IO tag                  */
	uint8_t              tsecs;          /*!< timeout in seconds          */
	bfa_hal_reqq_wait_t  reqq_wait;      /*!< to wait for room in reqq    */
	bfa_q_t              io_q;           /*!< queue of affected IOs       */
	bfa_wc_t             wc;             /*!< waiting counter             */
	bfa_hcb_qe_t         hcb_qe;         /*!< hal callback qelem          */
	bfa_tskim_status_t   tsk_status;     /*!< TM status                   */
};

/**
 * HAL i-t-n (initiator mode)
 */
struct bfa_hal_itnim_s {
	bfa_q_t         qe;		/*!< queue element               */
	bfa_sm_t        sm;		/*!< i-t-n im HAL state machine  */
	bfa_hal_t      *hal;		/*!< hal instance                */
	bfa_hal_rport_t *rport;		/*!< hal rport                   */
	void           *ditn;		/*!< driver i-t-n structure      */
	bfa_mhdr_t      mhdr;		/*!< pre-built mhdr              */
	uint8_t         msg_no;		/*!< itnim/rport firmware handle */
	uint8_t         reqq;		/*!< CQ for requests             */
	bfa_hcb_qe_t    hcb_qe;		/*!< hal callback qelem          */
	bfa_q_t         pending_q;	/*!< queue of pending IO requests*/
	bfa_q_t         io_q;		/*!< queue of active IO requests */
	bfa_q_t         io_cleanup_q;	/*!< IO being cleaned up         */
	bfa_q_t         tsk_q;		/*!< queue of active TM commands */
	bfa_boolean_t   seq_rec;	/*!< SQER supported              */
	bfa_boolean_t   is_online;	/*!< itnim is ONLINE for IO      */
	bfa_boolean_t   iotov_active;	/*!< IO TOV timer is active	 */
	bfa_wc_t        wc;		/*!< waiting counter             */
	bfa_hal_timer_t timer;		/*!< pending IO TOV		 */
	bfa_hal_reqq_wait_t reqq_wait;	/*!< to wait for room in reqq    */
	bfa_hal_fcpim_mod_t *fcpim;	/*!< fcpim module                */
	bfa_itnim_hal_stats_t	stats;
};

#define bfa_hal_itnim_is_online(_itnim) (_itnim)->is_online
#define BFA_HAL_FCPIM_MOD(_hal) (&(_hal)->modules.fcpim_mod)
#define BFA_HAL_IOIM_FROM_TAG(_fcpim, _iotag)               \
    (&fcpim->ioim_arr[_iotag & (fcpim->num_ioim_reqs - 1)])
#define BFA_HAL_TSKIM_FROM_TAG(_fcpim, _tmtag)                  \
    (&fcpim->tskim_arr[_tmtag & (fcpim->num_tskim_reqs - 1)])

/*
 * function prototypes
 */
void            bfa_hal_ioim_attach(bfa_hal_fcpim_mod_t *fcpim,
				    bfa_hal_meminfo_t *minfo);
void            bfa_hal_ioim_detach(bfa_hal_fcpim_mod_t *fcpim);
void            bfa_hal_ioim_isr(bfa_hal_t *hal, bfa_msg_t *msg);
void            bfa_hal_ioim_good_comp_isr(bfa_hal_t *hal, bfa_msg_t *msg);
void            bfa_hal_ioim_cleanup(bfa_hal_ioim_t *ioim);
void            bfa_hal_ioim_cleanup_tm(bfa_hal_ioim_t *ioim,
					bfa_hal_tskim_t *tskim);
void            bfa_hal_ioim_iocdisable(bfa_hal_ioim_t *ioim);
void            bfa_hal_ioim_tov(bfa_hal_ioim_t *ioim);

void            bfa_hal_tskim_attach(bfa_hal_fcpim_mod_t *fcpim,
				     bfa_hal_meminfo_t *minfo);
void            bfa_hal_tskim_detach(bfa_hal_fcpim_mod_t *fcpim);
void            bfa_hal_tskim_isr(bfa_hal_t *hal, bfa_msg_t *msg);
void            bfa_hal_tskim_iodone(bfa_hal_tskim_t *tskim);
void            bfa_hal_tskim_iocdisable(bfa_hal_tskim_t *tskim);
void            bfa_hal_tskim_cleanup(bfa_hal_tskim_t *tskim);

void            bfa_hal_itnim_meminfo(bfa_ioc_cfg_t *cfg, uint32_t *km_len,
				      uint32_t *dm_len);
void            bfa_hal_itnim_attach(bfa_hal_fcpim_mod_t *fcpim,
				     bfa_hal_meminfo_t *minfo);
void            bfa_hal_itnim_detach(bfa_hal_fcpim_mod_t *fcpim);
void            bfa_hal_itnim_iocdisable(bfa_hal_itnim_t *itnim);
void            bfa_hal_itnim_isr(bfa_hal_t *hal, bfa_msg_t *msg);
void            bfa_hal_itnim_iodone(bfa_hal_itnim_t *itnim);
void            bfa_hal_itnim_tskdone(bfa_hal_itnim_t *itnim);
bfa_boolean_t   bfa_hal_itnim_is_offline(bfa_hal_itnim_t *itnim);

#endif /* __HAL_FCPIM_H__ */

