/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __HAL_INTR_H__
#define __HAL_INTR_H__

/**
 * Message handler
 */
typedef void (*bfa_int_func_t) (bfa_hal_t *hal, bfa_msg_t *m);
void bfa_hal_isr_unhandled(bfa_hal_t *hal, bfa_msg_t *m);

#define bfa_hal_reqq_pi(__hal, __reqq)	(__hal)->ioc.req_cq_pi[__reqq]
#define bfa_hal_reqq_ci(__hal, __reqq)					\
	*(uint32_t *)((__hal)->ioc.req_cq_shadow_ci[__reqq].kva)

#define bfa_hal_reqq_next(__hal, __reqq)				\
	((void *)((bfa_msg_t *)((__hal)->ioc.req_cq_ba[__reqq].kva)	\
			  + bfa_hal_reqq_pi((__hal), (__reqq))))

#define bfa_hal_reqq_produce(__hal, __reqq)	do {			\
	(__hal)->ioc.req_cq_pi[__reqq]++;				\
	(__hal)->ioc.req_cq_pi[__reqq] &=				\
				((__hal)->ioc.cfg.num_reqq_elems - 1);	\
	bfa_hal_reg_write((__hal)->ioc.bfa_regs.cpe_q_pi[__reqq],	\
					(__hal)->ioc.req_cq_pi[__reqq]);\
	bfa_os_mmiowb();						\
} while (0)

#define bfa_hal_rspq_pi(__hal, __rspq)					\
	*(uint32_t *)((__hal)->ioc.rsp_cq_shadow_pi[__rspq].kva)

#define bfa_hal_rspq_ci(__hal, __rspq)	(__hal)->ioc.rsp_cq_ci[__rspq]
#define bfa_hal_rspq_elem(__hal, __rspq, __ci)				\
	&((bfa_msg_t *)((__hal)->ioc.rsp_cq_ba[__rspq].kva))[__ci]

#define CQ_INCR(__index, __size)					\
			(__index)++; (__index) &= ((__size) - 1)

/**
 * @brief
 * Queue element to wait for room in request queue. FIFO order is
 * maintained when fullfilling requests.
 */
typedef struct {
	bfa_q_t		qe;
	void		(*qresume) (void *cbarg);
	void		*cbarg;
} bfa_hal_reqq_wait_t;

/**
 * Circular queue usage assignments
 */
enum {
	BFA_HAL_REQQ_IOC	= 0,	/* !< all low-priority IOC msgs	*/
	BFA_HAL_REQQ_FCXP	= 0,	/* !< all FCXP messages		*/
	BFA_HAL_REQQ_PORT	= 0,	/* !< all port messages		*/
	BFA_HAL_REQQ_FLASH	= 0,	/* !< for flash module		*/
	BFA_HAL_REQQ_DIAG	= 0,	/* !< for diag module		*/
	BFA_HAL_REQQ_RPORT	= 0,	/* !< all port messages		*/
	BFA_HAL_REQQ_SBOOT	= 0,	/* !< all san boot messages	*/
	BFA_HAL_REQQ_QOS_LO	= 1,	/* !< all low priority IO	*/
	BFA_HAL_REQQ_QOS_MD	= 2,	/* !< all medium priority IO	*/
	BFA_HAL_REQQ_QOS_HI	= 3,	/* !< all high priority IO	*/
};

static inline void
bfa_hal_reqq_winit(bfa_hal_reqq_wait_t *wqe, void (*qresume) (void *cbarg),
			void *cbarg)
{
	wqe->qresume = qresume;
	wqe->cbarg = cbarg;
}

/**
 * static inline void
 * bfa_hal_reqq_wait(bfa_hal_t *hal, int reqq, bfa_hal_reqq_wait_t *wqe)
 */
#define bfa_hal_reqq_wait(__hal, __reqq, __wqe) do {			\
									\
		bfa_q_t	*waitq = &(__hal)->reqq_waitq[__reqq];		\
									\
		bfa_assert(((__reqq) < BFA_IOC_MAX_CQS));		\
		bfa_assert((__wqe)->qresume && (__wqe)->cbarg);		\
									\
		bfa_q_enq(waitq, (__wqe));				\
} while (0)

#define bfa_hal_reqq_wcancel(__wqe)	bfa_q_qe_deq(__wqe)

#endif /* __HAL_INTR_H__ */
