/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __HAL_IOC_H__
#define __HAL_IOC_H__

#include <cs/bfa_sm.h>
#include <fwi/bfa_fwi.h>
#include <fwi/bfa_fwi_ioc.h>
#include <defs/bfa_defs_types.h>
#include <hcb/bfa_hcb.h>
#include <hal/bfa_hal_timer.h>
#include <fwi/bfa_fwi_boot.h>
#include "hal_reg.h"
#include "hal_callback.h"

/**
 * Structure used to remember the DMA-able memory block's KVA and Physical
 * Address
 */
typedef struct {
	void		*kva;	/*! Kernel virtual address	*/
	uint64_t        pa;	/*! Physical address		*/
} bfa_hal_dma_t;

#define BFA_DMA_ALIGN_SZ	256
#define BFA_ROUNDUP(_l, _s)	(((_l) + ((_s) - 1)) & ~((_s) - 1))

typedef struct bfa_hal_ioc {
	bfa_sm_t        	sm;
	bfa_hal_t		*hal;	/*! Back pointer to HAL */
	union {
		bfa_ioc_cfg_reply_t *cfg_reply;
	} event_args;

	bfa_hal_pcidev_t	pci_info;
	bfa_ioc_cfg_t		cfg;

	uint32_t        	req_cq_pi[BFA_IOC_MAX_CQS];
	uint32_t        	rsp_cq_ci[BFA_IOC_MAX_CQS];
	bfa_ioc_i2h_msg_t 	mbox_msg;
					/*!< Used to store msg from the MBOX */

	bfa_hcb_qe_t		init_hcb_qe;
	bfa_hcb_qe_t		stop_hcb_qe;
	bfa_hcb_qe_t		dis_hcb_qe;
	bfa_hcb_qe_t		stats_hcb_qe;
	bfa_boolean_t		init_cb;
	bfa_boolean_t		stop_cb;
	bfa_boolean_t		disable_cb;
	bfa_boolean_t		ioc_started;
	bfa_hal_timer_t		ioc_timer;
	bfa_hal_timer_t		sem_timer;
	uint32_t		hb_count;
	uint32_t		hb_fail;
	uint32_t		hwinit_count;

	bfa_hal_dma_t		cfg_info;
	bfa_ioc_cfg_info_t	*cfginfo;
	bfa_hal_dma_t		cfgrsp_dma;
	bfa_ioc_cfgrsp_info_t	*cfgrsp;

	uint8_t			*stats_kva;
	uint64_t		stats_pa;
	bfa_ioc_stats_t		*stats;
	bfa_hal_timer_t		stats_timer;	/*!< timer */
	bfa_ioc_stats_t		*stats_ret;	/*!< driver stats location */
	bfa_status_t		stats_status;	/*!< stats/statsclr status */
	bfa_boolean_t   	stats_busy;	/*!< outstanding stats */
	bfa_hcb_ioc_t   	stats_cbfn;	/*!< driver callback function */
	void           		*stats_cbarg;	/*!< user callback arg */

	bfa_hal_dma_t   	req_cq_ba[BFA_IOC_MAX_CQS];
	bfa_hal_dma_t   	req_cq_shadow_ci[BFA_IOC_MAX_CQS];
	bfa_hal_dma_t   	rsp_cq_ba[BFA_IOC_MAX_CQS];
	bfa_hal_dma_t   	rsp_cq_shadow_pi[BFA_IOC_MAX_CQS];
	bfa_chip_regs_t 	bfa_regs;	/*!< HAL device registers */
	bfa_boolean_t   	auto_recover;
} bfa_hal_ioc_t;

#define bfa_hal_ioc_lpuid(__hal)	(__hal)->ioc.pci_info.pci_func
#define bfa_hal_ioc_maxfrsize(__hal)	(__hal)->ioc.cfgrsp->maxfrsize
#define bfa_hal_ioc_rx_bbcredit(__hal)	(__hal)->ioc.cfgrsp->rx_bbcredit
#define bfa_hal_ioc_speed_sup(__hal)	\
	BFA_FWI_ADAPTER_GETP(SPEED, (__hal)->ioc.cfgrsp->adapter_prop)

void	bfa_hal_ioc_meminfo(bfa_ioc_cfg_t *cfg, uint32_t *km_len,
							uint32_t *dm_len);
void	bfa_hal_ioc_attach(bfa_hal_t *hal, void *bfad,
				   bfa_ioc_cfg_t *cfg,
				   bfa_hal_meminfo_t *meminfo,
				   bfa_hal_pcidev_t *pcidev);
void	bfa_hal_ioc_detach(bfa_hal_t *hal);
void	bfa_hal_ioc_init(bfa_hal_t *hal);
void	bfa_hal_ioc_start(bfa_hal_t *hal);
void	bfa_hal_ioc_stop(bfa_hal_t *hal);
void	bfa_hal_ioc_boot(bfa_hal_t *hal, uint32_t boot_type,
							uint32_t boot_param);
void	bfa_hal_ioc_isr(bfa_hal_ioc_t *ioc, bfa_ioc_i2h_msg_t *msg);

bfa_status_t	bfa_hal_ioc_pll_init(bfa_hal_t *hal);
bfa_boolean_t	bfa_hal_ioc_is_operational(bfa_hal_t *hal);
bfa_boolean_t	bfa_hal_adapter_is_disabled(bfa_hal_t *hal);
wwn_t		bfa_hal_ioc_get_pwwn(bfa_hal_t *hal);
void		bfa_hal_ioc_set_snsbase(bfa_hal_t *hal, uint64_t snsbase_pa);

#endif /* __HAL_IOC_H__ */

