/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __HAL_PORT_H__
#define __HAL_PORT_H__

#include <defs/bfa_defs_pport.h>
#include <fwi/bfa_fwi_pport.h>
#include <hcb/bfa_hcb_pport.h>
#include "hal_intr.h"

/**
 * HAL physical port data structure
 */
typedef struct {
	bfa_hal_t		*hal;	/*!< parent HAL instance */
	bfa_sm_t		sm;	/*!< port state machine */
	wwn_t			nwwn;	/*!< node wwn of physical port */
	wwn_t			pwwn;	/*!< port wwn of physical oprt */
	bfa_pport_speed_t	speed_sup;
					/*!< supported speeds */
	bfa_pport_speed_t	speed;	/*!< current speed */
	bfa_pport_topology_t	topology;	/*!< current topology */
	uint8_t			myalpa;	/*!< my ALPA in LOOP topology */
	uint8_t			rsvd[5];
	bfa_pport_cfg_t		cfg;	/*!< current port configuration */
	bfa_hal_reqq_wait_t	reqq_wait;
					/*!< to wait for room in reqq */
	bfa_hal_reqq_wait_t	svcreq_wait;
					/*!< to wait for room in reqq */
	bfa_hal_reqq_wait_t	stats_reqq_wait;
					/*!< to wait for room in reqq (stats) */
	void			*event_cbarg;
	void			(*event_cbfn) (void *cbarg,
						bfa_pport_event_t event);
	union {
		bfa_fwi_pport_i2h_msg_t i2hmsg;
	} event_arg;
	void			*bfad;	/*!< BFA driver handle */
	bfa_hcb_qe_t		hcb_qe;	/*!< HAL callback queue elem */
	bfa_pport_linkstate_t	hcb_event;
					/*!< link event for callback */
	uint32_t		msgtag;	/*!< fimrware msg tag for reply */
	uint8_t			*stats_kva;
	uint64_t		stats_pa;
	bfa_pport_stats_t	*stats;	/*!< pport stats */
	bfa_hal_timer_t		timer;	/*!< timer */
	bfa_pport_stats_t 	*stats_ret;
					/*!< driver stats location */
	bfa_status_t		stats_status;
					/*!< stats/statsclr status */
	bfa_boolean_t   	stats_busy;
					/*!< outstanding stats/statsclr */
	bfa_hcb_pport_t 	stats_cbfn;
					/*!< driver callback function */
	void			*stats_cbarg;
					/* *!< user callback arg */
} bfa_hal_pport_t;

#define BFA_HAL_PORT_MOD(__hal)	(&(__hal)->modules.pport)

/*
 * public functions
 */
void	bfa_hal_pport_isr(bfa_hal_t *hal, bfa_msg_t *msg);
#endif /* __HAL_PORT_H__ */
