/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __HAL_PRIV_H__
#define __HAL_PRIV_H__

#include "hal_ioc.h"
#include "hal_timer.h"
#include "hal_intr.h"
#include "hal_reg.h"
#include "hal_trcmod.h"
#include "hal_modules.h"
#include "hal_fwimg.h"
#include <cs/bfa_log.h>

/**
 * @brief
 * Macro to define a new HAL module
 */
#define BFA_HAL_MODULE(__mod) 						\
	static void bfa_hal_ ## __mod ## _meminfo(bfa_ioc_cfg_t *cfg,	\
				uint32_t *ndm_len, uint32_t *dm_len);	\
	static void bfa_hal_ ## __mod ## _attach(bfa_hal_t *hal,	\
				void *bfad, bfa_ioc_cfg_t *cfg, 	\
				bfa_hal_meminfo_t *meminfo,		\
				bfa_hal_pcidev_t *pcidev);		\
	static void bfa_hal_ ## __mod ## _initdone(bfa_hal_t *hal);	\
	static void bfa_hal_ ## __mod ## _detach(bfa_hal_t *hal);	\
	static void bfa_hal_ ## __mod ## _start(bfa_hal_t *hal);	\
	static void bfa_hal_ ## __mod ## _stop(bfa_hal_t *hal);		\
	static void bfa_hal_ ## __mod ## _iocdisable(bfa_hal_t *hal);	\
									\
	bfa_hal_mod_t hal_mod_ ## __mod = {				\
		bfa_hal_ ## __mod ## _meminfo,				\
		bfa_hal_ ## __mod ## _attach,				\
		bfa_hal_ ## __mod ## _initdone,				\
		bfa_hal_ ## __mod ## _detach,				\
		bfa_hal_ ## __mod ## _start,				\
		bfa_hal_ ## __mod ## _stop,				\
		bfa_hal_ ## __mod ## _iocdisable,			\
	}

#define BFA_HAL_CACHELINE_SZ	(256)

/**
 * @brief BFA HAL dma address assignment macro
 */
#define bfa_hal_dma_addr_set(dma_addr, pa)	\
		__bfa_hal_dma_addr_set(&dma_addr, (uint64_t)pa)

static inline void
__bfa_hal_dma_addr_set(bfa_addr_t *dma_addr, uint64_t pa)
{
	dma_addr->a32.addr_lo = (uint32_t) pa;
	dma_addr->a32.addr_hi = (uint32_t) (pa >> 32);
}

/**
 * @brief BFA HAL dma address assignment macro. (big endian format)
 */
#define bfa_hal_dmabe_addr_set(dma_addr, pa)	\
		__bfa_hal_dmabe_addr_set(&dma_addr, (uint64_t)pa)
static inline void
__bfa_hal_dmabe_addr_set(bfa_addr_t *dma_addr, uint64_t pa)
{
	dma_addr->a32.addr_lo = (uint32_t) bfa_os_htonl(pa);
	dma_addr->a32.addr_hi = (uint32_t) bfa_os_htonl(pa >> 32);
}

/**
 * Structure used to interact between different HAL sub modules
 *
 * Each sub module needs to implement only the entry points relevant to it (and
 * can leave entry points as NULL)
 */
typedef struct {
	void	(*meminfo) (bfa_ioc_cfg_t *cfg, uint32_t *km_len,
							uint32_t *dm_len);
	void	(*attach) (bfa_hal_t *hal, void *bfad,
				   bfa_ioc_cfg_t *cfg,
				   bfa_hal_meminfo_t *meminfo,
				   bfa_hal_pcidev_t *pcidev);
	void	(*initdone) (bfa_hal_t *hal);
	void	(*detach) (bfa_hal_t *hal);
	void	(*start) (bfa_hal_t *hal);
	void	(*stop) (bfa_hal_t *hal);
	void	(*iocdisable) (bfa_hal_t *hal);
} bfa_hal_mod_t;

extern bfa_hal_mod_t *hal_mods[];

struct bfa_hal_s {
	void		*bfad;	/*!< BFA driver instance */
	bfa_aen_t	*aen;	/*!< AEN module */
	bfa_plog_t	*plog;	/*!< portlog buffer */
	bfa_log_mod_t	*logm;	/*!< driver logging module */
	bfa_trc_mod_t	*trcmod;/*!< HAL tracing */
	bfa_hal_pcidev_t *pcidev;
				/*!< PCI device information */
	bfa_hal_ioc_t	ioc;	/*!< IOC module */
	bfa_hal_timer_mod_t timer_mod;
				/*!< timer module */
	bfa_hal_modules_t modules;/*!< HAL modules */
	bfa_q_t		comp_q;	/*!< queue of pending completions */
	bfa_q_t         reqq_waitq[BFA_IOC_MAX_CQS];
	bfa_boolean_t   fcs;	/*!< FCS is attached to HAL */
};

#endif /* __HAL_PRIV_H__ */
