/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file hal_sgpg.h BFA HAL SG page module
 */

#ifndef __HAL_SGPG_H__
#define __HAL_SGPG_H__

#include <cs/bfa_q.h>

#define BFA_HAL_SGPG_MIN	(512)

typedef struct {
	bfa_q_t	qe;		/*!< queue sg page element	*/
	int	nsgpg;		/*!< pages to be allocated	*/
	int	nsgpg_total;	/*!< total pages required	*/
	void	(*cbfn) (void *cbarg);
				/*!< callback function		*/
	void	*cbarg;		/*!< callback arg		*/
	bfa_q_t sgpg_q;		/*!< queue of alloced sgpgs	*/
} bfa_hal_sgpg_wqe_t;

typedef struct {
	bfa_q_t		qe;	/*!< queue sg page element	*/
	bfa_sgpg_t	*sgpg;	/*!< va of SG page		*/
	bfa_addr_t	sgpg_pa;/*!< pa of SG page		*/
} bfa_hal_sgpg_t;

typedef struct {
	bfa_hal_t	*hal;
	int		num_sgpgs;	/*!< number of SG pages		*/
	int		free_sgpgs;	/*!< number of free SG pages	*/
	bfa_hal_sgpg_t	*hsgpg_arr;	/*!< HAL SG page array		*/
	bfa_sgpg_t	*sgpg_arr;	/*!< actual SG page array	*/
	uint64_t	sgpg_arr_pa;	/*!< SG page array DMA addr	*/
	bfa_q_t		sgpg_q;		/*!< queue of free SG pages	*/
	bfa_q_t		sgpg_wait_q;	/*!< wait queue for SG pages	*/
} bfa_hal_sgpg_mod_t;
#define BFA_HAL_SGPG_MOD(__hal)	(&(__hal)->modules.sgpg_mod)

bfa_status_t	bfa_hal_sgpg_malloc(bfa_hal_t *hal, bfa_q_t *sgpg_q,
								int nsgpgs);
void		bfa_hal_sgpg_mfree(bfa_hal_t *hal, bfa_q_t *sgpg_q,
								int nsgpgs);
void		bfa_hal_sgpg_winit(bfa_hal_sgpg_wqe_t *wqe,
				   void (*cbfn) (void *cbarg), void *cbarg);
void		bfa_hal_sgpg_wait(bfa_hal_t *hal, bfa_hal_sgpg_wqe_t *wqe,
								int nsgpgs);
void		bfa_hal_sgpg_wcancel(bfa_hal_t *hal, bfa_hal_sgpg_wqe_t *wqe);

#endif /* __HAL_SGPG_H__ */
