/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfa_wc.h Generic wait counter.
 */

#ifndef __BFA_WC_H__
#define __BFA_WC_H__

typedef void	(*bfa_wc_resume_t) (void *cbarg);

typedef struct {
	bfa_wc_resume_t wc_resume;
	void		*wc_cbarg;
	int		wc_count;
} bfa_wc_t;

static inline void
bfa_wc_up(bfa_wc_t *wc)
{
	wc->wc_count++;
}

static inline void
bfa_wc_down(bfa_wc_t *wc)
{
	wc->wc_count--;
	if (wc->wc_count == 0)
		wc->wc_resume(wc->wc_cbarg);
}

/**
 * Initialize a waiting counter.
 */
static inline void
bfa_wc_init(bfa_wc_t *wc, bfa_wc_resume_t wc_resume, void *wc_cbarg)
{
	wc->wc_resume = wc_resume;
	wc->wc_cbarg = wc_cbarg;
	wc->wc_count = 0;
	bfa_wc_up(wc);
}

/**
 * Wait for counter to reach zero
 */
static inline void
bfa_wc_wait(bfa_wc_t *wc)
{
	bfa_wc_down(wc);
}

#endif
