/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_DEFS_ADAPTER_H__
#define __BFA_DEFS_ADAPTER_H__

#include <protocol/types.h>
#include <defs/bfa_defs_version.h>
#include <defs/bfa_defs_mfg.h>

/**
 * BFA adapter level attributes.
 */
enum {
	BFA_ADAPTER_SERIAL_NUM_LEN = STRSZ(BFA_MFG_SERIALNUM_SIZE),
					/*
	 				 *!< adapter serial num length
	 				 */
	BFA_ADAPTER_MODEL_NAME_LEN  = 16,  /*!< model name length */
	BFA_ADAPTER_MODEL_DESCR_LEN = 128, /*!< model description length */
	BFA_ADAPTER_MFG_NAME_LEN    = 16,  /*!< manufacturer name length */
	BFA_ADAPTER_SYM_NAME_LEN    = 64,  /*!< adapter symbolic name length */
	BFA_ADAPTER_OS_TYPE_LEN	    = 64,  /*!< adapter os type length */
};

typedef struct {
	char            manufacturer[BFA_ADAPTER_MFG_NAME_LEN];
	char            serial_num[BFA_ADAPTER_SERIAL_NUM_LEN];
	uint32_t	rsvd1;
	char            model[BFA_ADAPTER_MODEL_NAME_LEN];
	char            model_descr[BFA_ADAPTER_MODEL_DESCR_LEN];
	wwn_t           pwwn;
	char            node_symname[FC_SYMNAME_MAX];
	char            hw_ver[BFA_VERSION_LEN];
	char            fw_ver[BFA_VERSION_LEN];
	char            optrom_ver[BFA_VERSION_LEN];
	char            os_type[BFA_ADAPTER_OS_TYPE_LEN];

	uint8_t		nports;
	uint8_t		max_speed;
	uint8_t		prototype;
	char	        asic_rev;

	uint8_t         pcie_gen;
	uint8_t         pcie_lanes_orig;
	uint8_t         pcie_lanes;
	uint8_t	        rsvd2;
} bfa_adapter_attr_t;

/**
 * BFA adapter level events
 * Arguments below are in BFAL context from Mgmt
 * BFA_PORT_AEN_ADD:        [in]: None     [out]: serial_num, pwwn, nports
 * BFA_PORT_AEN_REMOVE:     [in]: pwwn     [out]: serial_num, pwwn, nports
 */
typedef enum {
	BFA_ADAPTER_AEN_ADD 	= 1,	/*!< New Adapter found event */
	BFA_ADAPTER_AEN_REMOVE 	= 2,	/*!< Adapter removed event */
} bfa_adapter_aen_event_t;

typedef struct {
	char            serial_num[BFA_ADAPTER_SERIAL_NUM_LEN];
	uint32_t        nports;	/*!< Number of NPorts */
	wwn_t           pwwn;	/*!< WWN of one of its physical port */
} bfa_adapter_aen_data_t;

#endif /* __BFA_DEFS_ADAPTER_H__ */
