/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_DEFS_AUTH_H__
#define __BFA_DEFS_AUTH_H__

#include <defs/bfa_defs_types.h>

#define PUBLIC_KEY			15409
#define PRIVATE_KEY			19009
#define KEY_LEN				32399
#define BFA_AUTH_SECRET_STRING_LEN	256
#define BFA_AUTH_FAIL_TIMEOUT		0xFF

/**
 * Authentication status
 */
typedef enum {
	BFA_AUTH_STATUS_NONE 	= 0,	/*!< no authentication */
	BFA_AUTH_UNINIT 	= 1,	/*!< state - uninit */
	BFA_AUTH_NEG_SEND 	= 2,	/*!< state - negotiate send */
	BFA_AUTH_CHAL_WAIT 	= 3,	/*!< state - challenge wait */
	BFA_AUTH_NEG_RETRY 	= 4,	/*!< state - negotiate retry */
	BFA_AUTH_REPLY_SEND 	= 5,	/*!< state - reply send */
	BFA_AUTH_STATUS_WAIT 	= 6,	/*!< state - status wait */
	BFA_AUTH_SUCCESS 	= 7,	/*!< state - success */
	BFA_AUTH_FAILED 	= 8,	/*!< state - failed */
	BFA_AUTH_STATUS_UNKNOWN = 9,	/*!< authentication status unknown */
} bfa_auth_status_t;

typedef struct {
	uint32_t        auth_rjts;
	uint32_t        auth_negs;
	uint32_t        auth_dones;

	uint32_t        dhchap_challenges;
	uint32_t        dhchap_replies;
	uint32_t        dhchap_successes;
} auth_proto_stats_t;

/**
 * Authentication related statistics
 */
typedef struct {
	uint32_t           auth_failures;	/*!< authentication failures */
	uint32_t           auth_successes;	/*!< authentication successes*/
	auth_proto_stats_t auth_rx_stats;	/*!< Rx protocol stats */
	auth_proto_stats_t auth_tx_stats;	/*!< Tx protocol stats */
} bfa_auth_stats_t;

/**
 * Authentication hash function algorithms
 */
typedef enum {
	BFA_AUTH_ALGO_MD5 	= 1,	/*!< Message-Digest algorithm 5 */
	BFA_AUTH_ALGO_SHA1 	= 2,	/*!< Secure Hash Algorithm 1 */
	BFA_AUTH_ALGO_MS 	= 3,	/*!< MD5, then SHA-1 */
	BFA_AUTH_ALGO_SM 	= 4,	/*!< SHA-1, then MD5 */
} bfa_auth_algo_t;

/**
 * DH Groups
 *
 * Current value could be combination of one or more of the following values
 */
typedef enum {
	BFA_AUTH_GROUP_DHNULL 	= 0,	/*!< DH NULL (value == 0) */
	BFA_AUTH_GROUP_DH768 	= 1,	/*!< DH group 768 (value == 1) */
	BFA_AUTH_GROUP_DH1024 	= 2,	/*!< DH group 1024 (value == 2) */
	BFA_AUTH_GROUP_DH1280 	= 4,	/*!< DH group 1280 (value == 3) */
	BFA_AUTH_GROUP_DH1536 	= 8,	/*!< DH group 1536 (value == 4) */

	BFA_AUTH_GROUP_ALL 	= 256	/*!< Use default DH group order
					 *    0, 1, 2, 3, 4 */
} bfa_auth_group_t;

/**
 * Authentication secret sources
 */
typedef enum {
	BFA_AUTH_SECSRC_LOCAL 	= 1,	/*!< locally configured */
	BFA_AUTH_SECSRC_RADIUS 	= 2,	/*!< use radius server */
	BFA_AUTH_SECSRC_TACACS 	= 3,	/*!< TACACS server */
} bfa_auth_secretsource_t;

/**
 * Authentication attributes
 */
typedef struct {
	bfa_auth_status_t 	status;
	bfa_auth_algo_t 	algo;
	bfa_auth_group_t 	dh_grp;
	uint16_t		rjt_code;
	uint16_t		rjt_code_exp;
	uint8_t			secret_set;
	uint8_t			resv[7];
} bfa_auth_attr_t;

#endif /* __BFA_DEFS_AUTH_H__ */
