/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_DEFS_DIAG_H__
#define __BFA_DEFS_DIAG_H__

#include <bfa_os_inc.h>
#include <defs/bfa_defs_types.h>
#include <defs/bfa_defs_status.h>
#include <defs/bfa_defs_pport.h>
#include <defs/bfa_defs_led.h>
#include <fwi/bfa_fwi.h>
#include <protocol/sfp.h>

/**
 * @brief
 * 		Memory test algorithms.
 */
typedef enum {
	BFA_DIAG_MEMTEST_UNKNOWN 	= 0x00,
	BFA_DIAG_MEMTEST_WALK01 	= 0x01,
	BFA_DIAG_MEMTEST_MARCHY 	= 0x02,
	BFA_DIAG_MEMTEST_MATSPLUS 	= 0x04,
	BFA_DIAG_MEMTEST_MOVI 		= 0x08,
	BFA_DIAG_MEMTEST_TLAPNPSF1T 	= 0x10,
} bfa_diag_memtest_algo_t;

/**
 * @brief
 * 		Firmware ping test results
 */
typedef struct bfa_hal_diag_results_fwping {
	uint32_t        data[BFA_IOC_MAX_CQS];	/* store the corrupted data */
	uint32_t        status[BFA_IOC_MAX_CQS];
	uint32_t		dmastatus;
	uint8_t			rsvd[4];
} bfa_diag_fwping_result_t;

/**
 * RAM test params
 */
typedef struct bfa_diag_memtest {
	uint8_t         algo;
	uint8_t         rsvd[7];
} bfa_diag_memtest_t;

/*
 * User are expected to allocate one result data struct for each alog
 */
typedef struct bfa_diag_memtest_result {
	uint32_t        status;
	uint32_t        addr;
	uint32_t        exp;		/* expect value read from reg */
	uint32_t        act;		/* actually value read */
	uint32_t        err_status;	/* error status reg */
	uint32_t        err_addr;	/* error address reg */
	uint8_t         algo;
	uint8_t         rsv[7];
} bfa_diag_memtest_result_t;

/**
 * Post diiagnostics test results
 */
typedef struct {
	uint8_t         imist_pass;	/*!< internal SRAM BIST test result */
	uint8_t         emist_pass;	/*!< external SRAM BIST test result */
	uint8_t         serdes_pass;	/*!< serdes test result */
	uint8_t         rsv;
	uint32_t        rsvd;
} bfa_diag_post_result_t;

#define LB_LOOPCNT_DEFAULT	(128 * 1024)
#define LB_PATTERN_DEFAULT	0xB5B5B5B5

/**
 * Loopback Test
 */
typedef struct {
	uint32_t        loopcnt;
	uint32_t        pattern;
	uint8_t         lb_mode;	/*!< bfa_pport_opmode_t */
	uint8_t         speed;		/*!< bfa_pport_speed_t */
	uint8_t         rsvd[2];
} bfa_diag_loopback_t;

typedef struct {
	uint32_t        numrcvfrm;	/* no. of received good frame */
	uint32_t        badfrmoff;	/* mis-match byte location */
	uint32_t        badfrmnum;	/* mis-match fram number */
	uint8_t         status;		/* loopback test result */
	uint8_t         rsvd[3];
} bfa_diag_loopback_result_t;

/**
 * LED Test command
 */
typedef struct {
	uint32_t        cmd;	/*!< bfa_led_op_t */
	uint32_t        color;	/*!< bfa_led_color_t */
	uint16_t        freq;	/*!< no. of blinks every 10 secs */
	uint8_t         led;	/*!< bitmap of LEDs to be tested */
	uint8_t         rsvd[5];
} bfa_diag_ledtest_t;

/**
 * SFP event
 */
#define	BFA_DIAG_SFP_EVENT_INSERTED		1
#define	BFA_DIAG_SFP_EVENT_REMOVED		0



#endif /* __BFA_DEFS_DIAG_H__ */
