/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_DEFS_ITNIM_H__
#define __BFA_DEFS_ITNIM_H__

#include <bfa_os_inc.h>
#include <protocol/types.h>

/**
 * FCS itnim states
 */
typedef enum {
	BFA_ITNIM_OFFLINE 	= 0,	/*!< offline */
	BFA_ITNIM_PRLI_SEND 	= 1,	/*!< prli send */
	BFA_ITNIM_PRLI_SENT 	= 2,	/*!< prli sent */
	BFA_ITNIM_PRLI_RETRY 	= 3,	/*!< prli retry */
	BFA_ITNIM_HCB_ONLINE 	= 4,	/*!< online callback */
	BFA_ITNIM_ONLINE 	= 5,	/*!< online */
	BFA_ITNIM_HCB_OFFLINE 	= 6,	/*!< offline callback */
	BFA_ITNIM_INITIATIOR 	= 7,	/*!< initiator */
} bfa_itnim_state_t;

typedef struct {
	uint32_t	onlines;
	uint32_t	offlines;
	uint32_t	creates;
	uint32_t	deletes;
	uint32_t	create_comps;
	uint32_t	delete_comps;
	uint32_t	sler_events;
	uint32_t	ioc_disabled;
	uint32_t	cleanup_comps;
	uint32_t	tm_cmnds;
	uint32_t	tm_fw_rsps;
	uint32_t	tm_success;
	uint32_t	tm_failures;
	uint32_t	tm_io_comps;
	uint32_t	tm_qresumes;
	uint32_t	tm_iocdowns;
	uint32_t	tm_cleanups;
	uint32_t	tm_cleanup_comps;
	uint32_t	ios;
	uint32_t	io_comps;
} bfa_itnim_hal_stats_t;

/**
 * FCS remote port statistics
 */
typedef struct {
	uint32_t        onlines;	/*!< num rport online */
	uint32_t        offlines;	/*!< num rport offline */
	uint32_t        prli_sent;	/*!< num prli sent out */
	uint32_t        fcxp_alloc_wait;/*!< num fcxp alloc waits */
	uint32_t        prli_rsp_err;	/*!< num prli rsp errors */
	uint32_t        prli_rsp_acc;	/*!< num prli rsp accepts */
	uint32_t        initiator;	/*!< rport is an initiator */
	uint32_t        prli_rsp_parse_err;	/*!< prli rsp parsing errors */
	uint32_t        prli_rsp_rjt;	/*!< num prli rsp rejects */
	uint32_t        timeout;	/*!< num timeouts detected */
	uint32_t        sler;		/*!< num sler notification from HAL */
	uint32_t	rsvd;
	bfa_itnim_hal_stats_t	hal_stats;
} bfa_itnim_stats_t;

/**
 * FCS itnim attributes returned in queries
 */
typedef struct {
	bfa_itnim_state_t	state;          /*!< FCS itnim state            */
	uint8_t        		retry;		/*!< data retransmision support */
	uint8_t			task_retry_id;  /*!< task retry ident support   */
	uint8_t        	    	rec_support;    /*!< REC supported              */
	uint8_t        	    	conf_comp;      /*!< confirmed completion supp  */
} bfa_itnim_attr_t;

/**
 * BFA ITNIM events.
 * Arguments below are in BFAL context from Mgmt
 * BFA_ITNIM_AEN_NEW:       [in]: None  [out]: vf_id, lpwwn
 * BFA_ITNIM_AEN_DELETE:    [in]: vf_id, lpwwn, rpwwn (0 = all fcp4 targets),
 *				  [out]: vf_id, lpwwn, rpwwn
 * BFA_ITNIM_AEN_ONLINE:    [in]: vf_id, lpwwn, rpwwn (0 = all fcp4 targets),
 *				  [out]: vf_id, lpwwn, rpwwn
 * BFA_ITNIM_AEN_OFFLINE:   [in]: vf_id, lpwwn, rpwwn (0 = all fcp4 targets),
 *				  [out]: vf_id, lpwwn, rpwwn
 * BFA_ITNIM_AEN_DISCONNECT:[in]: vf_id, lpwwn, rpwwn (0 = all fcp4 targets),
 *				  [out]: vf_id, lpwwn, rpwwn
 */
typedef enum {
	BFA_ITNIM_AEN_NEW 	= 1,	/*!< New target found */
	BFA_ITNIM_AEN_DELETE 	= 2,	/*!< Target removed */
	BFA_ITNIM_AEN_ONLINE 	= 3,	/*!< Target online */
	BFA_ITNIM_AEN_OFFLINE 	= 4,	/*!< Target offline */
	BFA_ITNIM_AEN_DISCONNECT= 5,	/*!< Target disconnected */
} bfa_itnim_aen_event_t;

/**
 * BFA ITNIM event data structure.
 */
typedef struct {
	uint16_t        vf_id;	/*!< vf_id of the IT nexus */
	uint16_t        rsvd[3];
	wwn_t           lpwwn;	/*!< WWN of logical port */
	wwn_t           rpwwn;	/*!< WWN of remote(target) port */
} bfa_itnim_aen_data_t;
#endif /* __BFA_DEFS_ITNIM_H__ */
