/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_DEFS_MFG_H__
#define __BFA_DEFS_MFG_H__

#include <bfa_os_inc.h>

/**
 * Manufacturing block version
 */
#define BFA_MFG_VERSION				1

/**
 * Manufacturing block format
 */
#define BFA_MFG_SERIALNUM_SIZE			11
#define BFA_MFG_PARTNUM_SIZE			14
#define BFA_MFG_SUPPLIER_ID_SIZE		10
#define BFA_MFG_SUPPLIER_PARTNUM_SIZE		20
#define BFA_MFG_SUPPLIER_SERIALNUM_SIZE		20
#define BFA_MFG_SUPPLIER_REVISION_SIZE		4
#define STRSZ(_n)	(((_n) + 4) & ~3)

#pragma pack(1)

/**
 * @brief BFA adapter manufacturing block definition.
 *
 * All numerical fields are in big-endian format.
 */
typedef struct {
	uint8_t         version;	/*!< manufacturing block version */
	uint8_t         mfg_sig[3];	/*!< characters 'M', 'F', 'G' */
	uint16_t        mfgsize;	/*!< mfg block size */
	uint16_t        chksum;		/*!< checksum */
	char            brcd_serialnum[STRSZ(BFA_MFG_SERIALNUM_SIZE)];
	char            brcd_partnum[STRSZ(BFA_MFG_PARTNUM_SIZE)];
	uint8_t         mfg_day;	/*!< manufacturing day */
	uint8_t         mfg_month;	/*!< manufacturing month */
	uint16_t        mfg_year;	/*!< manufacturing year */
	wwn_t           mfg_wwn;	/*!< wwn base for this adapter */
	uint8_t         num_wwn;	/*!< number of wwns assigned */
	uint8_t         mfg_speeds;	/*!< speeds allowed for this adapter */
	uint8_t         rsvd[2];
	char            supplier_id[STRSZ(BFA_MFG_SUPPLIER_ID_SIZE)];
	char            supplier_partnum[STRSZ(BFA_MFG_SUPPLIER_PARTNUM_SIZE)];
	char
		supplier_serialnum[STRSZ(BFA_MFG_SUPPLIER_SERIALNUM_SIZE)];
	char
		supplier_revision[STRSZ(BFA_MFG_SUPPLIER_REVISION_SIZE)];
	uint8_t         rsvd1[8];
} bfa_mfg_block_t;

#pragma pack()

#endif /* __BFA_DEFS_MFG_H__ */
