/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_DEFS_POM_H__
#define __BFA_DEFS_POM_H__

#include <bfa_os_inc.h>
#include <defs/bfa_defs_types.h>

/**
 * POM health status levels for each attributes.
 */
typedef enum {
	BFA_POM_HEALTH_NOINFO  = 1,	/*!< no information */
	BFA_POM_HEALTH_NORMAL  = 2,	/*!< health is normal */
	BFA_POM_HEALTH_WARNING = 3,	/*!< warning level */
	BFA_POM_HEALTH_ALARM   = 4,	/*!< alarming level */
} bfa_pom_entry_health_t;

/**
 * Reading of temperature/voltage/current/power
 */
typedef struct {
	bfa_pom_entry_health_t health;	/*!< POM entry health */
	uint32_t        curr_value;	/*!< current value */
	uint32_t        thr_warn_high;	/*!< threshold warning high */
	uint32_t        thr_warn_low;	/*!< threshold warning low */
	uint32_t        thr_alarm_low;	/*!< threshold alaram low */
	uint32_t        thr_alarm_high;	/*!< threshold alarm high */
} bfa_pom_entry_t;

/**
 * POM attributes
 */
typedef struct {
	bfa_pom_entry_t temperature;	/*!< centigrade */
	bfa_pom_entry_t voltage;	/*!< volts */
	bfa_pom_entry_t curr;	/*!< milli amps */
	bfa_pom_entry_t txpower;	/*!< micro watts */
	bfa_pom_entry_t rxpower;	/*!< micro watts */
} bfa_pom_attr_t;

#endif /* __BFA_DEFS_POM_H__ */
