/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_DEFS_PPORT_H__
#define __BFA_DEFS_PPORT_H__

#include <bfa_os_inc.h>
#include <protocol/fc.h>
#include <defs/bfa_defs_types.h>

/* Modify char* port_stt[] in bfal_port.c if a new state was added */
typedef enum {
	BFA_HAL_PPORT_UNINIT 		= 1,
	BFA_HAL_PPORT_ENABLING_QWAIT 	= 2,
	BFA_HAL_PPORT_ENABLING 		= 3,
	BFA_HAL_PPORT_LINKDOWN 		= 4,
	BFA_HAL_PPORT_LINKUP 		= 5,
	BFA_HAL_PPORT_DISABLING_QWAIT 	= 6,
	BFA_HAL_PPORT_DISABLING 	= 7,
	BFA_HAL_PPORT_DISABLED 		= 8,
	BFA_HAL_PPORT_STOPPED 		= 9,
	BFA_HAL_PPORT_IOCDOWN 		= 10,
	BFA_HAL_PPORT_MAX_STATE,
} bfa_hal_pport_states_t;

/**
 * @brief
 * 	Port speed settings. Each specific speed is a bit field. Use multiple
 *      bits to specify speeds to be selected for auto-negotiation.
 */
typedef enum {
	BFA_PPORT_SPEED_UNKNOWN = 0,
	BFA_PPORT_SPEED_1GBPS 	= 1,
	BFA_PPORT_SPEED_2GBPS 	= 2,
	BFA_PPORT_SPEED_4GBPS 	= 4,
	BFA_PPORT_SPEED_8GBPS 	= 8,
	BFA_PPORT_SPEED_AUTO =
		(BFA_PPORT_SPEED_1GBPS | BFA_PPORT_SPEED_2GBPS |
		 BFA_PPORT_SPEED_4GBPS | BFA_PPORT_SPEED_8GBPS),
} bfa_pport_speed_t;

/**
 * @brief
 * 		Port operational type (in sync with SNIA port type).
 */
typedef enum {
	BFA_PPORT_TYPE_UNKNOWN = 1,	/*!< port type is unkown */
	BFA_PPORT_TYPE_TRUNKED = 2,	/*!< Trunked mode */
	BFA_PPORT_TYPE_NPORT   = 5,	/*!< P2P with switched fabric */
	BFA_PPORT_TYPE_NLPORT  = 6,	/*!< public loop */
	BFA_PPORT_TYPE_LPORT   = 20,	/*!< private loop */
	BFA_PPORT_TYPE_P2P     = 21,	/*!< P2P with no switched fabric */
	BFA_PPORT_TYPE_VPORT   = 22,	/*!< NPIV - virtual port */
} bfa_pport_type_t;

/**
 * @brief
 * 		Port topology setting. A port's topology and fabric login status
 * 		determine its operational type.
 */
typedef enum {
	BFA_PPORT_TOPOLOGY_NONE = 0,	/*!< No valid topology */
	BFA_PPORT_TOPOLOGY_P2P  = 1,	/*!< P2P only */
	BFA_PPORT_TOPOLOGY_LOOP = 2,	/*!< LOOP topology */
	BFA_PPORT_TOPOLOGY_AUTO = 3,	/*!< auto topology selection */
} bfa_pport_topology_t;

/**
 * @brief
 * 		Physical port loopback types.
 */
typedef enum {
	BFA_PPORT_OPMODE_NORMAL   = 0x00, /*!< normal non-loopback mode */
	BFA_PPORT_OPMODE_LB_INT   = 0x01, /*!< internal loop back */
	BFA_PPORT_OPMODE_LB_SLW   = 0x02, /*!< serial link wrapback (serdes) */
	BFA_PPORT_OPMODE_LB_EXT   = 0x04, /*!< external loop back (serdes) */
	BFA_PPORT_OPMODE_LB_CBL   = 0x08, /*!< cabled loop back */
	BFA_PPORT_OPMODE_LB_NLINT = 0x20, /*!< NL_Port internal loopback */
} bfa_pport_opmode_t;

#define BFA_PPORT_OPMODE_LB_HARD(_mode)			\
	((_mode == BFA_PPORT_OPMODE_LB_INT) ||		\
     (_mode == BFA_PPORT_OPMODE_LB_SLW) ||		\
     (_mode == BFA_PPORT_OPMODE_LB_EXT))

/**
 * @brief
		Port State (in sync with SNIA port state).
 */
typedef enum {
	BFA_PPORT_STATE_UNKNOWN  = 1,	/*!< port is not initialized */
	BFA_PPORT_STATE_ONLINE   = 2,	/*!< port is ONLINE */
	BFA_PPORT_STATE_DISABLED = 3,	/*!< port is disabled by user */
	BFA_PPORT_STATE_BYPASSED = 4,	/*!< port is bypassed (in LOOP) */
	BFA_PPORT_STATE_DIAG     = 5,	/*!< port diagnostics is active */
	BFA_PPORT_STATE_LINKDOWN = 6,	/*!< link is down */
	BFA_PPORT_STATE_LOOPBACK = 8,	/*!< port is looped back */
} bfa_pport_snia_state_t;

/**
 * @brief
 * 		Port link state
 */
typedef enum {
	BFA_PPORT_LINKUP 	 = 1,	/*!< Physical port/Trunk link up */
	BFA_PPORT_LINKDOWN 	 = 2,	/*!< Physical port/Trunk link down */
	BFA_PPORT_TRUNK_LINKDOWN = 3,	/*!< Trunk link down (new tmaster) */
} bfa_pport_linkstate_t;

/**
 * @brief
 * 		Port link state event
 */
typedef bfa_pport_linkstate_t bfa_pport_event_t;

/**
 * @brief
 * 		Port link state reason code
 */
typedef enum {
	BFA_PPORT_LINKSTATE_RSN_NONE		= 0,
	BFA_PPORT_LINKSTATE_RSN_DISABLED 	= 1,
	BFA_PPORT_LINKSTATE_RSN_RX_NOS 		= 2,
	BFA_PPORT_LINKSTATE_RSN_RX_OLS 		= 3,
	BFA_PPORT_LINKSTATE_RSN_RX_LIP 		= 4,
	BFA_PPORT_LINKSTATE_RSN_RX_LIPF7 	= 5,
	BFA_PPORT_LINKSTATE_RSN_SFP_REMOVED 	= 6,
	BFA_PPORT_LINKSTATE_RSN_PORT_FAULT 	= 7,
} bfa_pport_linkstate_rsn_t;

/**
 * @brief
 *      Physical port configuration
 */
typedef struct {
	uint8_t         topology;	/*!< bfa_pport_topology_t */
	uint8_t         speed;		/*!< bfa_pport_speed_t */
	uint8_t         trunked;	/*!< trunked or not */
	uint8_t         qos_enabled;	/*!< qos enabled or not */
	uint8_t         trunk_ports;	/*!< bitmap of trunked ports */
	uint8_t         cfg_hardalpa;	/*!< is hard alpa configured */
	uint16_t        maxfrsize;	/*!< maximum frame size */
	uint8_t         hardalpa;	/*!< configured hard alpa */
	uint8_t         rx_bbcredit;	/*!< receive buffer credits */
	uint8_t         tx_bbcredit;	/*!< transmit buffer credits */
	uint8_t         ratelimit;	/*!< ratelimit enabled or not */
	bfa_boolean_t   mpio_mode;	/*!< mpio mode (fast failover) */
	uint16_t   	path_tov;	/*!< path TOV value in non-mpio mode */
	uint16_t   	q_depth;	/*!< SCSI Queue depth */
	uint32_t   	rsvd;
} bfa_pport_cfg_t;

/**
 * @brief
 * 		Port attribute values.
 */
typedef struct {
	/*
	 * Static fields
	 */
	wwn_t           nwwn;		/*!< node wwn */
	wwn_t           pwwn;		/*!< port wwn */
	fc_cos_t        cos_supported;	/*!< supported class of services */
	uint32_t        rsvd;
	fc_symname_t    port_symname;	/*!< port symbolic name */
	bfa_pport_speed_t speed_supported; /*!< supported speeds */
	bfa_boolean_t   pbind_enabled;	/*!< Will be set if Persistent binding 
					 *   enabled. Relevant only in Windows 
					 */

	/*
	 * Configured values
	 */
	bfa_pport_cfg_t pport_cfg;	/*!< pport cfg */

	/*
	 * Dynamic field - info from HAL
	 */
	bfa_hal_pport_states_t 	port_state;	/*!< current port state */
	bfa_pport_speed_t 	speed;		/*!< current speed */
	bfa_pport_topology_t 	topology;	/*!< current topology */
	bfa_boolean_t		beacon;		/*!< current beacon status */
	bfa_boolean_t		link_e2e_beacon;/*!< set if link beacon on */
	bfa_boolean_t		plog_enabled;	/*!< set if portlog is enabled*/

	/*
	 * Dynamic field - info from FCS
	 */
	uint32_t        	pid;		/*!< port ID */
	bfa_pport_type_t 	port_type;	/*!< current topology */
	uint32_t        	loopback;	/*!< external loopback */
	uint32_t		rsvd1;
} bfa_pport_attr_t;

/**
 * @brief
 * 		Port statistics.
 */
typedef struct {
	uint64_t        secs_since_reset;
					/*!< seconds since stats is reset */
	uint64_t        tx_frames;	/*!< transmitted frames */
	uint64_t        tx_words;	/*!< transmitted words */
	uint64_t        rx_frames;	/*!< received frames */
	uint64_t        rx_words;	/*!< received words */
	uint64_t        lip_count;	/*!< LIPs seen */
	uint64_t        nos_count;	/*!< NOS count */
	uint64_t        error_frames;	/*!< errored frames (sent?) */
	uint64_t        dropped_frames;	/*!< dropped frames */
	uint64_t        link_failures;	/*!< link failure count */
	uint64_t        loss_of_syncs;	/*!< loss of sync count */
	uint64_t        loss_of_signals;/*!< loss of signal count */
	uint64_t        primseq_errs;	/*!< primitive sequence protocol
					 *   errors
					 */
	uint64_t        bad_os_count;	 /*!< invalid ordered set */
	uint64_t        err_enc_out;	 /*!< Encoding error outside frame */
	uint64_t        invalid_crcs;	/*!< frames received with invalid CRC*/
	uint64_t	undersized_frm; /*!< undersized frames */
	uint64_t	oversized_frm;	/*!< oversized frames */
	uint64_t	bad_eof_frm;	/*!< frames with bad EOF */
} bfa_pport_stats_t;

/**
 * @brief
 * 		Link state information
 */
typedef struct {
	uint8_t         linkstate;	/*!< Link state bfa_pport_linkstate_t*/
	uint8_t         linkstate_rsn;	/*!< bfa_pport_linkstate_rsn_t */
	uint8_t         topology;	/*!< P2P/LOOP bfa_pport_topology_t */
	uint8_t         speed;		/*!< Link speed (1/2/4/8 G) */
	uint32_t        linkstate_opt;	/*!< Linkstate optional data (debug) */
	uint8_t         trunked;	/*!< Trunked or not (1 or 0) */
	union {
		struct {
			uint8_t         tmaster;/*!< Trunk Master or
						 *    not (1 or 0) */
			uint8_t         tlinks;	/*!< Trunk links bitmap
						 *    (linkup) */
			uint8_t         resv1;	/*!< Reserved */
		} trunk_info;

		struct {
			uint8_t         myalpa;	   /*!< alpa claimed */
			uint8_t         login_req; /*!< Login required or
						    *    not (1 or 0) */
			uint8_t         alpabm_val;/*!< alpa bitmap valid
						    *    or not (1 or 0) */
			fc_alpabm_t     alpabm;	   /*!< alpa bitmap */
		} loop_info;
	} tl;
} bfa_pport_link_t;

#endif /* __BFA_DEFS_PPORT_H__ */
