/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_DEFS_RPORT_H__
#define __BFA_DEFS_RPORT_H__

#include <bfa_os_inc.h>
#include <protocol/types.h>
#include <defs/bfa_defs_pport.h>
#include <defs/bfa_defs_port.h>

/**
 * FCS remote port states
 */
typedef enum {
	BFA_RPORT_UNINIT 	= 0,	/*!< PORT is not yet initialized */
	BFA_RPORT_OFFLINE 	= 1,	/*!< rport is offline */
	BFA_RPORT_PLOGI 	= 2,	/*!< PLOGI to rport is in progress */
	BFA_RPORT_ONLINE 	= 3,	/*!< login to rport is complete */
	BFA_RPORT_PLOGI_RETRY 	= 4,	/*!< retrying login to rport */
	BFA_RPORT_NSQUERY 	= 5,	/*!< nameserver query */
	BFA_RPORT_ADISC 	= 6,	/*!< ADISC authentication */
	BFA_RPORT_LOGO 		= 7,	/*!< logging out with rport */
	BFA_RPORT_LOGORCV 	= 8,	/*!< handling LOGO from rport */
	BFA_RPORT_NSDISC 	= 9,	/*!< re-discover rport */
} bfa_rport_state_t;

/**
 *  Rport Scsi Function : Initiator/Target.
 */
typedef enum {
	BFA_RPORT_INITIATOR 	= 0x01,	/*!< SCSI Initiator	*/
	BFA_RPORT_TARGET 		= 0x02,	/*!< SCSI Target	*/
} bfa_rport_function_t;

/**
 * port/node symbolic names for rport
 */
#define BFA_RPORT_SYMNAME_MAXLEN	255
typedef struct {
	char            symname[BFA_RPORT_SYMNAME_MAXLEN];
} bfa_rport_symname_t;

typedef struct {
	uint32_t        sm_un_cr;	    /*!< uninit: create events      */
	uint32_t        sm_un_unexp;	    /*!< uninit: exception events   */
	uint32_t        sm_cr_on;	    /*!< created: online events     */
	uint32_t        sm_cr_del;	    /*!< created: delete events     */
	uint32_t        sm_cr_hwf;	    /*!< created: IOC down          */
	uint32_t        sm_cr_unexp;	    /*!< created: exception events  */
	uint32_t        sm_fwc_rsp;	    /*!< fw create: f/w responses   */
	uint32_t        sm_fwc_del;	    /*!< fw create: delete events   */
	uint32_t        sm_fwc_off;	    /*!< fw create: offline events  */
	uint32_t        sm_fwc_hwf;	    /*!< fw create: IOC down        */
	uint32_t        sm_fwc_unexp;	    /*!< fw create: exception events*/
	uint32_t        sm_on_off;	    /*!< online: offline events     */
	uint32_t        sm_on_del;	    /*!< online: delete events      */
	uint32_t        sm_on_hwf;	    /*!< online: IOC down events    */
	uint32_t        sm_on_unexp;	    /*!< online: exception events   */
	uint32_t        sm_fwd_rsp;	    /*!< fw delete: fw responses    */
	uint32_t        sm_fwd_del;	    /*!< fw delete: delete events   */
	uint32_t        sm_fwd_hwf;	    /*!< fw delete: IOC down events */
	uint32_t        sm_fwd_unexp;	    /*!< fw delete: exception events*/
	uint32_t        sm_off_del;	    /*!< offline: delete events     */
	uint32_t        sm_off_on;	    /*!< offline: online events     */
	uint32_t        sm_off_hwf;	    /*!< offline: IOC down events   */
	uint32_t        sm_off_unexp;	    /*!< offline: exception events  */
	uint32_t        sm_del_fwrsp;	    /*!< delete: fw responses       */
	uint32_t        sm_del_hwf;	    /*!< delete: IOC down events    */
	uint32_t        sm_del_unexp;	    /*!< delete: exception events   */
	uint32_t        sm_delp_fwrsp;	    /*!< delete pend: fw responses  */
	uint32_t        sm_delp_hwf;	    /*!< delete pend: IOC downs     */
	uint32_t        sm_delp_unexp;	    /*!< delete pend: exceptions    */
	uint32_t        sm_offp_fwrsp;	    /*!< off-pending: fw responses  */
	uint32_t        sm_offp_del;	    /*!< off-pending: deletes       */
	uint32_t        sm_offp_hwf;	    /*!< off-pending: IOC downs     */
	uint32_t        sm_offp_unexp;	    /*!< off-pending: exceptions    */
	uint32_t        sm_iocd_off;	    /*!< IOC down: offline events   */
	uint32_t        sm_iocd_del;	    /*!< IOC down: delete events    */
	uint32_t        sm_iocd_on;	    /*!< IOC down: online events    */
	uint32_t        sm_iocd_unexp;	    /*!< IOC down: exceptions       */
	uint32_t        rsvd;
} bfa_rport_hal_stats_t;

/**
 * FCS remote port statistics
 */
typedef struct {
	uint32_t        offlines;           /*!< remote port offline count  */
	uint32_t        onlines;            /*!< remote port online count   */
	uint32_t        rscns;              /*!< RSCN affecting rport       */
	uint32_t        plogis;		    /*!< plogis sent                */
	uint32_t        plogi_accs;	    /*!< plogi accepts              */
	uint32_t        plogi_timeouts;	    /*!< plogi timeouts             */
	uint32_t        plogi_rejects;	    /*!< rcvd plogi rejects         */
	uint32_t        plogi_failed;	    /*!< local failure              */
	uint32_t        plogi_rcvd;	    /*!< plogis rcvd                */
	uint32_t        prli_rcvd;          /*!< inbound PRLIs              */
	uint32_t        adisc_rcvd;         /*!< ADISCs received            */
	uint32_t        adisc_rejects;      /*!< recvd  ADISC rejects       */
	uint32_t        adisc_sent;         /*!< ADISC requests sent        */
	uint32_t        adisc_accs;         /*!< ADISC accepted by rport    */
	uint32_t        adisc_failed;       /*!< ADISC failed (no response) */
	uint32_t        adisc_rejected;     /*!< ADISC rejected by rport    */
	uint32_t        logos;              /*!< logos sent                 */
	uint32_t        logo_accs;          /*!< LOGO accepts from rport    */
	uint32_t        logo_failed;        /*!< LOGO failures              */
	uint32_t        logo_rejected;      /*!< LOGO rehects from rport    */
	uint32_t        logo_rcvd;          /*!< LOGO from remote port      */
	uint32_t        rsvd;
	bfa_rport_hal_stats_t	hal_stats;  /*!< HAL rport stats	    */
} bfa_rport_stats_t;

/**
 * FCS remote port attributes returned in queries
 */
typedef struct {
	wwn_t           		nwwn;	/*!< node wwn */
	wwn_t           		pwwn;	/*!< port wwn */
	fc_cos_t        		cos_supported;	/*!< supported class of services */
	uint32_t        		pid;	/*!< port ID */
	uint32_t        		df_sz;	/*!< Max payload size */
	bfa_rport_state_t 		state;	/*!< Rport State machine state */
	fc_cos_t        		fc_cos;	/*!< FC classes of services */
	bfa_boolean_t   		cisc;	/*!< CISC capable device */
	bfa_rport_symname_t 	symname;		/*!< Symbolic Name */
	bfa_rport_function_t	scsi_function;  /*!< Initiator/Target */
	uint32_t        		rsvd;
	/* TODO SPEED, QOS-Info */
} bfa_rport_attr_t;

/**
 * BFA remote port events
 * Arguments below are in BFAL context from Mgmt
 * BFA_RPORT_AEN_ONLINE:    [in]: lpwwn             [out]: vf_id, lpwwn, rpwwn
 * BFA_RPORT_AEN_OFFLINE:   [in]: lpwwn             [out]: vf_id, lpwwn, rpwwn
 * BFA_RPORT_AEN_DISCONNECT:[in]: lpwwn             [out]: vf_id, lpwwn, rpwwn
 */
typedef enum {
	BFA_RPORT_AEN_ONLINE 	= 1,	/*!< RPort online event */
	BFA_RPORT_AEN_OFFLINE 	= 2,	/*!< RPort offline event */
	BFA_RPORT_AEN_DISCONNECT= 3,	/*!< RPort disconnect event */
} bfa_rport_aen_event_t;

typedef struct {
	uint16_t        vf_id;	/*!< vf_id of this logical port */
	uint16_t        rsvd[3];
	wwn_t           lpwwn;	/*!< WWN of this logical port */
	wwn_t           rpwwn;	/*!< WWN of this remote port */
} bfa_rport_aen_data_t;

#endif /* __BFA_DEFS_RPORT_H__ */
