/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_DEFS_VPORT_H__
#define __BFA_DEFS_VPORT_H__

#include <bfa_os_inc.h>
#include <defs/bfa_defs_port.h>
#include <protocol/types.h>

/**
 * VPORT states
 */
typedef enum {
	BFA_FCS_VPORT_UNINIT 		= 0,
	BFA_FCS_VPORT_CREATED 		= 1,
	BFA_FCS_VPORT_OFFLINE 		= 1,
	BFA_FCS_VPORT_FDISC_SEND 	= 2,
	BFA_FCS_VPORT_FDISC 		= 3,
	BFA_FCS_VPORT_FDISC_RETRY 	= 4,
	BFA_FCS_VPORT_ONLINE 		= 5,
	BFA_FCS_VPORT_DELETING 		= 6,
	BFA_FCS_VPORT_CLEANUP 		= 6,
	BFA_FCS_VPORT_LOGO_SEND 	= 7,
	BFA_FCS_VPORT_LOGO 			= 8,
	BFA_FCS_VPORT_ERROR			= 9,
	BFA_FCS_VPORT_MAX_STATE,
} bfa_vport_state_t;

/**
 * vport statistics
 */
typedef struct {
	bfa_port_stats_t port_stats;	/*!< base class (port) stats */
	/*
	 * TODO - remove
	 */

	uint32_t        fdisc_sent;	/*!< num fdisc sent */
	uint32_t        fdisc_accepts;	/*!< fdisc accepts */
	uint32_t        fdisc_retries;	/*!< fdisc retries */
	uint32_t        fdisc_timeouts;	/*!< fdisc timeouts */
	uint32_t        fdisc_rsp_err;	/*!< fdisc response error */
	uint32_t        fdisc_acc_bad;	/*!< bad fdisc accepts */
	uint32_t        fdisc_rejects;	/*!< fdisc rejects */
	uint32_t        fdisc_unknown_rsp;
	/*
	 *!< fdisc rsp unknown error
	 */
	uint32_t        fdisc_alloc_wait;/*!< fdisc req (fcxp)alloc wait */

	uint32_t        logo_alloc_wait;/*!< logo req (fcxp) alloc wait */
	uint32_t        logo_sent;	/*!< logo sent */
	uint32_t        logo_accepts;	/*!< logo accepts */
	uint32_t        logo_rejects;	/*!< logo rejects */
	uint32_t        logo_rsp_err;	/*!< logo rsp errors */
	uint32_t        logo_unknown_rsp;
			/*!< logo rsp unknown errors */

	uint32_t        fab_no_npiv;	/*!< fabric does not support npiv */

	uint32_t        fab_offline;	/*!< offline events from fab SM */
	uint32_t        fab_online;	/*!< online events from fab SM */
	uint32_t        fab_cleanup;	/*!< cleanup request from fab SM */
	uint32_t        rsvd;
} bfa_vport_stats_t;

/**
 * BFA vport attribute returned in queries
 */
typedef struct {
	bfa_port_attr_t   port_attr;	/*!< base class (port) attributes */
	bfa_vport_state_t vport_state;	/*!< vport state */
	uint32_t          rsvd;
} bfa_vport_attr_t;

#endif /* __BFA_DEFS_VPORT_H__ */
