/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_FCS_H__
#define __BFA_FCS_H__

#include <cs/bfa_debug.h>
#include <defs/bfa_defs_status.h>
#include <defs/bfa_defs_version.h>
#include <hal/bfa_hal.h>
#include <bfad.h>
#include <fcs/bfa_fcs_fabric.h>

#define BFA_FCS_OS_STR_LEN  		64

typedef struct {
	struct {
		uint32_t        untagged;	/*!< untagged receive frames */
		uint32_t        tagged;	/*!< tagged receive frames */
		uint32_t        vfid_unknown;	/*!< VF id is unknown */
	} uf;
} bfa_fcs_stats_t;

typedef struct {
	uint8_t  version[BFA_VERSION_LEN];					/*!< Driver Version */
	uint8_t  host_machine_name[BFA_FCS_OS_STR_LEN];
	uint8_t  host_os_name[BFA_FCS_OS_STR_LEN]; 			/*!< OS name and version */
	uint8_t  host_os_patch[BFA_FCS_OS_STR_LEN];     	/*!< patch or service pack */ 
	uint8_t  os_device_name[BFA_FCS_OS_STR_LEN];    	/*!< Driver Device Name */
} bfa_fcs_driver_info_t;

typedef struct bfa_fcs_s {
	bfa_hal_t      *hal;	/*!< corresponding BFA hal instance */
	bfad_t         *bfad;	/*!< corresponding BDA driver instance */
	bfa_log_mod_t  *logm;	/*!< driver logging module instance */
	bfa_trc_mod_t  *trcmod;	/*!< tracing module */
	bfa_aen_t      *aen;	/*!< aen component */
	bfa_boolean_t   vf_enabled;	/*!< VF mode is enabled */
	uint16_t        port_vfid;	/*!< port default VF ID */
	bfa_fcs_driver_info_t driver_info;
	bfa_fcs_fabric_t 	  fabric;	/*!< base fabric state machine */
	bfa_fcs_stats_t		  stats;	/*!< FCS statistics */
	bfa_wc_t       		  wc;	/*!< waiting counter */
} bfa_fcs_t;

/*
 * bfa fcs API functions
 */
void 	  		bfa_fcs_init(bfa_fcs_t *fcs, bfa_hal_t *hal, bfad_t *bfad);
void 			bfa_fcs_driver_info_init(bfa_fcs_t *fcs, 
										bfa_fcs_driver_info_t *driver_info);
void            bfa_fcs_exit(bfa_fcs_t *fcs);
void            bfa_fcs_trc_init(bfa_fcs_t *fcs, bfa_trc_mod_t *trcmod);
void            bfa_fcs_log_init(bfa_fcs_t *fcs, bfa_log_mod_t *logmod);
void            bfa_fcs_aen_init(bfa_fcs_t *fcs, bfa_aen_t *aen);

#endif /* __BFA_FCS_H__ */
