/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_FCS_AUTH_H__
#define __BFA_FCS_AUTH_H__

struct bfa_fcs_s;

#include <defs/bfa_defs_status.h>
#include <defs/bfa_defs_auth.h>
#include <defs/bfa_defs_vf.h>
#include <cs/bfa_q.h>
#include <cs/bfa_sm.h>
#include <defs/bfa_defs_pport.h>
#include <hal/bfa_hal_timer.h>
#include <fcs/bfa_fcs_port.h>
#include <protocol/fc_sp.h>

struct bfa_fcs_fabric_s;

/**
 * @brief authentication data structure
 */

typedef struct bfa_fcs_auth_s {
	bfa_sm_t        sm;	/*!< state machine */
	bfa_boolean_t   policy;	/*!< authentication enabled/disabled */
	bfa_auth_status_t status;	/*!< authentication status */
	auth_rjt_codes_t  rjt_code;	/*!< auth reject status */
	auth_rjt_code_exps_t  rjt_code_exp;	/*!< auth reject reason */
	bfa_auth_algo_t algo;	/*!< Authentication algorithm */
	bfa_auth_stats_t stats;	/*!< Statistics */
	auth_dh_gid_t   group;	/*!< DH(diffie-hellman) Group */
	bfa_auth_secretsource_t source;	/*!< Secret source */
	char            secret[BFA_AUTH_SECRET_STRING_LEN];
				/*!< secret string */
	uint8_t         secret_len;
				/*!< secret string length */
	uint8_t         nretries;
				/*!< number of retries */
	struct bfa_fcs_fabric_s *fabric;/*!< pointer to fabric */
	uint8_t         sentcode;	/*!< pointer to response data */
	uint8_t        *response;	/*!< pointer to response data */
	bfa_hal_timer_t delay_timer; 	/*!< delay timer */
	bfa_hal_fcxp_t *fcxp;		/*!< pointer to fcxp */
} bfa_fcs_auth_t;

/**
 * bfa fcs authentication public functions
 */
bfa_status_t    bfa_fcs_auth_get_attr(struct bfa_fcs_s *port,
				      bfa_auth_attr_t *attr);
bfa_status_t    bfa_fcs_auth_set_policy(struct bfa_fcs_s *port,
					bfa_boolean_t policy);
bfa_auth_status_t bfa_fcs_auth_get_status(struct bfa_fcs_s *port);
bfa_status_t    bfa_fcs_auth_set_algo(struct bfa_fcs_s *port,
				      bfa_auth_algo_t algo);
bfa_status_t    bfa_fcs_auth_get_stats(struct bfa_fcs_s *port,
				       bfa_auth_stats_t *stats);
bfa_status_t    bfa_fcs_auth_set_dh_group(struct bfa_fcs_s *port, int group);
bfa_status_t    bfa_fcs_auth_set_secretstring(struct bfa_fcs_s *port,
					      char *secret);
bfa_status_t    bfa_fcs_auth_set_secretstring_encrypt(struct bfa_fcs_s *port,
					      uint32_t secret[], uint32_t len);
bfa_status_t    bfa_fcs_auth_set_secretsource(struct bfa_fcs_s *port,
					      bfa_auth_secretsource_t src);
bfa_status_t    bfa_fcs_auth_reset_stats(struct bfa_fcs_s *port);
bfa_status_t    bfa_fcs_auth_reinit(struct bfa_fcs_s *port);
#endif /* __BFA_FCS_AUTH_H__ */
