/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_FCS_FABRIC_H__
#define __BFA_FCS_FABRIC_H__

struct bfa_fcs_s;

#include <defs/bfa_defs_status.h>
#include <defs/bfa_defs_vf.h>
#include <cs/bfa_q.h>
#include <cs/bfa_sm.h>
#include <defs/bfa_defs_pport.h>
#include <hal/bfa_hal_timer.h>
#include <fcs/bfa_fcs_port.h>
#include <protocol/fc_sp.h>
#include <fcs/bfa_fcs_auth.h>

/*
 * forward declaration
 */
typedef struct bfad_vf_s bfad_vf_t;

typedef enum {
	BFA_FCS_FABRIC_UNKNOWN = 0,
	BFA_FCS_FABRIC_SWITCHED = 1,
	BFA_FCS_FABRIC_PLOOP = 2,
	BFA_FCS_FABRIC_N2N = 3,
} bfa_fcs_fabric_type_t;

/**
 * @brief fabric data structure
 */
typedef struct bfa_fcs_fabric_s {
	bfa_q_t         qe;				/*!< queue element */
	bfa_sm_t        sm;				/*!< state machine */
	struct bfa_fcs_s *fcs;			/*!< FCS instance */
	bfa_fcs_port_t  bport;			/*!< base logical port */
	bfa_fcs_fabric_type_t fab_type;	/*!< fabric type */
	bfa_pport_type_t oper_type;		/*!< current link topology */
	uint8_t         is_vf;			/*!< is virtual fabric? */
	uint8_t         is_npiv;		/*!< is NPIV supported ? */
	uint8_t         is_auth;		/*!< is Security/Auth supported ? */
	uint16_t        bb_credit;		/*!< BB credit from fabric */
	uint16_t        vf_id;			/*!< virtual fabric ID */
	uint16_t        num_vports;		/*!< num vports */
	uint16_t        rsvd;
	bfa_q_t         vport_q;		/*!< queue of virtual ports */
	bfa_q_t         vf_q;			/*!< queue of virtual fabrics */
	bfad_vf_t      *vf_drv;			/*!< driver vf structure */
	bfa_hal_timer_t link_timer;		/*!< Link Failure timer. Vport */
	wwn_t           fabric_name;	/*!< attached fabric name */
	bfa_boolean_t   auth_reqd;		/*!< authentication required */
	bfa_hal_timer_t delay_timer;	/*!< delay timer */
	bfa_hal_fcxp_wqe_t fcxp_wqe;	/*!< async fcxp alloc */
	bfa_hal_fcxp_t *fcxp;			/*!< fcxp for sending flogi */
	union {
		uint16_t        swp_vfid;	/*!< switch port VF id */
	} event_arg;
	bfa_fcs_auth_t  auth;			/*!< authentication config */
	bfa_wc_t        wc;				/*!< wait counter for delete */
    bfa_vf_stats_t  stats; 			/*!< fabric/vf stats */
} bfa_fcs_fabric_t;

#define bfa_fcs_fabric_npiv_capable(__f)    (__f)->is_npiv
#define bfa_fcs_fabric_is_switched(__f)     \
			((__f)->fab_type == BFA_FCS_FABRIC_SWITCHED)

/**
 * @brief
 *   The design calls for a single implementation of base fabric and vf.
 */
typedef bfa_fcs_fabric_t bfa_fcs_vf_t;

typedef struct bfa_vf_event_s {
	uint32_t        undefined;
} bfa_vf_event_t;

/**
 * bfa fcs vf public functions
 */
bfa_status_t    bfa_fcs_vf_mode_enable(struct bfa_fcs_s *fcs, uint16_t vf_id);
bfa_status_t    bfa_fcs_vf_mode_disable(struct bfa_fcs_s *fcs);
bfa_status_t    bfa_fcs_vf_create(bfa_fcs_vf_t *vf, struct bfa_fcs_s *fcs,
				  uint16_t vf_id, bfa_port_cfg_t *port_cfg,
				  bfad_vf_t *vf_drv);
bfa_status_t    bfa_fcs_vf_delete(bfa_fcs_vf_t *vf);
void            bfa_fcs_vf_start(bfa_fcs_vf_t *vf);
bfa_status_t    bfa_fcs_vf_stop(bfa_fcs_vf_t *vf);
void            bfa_fcs_vf_list(struct bfa_fcs_s *fcs, uint16_t *vf_ids,
				int *nvfs);
void            bfa_fcs_vf_list_all(struct bfa_fcs_s *fcs, uint16_t *vf_ids,
				    int *nvfs);
void            bfa_fcs_vf_get_attr(bfa_fcs_vf_t *vf, bfa_vf_attr_t *vf_attr);
void            bfa_fcs_vf_get_stats(bfa_fcs_vf_t *vf,
				     bfa_vf_stats_t *vf_stats);
void            bfa_fcs_vf_clear_stats(bfa_fcs_vf_t *vf);
void            bfa_fcs_vf_get_ports(bfa_fcs_vf_t *vf, wwn_t vpwwn[],
				     int *nports);
bfa_fcs_vf_t   *bfa_fcs_vf_lookup(struct bfa_fcs_s *fcs, uint16_t vf_id);
bfad_vf_t      *bfa_fcs_vf_get_drv_vf(bfa_fcs_vf_t *vf);

#endif /* __BFA_FCS_FABRIC_H__ */
