/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfa_fcs_fcpim.h BFA FCS FCP Initiator Mode interfaces/defines.
 */

#ifndef __BFA_FCS_FCPIM_H__
#define __BFA_FCS_FCPIM_H__

#include <defs/bfa_defs_status.h>
#include <defs/bfa_defs_itnim.h>
#include <fcs/bfa_fcs.h>
#include <fcs/bfa_fcs_rport.h>
#include <fcs/bfa_fcs_port.h>
#include <hal/bfa_hal_timer.h>
#include <hal/bfa_hal_itnim.h>

/*
 * forward declarations
 */
typedef struct bfad_itnim_s bfad_itnim_t;

struct bfa_fcs_itnim_s {
	bfa_sm_t        	sm;		/*!< state machine        */
	bfa_fcs_rport_t		*rport;		/*!< parent remote rport  */
	bfad_itnim_t   		*itnim_drv;	/*!< driver peer instance */
	bfa_fcs_t      		*fcs;		/*!< fcs instance         */
	bfa_hal_timer_t 	timer;		/*!< timer functions      */
	bfa_hal_itnim_t 	*hal_itnim;	/*!< HAL itnim struct     */
	bfa_boolean_t	 	seq_rec;	/*!< seq recovery support */
	bfa_boolean_t	 	rec_support;	/*!< REC supported        */
	bfa_boolean_t	 	conf_comp;	/*!< FCP_CONF     support */
	bfa_boolean_t	 	task_retry_id;	/*!< task retry id supp   */
	bfa_hal_fcxp_wqe_t 	fcxp_wqe;	/*!< wait qelem for fcxp  */
	bfa_hal_fcxp_t *	fcxp;		/*!< FCXP in use          */
	bfa_itnim_stats_t 	stats;		/*!< itn statistics       */
};

/**
 * @brief Return driver port instance for a itnim.
 */
static inline struct bfad_port_s *
bfa_fcs_itnim_get_drvport(bfa_fcs_itnim_t *itnim)
{
	return itnim->rport->port->bfad_port;
}

/**
 * @brief Return fcs port instance for a itnim.
 */
static inline bfa_fcs_port_t *
bfa_fcs_itnim_get_port(bfa_fcs_itnim_t *itnim)
{
	return itnim->rport->port;
}

/**
 * @brief Return node wwn of itnim
 */
static inline   wwn_t
bfa_fcs_itnim_get_nwwn(bfa_fcs_itnim_t *itnim)
{
	return itnim->rport->nwwn;
}

/**
 * @brief Return port wwn of itnim
 */
static inline   wwn_t
bfa_fcs_itnim_get_pwwn(bfa_fcs_itnim_t *itnim)
{
	return itnim->rport->pwwn;
}

/**
 * @brief Return remote port FC address.
 */
static inline   uint32_t
bfa_fcs_itnim_get_fcid(bfa_fcs_itnim_t *itnim)
{
	return itnim->rport->pid;
}

/**
 * @brief Return maximum receive size of remote port.
 */
static inline   uint32_t
bfa_fcs_itnim_get_maxfrsize(bfa_fcs_itnim_t *itnim)
{
	return itnim->rport->maxfrsize;
}

/**
 * @brief Return FC classes of service supported.
 */
static inline   fc_cos_t
bfa_fcs_itnim_get_cos(bfa_fcs_itnim_t *itnim)
{
	return itnim->rport->fc_cos;
}

/**
 * @brief Return driver itnim instance.
 */
static inline bfad_itnim_t *
bfa_fcs_itnim_get_drvitn(bfa_fcs_itnim_t *itnim)
{
	return itnim->itnim_drv;
}

/**
 * @brief Return hal itnim instance.
 */
static inline bfa_hal_itnim_t *
bfa_fcs_itnim_get_halitn(bfa_fcs_itnim_t *itnim)
{
	return itnim->hal_itnim;
}

/**
 * bfa fcs FCP Initiator mode API functions
 */
void            bfa_fcs_itnim_get_attr(bfa_fcs_itnim_t *itnim,
				       bfa_itnim_attr_t *attr);
void            bfa_fcs_itnim_get_stats(bfa_fcs_itnim_t *itnim,
					bfa_itnim_stats_t *stats);
bfa_fcs_itnim_t *bfa_fcs_itnim_lookup(bfa_fcs_port_t *port, wwn_t rpwwn);
bfa_status_t    bfa_fcs_itnim_attr_get(bfa_fcs_port_t *port, wwn_t rpwwn,
				       bfa_itnim_attr_t *attr);
bfa_status_t    bfa_fcs_itnim_stats_get(bfa_fcs_port_t *port, wwn_t rpwwn,
					bfa_itnim_stats_t *stats);
bfa_status_t    bfa_fcs_itnim_stats_clear(bfa_fcs_port_t *port, wwn_t rpwwn);
#endif /* __BFA_FCS_FCPIM_H__ */
