/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_FCS_RPORT_H__
#define __BFA_FCS_RPORT_H__

#include <defs/bfa_defs_status.h>
#include <cs/bfa_q.h>
#include <fcs/bfa_fcs.h>
#include <defs/bfa_defs_rport.h>

#define BFA_FCS_RPORT_DEF_DEL_TIMEOUT 	90 	/* in secs */
/*
 * forward declarations
 */
typedef struct bfad_rport_s bfad_rport_t;

typedef struct bfa_fcs_itnim_s bfa_fcs_itnim_t;
typedef struct bfa_fcs_itntm_s bfa_fcs_itntm_t;
typedef struct bfa_fcs_iprp_s bfa_fcs_iprp_t;

typedef struct bfa_fcs_rport_s {
	bfa_q_t         qe;	/*!< used by port/vport */
	bfa_fcs_port_t *port;	/*!< parent FCS port */
	bfa_fcs_t      *fcs;	/*!< fcs instance */
	bfad_rport_t   *rp_drv;	/*!< driver peer instance */
	uint32_t        pid;	/*!< port ID of rport */
	uint16_t        maxfrsize;	/*!< maximum frame size */
	uint16_t        reply_oxid;	/*!< OX_ID of inbound requests */
	fc_cos_t        fc_cos;	/*!< FC classes of service supp */
	bfa_boolean_t   cisc;	/*!< CISC capable device */
	wwn_t           pwwn;	/*!< port wwn of rport */
	wwn_t           nwwn;	/*!< node wwn of rport */
	bfa_rport_symname_t	psym_name;	/*!< port symbolic name  */
	bfa_sm_t        sm;	/*!< state machine */
	bfa_hal_timer_t timer;	/*!< general purpose timer */
	bfa_fcs_itnim_t *itnim;	/*!< ITN initiator mode role */
	bfa_fcs_itntm_t *itntm;	/*!< ITN initiator mode role */
	bfa_fcs_iprp_t *iprp;	/*!< IP/FC role */
	bfa_hal_rport_t *hal_rport;	/*!< HAL Rport */
	bfa_hal_fcxp_t *fcxp;	/*!< FCXP needed for discarding */
	int             plogi_retries;	/*!< max plogi retry attempts */
	int             ns_retries;	/*!< max NS query retry attempts */
	bfa_hal_fcxp_wqe_t 		fcxp_wqe;		/*!< fcxp wait queue element */
	bfa_rport_stats_t 		stats;			/*!< rport stats */
	bfa_rport_function_t	scsi_function;  /*!< Initiator/Target */
} bfa_fcs_rport_t;

static inline bfa_hal_rport_t *
bfa_fcs_rport_get_halrport(bfa_fcs_rport_t *rport)
{
	return rport->hal_rport;
}

/**
 * bfa fcs rport API functions
 */
bfa_status_t    bfa_fcs_rport_add(bfa_fcs_port_t *port, wwn_t *pwwn,
				  bfa_fcs_rport_t *rport,
				  bfad_rport_t *rport_drv);
bfa_status_t    bfa_fcs_rport_remove(bfa_fcs_rport_t *rport);
void            bfa_fcs_rport_get_attr(bfa_fcs_rport_t *rport,
				       bfa_rport_attr_t *attr);
void            bfa_fcs_rport_get_stats(bfa_fcs_rport_t *rport,
					bfa_rport_stats_t *stats);
void            bfa_fcs_rport_clear_stats(bfa_fcs_rport_t *rport);
bfa_fcs_rport_t *bfa_fcs_rport_lookup(bfa_fcs_port_t *port, wwn_t rpwwn);
bfa_fcs_rport_t *bfa_fcs_rport_lookup_by_nwwn(bfa_fcs_port_t *port, wwn_t rnwwn);
void 			bfa_fcs_rport_set_del_timeout(uint8_t rport_tmo);
#endif /* __BFA_FCS_RPORT_H__ */
