/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfa_fcs_vport.h BFA fcs vport module public interface
 */

#ifndef __BFA_FCS_VPORT_H__
#define __BFA_FCS_VPORT_H__

#include <defs/bfa_defs_status.h>
#include <defs/bfa_defs_port.h>
#include <defs/bfa_defs_vport.h>
#include <fcs/bfa_fcs.h>
#include <fcb/bfa_fcb_vport.h>

typedef struct bfa_fcs_vport_s {
	bfa_q_t         qe;	/*!< queue elem */
	bfa_sm_t        sm;	/*!< vport state machine */
	bfa_fcs_lport_t lport;	/*!< logical port */
	bfa_hal_timer_t timer;	/*!< general purpose timer */
	bfad_vport_t   *vport_drv;	/*!< Driver private */
	bfa_vport_stats_t vport_stats;	/*!< vport statistics */
	bfa_hal_fcxp_t *fcxp;	/*!< active fcxp */
	bfa_hal_fcxp_wqe_t fcxp_wqe;	/*!< fcxp wait queue elem */
	int 		 	fdisc_retries;
} bfa_fcs_vport_t;

#define bfa_fcs_vport_get_port(vport)  ((bfa_fcs_port_t	 *)(&vport->port))

/**
 * bfa fcs vport public functions
 */
bfa_status_t    bfa_fcs_vport_create(bfa_fcs_vport_t *vport, bfa_fcs_t *fcs,
				     uint16_t vf_id, bfa_port_cfg_t *port_cfg,
				     bfad_vport_t *vport_drv);
bfa_status_t    bfa_fcs_vport_delete(bfa_fcs_vport_t *vport);
bfa_status_t    bfa_fcs_vport_start(bfa_fcs_vport_t *vport);
bfa_status_t    bfa_fcs_vport_stop(bfa_fcs_vport_t *vport);
void            bfa_fcs_vport_get_attr(bfa_fcs_vport_t *vport,
				       bfa_vport_attr_t *vport_attr);
void            bfa_fcs_vport_get_stats(bfa_fcs_vport_t *vport,
					bfa_vport_stats_t *vport_stats);
void            bfa_fcs_vport_clr_stats(bfa_fcs_vport_t *vport);
bfa_fcs_vport_t *bfa_fcs_vport_lookup(bfa_fcs_t *fcs, uint16_t vf_id,
				      wwn_t vpwwn);
#endif /* __BFA_FCS_VPORT_H__ */
