/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_FWI_FCXP_H__
#define __BFA_FWI_FCXP_H__

#include "bfa_fwi.h"

#pragma pack(1)

typedef enum {
	BFA_FWI_FCXP_H2I_SEND_REQ = 1,
} bfa_fwi_fcxp_h2i_t;

typedef enum {
	BFA_FWI_FCXP_I2H_SEND_RSP = BFA_I2HM(1),
} bfa_fwi_fcxp_i2h_t;

#define BFA_FCXP_MAX_SGES	2

/**
 * FCXP send request structure
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< Common msg header */
	uint16_t        fcxp_tag;	/*!< driver request tag */
	uint16_t        max_frmsz;	/*!< max send frame size */
	uint16_t        vf_id;	/*!< vsan tag if applicable */
	uint16_t        rport_fw_hndl;	/*!< FW Handle for the remote port */
	uint8_t         req_type;	/*!< Request type. *See
					 *bfa_fcxp_req_type_t */
	uint8_t         class;	/*!< FC class for the request/response */
	uint8_t         rsp_timeout;	/*!< timeout in seconds, 0-no response
					 */
	uint8_t         cts;	/*!< continue sequence */
	fchs_t          fchs;	/*!< request FC header structure */
	uint32_t        req_len;	/*!< request payload length */
	uint32_t        rsp_maxlen;	/*!< max response length expected */
	bfa_sge_t       req_sge[BFA_FCXP_MAX_SGES];	/*!< request SG
							 *elements */
	bfa_sge_t       rsp_sge[BFA_FCXP_MAX_SGES];	/*!< response SG
							 *elements */
} bfa_fwi_fcxp_send_req_t;

/**
 * FCXP send response structure
 */
typedef struct bfa_fcxp_send_rsp {
	bfa_mhdr_t      mh;	/*!< Common msg header */
	uint16_t        fcxp_tag;	/*!< send request tag */
	uint8_t         req_status;	/*!< request status */
	uint8_t         rsvd;
	uint32_t        rsp_len;	/*!< actual response length */
	uint32_t        residue_len;	/*!< residual response length */
	fchs_t          fchs;	/*!< response FC header structure */
} bfa_fwi_fcxp_send_rsp_t, bfa_fcxp_send_rsp_t;

#pragma pack()

#endif /* __BFA_FWI_FCXP_H__ */
