/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_FWI_FLASH_H__
#define __BFA_FWI_FLASH_H__

#include "bfa_fwi.h"

#pragma pack(1)

typedef enum {
	BFA_FLASH_H2I_QUERY_REQ = 1,
	BFA_FLASH_H2I_ERASE_REQ = 2,
	BFA_FLASH_H2I_WRITE_REQ = 3,
	BFA_FLASH_H2I_READ_REQ = 4,
} bfa_flash_h2i_msgs_t;

typedef enum {
	BFA_FLASH_I2H_QUERY_RSP = BFA_I2HM(1),
	BFA_FLASH_I2H_ERASE_RSP = BFA_I2HM(2),
	BFA_FLASH_I2H_WRITE_RSP = BFA_I2HM(3),
	BFA_FLASH_I2H_READ_RSP = BFA_I2HM(4),
} bfa_flash_i2h_msgs_t;

#define BFA_FWI_FLASH_MAX_SGES	2

/**
 * Flash query request
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< Common msg header */
	bfa_sge_t       req_sge[BFA_FWI_FLASH_MAX_SGES];
} bfa_fwi_flash_query_req_t;

/**
 * Flash erase request
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< Common msg header */
	uint32_t        type;	/*!< partition type */
	uint32_t        instance;	/*!< partition instance */
} bfa_fwi_flash_erase_req_t;

/**
 * Flash write request
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< Common msg header */
	uint32_t        type;	/*!< partition type */
	uint32_t        instance;	/*!< partition instance */
	uint32_t        offset;
	uint32_t        length;
	uint32_t        last;
	bfa_sge_t       req_sge[BFA_FWI_FLASH_MAX_SGES];
} bfa_fwi_flash_write_req_t;

/**
 * Flash read request
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< Common msg header */
	uint32_t        type;	/*!< partition type */
	uint32_t        instance;	/*!< partition instance */
	uint32_t        offset;
	uint32_t        length;
	bfa_sge_t       req_sge[BFA_FWI_FLASH_MAX_SGES];
} bfa_fwi_flash_read_req_t;

/**
 * Flash query response
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< Common msg header */
	uint32_t        status;
} bfa_fwi_flash_query_rsp_t;

/**
 * Flash read response
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< Common msg header */
	uint32_t        type;	/*!< partition type */
	uint32_t        instance;	/*!< partition instance */
	uint32_t        status;
	uint32_t        length;
} bfa_fwi_flash_read_rsp_t;

/**
 * Flash write response
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< Common msg header */
	uint32_t        type;	/*!< partition type */
	uint32_t        instance;	/*!< partition instance */
	uint32_t        status;
	uint32_t        length;
} bfa_fwi_flash_write_rsp_t;

/**
 * Flash erase response
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< Common msg header */
	uint32_t        type;	/*!< partition type */
	uint32_t        instance;	/*!< partition instance */
	uint32_t        status;
} bfa_fwi_flash_erase_rsp_t;

#pragma pack()

#endif /* __BFA_FWI_FLASH_H__ */
