/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_IOC_MSGS_H__
#define __BFA_IOC_MSGS_H__

#include "bfa_fwi.h"
#include <defs/bfa_defs_ioc.h>

#pragma pack(1)

typedef enum {
	BFA_IOC_H2I_CFG_REQ 		= 1,
	BFA_IOC_H2I_ENABLE_REQ 		= 2,
	BFA_IOC_H2I_DISABLE_REQ 	= 3,
	BFA_IOC_H2I_GET_STATS_REQ 	= 4,
	BFA_IOC_H2I_CLEAR_STATS_REQ 	= 5,
	BFA_IOC_H2I_SET_INTR_REQ 	= 6,
} bfa_ioc_h2i_msgs_t;

typedef enum {
	BFA_IOC_I2H_CFG_REPLY 		= BFA_I2HM(1),
	BFA_IOC_I2H_ENABLE_REPLY	= BFA_I2HM(2),
	BFA_IOC_I2H_DISABLE_REPLY 	= BFA_I2HM(3),
	BFA_IOC_I2H_GET_STATS_RSP 	= BFA_I2HM(4),
	BFA_IOC_I2H_CLEAR_STATS_RSP 	= BFA_I2HM(5),
	BFA_IOC_I2H_READY_EVENT 	= BFA_I2HM(6),
} bfa_ioc_i2h_msgs_t;

#define BFA_IOC_PROTO_FCP_INI   1
#define BFA_IOC_PROTO_FCP_TGT   2
#define BFA_IOC_PROTO_IPoFC     4
#define BFA_IOC_PROTO_FC_PASS   8

/**
 * Firmware trace offset
 */
#define BFA_FWI_IOC_TRC_OFF		(0x4980)
#define BFA_FWI_IOC_FW_SIGNATURE	(0xbfadbfad)
typedef struct {
	uint32_t        signature;	/*!< constant signature */
	uint32_t        cksum;		/*!< firmware checksum  */
	uint32_t        exec;		/*!< exec vector        */
	uint32_t        param;		/*!< parameters         */
} bfa_fwi_ioc_image_hdr_t;

/**
 *  BFA_IOC_I2H_READY_EVENT message
 */
typedef struct bfa_ioc_rdy_event {
	bfa_mhdr_t      mh;		/*!< common msg header */
	uint8_t         init_status;	/*!< init event status */
	uint8_t         rsvd[3];
} bfa_ioc_rdy_event_t;

/**
 * IOC hardware/firmware state
 */
typedef enum {
	BFA_FWI_IOC_UNINIT 	 = 0,	/*!< not initialized                 */
	BFA_FWI_IOC_INITING 	 = 1,	/*!< h/w is being initialized        */
	BFA_FWI_IOC_HWINIT 	 = 2,	/*!< h/w is initialized              */
	BFA_FWI_IOC_CFG 	 = 3,	/*!< IOC configuration in progress   */
	BFA_FWI_IOC_OP 		 = 4,	/*!< IOC is operational              */
	BFA_FWI_IOC_DISABLING 	 = 5,	/*!< IOC is being disabled           */
	BFA_FWI_IOC_DISABLED 	 = 6,	/*!< IOC is disabled                 */
	BFA_FWI_IOC_CFG_DISABLED = 7,	/*!< IOC is being disabled;transient */
	BFA_FWI_IOC_HBFAIL       = 8,	/*!< IOC heart-beat failure          */
} bfa_fwi_ioc_state_t;

/**
 * Identifies who initialized the IOC
 */
typedef enum {
	BFA_IOCI_HOST_DRIVER = 1,	/* FCP host driver */
	BFA_IOCI_SBOOT 	     = 2,	/* system boot driver */
} bfa_ioc_inited_by_t;

#define BFA_FWI_IOC_ENDIAN_SIG  0x12345678

typedef struct bfa_ioc_cfg_info {
	uint8_t         num_cqs; 	/*!< Number of CQs to be used     */
	uint8_t         sense_buf_len;	/*!< SCSI sense length            */
	uint8_t         trunk_enabled;	/*!< port trunking enabled        */
	uint8_t         trunk_ports;	/*!< trunk ports bit map          */
	uint32_t        endian_sig;	/*!< endian signature of host     */

	/**
	 * Request and response circular queue base addresses, size and
	 * shadow index pointers.
	 */
	bfa_addr_t      req_cq_ba[ BFA_IOC_MAX_CQS ];
	bfa_addr_t      req_shadow_ci[ BFA_IOC_MAX_CQS ];
	uint16_t        req_cq_elems[ BFA_IOC_MAX_CQS ];
	bfa_addr_t      rsp_cq_ba[ BFA_IOC_MAX_CQS ];
	bfa_addr_t      rsp_shadow_pi[ BFA_IOC_MAX_CQS ];
	uint16_t        rsp_cq_elems[ BFA_IOC_MAX_CQS ];

	bfa_addr_t      stats_addr;	/*!< DMA-able address for stats	  */
	bfa_addr_t      cfgrsp_addr;	/*!< config response dma address  */
    	bfa_addr_t      ioim_snsbase;   /*!< IO sense buffer base address */
	bfa_ioc_intr_attr_t intr_attr;	/*!< IOC interrupt attributes     */
} bfa_ioc_cfg_info_t;

enum {
	BFA_FWI_ADAPTER_TYPE_FC   = 0x01,	/*!< FC adapters           */
	BFA_FWI_ADAPTER_TYPE_MK   = 0x0f0000,	/*!< adapter type mask     */
	BFA_FWI_ADAPTER_TYPE_SH   = 16,	        /*!< adapter type shift    */
	BFA_FWI_ADAPTER_NPORTS_MK = 0xff00,	/*!< number of ports mask  */
	BFA_FWI_ADAPTER_NPORTS_SH = 8,	        /*!< number of ports shift */
	BFA_FWI_ADAPTER_SPEED_MK  = 0xff,	/*!< adapter speed mask    */
	BFA_FWI_ADAPTER_SPEED_SH  = 0,	        /*!< adapter speed shift   */
	BFA_FWI_ADAPTER_PROTO     = 0x100000,	/*!< prototype adapaters   */
	BFA_FWI_ADAPTER_TTV       = 0x200000,	/*!< TTV debug capable     */
	BFA_FWI_ADAPTER_UNSUPP    = 0x400000,	/*!< unknown adapter type  */
};

#define BFA_FWI_ADAPTER_GETP(__prop,__adap_prop)                        \
    (((__adap_prop) & BFA_FWI_ADAPTER_ ## __prop ## _MK) >>             \
     BFA_FWI_ADAPTER_ ## __prop ## _SH)
#define BFA_FWI_ADAPTER_SETP(__prop, __val)         \
    ((__val) << BFA_FWI_ADAPTER_ ## __prop ## _SH)
#define BFA_FWI_ADAPTER_IS_PROTO(__adap_type)   \
    ((__adap_type) & BFA_FWI_ADAPTER_PROTO)
#define BFA_FWI_ADAPTER_IS_TTV(__adap_type)     \
    ((__adap_type) & BFA_FWI_ADAPTER_TTV)
#define BFA_FWI_ADAPTER_IS_UNSUPP(__adap_type)  \
    ((__adap_type) & BFA_FWI_ADAPTER_UNSUPP)
#define BFA_FWI_ADAPTER_IS_SPECIAL(__adap_type)                         \
    ((__adap_type) & (BFA_FWI_ADAPTER_TTV | BFA_FWI_ADAPTER_PROTO |     \
			BFA_FWI_ADAPTER_UNSUPP))

typedef struct {
	wwn_t           mfg_wwn;
	char            brcd_serialnum[STRSZ(BFA_MFG_SERIALNUM_SIZE)];
	uint8_t         pcie_gen;
	uint8_t         pcie_lanes_orig;
	uint8_t         pcie_lanes;
	uint8_t         rx_bbcredit;	/*!< receive buffer credits */
	uint32_t        adapter_prop;	/*!< adapter properties     */
	uint16_t        maxfrsize;	/*!< max receive frame size */
	char         	asic_rev;
	uint8_t         rsvd;
	bfa_ioc_fwcfg_t fwcfg;	        /*!< firmware configuration */
	bfa_ioc_intr_attr_t intr_attr;	/*!< interrupt attributes   */
	char            fw_version[BFA_VERSION_LEN];
} bfa_ioc_cfgrsp_info_t;

/**
 * BFA_IOC_H2I_CFG_REQ message
 */
typedef struct bfa_ioc_cfg_req {
	bfa_mhdr_t      mh;
	bfa_addr_t      ioc_cfg_dma_addr;
} bfa_ioc_cfg_req_t;

/**
 * BFA_IOC_I2H_CFG_REPLY message
 */
typedef struct bfa_ioc_cfg_reply {
	bfa_mhdr_t      mh;		/*!< Common msg header          */
	uint8_t         cfg_success;	/*!< cfg reply status           */
	uint8_t         lpu_bm;		/*!< LPUs assigned for this IOC */
	uint8_t         rsvd[2];
} bfa_ioc_cfg_reply_t;

/**
 * BFA_IOC_H2I_ENABLE_REQ & BFA_IOC_H2I_DISABLE_REQ messages
 */
typedef struct {
	bfa_mhdr_t      mh;
} bfa_ioc_enable_req_t, bfa_ioc_disable_req_t;

/**
 * BFA_IOC_I2H_ENABLE_REPLY & BFA_IOC_I2H_DISABLE_REPLY messages
 */
typedef struct {
	bfa_mhdr_t      mh;		/*!< Common msg header     */
	uint8_t         status;		/*!< enable/disable status */
	uint8_t         rsvd[3];
} bfa_ioc_enable_reply_t, bfa_ioc_disable_reply_t;

/**
 *  BFA_IOC_H2I_GET_STATS_REQ & BFA_IOC_H2I_CLEAR_STATS_REQ messages
 */
typedef struct {
	bfa_mhdr_t      mh;		/*!< msg header            */
	uint32_t        msgtag;		/*!< msgtag for reply      */
} bfa_ioc_stats_get_req_t, bfa_ioc_stats_clr_req_t;

/**
 * BFA_IOC_I2H_GET_STATS_RSP & BFA_IOC_I2H_CLEAR_STATS_RSP messages
 */
typedef struct {
	bfa_mhdr_t      mh;	       /*!< common msg header     */
	uint8_t         status;	       /*!< reply status          */
	uint8_t         rsvd[3];
	uint32_t        msgtag;	       /*!< msgtag for reply      */
} bfa_ioc_stats_get_rsp_t, bfa_ioc_stats_clr_rsp_t;

/**
 * BFA_IOC_H2I_SET_INTR_REQ message
 */
typedef struct {
	bfa_mhdr_t      mh;		/*!< common msg header     */
	uint8_t		coalesce;	/*!< enable intr coalescing*/
	uint8_t         rsvd[3];
	uint16_t	delay;		/*!< delay timer 0..1125us  */
	uint16_t	latency;	/*!< latency timer 0..225us */
} bfa_ioc_fwi_set_intr_req_t;

/**
 * H2I Messages
 */
typedef union {
	bfa_mhdr_t      	mh;
	bfa_ioc_cfg_req_t 	cfg_req;
	bfa_ioc_enable_req_t 	enable_req;
	bfa_ioc_disable_req_t 	disable_req;
	bfa_ioc_stats_get_req_t stats_get;
	bfa_ioc_stats_clr_req_t stats_clr;
} bfa_ioc_h2i_msg_t;

#define BFA_FWI_IOC_MSGSZ   8
/**
 * I2H Messages
 */
typedef union {
	bfa_mhdr_t      	mh;
	bfa_ioc_rdy_event_t 	rdy_event;
	bfa_ioc_cfg_reply_t 	cfg_reply;
	bfa_ioc_stats_get_rsp_t stats_get_rsp;
	bfa_ioc_stats_clr_rsp_t stats_clr_rsp;
	uint32_t        	mboxmsg[BFA_FWI_IOC_MSGSZ];
} bfa_ioc_i2h_msg_t;

#pragma pack()

#endif /* __BFA_IOC_MSGS_H__ */

