/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_FWI_ITNIM_H__
#define __BFA_FWI_ITNIM_H__

#include "bfa_fwi.h"

#pragma pack(1)

typedef enum {
	BFA_FWI_ITNIM_H2I_CREATE_REQ = 1,	/*!< i-t nexus creation */
	BFA_FWI_ITNIM_H2I_DELETE_REQ = 2,	/*!< i-t nexus deletion */

} bfa_fwi_itnim_h2i_t;

typedef enum {
	BFA_FWI_ITNIM_I2H_CREATE_RSP = BFA_I2HM(1),
	BFA_FWI_ITNIM_I2H_DELETE_RSP = BFA_I2HM(2),
	BFA_FWI_ITNIM_I2H_SLER_EVENT = BFA_I2HM(3),
} bfa_fwi_itnim_i2h_t;

typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
	uint16_t        fw_handle;	/*!< f/w handle for itnim */
	uint8_t         class;	/*!< FC class for IO */
	uint8_t         seq_rec;	/*!< seq level error recovery support */
	uint8_t         msg_no;	/*!< seq id of the msg */
} bfa_fwi_itnim_create_req_t;

typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
	uint16_t        bfa_handle;	/*!< bfa handle for itnim */
	uint8_t         status;	/*!< fcp request status */
	uint8_t         seq_id;
} bfa_fwi_itnim_create_rsp_t;

typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
	uint16_t        fw_handle;	/*!< f/w itnim handle */
	uint8_t         seq_id;	/*!< seq id of the msg */
	uint8_t         rsvd;
} bfa_fwi_itnim_delete_req_t;

typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
	uint16_t        bfa_handle;	/*!< bfa handle for itnim */
	uint8_t         status;	/*!< fcp request status */
	uint8_t         seq_id;
} bfa_fwi_itnim_delete_rsp_t;

typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
	uint16_t        bfa_handle;	/*!< bfa handle for itnim */
	uint16_t        rsvd;
} bfa_fwi_itnim_sler_event_t;

typedef union {
	bfa_fwi_itnim_create_req_t *create_req;
	bfa_fwi_itnim_delete_req_t *delete_req;
	bfa_msg_t      *msg;
} bfa_fwi_itnim_h2i_msg_t;

typedef union {
	bfa_fwi_itnim_create_rsp_t *create_rsp;
	bfa_fwi_itnim_delete_rsp_t *delete_rsp;
	bfa_fwi_itnim_sler_event_t *sler_event;
	bfa_msg_t      *msg;
} bfa_fwi_itnim_i2h_msg_t;

#pragma pack()

#endif /* __BFA_FWI_ITNIM_H__ */
