/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_FWI_PPORT_H__
#define __BFA_FWI_PPORT_H__

#include <fwi/bfa_fwi.h>
#include <defs/bfa_defs_pport.h>

#pragma pack(1)

typedef enum {
	BFA_FWI_PPORT_H2I_ENABLE_REQ = (1),
	BFA_FWI_PPORT_H2I_DISABLE_REQ = (2),
	BFA_FWI_PPORT_H2I_GET_STATS_REQ = (3),
	BFA_FWI_PPORT_H2I_CLEAR_STATS_REQ = (4),
	BFA_FWI_PPORT_H2I_SET_SVC_PARAMS_REQ = (5),
	BFA_FWI_PPORT_H2I_ENABLE_RX_VF_TAG_REQ = (6),
	BFA_FWI_PPORT_H2I_ENABLE_TX_VF_TAG_REQ = (7),
} bfa_fwi_pport_h2i_t;

typedef enum {
	BFA_FWI_PPORT_I2H_ENABLE_RSP = BFA_I2HM(1),
	BFA_FWI_PPORT_I2H_DISABLE_RSP = BFA_I2HM(2),
	BFA_FWI_PPORT_I2H_GET_STATS_RSP = BFA_I2HM(3),
	BFA_FWI_PPORT_I2H_CLEAR_STATS_RSP = BFA_I2HM(4),
	BFA_FWI_PPORT_I2H_SET_SVC_PARAMS_RSP = BFA_I2HM(5),
	BFA_FWI_PPORT_I2H_ENABLE_RX_VF_TAG_RSP = BFA_I2HM(6),
	BFA_FWI_PPORT_I2H_ENABLE_TX_VF_TAG_RSP = BFA_I2HM(7),
	BFA_FWI_PPORT_I2H_EVENT = BFA_I2HM(8),
} bfa_fwi_pport_i2h_t;

/**
 * Generic REQ type
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< msg header */
	uint32_t        msgtag;	/*!< msgtag for reply */
} bfa_fwi_pport_generic_req_t;

/**
 * Generic RSP type
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
	uint8_t         status;	/*!< port enable status */
	uint8_t         rsvd[3];
	uint32_t        msgtag;	/*!< msgtag for reply */
} bfa_fwi_pport_generic_rsp_t;

/**
 * BFA_FWI_PPORT_H2I_ENABLE_REQ
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< msg header */
	uint32_t        resv1;
	wwn_t           nwwn;	/*!< node wwn of physical port */
	wwn_t           pwwn;	/*!< port wwn of physical port */
	bfa_pport_cfg_t port_cfg;	/*!< port configuration */
	bfa_addr_t      stats_dma_addr;	/*!< DMA address for stats. *See
					 *bfa_pport_stats_t */
	uint32_t        msgtag;	/*!< msgtag for reply */
	uint32_t        resv2;
} bfa_fwi_pport_enable_req_t;

/**
 * BFA_FWI_PPORT_I2H_ENABLE_RSP
 */
typedef bfa_fwi_pport_generic_rsp_t bfa_fwi_pport_enable_rsp_t;

/**
 * BFA_FWI_PPORT_H2I_DISABLE_REQ
 */
typedef bfa_fwi_pport_generic_req_t bfa_fwi_pport_disable_req_t;

/**
 * BFA_FWI_PPORT_I2H_DISABLE_RSP
 */
typedef bfa_fwi_pport_generic_rsp_t bfa_fwi_pport_disable_rsp_t;

/**
 * BFA_FWI_PPORT_H2I_GET_STATS_REQ
 */
typedef bfa_fwi_pport_generic_req_t bfa_fwi_pport_get_stats_req_t;

/**
 * BFA_FWI_PPORT_I2H_GET_STATS_RSP
 */
typedef bfa_fwi_pport_generic_rsp_t bfa_fwi_pport_get_stats_rsp_t;

/**
 * BFA_FWI_PPORT_H2I_CLEAR_STATS_REQ
 */
typedef bfa_fwi_pport_generic_req_t bfa_fwi_pport_clear_stats_req_t;

/**
 * BFA_FWI_PPORT_I2H_CLEAR_STATS_RSP
 */
typedef bfa_fwi_pport_generic_rsp_t bfa_fwi_pport_clear_stats_rsp_t;

/**
 * BFA_FWI_PPORT_H2I_SET_SVC_PARAMS_REQ
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< msg header */
	uint16_t        tx_bbcredit;	/*!< Tx credits */
	uint16_t        rsvd;
} bfa_fwi_pport_set_svc_params_req_t;

/**
 * BFA_FWI_PPORT_I2H_SET_SVC_PARAMS_RSP
 */
typedef bfa_fwi_pport_generic_rsp_t bfa_fwi_pport_set_svc_params_rsp_t;

/**
 * BFA_FWI_PPORT_I2H_EVENT
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
	union {
		bfa_pport_link_t link_state;
	} port_event;
} bfa_fwi_pport_event_t;

typedef union {
	bfa_fwi_pport_enable_req_t *penable;
	bfa_fwi_pport_disable_req_t *pdisable;
	bfa_fwi_pport_get_stats_req_t *pgetstats;
	bfa_fwi_pport_clear_stats_req_t *pclearstats;
	bfa_fwi_pport_set_svc_params_req_t *psetsvcparams;
	bfa_mhdr_t     *mhdr;
} bfa_fwi_pport_h2i_msg_t;

typedef union {
	bfa_fwi_pport_enable_rsp_t *enable_rsp;
	bfa_fwi_pport_disable_rsp_t *disable_rsp;
	bfa_fwi_pport_get_stats_rsp_t *getstats_rsp;
	bfa_fwi_pport_clear_stats_rsp_t *clearstats_rsp;
	bfa_fwi_pport_set_svc_params_rsp_t *setsvcparasm_rsp;
	bfa_fwi_pport_event_t *event;
	bfa_msg_t      *msg;
} bfa_fwi_pport_i2h_msg_t;

#pragma pack()

#endif /* __BFA_FWI_PPORT_H__ */
