/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_FWI_RPORT_H__
#define __BFA_FWI_RPORT_H__

#include <fwi/bfa_fwi.h>

#pragma pack(1)

typedef enum {
	BFA_FWI_RPORT_H2I_CREATE_REQ = 1,
	BFA_FWI_RPORT_H2I_DELETE_REQ = 2,
} bfa_rport_h2i_msgs_t;

typedef enum {
	BFA_FWI_RPORT_I2H_CREATE_RSP = BFA_I2HM(1),
	BFA_FWI_RPORT_I2H_DELETE_RSP = BFA_I2HM(2),
} bfa_rport_i2h_msgs_t;

typedef struct {
	bfa_mhdr_t      mh;		/*!< common msg header		*/
	uint16_t        bfa_handle;	/*!< host rport handle		*/
	uint16_t        max_frmsz;	/*!< max rcv pdu size		*/
	uint32_t        pid : 24,	/*!< remote port ID		*/
			rsvd_a : 8;
	uint32_t        local_pid : 24,	/*!< local port ID		*/
			cisc : 8;
	uint8_t         fc_class;	/*!< supported FC classes	*/
	uint8_t         vf_en;		/*!< virtual fabric enable	*/
	uint16_t        vf_id;		/*!< virtual fabric ID		*/
} bfa_fwi_rport_create_req_t;

typedef struct {
	bfa_mhdr_t      mh;		/*!< common msg header		*/
	uint8_t         status;		/*!< rport creation status	*/
	uint8_t         rsvd[3];
	uint16_t        bfa_handle;	/*!< host rport handle		*/
	uint16_t        fw_handle;	/*!< firmware rport handle	*/
} bfa_fwi_rport_create_rsp_t;

typedef struct bfa_rport_delete_req {
	bfa_mhdr_t      mh;		/*!< common msg header		*/
	uint16_t        fw_handle;	/*!< firmware rport handle	*/
	uint16_t        rsvd;
} bfa_fwi_rport_delete_req_t;

typedef struct {
	bfa_mhdr_t      mh;		/*!< common msg header		*/
	uint16_t        bfa_handle;	/*!< host rport handle		*/
	uint8_t         status;		/*!< rport deletion status	*/
	uint8_t         rsvd;
} bfa_fwi_rport_delete_rsp_t;

typedef union {
	bfa_msg_t      *msg;
	bfa_fwi_rport_create_req_t *create_req;
	bfa_fwi_rport_delete_req_t *delete_req;
} bfa_fwi_rport_h2i_msg_t;

typedef union {
	bfa_msg_t      *msg;
	bfa_fwi_rport_create_rsp_t *create_rsp;
	bfa_fwi_rport_delete_rsp_t *delete_rsp;
} bfa_fwi_rport_i2h_msg_t;

#pragma pack()

#endif /* __BFA_FWI_RPORT_H__ */

