/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/*
 * @file bfa_fwi_sboot.h BFA SAN boot services interface
 */

#ifndef __BFA_FWI_SBOOT_H__
#define __BFA_FWI_SBOOT_H__

#include <fwi/bfa_fwi.h>
#include <defs/bfa_defs_boot.h>
#include <protocol/scsi.h>

#pragma pack(1)

typedef enum {
	BFA_FWI_SBOOT_H2I_INIT_REQ = 1,	/*!< initialize request */
	BFA_FWI_SBOOT_H2I_OPENBLUN_REQ = 2,	/*!< open boot lun */
	BFA_FWI_SBOOT_H2I_DISC_RPORTS_REQ = 3,	/*!< list remote ports */
	BFA_FWI_SBOOT_H2I_DISC_LUNS_REQ = 4,	/*!< list LUNs of rport */
	BFA_FWI_SBOOT_H2I_IORW_REQ = 5,	/*!< block read request */
} bfa_fwi_sboot_h2i_t;

typedef enum {
	BFA_FWI_SBOOT_I2H_INIT_RSP = BFA_I2HM(1),
	BFA_FWI_SBOOT_I2H_OPENBLUN_RSP = BFA_I2HM(2),
	BFA_FWI_SBOOT_I2H_DISC_RPORTS_RSP = BFA_I2HM(3),
	BFA_FWI_SBOOT_I2H_DISC_LUNS_RSP = BFA_I2HM(4),
	BFA_FWI_SBOOT_I2H_IORW_RSP = BFA_I2HM(5),
} bfa_fwi_sboot_i2h_t;

/**
 * Structure of remote port DMAed by firmware
 */
typedef struct {
	wwn_t           pwwn;	/*!< port wwn of target port */
	uint32_t        fc_pid;	/*!< FC port ID */
	uint16_t        maxfrsz;	/*!< maximum receive pdu size */
	uint16_t        bbcredit;	/*!< tx bb-credits */
	scsi_inquiry_vendor_t vendor_id;	/*!< vendor ID from INQUIRY */
	scsi_inquiry_prodid_t product_id;	/*!< product ID from INQUIRY */
	scsi_inquiry_prodrev_t product_rev;	/*!< product revision from INQ
						 */
	uint32_t        rsvd_b;	/*!< for 8-byte multiples */
} bfa_fwi_sboot_rport_t;

/**
 * Structure of LUN list DMAed by firmware.
 */
typedef struct {
	lun_t           lun;	/*!< 8-byte lun number */
	uint32_t        nblocks;	/*!< lun size (read capacity) */
	uint32_t        blksize;	/*!< block size (read capacity) */
	scsi_inquiry_vendor_t vendor_id;	/*!< vendor ID from INQUIRY */
	scsi_inquiry_prodid_t product_id;	/*!< product ID from INQUIRY */
	scsi_inquiry_prodrev_t product_rev;	/*!< product revision from INQ
						 */
	uint32_t        rsvd_b;	/*!< for 8-byte multiples */
} bfa_fwi_sboot_lun_t;

/**
 * San boot configuration.
 */
typedef struct {
	uint8_t         speed;	/*!< bfa_pport_speed_t */
	uint8_t         topology;	/*!< bfa_pport_topology_t */
	uint8_t         auto_discover;	/*!< auto discover boot-lun */
	uint8_t         nbluns;	/*!< number of boot-luns */
	uint16_t        rports_max;	/*!< size of remote ports array */
	uint16_t        rluns_max;	/*!< size of luns array */
	bfa_boot_bootlun_t blun[BFA_BOOT_BOOTLUN_MAX];
	uint32_t        rports_off;	/*!< remote port list */
	uint32_t        luns_off;	/*!< remote lun list */
} bfa_sboot_config_t;

/**
 * San boot initialization request.
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
	bfa_addr_t      shmem;	/*!< shared memory address */
} bfa_fwi_sboot_init_req_t;

/**
 * san boot initialization response.
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
	uint8_t         status;	/*!< initialization status */
	uint8_t         cur_speed;	/*!< link speed */
	uint8_t         cur_topology;	/*!< link topology */
	uint8_t         cur_linkstate;	/*!< link state */
	uint32_t        mypid;	/*!< local FC port ID */
} bfa_fwi_sboot_init_rsp_t;

/**
 * San boot open boot-lun request
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
	wwn_t           pwwn;	/*!< target port name */
	lun_t           lun;	/*!< boot LU number */
	uint32_t        autodisc;	/*!< auto discover */
} bfa_fwi_sboot_openblun_req_t;

/**
 * san boot open boot-lun response.
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
	uint8_t         status;	/*!< open boot lun status */
	uint8_t         rsvd[3];
	uint32_t        nblocks;	/*!< lun size (read capacity) */
	uint32_t        blksize;	/*!< block size (read capacity) */
	scsi_inquiry_vendor_t vendor_id;	/*!< vendor ID from INQUIRY */
	scsi_inquiry_prodid_t product_id;	/*!< product ID from INQUIRY */
	scsi_inquiry_prodrev_t product_rev;	/*!< product revision from INQ
						 */
} bfa_fwi_sboot_openblun_rsp_t;

/**
* san boot: remote port list request
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
} bfa_fwi_sboot_disc_rports_req_t;

/**
* san boot: remote port list response
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
	uint8_t         status;	/*!< initialization status */
	uint8_t         rsvd;
	uint16_t        nrports;	/*!< number of remote ports */
} bfa_fwi_sboot_disc_rports_rsp_t;

/**
* san boot: request for list of LUNs of a remote port
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
	uint32_t        rpid;	/*!< remote port PID */
	wwn_t           pwwn;	/*!< remote port name */
} bfa_fwi_sboot_disc_luns_req_t;

/**
* san boot: LUN list response
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
	uint8_t         status;	/*!< initialization status */
	uint8_t         rsvd;
	uint16_t        nluns;	/*!< number of remote ports */
} bfa_fwi_sboot_disc_luns_rsp_t;

/**
 * IO type values for IO read/write request
 */
typedef enum {
	BFA_FWI_SBOOT_IOWRITE = 1,
	BFA_FWI_SBOOT_IOREAD = 2,
} bfa_fwi_sboot_iotype_t;

/**
* san boot: IO read/write request
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
uint8_t         iotype;	/*!< IO type: read/write */
	uint8_t         rsvd[3];
	uint32_t        blkoff;	/*!< offset in disk blocks */
	uint32_t        nblks;	/*!< IO size in blocks */
	bfa_addr_t      bufaddr;	/*!< IO buffer address */
} bfa_fwi_sboot_iorw_req_t;

/**
 * IO read/write request response
 */
typedef struct {
	bfa_mhdr_t      mh;	/*!< common msg header */
	uint8_t         status;	/*!< IO completion status */
	uint8_t         rsvd[3];
} bfa_fwi_sboot_iorw_rsp_t;

/**
 * Host to firmware message.
 */
typedef union {
	bfa_mhdr_t     *mhdr;
	bfa_fwi_sboot_init_req_t *init;
	bfa_fwi_sboot_openblun_req_t *openblun;
	bfa_fwi_sboot_disc_rports_req_t *disc_rports;
	bfa_fwi_sboot_disc_luns_req_t *disc_luns;
	bfa_fwi_sboot_iorw_req_t *iorw;
} bfa_fwi_sboot_h2i_msg_t;

/**
 * Firmware to host message.
 */
typedef union {
	bfa_msg_t      *msg;
	bfa_fwi_sboot_init_rsp_t *init;
	bfa_fwi_sboot_openblun_rsp_t *openblun;
	bfa_fwi_sboot_disc_rports_rsp_t *disc_rports;
	bfa_fwi_sboot_disc_luns_rsp_t *disc_luns;
	bfa_fwi_sboot_iorw_rsp_t *iorw;
} bfa_fwi_sboot_i2h_msg_t;

#pragma pack()

#endif /* __BFA_FWI_SBOOT_H__ */
