/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_FWI_TSKIM_H__
#define __BFA_FWI_TSKIM_H__

#include "bfa_fwi.h"

#pragma pack(1)

typedef enum {
	BFA_FWI_TSKIM_H2I_TM_REQ    = 1, /*!< task-mgmt command         */
	BFA_FWI_TSKIM_H2I_ABORT_REQ = 2, /*!< task-mgmt command         */
} bfa_fwi_tskim_h2i_t;

typedef enum {
	BFA_FWI_TSKIM_I2H_TM_RSP = BFA_I2HM(1),
} bfa_fwi_tskim_i2h_t;

typedef struct {
	bfa_mhdr_t      mh;             /*!< Common msg header          */
	uint16_t        tsk_tag;        /*!< task management tag        */
	uint16_t        itn_fhdl;       /*!< itn firmware handle        */
	lun_t           lun;            /*!< LU number                  */
	uint8_t         tm_flags;       /*!< see fcp_tm_cmnd_t          */
	uint8_t         t_secs;         /*!< Timeout value in seconds   */
	uint8_t         rsvd[2];
} bfa_fwi_tskim_req_t;

typedef struct {
	bfa_mhdr_t      mh;             /*!< Common msg header          */
	uint16_t        tsk_tag;        /*!< task management tag        */
	uint16_t        rsvd;
} bfa_fwi_tskim_abortreq_t;

typedef enum {
	/*
	 * Following are FCP-4 spec defined status codes,
	 * **DO NOT CHANGE THEM **
	 */
	BFA_TSKIM_STS_OK          = 0,
	BFA_TSKIM_STS_TM_NOT_SUPP = 4,
	BFA_TSKIM_STS_TM_FAILED   = 5,

	/**
	 * Defined by BFA
	 */
	BFA_TSKIM_STS_TM_TIMEOUT  = 10,	    /*!< TM request timedout     */
	BFA_TSKIM_STS_TM_ABORTED  = 11,     /*!< Aborted on host request */
} bfa_tskim_status_t;

typedef struct {
	bfa_mhdr_t      mh;	/*!< Common msg header */
	uint16_t        tsk_tag;	/*!< task mgmt cmnd tag */
	uint8_t         tsk_status;	/*!< @ref bfa_tskim_status_t */
	uint8_t         rsvd;
} bfa_fwi_tskim_rsp_t;

#pragma pack()

#endif /* __BFA_FWI_TSKIM_H__ */
