/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_HAL_H__
#define __BFA_HAL_H__

#include <cs/bfa_debug.h>
#include <bfa_os_inc.h>
#include <bfa_os.h>
#include <defs/bfa_defs_status.h>
#include <cs/bfa_q.h>
#include <cs/bfa_trc.h>
#include <cs/bfa_log.h>
#include <aen/bfa_aen.h>
#include <cs/bfa_plog.h>
#include <defs/bfa_defs_ioc.h>
#include <hal/bfa_hal_mem.h>
#include <fwi/bfa_fwi.h>
#include <hcb/bfa_hcb.h>

typedef struct bfa_hal_s bfa_hal_t;
#include <hal_intr.h>

/**
 * HAL interrupt return enums
 */
typedef enum {
	BFA_HAL_INTR_NOTCLAIMED = 1,	/*!< interrupt not ours */
	BFA_HAL_INTR_CLAIMED_COMP = 2,	/*!< claimed; completions pending */
	BFA_HAL_INTR_CLAIMED_NO_COMP = 3,	/*!< claimed; no completions */
} bfa_hal_intr_status_t;

/**
 * PCI device information required by HAL
 */
typedef struct {
	int             pci_slot;
	uint8_t         pci_func;
	bfa_os_addr_t   pci_bar_kva;
} bfa_hal_pcidev_t;

/**
 * PCI devices supported by the current HAL
 */
typedef struct {
	uint16_t        device_id;
	uint16_t        vendor_id;
} bfa_hal_pciid_t;

extern char     bfa_version[];

/*
 * bfa hal API functions
 */
void            bfa_hal_get_pciids(bfa_hal_pciid_t **pciids, int *npciids);
void            bfa_hal_cfg_get_default(bfa_ioc_cfg_t *cfg);
void            bfa_hal_cfg_get_meminfo(bfa_ioc_cfg_t *cfg,
					bfa_hal_meminfo_t *meminfo);
void            bfa_hal_attach(bfa_hal_t *hal, void *bfad, bfa_ioc_cfg_t *cfg,
			       bfa_hal_meminfo_t *meminfo,
			       bfa_hal_pcidev_t *pcidev);
void            bfa_hal_trc_init(bfa_hal_t *hal, bfa_trc_mod_t *trcmod);
void            bfa_hal_log_init(bfa_hal_t *hal, bfa_log_mod_t *logmod);
void            bfa_hal_aen_init(bfa_hal_t *hal, bfa_aen_t *aen);
void            bfa_hal_plog_init(bfa_hal_t *hal, bfa_plog_t *plog);
void            bfa_hal_detach(bfa_hal_t *hal);
void            bfa_hal_init(bfa_hal_t *hal);
void            bfa_hal_start(bfa_hal_t *hal);
void            bfa_hal_stop(bfa_hal_t *hal);
void            bfa_hal_fcs_attach(bfa_hal_t *hal);

#define BFA_HAL_TIMER_FREQ 500 /**< specified in millisecs */	/* TODO VRK -
								 * need to find
								 * a better
								 * place.
								 */

void            bfa_hal_timer_tick(bfa_hal_t *hal);
void            bfa_hal_msix_getvecs(bfa_hal_t *hal, uint32_t *msix_vecs_bmap,
				     uint32_t *num_vecs,
				     uint32_t *max_vec_bit);
bfa_hal_intr_status_t bfa_hal_intx(bfa_hal_t *hal);
bfa_hal_intr_status_t bfa_hal_msix_reqq(bfa_hal_t *hal, int qid);
bfa_hal_intr_status_t bfa_hal_msix_rspq(bfa_hal_t *hal, int qid);
bfa_hal_intr_status_t bfa_hal_msix_lpu(bfa_hal_t *hal);
bfa_hal_intr_status_t bfa_hal_msix_errint(bfa_hal_t *hal);
void            bfa_hal_intr_enable(bfa_hal_t *hal);
void            bfa_hal_intr_disable(bfa_hal_t *hal);

void            bfa_hal_comp_deq(bfa_hal_t *hal, bfa_q_t *comp_q);
void            bfa_hal_comp_process(bfa_hal_t *hal, bfa_q_t *comp_q);
void            bfa_hal_comp_free(bfa_hal_t *hal, bfa_q_t *comp_q);

bfa_status_t    bfa_hal_ioc_get_stats(bfa_hal_t *hal, bfa_ioc_stats_t *stats,
				      bfa_hcb_ioc_t cbfn, void *cbarg);
bfa_status_t    bfa_hal_ioc_clear_stats(bfa_hal_t *hal, bfa_hcb_ioc_t cbfn,
					void *cbarg);
void            bfa_hal_ioc_get_attr(bfa_hal_t *hal,
				     bfa_ioc_attr_t *ioc_attr);
bfa_status_t    bfa_hal_intr_attr_set(bfa_hal_t *hal,
				      bfa_ioc_intr_attr_t *attr);

bfa_status_t    bfa_hal_ioc_enable(bfa_hal_t *hal);
void            bfa_hal_ioc_disable(bfa_hal_t *hal);

#include "hal_priv.h"

#endif /* __BFA_HAL_H__ */
