/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_HAL_FCXP_H__
#define __BFA_HAL_FCXP_H__

#include <defs/bfa_defs_status.h>
#include <defs/bfa_defs_types.h>
#include <hal/bfa_hal_rport.h>

/*
 * forward declarations
 */
typedef struct bfa_hal_fcxp bfa_hal_fcxp_t;

#include <hcb/bfa_hcb_fcxp.h>

/**
 * HAL fcxp allocation (asynchronous)
 */
typedef void    (*bfa_fcxp_alloc_cbfn_t) (void *cbarg, bfa_hal_fcxp_t *fcxp);
typedef struct {
	bfa_q_t         qe;
	bfa_fcxp_alloc_cbfn_t alloc_cbfn;
	void           *alloc_cbarg;
} bfa_hal_fcxp_wqe_t;

typedef         uint64_t(*bfa_fcxp_get_sgaddr_t) (void *bfad_fcxp, int sgeid);
typedef         uint32_t(*bfa_fcxp_get_sglen_t) (void *bfad_fcxp, int sgeid);

/*
 * bfa hal fcxp API functions
 */
bfa_hal_fcxp_t *bfa_hal_fcxp_alloc(void *bfad_fcxp, bfa_hal_t *hal,
				   int nreq_sgles, int nrsp_sgles,
				   bfa_fcxp_get_sgaddr_t get_req_sga,
				   bfa_fcxp_get_sglen_t get_req_sglen,
				   bfa_fcxp_get_sgaddr_t get_rsp_sga,
				   bfa_fcxp_get_sglen_t get_rsp_sglen);
void            bfa_hal_fcxp_alloc_wait(bfa_hal_t *hal,
					bfa_hal_fcxp_wqe_t *wqe,
					bfa_fcxp_alloc_cbfn_t alloc_cbfn,
					void *cbarg);
void            bfa_hal_fcxp_walloc_cancel(bfa_hal_t *hal,
					   bfa_hal_fcxp_wqe_t *wqe);
void            bfa_hal_fcxp_discard(bfa_hal_fcxp_t *fcxp);

void           *bfa_hal_fcxp_get_reqbuf(bfa_hal_fcxp_t *fcxp);
void           *bfa_hal_fcxp_get_rspbuf(bfa_hal_fcxp_t *fcxp);

void            bfa_hal_fcxp_free(bfa_hal_fcxp_t *fcxp);
void            bfa_hal_fcxp_send(bfa_hal_fcxp_t *fcxp,
				  bfa_hal_rport_t *rport, uint16_t vf_id,
				  bfa_boolean_t cts, fc_cos_t cos,
				  uint32_t reqlen, fchs_t *fchs,
				  bfa_hcb_fcxp_send_t cbfn, void *cbarg,
				  uint32_t rsp_maxlen, uint8_t rsp_timeout);
bfa_status_t    bfa_hal_fcxp_abort(bfa_hal_fcxp_t *fcxp);
#endif /* __BFA_HAL_FCXP_H__ */
