/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_HAL_MEM_H__
#define __BFA_HAL_MEM_H__

#include <bfa_os_inc.h>

/**
 * HAL memory resources
 */
typedef enum {
	BFA_HAL_MEM_TYPE_KVA = 1,	/*! Kernel Virtual Memory
					 *(non-dma-able) */
	BFA_HAL_MEM_TYPE_DMA = 2,	/*! DMA-able memory */
	BFA_HAL_MEM_TYPE_MAX = BFA_HAL_MEM_TYPE_DMA,
} bfa_hal_mem_type_t;

typedef struct {
	bfa_hal_mem_type_t mem_type;	/*! see bfa_hal_mem_type_t */
	uint32_t        mem_len;	/*!< Total Length in Bytes */
	uint8_t        *kva;	/*!< kernel virtual address */
	uint64_t        dma;	/*!< dma address valid only when mem_type *
				 *is BFA_HAL_MEM_TYPE_DMA */
	uint8_t        *kva_curp;	/*!< kva allocation cursor */
	uint64_t        dma_curp;	/*!< dma allocation cursor */

} bfa_hal_mem_elem_t;

typedef struct {
	bfa_hal_mem_elem_t meminfo[BFA_HAL_MEM_TYPE_MAX];
} bfa_hal_meminfo_t;
#define bfa_hal_meminfo_kva(_m)	\
	(_m)->meminfo[BFA_HAL_MEM_TYPE_KVA - 1].kva_curp
#define bfa_hal_meminfo_dma_virt(_m)	\
	(_m)->meminfo[BFA_HAL_MEM_TYPE_DMA - 1].kva_curp
#define bfa_hal_meminfo_dma_phys(_m)	\
	(_m)->meminfo[BFA_HAL_MEM_TYPE_DMA - 1].dma_curp

/**
 * Generic Scatter Gather Element used by driver
 */
typedef struct {
	uint32_t        sg_len;
	void           *sg_addr;
} bfa_hal_sge_t;

#endif /* __BFA_HAL_MEM_H__ */
