/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_HAL_RPORT_H__
#define __BFA_HAL_RPORT_H__

#include <defs/bfa_defs_status.h>
#include <bfa_os_inc.h>
#include <cs/bfa_q.h>
#include <cs/bfa_sm.h>
#include <defs/bfa_defs_rport.h>
#include <hal/bfa_hal.h>

/**
 * @brief
 * 		HAL rport information.
 */
typedef struct {
	uint16_t        max_frmsz;	/*!< max rcv pdu size               */
	uint32_t        pid:24,		/*!< remote port ID                 */
			rsvd1:8;
	uint32_t        local_pid:24,	/*!< local port ID                  */
			cisc:8;		/*!< CIRO supported		    */
	uint8_t         fc_class;	/*!< supported FC classes. fc_cos_t */
	uint8_t         vf_en;		/*!< virtual fabric enable          */
	uint16_t        vf_id;		/*!< virtual fabric ID              */
} bfa_hal_rport_info_t;

/**
 * BFA HAL rport data structure
 */
typedef struct {
	bfa_q_t               qe;	 /*!< queue element                 */
	bfa_sm_t              sm;	 /*!< state machine                 */
	bfa_hal_t             *hal;	 /*!< backpointer to HAL            */
	void                  *rport_drv;/*!< fcs/driver rport object       */
	uint16_t              fw_handle; /*!< firmware rport handle         */
	uint16_t              rport_tag; /*!< HAL rport tag                 */
	bfa_hal_rport_info_t  rport_info;/*!< rport info from *fcs/driver   */
	bfa_hal_reqq_wait_t   reqq_wait; /*!< to wait for room in reqq      */
	bfa_hcb_qe_t          hcb_qe;	 /*!< HAL callback qelem            */
	bfa_rport_hal_stats_t stats;	 /*!< HAL rport statistics          */
	union {
		bfa_status_t    status;	 /*!< f/w status                    */
	} event_arg;
} bfa_hal_rport_t;

#define BFA_HAL_RPORT_FC_COS(_rport)	((_rport)->rport_info.fc_class)

/*
 * bfa hal API functions
 */
bfa_hal_rport_t *bfa_hal_rport_create(bfa_hal_t *hal, void *rport_drv);
void            bfa_hal_rport_delete(bfa_hal_rport_t *rport);
void            bfa_hal_rport_online(bfa_hal_rport_t *rport,
				     bfa_hal_rport_info_t *rport_info);
void            bfa_hal_rport_offline(bfa_hal_rport_t *rport);
void		bfa_hal_rport_get_stats(bfa_hal_rport_t *rport,
					bfa_rport_hal_stats_t *stats);
void		bfa_hal_rport_clear_stats(bfa_hal_rport_t *rport);

#endif /* __BFA_HAL_RPORT_H__ */

