/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_HAL_UF_H__
#define __BFA_HAL_UF_H__

#include <defs/bfa_defs_status.h>
#include <cs/bfa_q.h>
#include <hal/bfa_hal.h>

#define BFA_HAL_UF_BUFSZ	(2 * 1024 + 256)

/**
 * @todo hal private
 */
typedef struct {
	uint8_t         d[BFA_HAL_UF_BUFSZ];
} bfa_hal_uf_buf_t;

/**
 * @brief
 */
typedef struct {
	bfa_q_t         qe;	/*!< queue element */
	bfa_hal_t      *hal;	/*!< hal instance */

	uint16_t        uf_tag;	/*!< identifying tag f/w messages */
	uint16_t        vf_id;

	uint16_t        src_rport_handle;
	uint16_t        rsvd;

	uint8_t        *data_ptr;
	uint16_t        data_len; /*!< actual receive length */

	uint16_t        pb_len;	/*!< posted buffer length */

	void           *buf_kva;	/*!< buffer virtual address */
	uint64_t        buf_pa;	/*!< buffer physical address */
	bfa_hcb_qe_t    hcb_qe;	/*!< comp: HAL comp qelem */

	bfa_hal_sge_t   sges[BFA_SGE_MAX_INLINE];	/*!< prebuilt sge
							 *entries */
} bfa_hal_uf_t;

static inline void *
bfa_hal_uf_get_frmbuf(bfa_hal_uf_t *uf)
{
	return uf->data_ptr;
}

static inline   uint16_t
bfa_hal_uf_get_frmlen(bfa_hal_uf_t *uf)
{
	return uf->data_len;
}

/**
 * @brief
 * 		Callback prototype for unsolicited frame receive handler.
 *
 * @param[in]		cbarg		callback arg for receive handler
 * @param[in]		uf		unsolicited frame descriptor
 *
 * @return None
 */
typedef void    (*bfa_hcb_uf_recv_t) (void *cbarg, bfa_hal_uf_t *uf);

/*
 * bfa hal uf API functions
 */
void            bfa_hal_uf_recv_register(bfa_hal_t *hal,
					 bfa_hcb_uf_recv_t ufrecv, void *cbarg);
void            bfa_hal_uf_free(bfa_hal_uf_t *uf);

#endif /* __BFA_HAL_UF_H__ */
