/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfa_hcb_ioim.h BFA HAL callbacks for initiator mode IO handling.
 */

#ifndef __BFA_HCB_IOIM_H__
#define __BFA_HCB_IOIM_H__

#include <hal/bfa_hal.h>
#include <fwi/bfa_fwi_ioim.h>

/*
 * forward declarations
 */
typedef struct bfad_ioim_s bfad_ioim_t;

/**
 * @dg bfa_hcb_ioim HAL initiator mode IO handling
 * @{
 */

/**
 * @brief
 * 		I/O completion notification.
 *
 * @param[in]		dio			driver IO structure
 * @param[in]		io_status		IO completion status
 * @param[in]		scsi_status		SCSI status returned by target
 * @param[in]		sns_len			SCSI sense length, 0 if none
 * @param[in]		sns_info		SCSI sense data, if any
 * @param[in]		residue			Residual length
 *
 * @return None
 */
void            bfa_hcb_ioim_done(void *bfad, bfad_ioim_t *dio,
				  bfa_ioim_status_t io_status,
				  uint8_t scsi_status, int sns_len,
				  uint8_t *sns_info, int32_t residue);

/**
 * @brief
 * 		I/O good completion notification.
 *
 * @param[in]		dio			driver IO structure
 *
 * @return None
 */
void            bfa_hcb_ioim_good_comp(void *bfad, bfad_ioim_t *dio);

/**
 * @brief
 * 		I/O abort completion notification
 *
 * @param[in]		dio			driver IO that was aborted
 *
 * @return None
 */
void            bfa_hcb_ioim_abort(void *bfad, bfad_ioim_t *dio);

/**
 * @}
 */

#endif /* __BFA_HCB_IOIM_H__ */
