/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __CT_H__
#define __CT_H__

#include <protocol/types.h>

#pragma pack(1)

typedef struct {
	uint32_t	rev_id:8;	/* Revision of the CT */
	uint32_t	in_id:24;	/* Initiator Id */
	uint32_t	gs_type:8;	/* Generic service Type */
	uint32_t	gs_sub_type:8;	/* Generic service sub type */
	uint32_t	options:8;	/* options */
	uint32_t	rsvrd:8;	/* reserved */
	uint32_t	cmd_rsp_code:16;/* ct command/response code */
	uint32_t	max_res_size:16;/* maximum/residual size */
	uint32_t	frag_id:8;	/* fragment ID */
	uint32_t	reason_code:8;	/* reason code */
	uint32_t	exp_code:8;	/* explanation code */
	uint32_t	vendor_unq:8;	/* vendor unique */
} ct_hdr_t;

/*
 * defines for the Revision
 */
enum {
	CT_GS3_REVISION = 0x01,
};

/*
 * defines for gs_type
 */
enum {
	CT_GSTYPE_KEYSERVICE	= 0xF7,
	CT_GSTYPE_ALIASSERVICE	= 0xF8,
	CT_GSTYPE_MGMTSERVICE	= 0xFA,
	CT_GSTYPE_TIMESERVICE	= 0xFB,
	CT_GSTYPE_DIRSERVICE	= 0xFC,
};

/*
 * defines for gs_sub_type for gs type directory service
 */
enum {
	CT_GSSUBTYPE_NAMESERVER = 0x02,
};

/*
 * defines for gs_sub_type for gs type management service
 */
enum {
	CT_GSSUBTYPE_CFGSERVER	= 0x01,
	CT_GSSUBTYPE_UNZONED_NS = 0x02,
	CT_GSSUBTYPE_ZONESERVER = 0x03,
	CT_GSSUBTYPE_LOCKSERVER = 0x04,
	CT_GSSUBTYPE_HBA_MGMTSERVER = 0x10,	/* for FDMI */
};

/*
 * defines for CT response code field
 */
enum {
	CT_RSP_REJECT = 0x8001,
	CT_RSP_ACCEPT = 0x8002,
};

/*
 * defintions for CT reason code
 */
enum {
	CT_RSN_INV_CMD		= 0x01,
	CT_RSN_INV_VER		= 0x02,
	CT_RSN_LOGIC_ERR	= 0x03,
	CT_RSN_INV_SIZE		= 0x04,
	CT_RSN_LOGICAL_BUSY	= 0x05,
	CT_RSN_PROTO_ERR	= 0x07,
	CT_RSN_UNABLE_TO_PERF	= 0x09,
	CT_RSN_NOT_SUPP			= 0x0B,
	CT_RSN_SERVER_NOT_AVBL  = 0x0D,
	CT_RSN_SESSION_COULD_NOT_BE_ESTBD = 0x0E,
	CT_RSN_VENDOR_SPECIFIC  = 0xFF,
	
};

/*
 * definitions for explanations code for Name server
 */
enum {
	CT_NS_EXP_NOADDITIONAL	= 0x00,
	CT_NS_EXP_ID_NOT_REG	= 0x01,
	CT_NS_EXP_PN_NOT_REG	= 0x02,
	CT_NS_EXP_NN_NOT_REG	= 0x03,
	CT_NS_EXP_CS_NOT_REG	= 0x04,
	CT_NS_EXP_IPN_NOT_REG	= 0x05,
	CT_NS_EXP_IPA_NOT_REG	= 0x06,
	CT_NS_EXP_FT_NOT_REG	= 0x07,
	CT_NS_EXP_SPN_NOT_REG	= 0x08,
	CT_NS_EXP_SNN_NOT_REG	= 0x09,
	CT_NS_EXP_PT_NOT_REG	= 0x0A,
	CT_NS_EXP_IPP_NOT_REG	= 0x0B,
	CT_NS_EXP_FPN_NOT_REG	= 0x0C,
	CT_NS_EXP_HA_NOT_REG	= 0x0D,
	CT_NS_EXP_FD_NOT_REG	= 0x0E,
	CT_NS_EXP_FF_NOT_REG	= 0x0F,
	CT_NS_EXP_ACCESSDENIED	= 0x10,
	CT_NS_EXP_UNACCEPTABLE_ID = 0x11,
	CT_NS_EXP_DATABASEEMPTY			= 0x12,
	CT_NS_EXP_NOT_REG_IN_SCOPE 		= 0x13,
	CT_NS_EXP_DOM_ID_NOT_PRESENT 	= 0x14,
	CT_NS_EXP_PORT_NUM_NOT_PRESENT  = 0x15,
	CT_NS_EXP_NO_DEVICE_ATTACHED 	= 0x16
};

/*
 * defintions for the explanation code for all servers
 */
enum {
	CT_EXP_AUTH_EXCEPTION			= 0xF1,
	CT_EXP_DB_FULL					= 0xF2,
	CT_EXP_DB_EMPTY					= 0xF3,
	CT_EXP_PROCESSING_REQ			= 0xF4,
	CT_EXP_UNABLE_TO_VERIFY_CONN	= 0xF5,
	CT_EXP_DEVICES_NOT_IN_CMN_ZONE  = 0xF6
};

/*
 * Command codes for Name server
 */
enum {
	GS_GID_PN	= 0x0121,	/* Get Id on port name */
	GS_GPN_ID	= 0x0112,	/* Get port name on ID */
	GS_GNN_ID	= 0x0113,	/* Get node name on ID */
	GS_GID_FT	= 0x0171,	/* Get Id on FC4 type */
	GS_GSPN_ID	= 0x0118,	/* Get symbolic PN on ID */
	GS_RFT_ID	= 0x0217,	/* Register fc4type on ID */
	GS_RSPN_ID	= 0x0218,	/* Register symbolic PN on ID */
	GS_RPN_ID	= 0x0212,	/* Register port name */
	GS_RNN_ID	= 0x0213,	/* Register node name */
	GS_RCS_ID	= 0x0214,	/* Register class of service */
	GS_RPT_ID	= 0x021A,	/* Register port type */
	GS_GA_NXT	= 0x0100,	/* Get all next */
	GS_RFF_ID	= 0x021F,	/* Register FC4 Feature		*/
};

typedef struct {
	uint32_t	rsvd:8;
	uint32_t	dap:24;	/* port identifier */
} fcgs_gpnid_req_t, fcgs_gnnid_req_t, fcgs_gspnid_req_t;

typedef struct {
	wwn_t	port_name;	/* port wwn */
} fcgs_gidpn_req_t;

typedef struct {
	uint32_t	rsvd:8;
	uint32_t	dap:24;	/* port identifier */
} fcgs_gidpn_resp_t;

/**
 * RFT_ID
 */
typedef struct {
	uint32_t	rsvd:8;
	uint32_t	dap:24;		/* port identifier */
	uint32_t	fc4_type[8];	/* fc4 types */
} fcgs_rftid_req_t;

/**
 * RFF_ID : Register FC4 features.
 */

#define FC_GS_FCP_FC4_FEATURE_INITIATOR  0x02
#define FC_GS_FCP_FC4_FEATURE_TARGET	 0x01

typedef struct {
    uint32_t    rsvd          :8;
    uint32_t    dap        	  :24;		/* port identifier	*/
    uint32_t    rsvd1         :16;
    uint32_t    fc4_type      :8;		/* corresponding FC4 Type */
    uint32_t    fc4ftr_bits   :8;		/* fc4 feature bits	*/
} fcgs_rffid_req_t;

/**
 * GID_FT Request
 */
typedef struct {
	uint8_t	reserved;
	uint8_t	domain_id;	/* domain, 0 - all fabric */
	uint8_t	area_id;	/* area, 0 - whole domain */
	uint8_t	fc4_type;	/* FC_TYPE_FCP for SCSI devices */
} fcgs_gidft_req_t;		/* GID_FT Request */

/**
 * GID_FT Response
 */
typedef struct {
	uint8_t		last:1;	/* last port identifier flag */
	uint8_t		reserved:7;
	uint32_t	pid:24;	/* port identifier */
} fcgs_gidft_resp_t;		/* GID_FT Response */

/**
 * RSPN_ID
 */
typedef struct {
	uint32_t	rsvd:8;
	uint32_t	dap:24;		/* port identifier */
	uint8_t		spn_len;	/* symbolic port name length */
	uint8_t		spn[256];	/* symbolic port name */
} fcgs_rspnid_req_t;

/**
 * RPN_ID
 */
typedef struct {
	uint32_t	rsvd:8;
	uint32_t	port_id:24;
	wwn_t		port_name;
} fcgs_rpnid_req_t;

/**
 * RNN_ID
 */
typedef struct {
	uint32_t	rsvd:8;
	uint32_t	port_id:24;
	wwn_t		node_name;
} fcgs_rnnid_req_t;

/**
 * RCS_ID
 */
typedef struct {
	uint32_t	rsvd:8;
	uint32_t	port_id:24;
	uint32_t	cos;
} fcgs_rcsid_req_t;

/**
 * RPT_ID
 */
typedef struct {
	uint32_t	rsvd:8;
	uint32_t	port_id:24;
	uint32_t	port_type:8;
	uint32_t	rsvd1:24;
} fcgs_rptid_req_t;

/**
 * GA_NXT Request
 */
typedef struct {
	uint32_t	rsvd:8;
	uint32_t	port_id:24;
} fcgs_ganxt_req_t;

/**
 * GA_NXT Response
 */
typedef struct {
	uint32_t	port_type:8;	/* Port Type */
	uint32_t	port_id:24;	/* Port Identifier */
	wwn_t		port_name;	/* Port Name */
	uint8_t		spn_len;	/* Length of Symbolic Port Name */
	char		spn[255];	/* Symbolic Port Name */
	wwn_t		node_name;	/* Node Name */
	uint8_t		snn_len;	/* Length of Symbolic Node Name */
	char		snn[255];	/* Symbolic Node Name */
	uint8_t		ipa[8];		/* Initial Process Associator */
	uint8_t		ip[16];		/* IP Address */
	uint32_t	cos;		/* Class of Service */
	uint32_t	fc4types[8];	/* FC-4 TYPEs */
	wwn_t		fabric_port_name;
					/* Fabric Port Name */
	uint32_t	rsvd:8;		/* Reserved */
	uint32_t	hard_addr:24;	/* Hard Address */
} fcgs_ganxt_rsp_t;

/*
 * Fabric Config Server
 */

/*
 * Command codes for Fabric Configuration Server
 */
enum {
	GS_FC_TRACE_CMD	= 0x0400,	/* GS FC Trace Route */
	GS_FC_PING_CMD	= 0x0401,	/* GS FC Ping */
};

/*
 * Source or Destination Port Tags.
 */
enum {
	GS_FTRACE_TAG_NPORT_ID		= 1,
	GS_FTRACE_TAG_NPORT_NAME	= 2,
};

/*
* Port Value : Could be a Port id or wwn
 */
typedef union {
	uint32_t	nport_id;
	wwn_t		nport_wwn;
} fcgs_port_val_t;

#define GS_FTRACE_MAX_HOP_COUNT	20
#define GS_FTRACE_REVISION	1

/*
 * Ftrace Related Structures.
 */

/*
 * STR (Switch Trace) Reject Reason Codes. From FC-SW.
 */
enum {
	GS_FTRACE_STR_CMD_COMPLETED_SUCC	= 0,
	GS_FTRACE_STR_CMD_NOT_SUPP_IN_NEXT_SWITCH,
	GS_FTRACE_STR_NO_RESP_FROM_NEXT_SWITCH,
	GS_FTRACE_STR_MAX_HOP_CNT_REACHED,
	GS_FTRACE_STR_SRC_PORT_NOT_FOUND,
	GS_FTRACE_STR_DST_PORT_NOT_FOUND,
	GS_FTRACE_STR_DEVICES_NOT_IN_COMMON_ZONE,
	GS_FTRACE_STR_NO_ROUTE_BW_PORTS,
	GS_FTRACE_STR_NO_ADDL_EXPLN,
	GS_FTRACE_STR_FABRIC_BUSY,
	GS_FTRACE_STR_FABRIC_BUILD_IN_PROGRESS,
	GS_FTRACE_STR_VENDOR_SPECIFIC_ERR_START = 0xf0,
	GS_FTRACE_STR_VENDOR_SPECIFIC_ERR_END = 0xff,
};

/*
 * Ftrace Request
 */
typedef struct {
	uint32_t	revision;
	uint16_t	src_port_tag;	/* Source Port tag */
	uint16_t	src_port_len;	/* Source Port len */
	fcgs_port_val_t src_port_val;	/* Source Port value */
	uint16_t	dst_port_tag;	/* Destination Port tag */
	uint16_t	dst_port_len;	/* Destination Port len */
	fcgs_port_val_t dst_port_val;	/* Destination Port value */
	uint32_t	token;
	uint8_t		vendor_id[8];	/* T10 Vendor Identifier */
	uint8_t		vendor_info[8];	/* Vendor specific Info */
	uint32_t	max_hop_cnt;	/* Max Hop Count */
} fcgs_ftrace_req_t;

/*
 * Path info structure
 */
typedef struct {
	wwn_t		switch_name;		/* Switch WWN */
	uint32_t	domain_id;
	wwn_t		ingress_port_name;	/* Ingress ports wwn */
	uint32_t	ingress_phys_port_num;	/* Ingress ports physical port
						 				 * number
										 */
	wwn_t		egress_port_name;	/* Ingress ports wwn */
	uint32_t	egress_phys_port_num;	/* Ingress ports physical port
										 * number
										 */
} fcgs_ftrace_path_info;

/*
 * Ftrace Acc Response
 */
typedef struct {
	uint32_t	revision;
	uint32_t	token;
	uint8_t		vendor_id[8];		/* T10 Vendor Identifier */
	uint8_t		vendor_info[8];		/* Vendor specific Info */
	uint32_t	str_rej_reason_code;	/* STR Reject Reason Code */
	uint32_t	num_path_info_entries;	/* No. of path info entries */
	/*
	 * path info entry/entries.
	 */
	fcgs_ftrace_path_info path_info[1];

} fcgs_ftrace_resp_t;

/*
* Fabric Config Server : FCPing
 */

/*
 * FC Ping Request
 */
typedef struct {
	uint32_t	revision;
	uint16_t	port_tag;
	uint16_t	port_len;	/* Port len */
	fcgs_port_val_t port_val;	/* Port value */
	uint32_t	token;
} fcgs_fcping_req_t;

/*
 * FC Ping Response
 */
typedef struct {
	uint32_t	token;
} fcgs_fcping_resp_t;

/*
 * Command codes for zone server query.
 */
enum {
	ZS_GZME = 0x0124,	/* Get zone member extended */
};

/*
 * ZS GZME request
 */
#define ZS_GZME_ZNAMELEN	32
typedef struct {
	uint8_t	znamelen;
	uint8_t	rsvd[3];
	uint8_t	zname[ZS_GZME_ZNAMELEN];
} zs_gzme_req_t;

typedef enum {
	ZS_MBR_TYPE_PWWN	= 1,
	ZS_MBR_TYPE_DOMPORT	= 2,
	ZS_MBR_TYPE_PORTID	= 3,
	ZS_MBR_TYPE_NWWN	= 4,
} zs_mbr_type_t;

typedef struct {
	uint8_t	mbr_type;
	uint8_t	rsvd[3];
	wwn_t	wwn;
} zs_mbr_wwn_t;

typedef struct {
	uint32_t	nmbrs;	/* !< number of zone members */
	zs_mbr_wwn_t	mbr[1];
} zs_query_resp_t;

#pragma pack()

#endif
