/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __FC_H__
#define __FC_H__

#include <protocol/types.h>

#pragma pack(1)

/**
 * crossbow fds header
 * @todo remove from generic protocol defines
 */
typedef struct {
#ifdef __BIGENDIAN
	uint32_t	rsvd_1:5,
			eof:3,
			rsvd_2:4,
			len:12,
			rsvd_3:1,
			fid_perr:1,
			frm_perr:1,
			maxsz:1,
			rsvd_4:1,
			inv_eof:1,
			code_vio:1,
			crc_err:1;
	uint32_t	rsvd_5:3,
			shim_pres:1,	/* shim present */
			sof:4,		/* SOF type */
			lport:6,	/* receiving logical port */
			arbvc:2,	/* ARB(VC) validation */
			vc:5,		/* vc number of received frame */
			vf_tagged:1,
			pport:6,	/* receiving physical port */
			rsvd_6:2,
			opn_err:1,
			rsvd_7:1;
#else
	uint32_t        eof:3,		/* EOF */
			rsvd_1:29;
	uint32_t        sof:4,		/* SOF */
			rsvd_2:28;
#endif
} fcif_t;

/*
 * SOF delimiter encodings for crossbow
 */
enum {
	FC_SOFf	= 0,
	FC_SOFc1 = 1,
	FC_SOFi1 = 2,
	FC_SOFn1 = 3,
	FC_SOFi2 = 4,
	FC_SOFn2 = 5,
	FC_SOFi3 = 6,
	FC_SOFn3 = 7,
	FC_SOFc4 = 8,
	FC_SOFi4 = 9,
	FC_SOFn4 = 10,
};

/*
 * EOF delimiter encodings for crossbow
 */
enum {
	FC_EOFt		= 0,
	FC_EOFdt	= 1,
	FC_EOFa		= 2,
	FC_EOFn		= 3,
	FC_EOFdti	= 4,
	FC_EOFni	= 5,
	FC_EOFrt	= 6,
	FC_EOFrti	= 7,
};

/*
 * Fibre Channel Header Structure (FCHS) definition
 */
typedef struct {
	fcif_t          fcif;
#ifdef __BIGENDIAN
	uint32_t        routing:4;	/* routing bits */
	uint32_t        cat_info:4;	/* category info */
#else
	uint32_t        cat_info:4;	/* category info */
	uint32_t        routing:4;	/* routing bits */
#endif
	uint32_t        d_id:24;	/* destination identifier */

	uint32_t        cs_ctl:8;	/* class specific control */
	uint32_t        s_id:24;	/* source identifier */

	uint32_t        type:8;		/* data structure type */
	uint32_t        f_ctl:24;	/* initial frame control */

	uint8_t         seq_id;		/* sequence identifier */
	uint8_t         df_ctl;		/* data field control */
	uint16_t        seq_cnt;	/* sequence count */

	uint16_t        ox_id;		/* originator exchange ID */
	uint16_t        rx_id;		/* responder exchange ID */

	uint32_t        ro;		/* relative offset */
} fchs_t;

#define FC_SEQ_ID_MAX		256

/*
 * routing bit definitions
 */
enum {
	FC_RTG_FC4_DEV_DATA	= 0x0,	/* FC-4 Device Data */
	FC_RTG_EXT_LINK		= 0x2,	/* Extended Link Data */
	FC_RTG_FC4_LINK_DATA	= 0x3,	/* FC-4 Link Data */
	FC_RTG_VIDEO_DATA	= 0x4,	/* Video Data */
	FC_RTG_EXT_HDR		= 0x5,	/* VFT, IFR or Encapsuled */
	FC_RTG_BASIC_LINK	= 0x8,	/* Basic Link data */
	FC_RTG_LINK_CTRL	= 0xC,	/* Link Control */
};

/*
 * information category for extended link data and FC-4 Link Data
 */
enum {
	FC_CAT_LD_REQUEST	= 0x2,	/* Request */
	FC_CAT_LD_REPLY		= 0x3,	/* Reply */
	FC_CAT_LD_DIAG		= 0xF,	/* for DIAG use only */
};

/*
 * information category for extended headers (VFT, IFR or encapsulation)
 */
enum {
	FC_CAT_VFT_HDR = 0x0,	/* Virtual fabric tagging header */
	FC_CAT_IFR_HDR = 0x1,	/* Inter-Fabric routing header */
	FC_CAT_ENC_HDR = 0x2,	/* Encapsulation header */
};

/*
 * information category for FC-4 device data
 */
enum {
	FC_CAT_UNCATEG_INFO	= 0x0,	/* Uncategorized information */
	FC_CAT_SOLICIT_DATA	= 0x1,	/* Solicited Data */
	FC_CAT_UNSOLICIT_CTRL	= 0x2,	/* Unsolicited Control */
	FC_CAT_SOLICIT_CTRL	= 0x3,	/* Solicited Control */
	FC_CAT_UNSOLICIT_DATA	= 0x4,	/* Unsolicited Data */
	FC_CAT_DATA_DESC	= 0x5,	/* Data Descriptor */
	FC_CAT_UNSOLICIT_CMD	= 0x6,	/* Unsolicited Command */
	FC_CAT_CMD_STATUS	= 0x7,	/* Command Status */
};

/*
 * information category for Link Control
 */
enum {
	FC_CAT_ACK_1		= 0x00,
	FC_CAT_ACK_0_N		= 0x01,
	FC_CAT_P_RJT		= 0x02,
	FC_CAT_F_RJT		= 0x03,
	FC_CAT_P_BSY		= 0x04,
	FC_CAT_F_BSY_DATA	= 0x05,
	FC_CAT_F_BSY_LINK_CTL	= 0x06,
	FC_CAT_F_LCR		= 0x07,
	FC_CAT_NTY		= 0x08,
	FC_CAT_END		= 0x09,
};

/*
 * Type Field Definitions. FC-PH Section 18.5 pg. 165
 */
enum {
	FC_TYPE_BLS		= 0x0,	/* Basic Link Service */
	FC_TYPE_ELS		= 0x1,	/* Extended Link Service */
	FC_TYPE_IP		= 0x5,	/* IP */
	FC_TYPE_FCP		= 0x8,	/* SCSI-FCP */
	FC_TYPE_GPP		= 0x9,	/* SCSI_GPP */
	FC_TYPE_SERVICES	= 0x20,	/* Fibre Channel Services */
	FC_TYPE_FC_FSS		= 0x22,	/* Fabric Switch Services */
	FC_TYPE_FC_AL		= 0x23,	/* FC-AL */
	FC_TYPE_FC_SNMP		= 0x24,	/* FC-SNMP */
	FC_TYPE_MAX		= 256,	/* 256 FC-4 types */
};

typedef struct {
	uint8_t         bits[FC_TYPE_MAX / 8];
} fc_fc4types_t;

/*
 * Frame Control Definitions. FC-PH Table-45. pg. 168
 */
enum {
	FCTL_EC_ORIG = 0x000000,	/* exchange originator */
	FCTL_EC_RESP = 0x800000,	/* exchange responder */
	FCTL_SEQ_INI = 0x000000,	/* sequence initiator */
	FCTL_SEQ_REC = 0x400000,	/* sequence recipient */
	FCTL_FS_EXCH = 0x200000,	/* first sequence of xchg */
	FCTL_LS_EXCH = 0x100000,	/* last sequence of xchg */
	FCTL_END_SEQ = 0x080000,	/* last frame of sequence */
	FCTL_SI_XFER = 0x010000,	/* seq initiative transfer */
	FCTL_RO_PRESENT = 0x000008,	/* relative offset present */
	FCTL_FILLBYTE_MASK = 0x000003	/* , fill byte mask */
};

/*
 * Fabric Well Known Addresses
 */
enum {
	FC_MIN_WELL_KNOWN_ADDR		= 0xFFFFF0,
	FC_DOMAIN_CONTROLLER_MASK 	= 0xFFFC00,
	FC_ALIAS_SERVER			= 0xFFFFF8,
	FC_MGMT_SERVER			= 0xFFFFFA,
	FC_TIME_SERVER			= 0xFFFFFB,
	FC_NAME_SERVER			= 0xFFFFFC,
	FC_FABRIC_CONTROLLER		= 0xFFFFFD,
	FC_FABRIC_PORT			= 0xFFFFFE,
	FC_BROADCAST_SERVER		= 0xFFFFFF
};

/*
 * domain/area/port defines
 */
#define FC_DOMAIN_MASK  0xFF0000
#define FC_DOMAIN_SHIFT 16
#define FC_AREA_MASK    0x00FF00
#define FC_AREA_SHIFT   8
#define FC_PORT_MASK    0x0000FF
#define FC_PORT_SHIFT   0

#define FC_GET_DOMAIN(p)	(((p) & FC_DOMAIN_MASK) >> FC_DOMAIN_SHIFT)
#define FC_GET_AREA(p)		(((p) & FC_AREA_MASK) >> FC_AREA_SHIFT)
#define FC_GET_PORT(p)		(((p) & FC_PORT_MASK) >> FC_PORT_SHIFT)

enum {
	FC_RXID_ANY = 0xFFFFU,
};

/*
 * generic ELS command
 */
typedef struct {
uint32_t        els_code:8;	/* ELS Command Code */
uint32_t        reserved:24;
} fc_els_cmd_t;

/*
 * ELS Command Codes. FC-PH Table-75. pg. 223
 */
enum {
	FC_ELS_LS_RJT = 0x1,	/* Link Service Reject. */
	FC_ELS_ACC = 0x02,	/* Accept */
	FC_ELS_PLOGI = 0x03,	/* N_Port Login. */
	FC_ELS_FLOGI = 0x04,	/* F_Port Login. */
	FC_ELS_LOGO = 0x05,	/* Logout. */
	FC_ELS_ABTX = 0x06,	/* Abort Exchange */
	FC_ELS_RES = 0x08,	/* Read Exchange status */
	FC_ELS_RSS = 0x09,	/* Read sequence status block */
	FC_ELS_RSI = 0x0A,	/* Request Sequence Initiative */
	FC_ELS_ESTC = 0x0C,	/* Estimate Credit. */
	FC_ELS_RTV = 0x0E,	/* Read Timeout Value. */
	FC_ELS_RLS = 0x0F,	/* Read Link Status. */
	FC_ELS_ECHO = 0x10,	/* Echo */
	FC_ELS_TEST = 0x11,	/* Test */
	FC_ELS_RRQ = 0x12,	/* Reinstate Recovery Qualifier. */
	FC_ELS_REC = 0x13,	/* Add this for TAPE support in FCR */
	FC_ELS_PRLI = 0x20,	/* Process Login */
	FC_ELS_PRLO = 0x21,	/* Process Logout. */
	FC_ELS_SCN = 0x22,	/* State Change Notification. */
	FC_ELS_TPRLO = 0x24,	/* Third Party Process Logout. */
	FC_ELS_PDISC = 0x50,	/* Discover N_Port Parameters. */
	FC_ELS_FDISC = 0x51,	/* Discover F_Port Parameters. */
	FC_ELS_ADISC = 0x52,	/* Discover Address. */
	FC_ELS_FAN = 0x60,	/* Fabric Address Notification */
	FC_ELS_RSCN = 0x61,	/* Reg State Change Notification */
	FC_ELS_SCR = 0x62,	/* State Change Registration. */
	FC_ELS_RTIN = 0x77,	/* Mangement server request */
	FC_ELS_RNID = 0x78,	/* Mangement server request */
	FC_ELS_RLIR = 0x79,	/* Registered Link Incident Record */

	FC_ELS_RPSC = 0x7D,	/* Report Port Speed Capabilities */
	FC_ELS_QSA = 0x7E,	/* Query Security Attributes. Ref FC-SP */
	FC_ELS_E2E_LBEACON = 0x81,
				/* End-to-End Link Beacon */
	FC_ELS_AUTH = 0x90,	/* Authentication. Ref FC-SP */
	FC_ELS_RFCN = 0x97,	/* Request Fabric Change Notification. Ref
				 *FC-SP */

};

/*
 *  Version numbers for FC-PH standards,
 *  used in login to indicate what port
 *  supports. See FC-PH-X table 158.
 */
enum {
	FC_PH_VER_4_3 = 0x09,
	FC_PH_VER_PH_3 = 0x20,
};

/*
 * PDU size defines
 */
enum {
	FC_MIN_PDUSZ = 512,
	FC_MAX_PDUSZ = 2112,
};

/*
 * N_Port PLOGI Common Service Parameters.
 * FC-PH-x. Figure-76. pg. 308.
 */
typedef struct {
	uint8_t         verhi;	/* FC-PH high version */
	uint8_t         verlo;	/* FC-PH low version */
	uint16_t        bbcred;	/* BB_Credit */

#ifdef __BIGENDIAN
	uint8_t         ciro:1,		/* continuously increasing RO */
			rro:1,		/* random relative offset */
			npiv_supp:1,	/* NPIV supported */
			port_type:1,	/* N_Port/F_port */
			altbbcred:1,	/* alternate BB_Credit */
			resolution:1,	/* ms/ns ED_TOV resolution */
			vvl_info:1,	/* VVL Info included */
			reserved1:1;

	uint8_t         hg_supp:1,
			query_dbc:1,
			security:1,
			sync_cap:1,
			r_t_tov:1,
			dh_dup_supp:1,
			cisc:1,		/* continuously increasing seq count */
			payload:1;
#else
	uint8_t         reserved2:2,
			resolution:1,	/* ms/ns ED_TOV resolution */
			altbbcred:1,	/* alternate BB_Credit */
			port_type:1,	/* N_Port/F_port */
			npiv_supp:1,	/* NPIV supported */
			rro:1,		/* random relative offset */
			ciro:1;		/* continuously increasing RO */

	uint8_t         payload:1,
			cisc:1,		/* continuously increasing seq count */
			dh_dup_supp:1,
			r_t_tov:1,
			sync_cap:1,
			security:1,
			query_dbc:1,
			hg_supp:1;
#endif

	uint16_t        rxsz;		/* recieve data_field size */

	uint16_t        conseq;
	uint16_t        ro_bitmap;

	uint32_t        e_d_tov;
} fc_plogi_csp_t;

/*
 * N_Port PLOGI Class Specific Parameters.
 * FC-PH-x. Figure 78. pg. 318.
 */
typedef struct {
#ifdef __BIGENDIAN
	uint32_t        class_valid:1;
	uint32_t        intermix:1;	/* class intermix supported if set =1.
					 * valid only for class1. Reserved for
					 * class2 & class3
					 */
	uint32_t        reserved1:2;
	uint32_t        sequential:1;
	uint32_t        reserved2:3;
#else
	uint32_t        reserved2:3;
	uint32_t        sequential:1;
	uint32_t        reserved1:2;
	uint32_t        intermix:1;	/* class intermix supported if set =1.
					 * valid only for class1. Reserved for
					 * class2 & class3
					 */
	uint32_t        class_valid:1;
#endif

	uint32_t        reserved3:24;

	uint32_t        reserved4:16;
	uint32_t        rxsz:16;	/* Receive data_field size */

	uint32_t        reserved5:8;
	uint32_t        conseq:8;
	uint32_t        e2e_credit:16;	/* end to end credit */

	uint32_t        reserved7:8;
	uint32_t        ospx:8;
	uint32_t        reserved8:16;
} fc_plogi_clp_t;

/*
 * PLOGI els command and reply payload
 */
typedef struct {
	fc_els_cmd_t    els_cmd;	/* ELS command code */
	fc_plogi_csp_t  csp;		/* common service params */
	wwn_t           port_name;
	wwn_t           node_name;
	fc_plogi_clp_t  class1;		/* class 1 service parameters */
	fc_plogi_clp_t  class2;		/* class 2 service parameters */
	fc_plogi_clp_t  class3;		/* class 3 service parameters */
	fc_plogi_clp_t  class4;		/* class 4 service parameters */
	uint8_t         vvl[16];	/* vendor version level */
} fc_plogi_t, fc_flogi_t;

/*
 * LOGO els command payload
 */
typedef struct {
	fc_els_cmd_t    els_cmd;	/* ELS command code */
	uint32_t        res1:8;
	uint32_t        nport_id:24;	/* N_Port identifier of source */
	wwn_t           orig_port_name;	/* Port name of the LOGO originator */
} fc_logo_t;

/*
 * ADISC els command payload
 */
typedef struct {
	fc_els_cmd_t    els_cmd;	/* ELS command code */
	uint32_t        res1:8;
	uint32_t        orig_HA:24;	/* originator hard address */
	wwn_t           orig_port_name;	/* originator port name */
	wwn_t           orig_node_name;	/* originator node name */
	uint32_t        res2:8;
	uint32_t        nport_id:24;	/* originator NPortID */
} fc_adisc_t;

/*
 * Exchange status block
 */
typedef struct {
	uint32_t        oxid:16;
	uint32_t        rxid:16;
	uint32_t        res1:8;
	uint32_t        orig_np:24;	/* originator NPortID */
	uint32_t        res2:8;
	uint32_t        resp_np:24;	/* responder NPortID */
	uint32_t        es_bits;
	uint32_t        res3;
	/*
	 * un modified section of the fields
	 */
} fc_exch_status_blk_t;

/*
 * RES els command payload
 */
typedef struct {
	fc_els_cmd_t    els_cmd;	/* ELS command code */
	uint32_t        res1:8;
	uint32_t        nport_id:24;	/* N_Port identifier of source */
	uint32_t        oxid:16;
	uint32_t        rxid:16;
	uint8_t         assoc_hdr[32];
} fc_res_t;

/*
 * RES els accept payload
 */
typedef struct {
	fc_els_cmd_t		els_cmd;	/* ELS command code */
	fc_exch_status_blk_t	fc_exch_blk;	/* Exchange status block */
} fc_res_acc_t;

/*
 * REC els command payload
 */
typedef struct _fc_rec_t {
	fc_els_cmd_t    els_cmd;	/* ELS command code */
	uint32_t        res1:8;
	uint32_t        nport_id:24;	/* N_Port identifier of source */
	uint32_t        oxid:16;
	uint32_t        rxid:16;
} fc_rec_t;

#define FC_REC_ESB_OWN_RSP	 	0x80000000	/* responder owns 	*/
#define FC_REC_ESB_SI		 	0x40000000	/* SI is owned 		*/
#define FC_REC_ESB_COMP		 	0x20000000	/* exchange is complete	*/
#define FC_REC_ESB_ENDCOND_ABN	0x10000000	/* abnormal ending 	*/
#define FC_REC_ESB_RQACT	 	0x04000000	/* recovery qual active	*/
#define FC_REC_ESB_ERRP_MSK	 	0x03000000
#define FC_REC_ESB_OXID_INV	 	0x00800000	/* invalid OXID		*/
#define FC_REC_ESB_RXID_INV	 	0x00400000	/* invalid RXID		*/
#define FC_REC_ESB_PRIO_INUSE	0x00200000

/*
 * REC els accept payload
 */
typedef struct _fc_rec_acc_t {
	fc_els_cmd_t    els_cmd;	/* ELS command code */
	uint32_t        oxid:16;
	uint32_t        rxid:16;
	uint32_t        res1:8;
	uint32_t        orig_id:24;	/* N_Port id of exchange originator */
	uint32_t        res2:8;
	uint32_t        resp_id:24;	/* N_Port id of exchange responder */
	uint32_t        count;		/* data transfer count */
	uint32_t        e_stat;		/* exchange status */
} fc_rec_acc_t;

/*
 * RSI els payload
 */
typedef struct _fc_rsi_t {
	fc_els_cmd_t    els_cmd;
	uint32_t        res1:8;
	uint32_t        orig_sid:24;
	uint32_t        oxid:16;
	uint32_t        rxid:16;
} fc_rsi_t;

/*
 * structure for PRLI paramater pages, both request & response
 * see FC-PH-X table 113 & 115 for explanation also FCP table 8
 */
typedef struct {
	uint32_t        reserved: 16;
#ifdef __BIGENDIAN
	uint32_t        reserved1: 5;
	uint32_t        rec_support : 1;
	uint32_t        task_retry_id : 1;
	uint32_t        retry : 1;

	uint32_t        confirm : 1;
	uint32_t        doverlay:1;
	uint32_t        initiator:1;
	uint32_t        target:1;
	uint32_t        cdmix:1;
	uint32_t        drmix:1;
	uint32_t        rxrdisab:1;
	uint32_t        wxrdisab:1;
#else
	uint32_t        retry : 1;
	uint32_t        task_retry_id : 1;
	uint32_t        rec_support : 1;
	uint32_t        reserved1: 5;

	uint32_t        wxrdisab:1;
	uint32_t        rxrdisab:1;
	uint32_t        drmix:1;
	uint32_t        cdmix:1;
	uint32_t        target:1;
	uint32_t        initiator:1;
	uint32_t        doverlay:1;
	uint32_t        confirm : 1;
#endif
} fc_prli_params_t;

/*
 * valid values for rspcode in PRLI ACC payload
 */
enum {
	FC_PRLI_ACC_XQTD = 0x1,		/* request executed */
	FC_PRLI_ACC_PREDEF_IMG = 0x5,	/* predefined image - no prli needed */
};

typedef struct {
	uint32_t        type:8;
	uint32_t        codext:8;
#ifdef __BIGENDIAN
	uint32_t        origprocasv:1;
	uint32_t        rsppav:1;
	uint32_t        imagepair:1;
	uint32_t        reserved1:1;
	uint32_t        rspcode:4;
#else
	uint32_t        rspcode:4;
	uint32_t        reserved1:1;
	uint32_t        imagepair:1;
	uint32_t        rsppav:1;
	uint32_t        origprocasv:1;
#endif
	uint32_t        reserved2:8;

	uint32_t        origprocas;
	uint32_t        rspprocas;
	fc_prli_params_t servparams;
} fc_prli_params_page_t;

/*
 * PRLI request and accept payload, FC-PH-X tables 112 & 114
 */
typedef struct {
	uint32_t        command:8;
	uint32_t        pglen:8;
	uint32_t        pagebytes:16;
	fc_prli_params_page_t parampage;
} fc_prli_t;

/*
 * PRLO logout params page
 */
typedef struct {
	uint32_t        type:8;
	uint32_t        type_ext:8;
#ifdef __BIGENDIAN
	uint32_t        opa_valid:1;	/* originator process associator
					 * valid
					 */
	uint32_t        rpa_valid:1;	/* responder process associator valid */
	uint32_t        res1:14;
#else
	uint32_t        res1:14;
	uint32_t        rpa_valid:1;	/* responder process associator valid */
	uint32_t        opa_valid:1;	/* originator process associator
					 * valid
					 */
#endif
	uint32_t        orig_process_assc;
	uint32_t        resp_process_assc;

	uint32_t        res2;
} fc_prlo_params_page_t;

/*
 * PRLO els command payload
 */
typedef struct {
	uint32_t        	command:8;
	uint32_t        	page_len:8;
	uint32_t        	payload_len:16;
	fc_prlo_params_page_t 	prlo_params[1];
} fc_prlo_t;

/*
 * PRLO Logout response parameter page
 */
typedef struct {
	uint32_t        type:8;
	uint32_t        type_ext:8;

#ifdef __BIGENDIAN
	uint32_t        opa_valid:1;	/* originator process associator
					 * valid
					 */
	uint32_t        rpa_valid:1;	/* responder process associator valid */
	uint32_t        res1:14;
#else
	uint32_t        res1:14;
	uint32_t        rpa_valid:1;	/* responder process associator valid */
	uint32_t        opa_valid:1;	/* originator process associator
					 * valid
					 */
#endif
	uint32_t        orig_process_assc;
	uint32_t        resp_process_assc;

	uint32_t        fc4type_csp;
} fc_prlo_acc_params_page_t;

/*
 * PRLO els command ACC payload
 */
typedef struct {
	uint32_t        command:8;
	uint32_t        page_len:8;
	uint32_t        payload_len:16;
	fc_prlo_acc_params_page_t prlo_acc_params[1];
} fc_prlo_acc_t;

/*
 * SCR els command payload
 */
enum {
	FC_SCR_REG_FUNC_FABRIC_DETECTED = 0x01,
	FC_SCR_REG_FUNC_N_PORT_DETECTED = 0x02,
	FC_SCR_REG_FUNC_FULL = 0x03,
	FC_SCR_REG_FUNC_CLEAR_REG = 0xFF,
};

typedef struct {
	uint32_t command:8;
	uint32_t res:24;
	uint32_t res1:24;
	uint32_t reg_func:8;
} fc_scr_t;

/*
 * Information category for Basic link data
 */
enum {
	FC_CAT_NOP	= 0x0,
	FC_CAT_ABTS	= 0x1,
	FC_CAT_RMC	= 0x2,
	FC_CAT_BA_ACC	= 0x4,
	FC_CAT_BA_RJT	= 0x5,
	FC_CAT_PRMT	= 0x6,
};

/*
 * LS_RJT els reply payload
 */
typedef struct {
	fc_els_cmd_t    els_cmd;		/* ELS command code */
	uint32_t        res1:8;
	uint32_t        reason_code:8;		/* Reason code for reject */
	uint32_t        reason_code_expl:8;	/* Reason code explanation */
	uint32_t        vendor_unique:8;	/* Vendor specific */
} fc_ls_rjt_t;

/*
 * LS_RJT reason codes
 */
enum {
	FC_LS_RJT_RSN_INV_CMD_CODE	= 0x01,
	FC_LS_RJT_RSN_LOGICAL_ERROR	= 0x03,
	FC_LS_RJT_RSN_LOGICAL_BUSY	= 0x05,
	FC_LS_RJT_RSN_PROTOCOL_ERROR	= 0x07,
	FC_LS_RJT_RSN_UNABLE_TO_PERF_CMD = 0x09,
	FC_LS_RJT_RSN_CMD_NOT_SUPP	= 0x0B,
};

/*
 * LS_RJT reason code explanation
 */
enum {
	FC_LS_RJT_EXP_NO_ADDL_INFO		= 0x00,
	FC_LS_RJT_EXP_SPARMS_ERR_OPTIONS	= 0x01,
	FC_LS_RJT_EXP_SPARMS_ERR_INI_CTL	= 0x03,
	FC_LS_RJT_EXP_SPARMS_ERR_REC_CTL	= 0x05,
	FC_LS_RJT_EXP_SPARMS_ERR_RXSZ		= 0x07,
	FC_LS_RJT_EXP_SPARMS_ERR_CONSEQ		= 0x09,
	FC_LS_RJT_EXP_SPARMS_ERR_CREDIT		= 0x0B,
	FC_LS_RJT_EXP_INV_PORT_NAME		= 0x0D,
	FC_LS_RJT_EXP_INV_NODE_FABRIC_NAME	= 0x0E,
	FC_LS_RJT_EXP_INV_CSP			= 0x0F,
	FC_LS_RJT_EXP_INV_ASSOC_HDR		= 0x11,
	FC_LS_RJT_EXP_ASSOC_HDR_REQD		= 0x13,
	FC_LS_RJT_EXP_INV_ORIG_S_ID		= 0x15,
	FC_LS_RJT_EXP_INV_OXID_RXID_COMB	= 0x17,
	FC_LS_RJT_EXP_CMD_ALREADY_IN_PROG	= 0x19,
	FC_LS_RJT_EXP_LOGIN_REQUIRED		= 0x1E,
	FC_LS_RJT_EXP_INVALID_NPORT_ID		= 0x1F,
	FC_LS_RJT_EXP_INSUFF_RES		= 0x29,
	FC_LS_RJT_EXP_CMD_NOT_SUPP		= 0x2C,
	FC_LS_RJT_EXP_INV_PAYLOAD_LEN		= 0x2D,
};

/*
 * RRQ els command payload
 */
typedef struct {
	fc_els_cmd_t    els_cmd;	/* ELS command code */
	uint32_t        res1:8;
	uint32_t        s_id:24;	/* exchange originator S_ID */

	uint32_t        ox_id:16;	/* originator exchange ID */
	uint32_t        rx_id:16;	/* responder exchange ID */

	uint32_t        res2[8];	/* optional association header */
} fc_rrq_t;

/*
 * ABTS BA_ACC reply payload
 */
typedef struct {
	uint32_t        seq_id_valid:8;	/* set to 0x00 for Abort Exchange */
	uint32_t        seq_id:8;	/* invalid for Abort Exchange */
	uint32_t        res2:16;
	uint32_t        ox_id:16;	/* OX_ID from ABTS frame */
	uint32_t        rx_id:16;	/* RX_ID from ABTS frame */
	uint32_t        low_seq_cnt:16;	/* set to 0x0000 for Abort Exchange */
	uint32_t        high_seq_cnt:16;/* set to 0xFFFF for Abort Exchange */
} fc_ba_acc_t;

/*
 * ABTS BA_RJT reject payload
 */
typedef struct {
	uint32_t        res1:8;		/* Reserved */
	uint32_t        reason_code:8;	/* reason code for reject */
	uint32_t        reason_expl:8;	/* reason code explanation */
	uint32_t        vendor_unique:8;/* vendor unique reason code,set to 0 */
} fc_ba_rjt_t;

/*
 * TPRLO logout parameter page
 */
typedef struct {
uint32_t        type:8;
uint32_t        type_ext:8;

#ifdef __BIGENDIAN
	uint32_t        opa_valid:1;
	uint32_t        rpa_valid:1;
	uint32_t        tpo_nport_valid:1;
	uint32_t        global_process_logout:1;
	uint32_t        res1:12;
#else
	uint32_t        res1:12;
	uint32_t        global_process_logout:1;
	uint32_t        tpo_nport_valid:1;
	uint32_t        rpa_valid:1;
	uint32_t        opa_valid:1;
#endif

	uint32_t        orig_process_assc;
	uint32_t        resp_process_assc;

	uint32_t        res2:8;
	uint32_t        tpo_nport_id;
} fc_tprlo_params_page_t;

/*
 * TPRLO ELS command payload
 */
typedef struct {
	uint32_t        command:8;
	uint32_t        page_len:8;
	uint32_t        payload_len:16;

	fc_tprlo_params_page_t tprlo_params[1];
} fc_tprlo_t;

typedef enum {
	FC_GLOBAL_LOGO = 1,
	FC_TPR_LOGO
} fc_tprlo_type_t;

/*
 * TPRLO els command ACC payload
 */
typedef struct {
	uint32_t	command:8;
	uint32_t	page_len:8;
	uint32_t	payload_len:16;
	fc_prlo_acc_params_page_t tprlo_acc_params[1];
} fc_tprlo_acc_t;

/*
 * RSCN els command req payload
 */
#define FC_RSCN_PGLEN	0x4

typedef enum {
	FC_RSCN_FORMAT_PORTID	= 0x0,
	FC_RSCN_FORMAT_AREA	= 0x1,
	FC_RSCN_FORMAT_DOMAIN	= 0x2,
	FC_RSCN_FORMAT_FABRIC	= 0x3,
} fc_rscn_format_t;

typedef struct {
	uint32_t        format:2;
	uint32_t        qualifier:4;
	uint32_t        resvd:2;
	uint32_t        portid:24;
} fc_rscn_event_t;

typedef struct {
	uint8_t         command;
	uint8_t         pagelen;
	uint16_t        payldlen;
	fc_rscn_event_t event[1];
} fc_rscn_pl_t;

/*
 * ECHO els command req payload
 */
typedef struct _fc_echo_t {
	fc_els_cmd_t    els_cmd;
} fc_echo_t;

/*
 * RNID els command
 */

#define RNID_NODEID_DATA_FORMAT_COMMON    		 0x00
#define RNID_NODEID_DATA_FORMAT_FCP3        		 0x08
#define RNID_NODEID_DATA_FORMAT_DISCOVERY     		0xDF

#define RNID_ASSOCIATED_TYPE_UNKNOWN                    0x00000001
#define RNID_ASSOCIATED_TYPE_OTHER                      0x00000002
#define RNID_ASSOCIATED_TYPE_HUB                        0x00000003
#define RNID_ASSOCIATED_TYPE_SWITCH                     0x00000004
#define RNID_ASSOCIATED_TYPE_GATEWAY                    0x00000005
#define RNID_ASSOCIATED_TYPE_STORAGE_DEVICE             0x00000009
#define RNID_ASSOCIATED_TYPE_HOST                       0x0000000A
#define RNID_ASSOCIATED_TYPE_STORAGE_SUBSYSTEM          0x0000000B
#define RNID_ASSOCIATED_TYPE_STORAGE_ACCESS_DEVICE      0x0000000E
#define RNID_ASSOCIATED_TYPE_NAS_SERVER                 0x00000011
#define RNID_ASSOCIATED_TYPE_BRIDGE                     0x00000002
#define RNID_ASSOCIATED_TYPE_VIRTUALIZATION_DEVICE      0x00000003
#define RNID_ASSOCIATED_TYPE_MULTI_FUNCTION_DEVICE      0x000000FF

/*
 * RNID els command payload
 */
typedef struct {
	fc_els_cmd_t    els_cmd;
	uint32_t        node_id_data_format:8;
	uint32_t        reserved:24;
} fc_rnid_cmd_t;

/*
 * RNID els response payload
 */

typedef struct {
	wwn_t           port_name;
	wwn_t           node_name;
} fc_rnid_common_id_data_t;

typedef struct {
	uint32_t        vendor_unique[4];
	uint32_t        asso_type;
	uint32_t        phy_port_num;
	uint32_t        num_attached_nodes;
	uint32_t        node_mgmt:8;
	uint32_t        ip_version:8;
	uint32_t        udp_tcp_port_num:16;
	uint32_t        ip_address[4];
	uint32_t        reserved:16;
	uint32_t        vendor_specific:16;
} fc_rnid_general_topology_data_t;

typedef struct {
	fc_els_cmd_t    els_cmd;
	uint32_t        node_id_data_format:8;
	uint32_t        common_id_data_length:8;
	uint32_t        reserved:8;
	uint32_t        specific_id_data_length:8;
	fc_rnid_common_id_data_t common_id_data;
	fc_rnid_general_topology_data_t gen_topology_data;
} fc_rnid_acc_t;

#define RNID_ASSOCIATED_TYPE_UNKNOWN                    0x00000001
#define RNID_ASSOCIATED_TYPE_OTHER                      0x00000002
#define RNID_ASSOCIATED_TYPE_HUB                        0x00000003
#define RNID_ASSOCIATED_TYPE_SWITCH                     0x00000004
#define RNID_ASSOCIATED_TYPE_GATEWAY                    0x00000005
#define RNID_ASSOCIATED_TYPE_STORAGE_DEVICE             0x00000009
#define RNID_ASSOCIATED_TYPE_HOST                       0x0000000A
#define RNID_ASSOCIATED_TYPE_STORAGE_SUBSYSTEM          0x0000000B
#define RNID_ASSOCIATED_TYPE_STORAGE_ACCESS_DEVICE      0x0000000E
#define RNID_ASSOCIATED_TYPE_NAS_SERVER                 0x00000011
#define RNID_ASSOCIATED_TYPE_BRIDGE                     0x00000002
#define RNID_ASSOCIATED_TYPE_VIRTUALIZATION_DEVICE      0x00000003
#define RNID_ASSOCIATED_TYPE_MULTI_FUNCTION_DEVICE      0x000000FF

typedef enum {
	RPSC_SPEED_CAP_1G = 0x8000,
	RPSC_SPEED_CAP_2G = 0x4000,
	RPSC_SPEED_CAP_4G = 0x2000,
	RPSC_SPEED_CAP_10G = 0x1000,
	RPSC_SPEED_CAP_8G = 0x0800,
	RPSC_SPEED_CAP_16G = 0x0400,

	RPSC_SPEED_CAP_UNKNOWN = 0x0001,
} fc_rpsc_speed_cap_t;

typedef enum {
	RPSC_OP_SPEED_1G = 0x8000,
	RPSC_OP_SPEED_2G = 0x4000,
	RPSC_OP_SPEED_4G = 0x2000,
	RPSC_OP_SPEED_10G = 0x1000,
	RPSC_OP_SPEED_8G = 0x0800,
	RPSC_OP_SPEED_16G = 0x0400,

	RPSC_OP_SPEED_NOT_EST = 0x0001,	/*! speed not established */
} fc_rpsc_op_speed_t;

typedef struct {
	uint16_t        port_speed_cap;	/*! see fc_rpsc_speed_cap_t */
	uint16_t        port_op_speed;	/*! see fc_rpsc_op_speed_t */
} fc_rpsc_speed_info_t;

typedef enum {
	LINK_E2E_BEACON_ON = 1,
	LINK_E2E_BEACON_OFF = 2
} link_e2e_beacon_subcmd_t;

typedef enum {
	BEACON_TYPE_NORMAL	= 1,	/*! Normal Beaconing. Green */
	BEACON_TYPE_WARN	= 2,	/*! Warning Beaconing. Yellow/Amber */
	BEACON_TYPE_CRITICAL	= 3	/*! Critical Beaconing. Red */
} beacon_type_t;

typedef struct els_link_e2e_beacon_param {
	uint8_t         beacon_type;	/* Beacon Type. See beacon_type_t */
	uint8_t         beacon_frequency;
					/* Beacon frequency. Number of blinks
					 * per 10 seconds
					 */
	uint16_t        beacon_duration;/* Beacon duration (in Seconds). The
					 * command operation should be
					 * terminated at the end of this
					 * timeout value.
					 *
					 * Ignored if diag_sub_cmd is
					 * LINK_E2E_BEACON_OFF.
					 *
					 * If 0, beaconing will continue till a
					 * BEACON OFF request is received
					 */
} link_e2e_beacon_param_t;

/*
 * Link E2E beacon request/good response format. For LS_RJTs use fc_ls_rjt_t
 */
typedef struct {
	uint32_t        ls_code;	/*! FC_ELS_E2E_LBEACON in requests *
					 *or FC_ELS_ACC in good replies */
	uint32_t        ls_sub_cmd;	/*! See link_e2e_beacon_subcmd_t */
	link_e2e_beacon_param_t beacon_parm;
} link_e2e_beacon_req_t;

/**
 * If RPSC request is sent to the Domain Controller, the request is for
 * all the ports within that domain (TODO - I don't think FOS implements
 * this...).
 */
typedef struct {
	fc_els_cmd_t    els_cmd;
} fc_rpsc_cmd_t;

/*
 * RPSC Acc
 */
typedef struct {
	uint32_t        command:8;
	uint32_t        rsvd:8;
	uint32_t        num_entries:16;

	fc_rpsc_speed_info_t speed_info[1];
} fc_rpsc_acc_t;

/**
 * bit fields so that multiple classes can be specified
 */
typedef enum {
	FC_CLASS_2	= 0x04,
	FC_CLASS_3	= 0x08,
	FC_CLASS_2_3	= 0x0C,
} fc_cos_t;

/*
 * symbolic name
 */
typedef struct {
	uint8_t         symname[FC_SYMNAME_MAX];
} fc_symname_t;

typedef struct {
	uint8_t         alpa_bm[FC_ALPA_MAX / 8];
} fc_alpabm_t;

/*
 * protocol default timeout values
 */
#define FC_ED_TOV		2
#define FC_REC_TOV		(FC_ED_TOV + 1)
#define FC_RA_TOV		10
#define FC_ELS_TOV		(2 * FC_RA_TOV)

/*
 * virtual fabric related defines
 */
#define FC_VF_ID_NULL    0	/*!< must not be used as VF_ID */
#define FC_VF_ID_MIN     1
#define FC_VF_ID_MAX     0xEFF
#define FC_VF_ID_CTL     0xFEF	/*!< control VF_ID */

/**
 * Virtual Fabric Tagging header format
 * @caution This is defined only in BIG ENDIAN format.
 */
typedef struct {
	uint32_t        r_ctl:8;
	uint32_t        ver:2;
	uint32_t        type:4;
	uint32_t        res_a:2;
	uint32_t        priority:3;
	uint32_t        vf_id:12;
	uint32_t        res_b:1;
	uint32_t        hopct:8;
	uint32_t        res_c:24;
} fc_vft_t;

#pragma pack()

#endif
