/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __FC_SP_H__
#define __FC_SP_H__

#include <protocol/types.h>

#pragma pack(1)

typedef enum {
	FC_AUTH_ELS_MORE_FRAGS_FLAG 	= 0x80,	/*! bit-7. More Fragments
						 * Follow
						 */
	FC_AUTH_ELS_CONCAT_FLAG 	= 0x40,	/*! bit-6. Concatenation Flag */ 	FC_AUTH_ELS_SEQ_NUM_FLAG 	= 0x01 	/*! bit-0. Sequence Number */
} auth_els_flags_t;

typedef enum {
	FC_AUTH_MC_AUTH_RJT		= 0x0A,	/*! Auth Reject */
	FC_AUTH_MC_AUTH_NEG 		= 0x0B, /*! Auth Negotiate */
	FC_AUTH_MC_AUTH_DONE 		= 0x0C, /*! Auth Done */

	FC_AUTH_MC_DHCHAP_CHAL 		= 0x10, /*! DHCHAP Challenge */
	FC_AUTH_MC_DHCHAP_REPLY 	= 0x11, /*! DHCHAP Reply */
	FC_AUTH_MC_DHCHAP_SUCC 		= 0x12, /*! DHCHAP Success */

	FC_AUTH_MC_FCAP_REQ 		= 0x13, /*! FCAP Request */
	FC_AUTH_MC_FCAP_ACK 		= 0x14, /*! FCAP Acknowledge */
	FC_AUTH_MC_FCAP_CONF 		= 0x15, /*! FCAP Confirm */

	FC_AUTH_MC_FCPAP_INIT 		= 0x16, /*! FCPAP Init */
	FC_AUTH_MC_FCPAP_ACC 		= 0x17, /*! FCPAP Accept */
	FC_AUTH_MC_FCPAP_COMP 		= 0x18, /*! FCPAP Complete */

	FC_AUTH_MC_IKE_SA_INIT 		= 0x22, /*! IKE SA INIT */
	FC_AUTH_MC_IKE_SA_AUTH 		= 0x23, /*! IKE SA Auth */
	FC_AUTH_MC_IKE_CREATE_CHILD_SA	= 0x24, /*! IKE Create Child SA */
	FC_AUTH_MC_IKE_INFO 		= 0x25, /*! IKE informational */
} auth_msg_codes_t;

typedef enum {
	FC_AUTH_PROTO_VER_1 	= 1,	/*! Protocol Version 1 */
} auth_proto_version_t;

enum {
	FC_AUTH_ELS_COMMAND_CODE = 0x90,/*! Authentication ELS Command code  */
	FC_AUTH_PROTO_PARAM_LEN_SZ = 4,	/*! Size of Proto Parameter Len Field */
	FC_AUTH_PROTO_PARAM_VAL_SZ = 4,	/*! Size of Proto Parameter Val Field */
	FC_MAX_AUTH_SECRET_LEN     = 256,
					/*! Maximum secret string length */
	FC_AUTH_NUM_USABLE_PROTO_LEN_SZ = 4,
					/*! Size of usable protocols field */
	FC_AUTH_RESP_VALUE_LEN_SZ	= 4,
					/*! Size of response value length */
	FC_MAX_CHAP_KEY_LEN	= 256,	/*! Maximum md5 digest length */
	FC_MAX_AUTH_RETRIES     = 3,	/*! Maximum number of retries */
	FC_MD5_DIGEST_LEN       = 16,	/*! MD5 digest length */
	FC_SHA1_DIGEST_LEN      = 20,	/*! SHA1 digest length */
	FC_MAX_DHG_SUPPORTED    = 1,	/*! Maximum DH Groups supported */
	FC_MAX_ALG_SUPPORTED    = 1,	/*! Maximum algorithms supported */
	FC_MAX_PROTO_SUPPORTED  = 1,	/*! Maximum protocols supported */
	FC_START_TXN_ID         = 2,	/*! Starting transaction ID */
};

typedef enum {
	FC_AUTH_PROTO_DHCHAP		= 0x00000001,
	FC_AUTH_PROTO_FCAP 		= 0x00000002,
	FC_AUTH_PROTO_FCPAP 		= 0x00000003,
	FC_AUTH_PROTO_IKEv2 		= 0x00000004,
	FC_AUTH_PROTO_IKEv2_AUTH 	= 0x00000005,
} auth_proto_id_t;

typedef struct {
	uint16_t	name_tag;	/*! Name Tag = 1 for Authentication */
	uint16_t	name_len;	/*! Name Length = 8 for Authentication
					 */
	wwn_t		name;  		/*! Name. TODO - is this PWWN */
} auth_name_t;


typedef enum {
	FC_AUTH_HASH_FUNC_MD5 		= 0x00000005,
	FC_AUTH_HASH_FUNC_SHA_1 	= 0x00000006,
} auth_hash_func_t;

typedef enum {
	FC_AUTH_DH_GID_0_DHG_NULL	= 0x00000000,
	FC_AUTH_DH_GID_1_DHG_1024	= 0x00000001,
	FC_AUTH_DH_GID_2_DHG_1280	= 0x00000002,
	FC_AUTH_DH_GID_3_DHG_1536	= 0x00000003,
	FC_AUTH_DH_GID_4_DHG_2048	= 0x00000004,
	FC_AUTH_DH_GID_6_DHG_3072	= 0x00000006,
	FC_AUTH_DH_GID_7_DHG_4096	= 0x00000007,
	FC_AUTH_DH_GID_8_DHG_6144	= 0x00000008,
	FC_AUTH_DH_GID_9_DHG_8192	= 0x00000009,
} auth_dh_gid_t;

typedef struct auth_els_msg {
	uint8_t		auth_els_code;	/*!< Authentication ELS Code (0x90) */
	uint8_t 	auth_els_flag; 	/*!< Authentication ELS Flags */
	uint8_t 	auth_msg_code; 	/*!< Authentication Message Code */
	uint8_t 	proto_version; 	/*!< Protocol Version */
	uint32_t	msg_len; 	/*!< Message Length */
	uint32_t	trans_id; 	/*!< Transaction Identifier (T_ID) */

	/* Msg payload follows... */
} auth_els_msg_t;


typedef enum auth_neg_param_tags {
	FC_AUTH_NEG_DHCHAP_HASHLIST 	= 0x0001,
	FC_AUTH_NEG_DHCHAP_DHG_ID_LIST 	= 0x0002,
} auth_neg_param_tags_t;


typedef struct dhchap_param_format {
	uint16_t	tag;		/*! Parameter Tag. See
					 * auth_neg_param_tags_t
					 */
	uint16_t	word_cnt;

	/* followed by variable length parameter value... */
} dhchap_param_format_t;

typedef struct auth_proto_params {
	uint32_t	proto_param_len;
	uint32_t	proto_id;

	/*
	 * Followed by variable length Protocol specific parameters. DH-CHAP
	 * uses dhchap_param_format_t
	 */
} auth_proto_params_t;

typedef struct auth_neg_msg {
	auth_name_t		auth_ini_name;
	uint32_t		usable_auth_protos;
	auth_proto_params_t proto_params[1];	/*! (1..usable_auth_proto)
						 * protocol params
						 */
} auth_neg_msg_t;

typedef struct auth_dh_val {
	uint32_t dh_val_len;
	uint32_t dh_val[1];
} auth_dh_val_t;

typedef struct auth_dhchap_chal_msg {
	auth_els_msg_t	hdr;
	auth_name_t 	auth_responder_name;	/* TODO VRK - is auth_name_t
						 * type OK?
						 */
	uint32_t 	hash_id;
	uint32_t 	dh_grp_id;
	uint32_t 	chal_val_len;
	char		chal_val[1];

	/* ...followed by variable Challenge length/value and DH length/value */
} auth_dhchap_chal_msg_t;


typedef enum auth_rjt_codes_s {
	FC_AUTH_RJT_CODE_AUTH_FAILURE 	= 0x01,
	FC_AUTH_RJT_CODE_LOGICAL_ERR	= 0x02,
} auth_rjt_codes_t;

typedef enum auth_rjt_code_exps_s {
	FC_AUTH_CEXP_AUTH_MECH_NOT_USABLE	= 0x01,
	FC_AUTH_CEXP_DH_GROUP_NOT_USABLE 	= 0x02,
	FC_AUTH_CEXP_HASH_FUNC_NOT_USABLE 	= 0x03,
	FC_AUTH_CEXP_AUTH_XACT_STARTED		= 0x04,
	FC_AUTH_CEXP_AUTH_FAILED 			= 0x05,
	FC_AUTH_CEXP_INCORRECT_PLD 		= 0x06,
	FC_AUTH_CEXP_INCORRECT_PROTO_MSG 	= 0x07,
	FC_AUTH_CEXP_RESTART_AUTH_PROTO 	= 0x08,
	FC_AUTH_CEXP_AUTH_CONCAT_NOT_SUPP 	= 0x09,
	FC_AUTH_CEXP_PROTO_VER_NOT_SUPP 	= 0x0A,
} auth_rjt_code_exps_t;

typedef enum auth_status_s {
	FC_AUTH_STATE_INPROGRESS = 0, 	/*! authentication in progress 	*/
	FC_AUTH_STATE_FAILED	= 1, 	/*! authentication failed */
	FC_AUTH_STATE_SUCCESS	= 2 	/*! authentication successful	*/
} auth_status_t;

typedef struct auth_rjt_msg {
	auth_els_msg_t	hdr;
	uint8_t		reason_code;
	uint8_t		reason_code_exp;
	uint8_t		rsvd[2];
} auth_rjt_msg_t;

typedef auth_els_msg_t auth_done_msg_t; /*! Auth done msg has no payload */

typedef struct auth_dhchap_neg_msg_s {
	auth_els_msg_t hdr;
	auth_neg_msg_t nego;
} auth_dhchap_neg_msg_t;

typedef struct auth_dhchap_reply_msg_s {
	auth_els_msg_t	hdr;

	/*
	 * followed by response value length & Value + DH Value Length & Value
	 */
} auth_dhchap_reply_msg_t;

#pragma pack()

#endif /* __FC_SP_H__ */
