/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __FCPPROTO_H__
#define __FCPPROTO_H__

#include <protocol/scsi.h>

#pragma pack(1)

enum {
	FCP_RJT		= 0x01000000,	/* SRR reject */
	FCP_SRR_ACCEPT	= 0x02000000,	/* SRR accept */
	FCP_SRR		= 0x14000000,	/* Sequence Retransmission Request */
};

/*
 * SRR FC-4 LS payload
 */
typedef struct {
	uint32_t	ls_cmd;
	uint32_t        ox_id:16;	/* ox-id */
	uint32_t        rx_id:16;	/* rx-id */
	uint32_t        ro;		/* relative offset */
	uint32_t        r_ctl:8;		/* R_CTL for I.U. */
	uint32_t        res:24;
} fc_srr_t;


/*
 * FCP_CMND definitions
 */
#define FCP_CMND_CDB_LEN    16
#define FCP_CMND_LUN_LEN    8

typedef struct {
	lun_t           lun;		/* 64-bit LU number */
	uint8_t         crn;		/* command reference number */
#ifdef __BIGENDIAN
	uint8_t         resvd:1,
			priority:4,	/* FCP-3: SAM-3 priority */
			taskattr:3;	/* scsi task attribute */
#else
	uint8_t         taskattr:3,	/* scsi task attribute */
			priority:4,	/* FCP-3: SAM-3 priority */
			resvd:1;
#endif
	uint8_t         tm_flags;	/* task management flags */
#ifdef __BIGENDIAN
	uint8_t         addl_cdb_len:6,	/* additional CDB length words */
			iodir:2;	/* read/write FCP_DATA IUs */
#else
	uint8_t         iodir:2,	/* read/write FCP_DATA IUs */
			addl_cdb_len:6;	/* additional CDB length */
#endif
	scsi_cdb_t      cdb;

	/*
	 * !!! additional cdb bytes follows here!!!
	 */
	uint32_t        fcp_dl;	/* bytes to be transferred */
} fcp_cmnd_t;

#define fcp_cmnd_cdb_len(_cmnd) ((_cmnd)->addl_cdb_len * 4 + FCP_CMND_CDB_LEN)
#define fcp_cmnd_fcpdl(_cmnd)	((&(_cmnd)->fcp_dl)[(_cmnd)->addl_cdb_len])

/*
 * fcp_cmnd_t.iodir field values
 */
typedef enum {
	FCP_IODIR_NONE	= 0,
	FCP_IODIR_WRITE = 1,
	FCP_IODIR_READ	= 2,
	FCP_IODIR_RW	= 3,
} fcp_iodir_t;

/*
 * Task attribute field
 */
enum {
	FCP_TASK_ATTR_SIMPLE	= 0,
	FCP_TASK_ATTR_HOQ	= 1,
	FCP_TASK_ATTR_ORDERED	= 2,
	FCP_TASK_ATTR_ACA	= 4,
	FCP_TASK_ATTR_UNTAGGED	= 5,	/* obsolete in FCP-3 */
};

/*
 * Task management flags field - only one bit shall be set
 */
#define BIT(_x)	(1 << (_x))
typedef enum {
	FCP_TM_ABORT_TASK_SET	= BIT(1),
	FCP_TM_CLEAR_TASK_SET	= BIT(2),
	FCP_TM_LUN_RESET	= BIT(4),
	FCP_TM_TARGET_RESET	= BIT(5),	/* obsolete in FCP-3 */
	FCP_TM_CLEAR_ACA	= BIT(6),
} fcp_tm_cmnd_t;

/*
 * FCP_XFER_RDY IU defines
 */
typedef struct {
	uint32_t        data_ro;
	uint32_t        burst_len;
	uint32_t        reserved;
} fcp_xfer_rdy_t;

/*
 * FCP_RSP residue flags
 */
typedef enum {
	FCP_NO_RESIDUE = 0,	/* no residue */
	FCP_RESID_OVER = 1,	/* more data left that was not sent */
	FCP_RESID_UNDER = 2,	/* less data than requested */
} fcp_residue_t;

enum {
	FCP_RSPINFO_GOOD = 0,
	FCP_RSPINFO_DATALEN_MISMATCH = 1,
	FCP_RSPINFO_CMND_INVALID = 2,
	FCP_RSPINFO_ROLEN_MISMATCH = 3,
	FCP_RSPINFO_TM_NOT_SUPP = 4,
	FCP_RSPINFO_TM_FAILED = 5,
};

typedef struct {
	uint32_t        res0:24;
	uint32_t        rsp_code:8;	/* response code (as above) */
	uint32_t        res1;
} fcp_rspinfo_t;

typedef struct {
	uint32_t        reserved[2];	/* 2 words reserved */
	uint16_t        reserved2;
#ifdef __BIGENDIAN
	uint8_t         reserved3:3;
	uint8_t         fcp_conf_req:1;	/* FCP_CONF is requested */
	uint8_t         resid_flags:2;	/* underflow/overflow */
	uint8_t         sns_len_valid:1;/* sense len is valid */
	uint8_t         rsp_len_valid:1;/* response len is valid */
#else
	uint8_t         rsp_len_valid:1;/* response len is valid */
	uint8_t         sns_len_valid:1;/* sense len is valid */
	uint8_t         resid_flags:2;	/* underflow/overflow */
	uint8_t         fcp_conf_req:1;	/* FCP_CONF is requested */
	uint8_t         reserved3:3;
#endif
	uint8_t         scsi_status;	/* one byte SCSI status */
	uint32_t        residue;	/* residual data bytes */
	uint32_t        sns_len;	/* length od sense info */
	uint32_t        rsp_len;	/* length of response info */
} fcp_resp_t;

#define fcp_snslen(__fcprsp)	((__fcprsp)->sns_len_valid ? 		\
					(__fcprsp)->sns_len : 0)
#define fcp_rsplen(__fcprsp)	((__fcprsp)->rsp_len_valid ? 		\
					(__fcprsp)->rsp_len : 0)
#define fcp_rspinfo(__fcprsp)	((fcp_rspinfo_t *)((__fcprsp) + 1))
#define fcp_snsinfo(__fcprsp)	(((uint8_t *)fcp_rspinfo(__fcprsp)) + 	\
						fcp_rsplen(__fcprsp))

typedef struct {
	fchs_t          fchs;
	fcp_cmnd_t      fcp;
} fcp_cmnd_fr_t;

#pragma pack()

#endif
